/*****************************************************************************
* @file     TestCode_ADC_Compare.c
* @brief    Contains the external function prototypes for TestCode
* @author   AE Team, ABOV Semiconductor Co., Ltd.
* @version  V0.0.1
* @date     16. MAY. 2019
*
* Copyright(C) 2019, ABOV Semiconductor
* All rights reserved.
*
*
********************************************************************************
* DISCLAIMER
*
* THE PRESENT FIRMWARE WHICH IS FOR GUIDANCE ONLY AIMS AT PROVIDING CUSTOMERS
* WITH CODING INFORMATION REGARDING THEIR PRODUCTS IN ORDER FOR THEM TO SAVE
* TIME. AS A RESULT, ABOV SEMICONDUCTOR DISCLAIMS ALL LIABILITIES FROM ANY
* DIRECT, INDIRECT OR CONSEQUENTIAL DAMAGES WITH RESPECT TO ANY CLAIMS ARISING
* FROM THE CONTENT OF SUCH FIRMWARE AND/OR THE USE MADE BY CUSTOMERS OF THE
* CODING INFORMATION CONTAINED HEREIN IN CONNECTION WITH THEIR PRODUCTS.
*
*******************************************************************************/

/*******************************************************************************
* Pre-processor Definition & Macro
*******************************************************************************/
#include "TestCode_ADC_Compare.h"
#include "main_conf.h"

/*******************************************************************************
* Private Typedef
*******************************************************************************/

/*******************************************************************************
* Private Variable
*******************************************************************************/
static uint32_t MilliSec;
static uint32_t ADCData[8];
static ADC_CFG_Type ADC_Config;
static ADC_CMP_CFG_Type CMPConfig;

static Bool DoneFlag;
static Bool CmpFlag;

/*******************************************************************************
* Private Function Prototype
*******************************************************************************/

/*******************************************************************************
* Public Function
*******************************************************************************/
/*******************************************************************************
* @brief      SysTick interrupt handler @ 1000Hz(1ms)
* @param      None
* @return     None
*******************************************************************************/
void SysTick_Handler_ADC_Compare(void) {
	if (MilliSec) {
		MilliSec--;
	}
}

/*******************************************************************************
* @brief      ADC interrupt handler
* @param      None
* @return     None
*******************************************************************************/
void ADC_IRQHandler_ADC_Compare(void) {
	ADC_STATUS_Type status; // check interrupt flag

	status = HAL_ADC_GetStatus(ADC);

	DoneFlag = TRUE;

	// EOC Flag
	if (status & ADC_STATUS_SINGLE_INT) {
		HAL_ADC_ClearStatus(ADC, ADC_STATUS_SINGLE_INT);
	}
	// EOS Flag
	if (status & ADC_STATUS_SEQUENCE_INT) {
		HAL_ADC_ClearStatus(ADC, ADC_STATUS_SEQUENCE_INT);
	}
	// Trigger Flag
	if (status & ADC_STATUS_TRIGGER_INT) {
		HAL_ADC_ClearStatus(ADC, ADC_STATUS_TRIGGER_INT);
	}
	// Compare Flag
	if (status & ADC_STATUS_COMPARE_INT) {
		CmpFlag = TRUE;
		HAL_ADC_ClearStatus(ADC, ADC_STATUS_COMPARE_INT);
	}    
}

/*******************************************************************************
* @brief      Main_AdcSampleCompare
* @param      None
* @return     None
*******************************************************************************/
void Main_AdcSampleCompare(void) {
	uint32_t Average;

//    SYSTICK_IRQ_Call_F = SysTick_Handler_ADC_Compare;
//    ADC_IRQ_Call_F = ADC_IRQHandler_ADC_Compare;
    
	// Set ADC input pin
	HAL_ADC_PortInit(0);
//	HAL_ADC_PortInit(1);
//	HAL_ADC_PortInit(2);
//	HAL_ADC_PortInit(3);
//	HAL_ADC_PortInit(4);
//	HAL_ADC_PortInit(5);
//	HAL_ADC_PortInit(6);
//	HAL_ADC_PortInit(7);
//	HAL_ADC_PortInit(8);
//	HAL_ADC_PortInit(9);
//	HAL_ADC_PortInit(10);
//	HAL_ADC_PortInit(11);
//	HAL_ADC_PortInit(12);
//	HAL_ADC_PortInit(13);
//	HAL_ADC_PortInit(14);
////	HAL_ADC_PortInit(15); // Excluded this channel for in use as UART1 debug port (FPGA)
//	HAL_ADC_PortInit(16);
//	HAL_ADC_PortInit(17);

	// Set ADC external clock
	HAL_SCU_HSI_ClockConfig(HSI_EN);
	HAL_SCU_MiscClockConfig(7, ADC_TYPE, CLKSRC_HSI, 8);   // 32/8 = 4MHz

	// Initialize and configure ADC
	ADC_Config.TriggerInfo = DISABLE;
	ADC_Config.ChannelInfo = ENABLE;
	ADC_Config.Dma = DISABLE;
	ADC_Config.Restart = DISABLE;

	ADC_Config.Mode = ADC_MODE_SINGLE;
	ADC_Config.TriggerSelection = ADC_TRIGGER_SEL_OLNY_SOFT;
	ADC_Config.SamplingTime = 1;
	ADC_Config.SequenceCounter = ADC_SEQ_NUM_1;
	ADC_Config.ExternalClock = ENABLE;
	ADC_Config.InternalClockDivider = 1;
	HAL_ADC_Init(ADC, &ADC_Config);
	HAL_ADC_ConfigureInterrupt(ADC, ADC_INTERRUPT_SINGLE, ENABLE);

	// the Setting of A/D Comparison
	CMPConfig.Channel = ADC_CHANNEL_0;
	CMPConfig.CompareValue = 0x800; // Set A/D reference value for comparison
	CMPConfig.Interrupt = ENABLE;
	CMPConfig.Operation = DISABLE;
	CMPConfig.TriggerSmall = ENABLE; // Condition : Measured Value < Reference Value
//	CMPConfig.TriggerSmall = DISABLE; // Condition : Measured Value > Reference Value
	HAL_ADC_ConfigureCompare(ADC, &CMPConfig);

	// Enable SysTick and ADC interrupt
	SysTick_Config(SystemCoreClock / 1000);

	NVIC_SetPriority(ADC_IRQn, 3);
	NVIC_EnableIRQ(ADC_IRQn);

	__enable_irq();

	// Start ADC test
	while(1) {
		HAL_ADC_ConfigureOneSequence(ADC, ADC_SEQUENCE_0, (ADC_CHANNEL_Type)(ADC_CHANNEL_0));
		DoneFlag = FALSE;

		HAL_ADC_Start(ADC); // START A/D Conversion
		HAL_ADC_CompareStart(ADC);	// Start A/D Comparison of A/D Comparison value with reference value.

		while(DoneFlag == FALSE);
		ADCData[0] = HAL_ADC_GetConversionData(ADC, ADC_DATA_SEL_LAST);

		cputs("--------------------------------------------------\r\n");
		cprintf("Selected Channel: AN%d\r\n", 0);

		Average = ADCData[0];
		cprintf("ADC Value: %04X\r\n", Average);

		// The comparison result of A and B is output.
		if (CmpFlag) { // Flag was occurred,
			if ((ADC->CMPR & ADC_CMPR_LTE_Msk) == ADC_CMPR_LTE_Msk) {
				cprintf("Measured Value is smaller than %04X\r\n", HAL_ADC_GetReferenceData(ADC));
			} else {
                cprintf("Measured Value is Larger than %04X\r\n", HAL_ADC_GetReferenceData(ADC));
			}
			CmpFlag = FALSE;
		} else { // When Flag wasn't occurred.
			if ((ADC->CMPR & ADC_CMPR_LTE_Msk) == ADC_CMPR_LTE_Msk) {
				cprintf("Measured Value is Larger than %04X\r\n", HAL_ADC_GetReferenceData(ADC));
			} else {
                cprintf("Measured Value is Smaller than %04X\r\n", HAL_ADC_GetReferenceData(ADC));
			}
		}

		// 1 second delay
		MilliSec = 1000;
		while(MilliSec);
	}
}

/* --------------------------------- End Of File ------------------------------ */
