/*****************************************************************************
* @file     TestCode_ADC_DmaTx.c
* @brief    Contains the external function prototypes for TestCode
* @author   AE Team, ABOV Semiconductor Co., Ltd.
* @version  V0.0.1
* @date     16. MAY. 2019
*
* Copyright(C) 2019, ABOV Semiconductor
* All rights reserved.
*
*
********************************************************************************
* DISCLAIMER
*
* THE PRESENT FIRMWARE WHICH IS FOR GUIDANCE ONLY AIMS AT PROVIDING CUSTOMERS
* WITH CODING INFORMATION REGARDING THEIR PRODUCTS IN ORDER FOR THEM TO SAVE
* TIME. AS A RESULT, ABOV SEMICONDUCTOR DISCLAIMS ALL LIABILITIES FROM ANY
* DIRECT, INDIRECT OR CONSEQUENTIAL DAMAGES WITH RESPECT TO ANY CLAIMS ARISING
* FROM THE CONTENT OF SUCH FIRMWARE AND/OR THE USE MADE BY CUSTOMERS OF THE
* CODING INFORMATION CONTAINED HEREIN IN CONNECTION WITH THEIR PRODUCTS.
*
*******************************************************************************/

/*******************************************************************************
* Pre-processor Definition & Macro
*******************************************************************************/
#include "TestCode_ADC_DmaTx.h"
#include "main_conf.h"
/*******************************************************************************
* Private Typedef
*******************************************************************************/

/*******************************************************************************
* Private Variable
*******************************************************************************/
static uint32_t MilliSec;
static ADC_CFG_Type ADC_Config;

static Bool DoneFlag;
static Bool DMA_Done;

 #define DMA_DATA_SIZE		(1)
//#define DMA_DATA_SIZE			(0xFFFUL)

#define DMA_ADDR_START	(0x20003000UL) // DMA Source Address for Test (SRAM)
#define DMA_ADDR_END	(0x20005000UL) // DMA Destination Address for Test (SRAM)
#define DMA_ADDR_PERI 	(0x40003450UL) // Peripheral Register for DMA Peri-to-Peri
#define DMA_ADDR_TARGET (ADC->TRG)	 // DMA Target Address for DMA Tx Test

#define DMA_TEST_CH			(DMAC0)
//#define DMA_TEST_CH			(DMAC1)
//#define DMA_TEST_CH			(DMAC2)
//#define DMA_TEST_CH			(DMAC3)
//#define DMA_TEST_CH			(DMAC4)
//#define DMA_TEST_CH			(DMAC5)
//#define DMA_TEST_CH			(DMAC6)
//#define DMA_TEST_CH			(DMAC7)

/*******************************************************************************
* Private Function Prototype
*******************************************************************************/

/*******************************************************************************
* Public Function
*******************************************************************************/
/*******************************************************************************
* @brief      SysTick interrupt handler @ 1000Hz(1ms)
* @param      None
* @return     None
*******************************************************************************/
void SysTick_Handler_ADC_DmaTx(void) {
	if (MilliSec) {
		MilliSec--;
	}
}

/*******************************************************************************
* @brief      ADC interrupt handler
* @param      None
* @return     None
*******************************************************************************/
void ADC_IRQHandler_ADC_DmaTx(void) {
	ADC_STATUS_Type status; // check interrupt flag
       
	status = HAL_ADC_GetStatus(ADC);

	DoneFlag = TRUE;

	// EOC Flag
	if (status & ADC_STATUS_SINGLE_INT) {
		HAL_ADC_ClearStatus(ADC, ADC_STATUS_SINGLE_INT);
	}
	// EOS Flag
	if (status & ADC_STATUS_SEQUENCE_INT) {
		HAL_ADC_ClearStatus(ADC, ADC_STATUS_SEQUENCE_INT);
	}
	// Trigger Flag
	if (status & ADC_STATUS_TRIGGER_INT) {
		HAL_ADC_ClearStatus(ADC, ADC_STATUS_TRIGGER_INT);
	}

	// DMA Done Flag
	if (status & ADC_STATUS_DMA_DONE) {
		DMA_Done = TRUE;		
		HAL_ADC_ClearStatus(ADC, ADC_STATUS_DMA_DONE);
		HAL_DMAC_SetTransferCount(DMA_TEST_CH, DMA_DATA_SIZE);
	}
	// DMA Overrun Flag
	if (status & ADC_STATUS_DMA_OVER) {
		HAL_ADC_ClearStatus(ADC, ADC_STATUS_DMA_OVER);
	}	    
}

/*******************************************************************************
* @brief      Main_AdcDma2Peri
* @param      None
* @return     None
*******************************************************************************/
void Main_AdcDma2Peri(void) {
	int i;
	uint32_t ADC_Data;
	DMAC_CFG_Type DMA_Config;

	// Set ADC input pin
	HAL_ADC_PortInit(0);

	// Set ADC external clock
	HAL_SCU_HSI_ClockConfig(HSI_EN);
	HAL_SCU_MiscClockConfig(7, ADC_TYPE, CLKSRC_HSI, 8);

	// Initialize and configure ADC
	ADC_Config.TriggerInfo = DISABLE;
	ADC_Config.ChannelInfo = ENABLE;
	ADC_Config.Dma = DISABLE; // DMA Enable
	ADC_Config.Restart = DISABLE;
	ADC_Config.Mode = ADC_MODE_SINGLE;
	ADC_Config.TriggerSelection = ADC_TRIGGER_SEL_OLNY_SOFT;
	ADC_Config.SamplingTime = 1;
	ADC_Config.SequenceCounter = ADC_SEQ_NUM_1;
	ADC_Config.ExternalClock = ENABLE;
	ADC_Config.InternalClockDivider = 1;
	HAL_ADC_Init(ADC, &ADC_Config);

	// DMA Configuration to use DAC Interrupt
	DMA_Config.Peripheral = DMAC_PERIPHERAL_ADC;					// Selected DMA ADC
	DMA_Config.Width = DMAC_WIDTH_HALF_WORD;						// Half-Word byte transfer (16-bit)
	DMA_Config.Direction = DMAC_DIRECTION_TX;						// Memory to Peripheral
	DMA_Config.PeripheralAddress = (uint32_t)&DMA_ADDR_TARGET;		// ADC Data register
	HAL_DMAC_Init(DMA_TEST_CH, &DMA_Config);						// DMA Channel Initization

	// Set DMA Memory Address
	HAL_DMAC_SetMemoryAddress(DMA_TEST_CH, DMA_ADDR_START);			// SET Memory Address

	// DMA Interrupt, EOS Interrupt Enable
	HAL_ADC_ConfigureInterrupt(ADC, (ADC_INTERRUPT_Type)(ADC_INTERRUPT_SINGLE | ADC_INTERRUPT_DMA), ENABLE);

	// Enable SysTick and ADC interrupt
	SysTick_Config(SystemCoreClock / 1000);

	NVIC_SetPriority(ADC_IRQn, 3);
	NVIC_EnableIRQ(ADC_IRQn);

	__enable_irq();

	// Clear SRAM area
	for (i = DMA_ADDR_START; i < DMA_ADDR_END; i+=4) {
		*(volatile uint32_t *)i = 0x00000000;
	}

	_DBG("ADC->TRG values are wrote at DMA memory address\r\n");
	*(volatile uint32_t *)DMA_ADDR_START = (0xF120);			// Write the value to memory


	// Start ADC test
	while(1) {             
		HAL_ADC_ConfigureOneSequence(ADC, ADC_SEQUENCE_0, (ADC_CHANNEL_Type)(ADC_CHANNEL_0));
		DoneFlag = FALSE;

		HAL_ADC_Start(ADC);
		HAL_ADC_DmaEnable(ADC);	// ADC DMA Enable should be set after ADC_CR<EN> is set to 1.

		while(DoneFlag == FALSE);
		ADC_Data = HAL_ADC_GetConversionData(ADC, ADC_DATA_SEL_LAST);	// Get A/D Conversion Data

		cprintf("ADC Value (hex): %04X\r\n", (ADC_Data & 0x0FFFUL));

		DMA_Done = FALSE;
		HAL_DMAC_SetTransferCount(DMA_TEST_CH, 1);							// 16-bit data will be write

		HAL_DMAC_Start(DMA_TEST_CH);										// DMA Ch.0 Operation Start

		while (DMA_Done == FALSE);										    // Check DMA Flag
		HAL_DMAC_Stop(DMA_TEST_CH);										    // If DMA is done, Stop the DMA operation

		// 1 second delay
		MilliSec = 1000;
		while (MilliSec);
	}
}

/* --------------------------------- End Of File ------------------------------ */
