#include "A31G22x_hal_scu.h"
#include "A31G22x_hal_pcu.h"

#define VECTOR_ADDR     (0x00002000UL)

volatile uint32_t msec;

void PCU_Init(void);
void SCU_ClockInit(void);

/**********************************************************************
 * @brief		SysTick handler sub-routine (1ms)
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void SysTick_Handler (void) 					// SysTick Interrupt Handler @ 1000Hz
{
	if(msec)msec--;
}

void mainloop(void)
{
	uint32_t i;
	
    for (i = 2; i < 8 ; i++) {
        HAL_GPIO_ConfigOutput(PE, i, PCU_MODE_PUSH_PULL);      // PE2 to PE7 GPIO Output
        HAL_GPIO_ConfigPullup(PE, i, PCU_PUPD_DISABLE);       // Pull-up/Pull-down Disable
        HAL_GPIO_ClearPin(PE, _BIT(i));                           // Output Low
    } 
    
	SysTick_Config(SystemCoreClock/1000);  //1 ms interrupt

	__enable_irq();	
	
	while(1) {
		msec = 100;
		while(msec);
		
		for(i = 2; i < 8; i++) {
			msec = 50;
			while(msec);
			HAL_GPIO_ClearPin(PE,_BIT(i));
		}
		
		for(i = 2; i < 8; i++) {
			msec = 50;
			while(msec);
			HAL_GPIO_SetPin(PE,_BIT(i));
		}
	}
}

int main(void)
{
  __disable_irq();	
	
	SCB->VTOR = VECTOR_ADDR;    //Vector Table Offset Register
	
	PCU_Init();                 // all port control setting  !!You have to set PORT function!!
	HAL_SCU_SetSystemClock(SYSCLK_HSI, SYSDIV_NONE);

	mainloop();
	
	return (0);
}
