#include "A31G22x_hal_pcu.h"
#include "A31G22x_hal_scu.h"
#include "A31G22x_hal_uartn.h"
#include "A31G22x_hal_cfmc.h"
#include "A31G22x_hal_usart1n.h"
#include "defboot.h"

/**********************************************************************
// Main Clock Selection
 **********************************************************************/
// #define USED_UART



// Assembly jump code
extern void _fjump(void);

void BootMain(void);

// system default I/O setting
extern void PCU_Init(void);

int	flash_erase_sector(unsigned long addr);
int	flash_write_sector(unsigned long addr,unsigned long *pdata);

#ifdef USED_UART
extern uint8_t uart_mode;                                            /*< 0:UART Polling / 1:UART Interrupt     */
#else
extern uint8_t usart_mode;                                           /*< 0:USART Polling / 1:USART Interrupt     */
#endif

DEV_INFO wrt;

int main(void)
{
	__disable_irq();
	
	BootMain();
	
	return (0);
}

void DelayMS(uint32_t t)			// Delay Time
{
	uint32_t i;
	
	do
	{
		for (i=0; i<4000; i++);
	} while(t--);
}

#ifdef USED_UART
void boot_uputch(uint8_t ch)
{
	while ((UART1->LSR & UART_LSR_TEMT) == 0)		
	{
		
	}

	UART1->THR = ch;
}

uint8_t boot_ugetch(void)
{
	uint8_t ch;
	
	while ((UART1->LSR & UART_LSR_RDR)==0)
	{
		
	}
	ch =  UART1->RBR;
	
	return ch;
}

uint8_t boot_ugetchar(uint8_t *ch)
{
	if( (UART1->LSR & UART_LSR_RDR) == UART_LSR_RDR)
	{
		*ch = UART1->RBR;
		return 1;
	}
	*ch=0;

	return 0;
}
#else // USART10
void boot_uputch(uint8_t ch)
{
	while(!(USART10->ST & USART_ST_DRE)){ }
	USART10->DR = ch;
}

uint8_t boot_ugetch(void)
{
	uint8_t ch;

	while(!(USART10->ST & USART_ST_RXC)){ }	
	ch =  USART10->DR;
	
	return ch;
}

uint8_t boot_ugetchar(uint8_t *ch)
{
	if( (USART10->ST & USART_ST_RXC) == USART_ST_RXC)
	{
		*ch = USART10->DR;
		return 1;
	}
	*ch=0;

	return 0;
}
#endif

uint32_t PageAddress;
uint32_t RealPageAddress;

void GetPageNumber(void)
{
	uint8_t PageAddressHigh;
	uint8_t PageAddressLow;
	
	PageAddressHigh = boot_ugetch();
	PageAddressLow = boot_ugetch();

	RealPageAddress = ((uint16_t)PageAddressHigh) * 0x100;
	RealPageAddress += PageAddressLow;

	PageAddress = ((uint32_t)RealPageAddress) << NSHIFTPAGE;
	
	boot_uputch('!');
}

/*****************************************************************************/
char WriteFlashPage(void)
{
	uint16_t i;
	uint8_t CheckSum;
	uint8_t tt;
	
	uint8_t rxdata[PageByte];

	CheckSum = 0;
	
	for (i = 0; i < PageByte; i++)
	{ 
		rxdata[i] = boot_ugetch();
	}	
	tt=boot_ugetch();

	for (i = 0; i < PageByte; i++)
	{ 
		CheckSum += rxdata[i];
	}
	
	if (CheckSum != tt)
	{
		return 0;
	}

	if (PageAddress < BOOTCODE_SIZE) // 0x0000~0x2000 is bootcode area, so we do not erase and write
		return 1;
		
	if (flash_erase_sector(PageAddress)) {
		return 0; //fail
    }
	if (flash_write_sector(PageAddress, (unsigned long *)rxdata)) {
		return 0; //fail
    }
	
	return 1;
}

void BootMain(void)
{
	uint8_t ch;
	uint8_t cnt;
#ifdef USED_UART
	UART_CFG_Type UARTConfigStruct;
#else
	USART_CFG_Type USARTConfigStruct; 
#endif
	
	PCU_Init(); // all port control setting  !!You have to set PORT function!!
//	SCU_ClockInit();
    HAL_SCU_SetSystemClock(SYSCLK_HSI, SYSDIV_NONE);    // Set System clock with HSI (32MHz)
	

#ifdef USED_UART
    uart_mode = USART_POLLING;
	HAL_UART_ConfigStructInit(&UARTConfigStruct); //38400-8-N-1	
	HAL_UART_Init(UART1, &UARTConfigStruct);
#else 
    usart_mode = USART_POLLING;
	HAL_USART_UART_Mode_Config(&USARTConfigStruct);	
	HAL_USART_Init(USART10, &USARTConfigStruct);
    HAL_USART_Enable(USART10, ENABLE);
#endif

	boot_uputch('>'); //start protocol
	for (cnt = 0; cnt < 100; cnt++)
	{
		boot_ugetchar(&ch);
		if (ch=='<') break;

		DelayMS(1); // about 1msec
	}
	
	if (cnt < 100) // program update start
	{
		wrt.pagesize = PageSize;
		wrt.deviceid.word = DeviceID;
		wrt.flashsize = FlashSize;
		wrt.bootsize = BootSize;	
		
		boot_uputch(wrt.pagesize);
		boot_uputch(wrt.deviceid.byte[0]);
		boot_uputch(wrt.deviceid.byte[1]);
		boot_uputch(wrt.flashsize);
		boot_uputch(wrt.bootsize);
		
		boot_uputch('!');

		while (1)
		{
			GetPageNumber();

			if (RealPageAddress == (uint32_t)END_ADDR)
			{
				__nop();
				goto EXIT;
			}

			if (WriteFlashPage())
			{
				boot_uputch('!');
			}
			else 
			{
				boot_uputch('@');
			}
		}
		
EXIT:
		;										
	}
	
	_fjump();										// jump to application	
}


int	flash_erase_sector(unsigned long addr)
{
	int	result;

    result = HAL_CFMC_EraseSector(0, addr);
	
	return result;
}

#define SECTOR_SIZE (0x200)

int	flash_write_sector(unsigned long addr, unsigned long *pdata)
{
	int	result;
	unsigned char	*fbuffer;

	fbuffer = (unsigned char *)(pdata);
    result = HAL_CFMC_ProgramPage(0, addr, SECTOR_SIZE, fbuffer);
	
	return result;
}
