#include "A31G22x_hal_scu.h"
#include "A31G22x_hal_pcu.h"
#include "A31G22x_hal_cfmc.h"
#include "debug_frmwrk.h"
#include "debug_cmd.h"
#include "debug.h"

#define VECTOR_ADDR     (0x00002000UL)

volatile uint32_t MilliSec;

void PCU_Init(void);
void SCU_ClockInit(void);

/**********************************************************************
 * @brief		SysTick handler sub-routine (1ms)
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void SysTick_Handler (void) 					// SysTick Interrupt Handler @ 1000Hz
{
	if (MilliSec) {
        MilliSec--;
    }
}

const char Memoryswap_bank0[] = 
"\r\n"
"   / __ )/   |  / | / / //_/   / ___/ |     / /   |  / __ \\   / __ \\  \r\n"
"  / __  / /| | /  |/ / ,<      \\__ \\| | /| / / /| | / /_/ /  / / / /  \r\n"
" / /_/ / ___ |/ /|  / /| |    ___/ /| |/ |/ / ___ |/ ____/  / /_/ /   \r\n"
"/_____/_/ _|_/_/ |_/_/_|_|__ /____/_|__/|__/_/__|_/_/       \\____/    \r\n\r\n"
;

const char Memoryswap_bank1[] =
"\r\n"
"   / __ )/   |  / | / / //_/   / ___/ |     / /   |  / __ \\   <  /    \r\n"
"  / __  / /| | /  |/ / ,<      \\__ \\| | /| / / /| | / /_/ /   / /     \r\n"
" / /_/ / ___ |/ /|  / /| |    ___/ /| |/ |/ / ___ |/ ____/   / /      \r\n"
"/_____/_/  |_/_/ |_/_/ |_|   /____/ |__/|__/_/  |_/_/       /_/       \r\n\r\n"
;                                                                    

void mainloop(void)
{
	uint32_t i;

    for (i = 2; i < 8 ; i++) {
        HAL_GPIO_ConfigOutput(PE, i, PCU_MODE_PUSH_PULL);      // PE2 to PE7 GPIO Output
        HAL_GPIO_ConfigPullup(PE, i, PCU_PUPD_DISABLE);        // Pull-up/Pull-down Disable
        HAL_GPIO_ClearPin(PE, _BIT(i));                        // Output Low
    } 
    
    debug_frmwrk_init();
    
    cprintf("Application Start\r\n");
    
	SysTick_Config(SystemCoreClock/1000);  //1 ms interrupt
	__enable_irq();	
    
	if ((HAL_CFMC_GetBankSwapStatus() & CFMC_BSR_BSWST_Msk) == CFMC_BSR_BSWST_Msk) {
        // When the current boot area is Bank 1, LED blinks from PE7 to PE2
        cputs((char *)Memoryswap_bank1);
                
        while(1){
            MilliSec = 100;
            while(MilliSec);
            
            for(i = 7; i > 1; i--) {
                MilliSec = 50;
                while(MilliSec);
                HAL_GPIO_ClearPin(PE,_BIT(i));
            }
            
            for(i = 7; i > 1; i--) {
                MilliSec = 50;
                while(MilliSec);
                HAL_GPIO_SetPin(PE,_BIT(i));
            }		
        }

    } else {
        // When the current boot area is Bank 0, LED blinks from PE2 to PE7
         cputs((char *)Memoryswap_bank0);
        // Blink LED PE2 to PE7
        while(1){
            MilliSec = 100;
            while(MilliSec);
            
            for(i = 2; i < 8; i++){
                MilliSec = 50;
                while(MilliSec);
                HAL_GPIO_ClearPin(PE,_BIT(i));
            }
            
            for(i = 2; i < 8; i++){
                MilliSec = 50;
                while(MilliSec);
                HAL_GPIO_SetPin(PE,_BIT(i));
            }
        }
    }
    
}

int main(void)
{
    __disable_irq();

    SCB->VTOR = VECTOR_ADDR; //Vector Table Offset Register

    PCU_Init(); // all port control setting  !!You have to set PORT function!!
    HAL_SCU_SetSystemClock(SYSCLK_HSI, SYSDIV_NONE);

    mainloop();

    return (0);
}
