#include "A31G22x_hal_pcu.h"
#include "A31G22x_hal_scu.h"
#include "A31G22x_hal_uartn.h"
#include "A31G22x_hal_cfmc.h"
#include "A31G22x_hal_usart1n.h"
#include "defboot.h"
#include "MemorySwap.h"

/**********************************************************************
// TARGET and Memory Information
 **********************************************************************/
// #define USED_UART               // Use UART Channel

/**********************************************************************
// Prototype
 **********************************************************************/
void BootMain(void);                                                 /*< Boot Main routine      */
extern void PCU_Init(void);                                          /*< Port Initialization    */
int flash_erase_sector(unsigned long addr);                          /*< Erase Sector           */
int flash_write_sector(unsigned long addr,unsigned long *pdata);     /*< Program Sector         */

typedef void (*fptr)(void);                                          /*< function pointer type definition */
void application_start(uint32_t addr);                               /*< Application pointer function     */

#ifdef USED_UART
extern uint8_t uart_mode;                                            /*< 0:UART Polling / 1:UART Interrupt     */
#else
extern uint8_t usart_mode;                                           /*< 0:USART Polling / 1:USART Interrupt     */
#endif
/**********************************************************************
// Variable
 **********************************************************************/
DEV_INFO wrt;

volatile fptr jump_to_app;               // function pointer variable
volatile uint32_t jump_addr;             // Jump Address

/**********************************************************************
// Function
 **********************************************************************/
int main(void)
{
	__disable_irq();
	
	BootMain();
	
	return (0);
}

void DelayMS(uint32_t t)			// Delay Time 
{
	uint32_t i;
	
	do
	{
		for (i=0; i<4000; i++); 
	} while(t--);
}

#ifdef USED_UART
void boot_uputch(uint8_t ch)
{
	while ((UART1->LSR & UART_LSR_TEMT) == 0)		
	{
		
	}

	UART1->THR = ch;
}

uint8_t boot_ugetch(void)
{
	uint8_t ch;
	
	while ((UART1->LSR & UART_LSR_RDR)==0)
	{
		
	}
	ch =  UART1->RBR;
	
	return ch;
}

uint8_t boot_ugetchar(uint8_t *ch)
{
	if( (UART1->LSR & UART_LSR_RDR) == UART_LSR_RDR)
	{
		*ch = UART1->RBR;
		return 1;
	}
	*ch=0;

	return 0;
}
#else // USART10
void boot_uputch(uint8_t ch)
{
	while(!(USART10->ST & USART_ST_DRE)) { }
	USART10->DR = ch;
}

uint8_t boot_ugetch(void)
{
	uint8_t ch;

	while(!(USART10->ST & USART_ST_RXC)) { }	
	ch =  USART10->DR;
	
	return ch;
}

uint8_t boot_ugetchar(uint8_t *ch)
{
	if( (USART10->ST & USART_ST_RXC) == USART_ST_RXC)
	{
		*ch = USART10->DR;
		return 1;
	}
	*ch=0;

	return 0;
}
#endif

uint32_t PageAddress;
uint32_t RealPageAddress;

void GetPageNumber(void)
{
	uint8_t PageAddressHigh;
	uint8_t PageAddressLow;
	
	PageAddressHigh = boot_ugetch();
	PageAddressLow = boot_ugetch();

	RealPageAddress = ((uint16_t)PageAddressHigh) * 0x100;
	RealPageAddress += PageAddressLow;

	PageAddress = ((uint32_t)RealPageAddress) << NSHIFTPAGE;
	
	boot_uputch('!');
}

/*****************************************************************************/
char WriteFlashPage(void)
{
	uint16_t i;
	uint8_t CheckSum;
	uint8_t tt;
	
	uint8_t rxdata[PageByte];

	CheckSum = 0;
	
	for (i = 0; i < PageByte; i++)
	{ 
		rxdata[i] = boot_ugetch();
	}	
	tt = boot_ugetch();

	for (i = 0; i < PageByte; i++)
	{ 
		CheckSum += rxdata[i];
	}
	
	if (CheckSum != tt)
	{
		return 0;
	}

    // Set Erase and Write Region
    // Boot Code Area that not use memory swap: 0x0000 ~ 0x2000.
    // Boot Code Area that use memory swap of A31G226: 0x20000 ~ 0x22000
    // Boot Code Area that use memory swap of A31G224: 0x10000 ~ 0x12000
    #ifndef MEMORY_SWAP
	if (PageAddress < APP_PGM_OFFSET) // Code Erase and Write Access except Boot Code Area
		return 1;
    #endif
    
	if (flash_erase_sector(PageAddress + APP_PGM_OFFSET)) {
		return 0; //fail
    }
	if (flash_write_sector(PageAddress + APP_PGM_OFFSET, (unsigned long *)rxdata)) {
		return 0; //fail
    }

	return 1;
}

void application_start(uint32_t addr) {
    jump_addr = *(__IO uint32_t*) (addr + 4);       // Assign application start address
    jump_to_app = (fptr) jump_addr;                 // Assign the app() address to function pointer
	    
    __set_MSP(*(__IO uint32_t*) addr);              // Set the address of Stack Pointer
    jump_to_app();                                  // jump to app() address
}

void BootMain(void)
{
	uint8_t ch;
	uint8_t cnt;
    
#ifdef USED_UART
	UART_CFG_Type UARTConfigStruct;
#else
	USART_CFG_Type USARTConfigStruct;
#endif
	
	PCU_Init(); // all port control setting  !!You have to set PORT function!!
    HAL_SCU_SetSystemClock(SYSCLK_HSI, SYSDIV_NONE);    // Set System clock with HSI (32MHz)

#ifdef USED_UART
    uart_mode = UART_POLLING;
	HAL_UART_ConfigStructInit(&UARTConfigStruct); //38400-8-N-1	
	HAL_UART_Init(UART1, &UARTConfigStruct);
#else
    usart_mode = USART_POLLING;
    HAL_USART_UART_Mode_Config(&USARTConfigStruct);
	HAL_USART_Init(USART10, &USARTConfigStruct);
    HAL_USART_Enable(USART10, ENABLE);
#endif

	boot_uputch('>'); //start protocol
	for (cnt = 0; cnt < 100; cnt++)
	{
		boot_ugetchar(&ch);
		if (ch=='<') break;

		DelayMS(1); // about 1msec
	}
	
	if (cnt < 100) // program update start
	{
		wrt.pagesize = PageSize;
		wrt.deviceid.word = DeviceID;
		wrt.flashsize = FlashSize;
		wrt.bootsize = BootSize;
		
		boot_uputch(wrt.pagesize);
		boot_uputch(wrt.deviceid.byte[0]);
		boot_uputch(wrt.deviceid.byte[1]);
		boot_uputch(wrt.flashsize);
		boot_uputch(wrt.bootsize);
		
		boot_uputch('!');

		while (1)
		{
			GetPageNumber();

			if (RealPageAddress == (uint32_t)END_ADDR)
			{
				__NOP();
                break;
			}

			if (WriteFlashPage())
			{
				boot_uputch('!');
			}
			else 
			{
				boot_uputch('@');
			}
		}
        #ifdef  MEMORY_SWAP
            memory_swap();      // Update Memory Swap Option Area (Swap will be applied after reset)
        #endif        
	}

    application_start(APP_START_ADDR); /*< Jump to application code */
}

int	flash_erase_sector(unsigned long addr)
{
	int	result;

    result = HAL_CFMC_EraseSector(0, addr);
	
	return result;
}

int	flash_write_sector(unsigned long addr, unsigned long *pdata)
{
	int	result;
	unsigned char	*fbuffer;

	fbuffer = (unsigned char *)(pdata);
    result = HAL_CFMC_ProgramPage(0, addr, SECTOR_SIZE, fbuffer);
	
	return result;
}
