/***************************************************************************//**
* @file     flash.c
* @brief    Contains all functions support for flash on A31G22x
* @author   AE Team, ABOV Semiconductor Co., Ltd.
* @version  V0.0.1
* @date     04. Sep. 2020
*
* Copyright(C) 2020, ABOV Semiconductor
* All rights reserved.
*
*
********************************************************************************
* DISCLAIMER
*
* THE PRESENT FIRMWARE WHICH IS FOR GUIDANCE ONLY AIMS AT PROVIDING CUSTOMERS
* WITH CODING INFORMATION REGARDING THEIR PRODUCTS IN ORDER FOR THEM TO SAVE
* TIME. AS A RESULT, ABOV SEMICONDUCTOR DISCLAIMS ALL LIABILITIES FROM ANY
* DIRECT, INDIRECT OR CONSEQUENTIAL DAMAGES WITH RESPECT TO ANY CLAIMS ARISING
* FROM THE CONTENT OF SUCH FIRMWARE AND/OR THE USE MADE BY CUSTOMERS OF THE
* CODING INFORMATION CONTAINED HEREIN IN CONNECTION WITH THEIR PRODUCTS.
*
*******************************************************************************/

/* Includes ------------------------------------------------------------------- */
#include "main_conf.h"
#include "TestCode_CFMC_MemorySwap.h"

#define A31G226
//#define A31G224

#define TEST_CODE_SIZE                  (0x6000)

#ifdef A31G226
#undef A31G224
#define BANK_SIZE                       (0x20000)
#define ACTIVE_BANK_START_ADDR          (0x0UL)
#define NONACTIVE_BANK_START_ADDR       (0x00020000UL)
#endif

#ifdef A31G224
#undef A31G226
#define BANK_SIZE                       (0x10000)
#define ACTIVE_BANK_START_ADDR          (0x0UL)
#define NONACTIVE_BANK_START_ADDR       (0x00010000UL)
#endif
#define MIRROR_OFFSET                   (0x30000000UL)

#define PAGE_SIZE		                (0x200) // 512Bytes = 0x128
#define BANK_PAGE_NUM                   (BANK_SIZE/PAGE_SIZE)


const uint32_t COTP_SWAP_TABLE[] =
{
	0x55555555, // Check Code A
	0xAAAAAAAA, // Check Code B
	0xED810148, 0xAA910010, // CFMC->BCR = 0xAA910010, Memory Bank Selection Enable
	0xED820148, 0xAA910011, // CFMC->BCR = 0xAA910011, Memory Bank Selection and Switching Bank	(Bank Switch occurrs after memory bank Selection Enable)
    0xED3B014C, 0x00000100, // CFMC->BSR = 0x00000100, Set Current Boot Bank Indicator Flag (This bit is set by user)
	0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, // < 0x0F010020UL
	0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, // < 0x0F010040UL
	0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, // < 0x0F010060UL
	0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, // < 0x0F010080UL
	0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, // < 0x0F0100A0UL
	0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, // < 0x0F0100C0UL
	0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, // < 0x0F0100E0UL
	0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, // < 0x0F010100UL
	0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, // < 0x0F010120UL
	0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, // < 0x0F010140UL
	0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, // < 0x0F010160UL
	0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, // < 0x0F010180UL
	0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, // < 0x0F0101A0UL
	0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, // < 0x0F0101C0UL
	0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, // < 0x0F0101E0UL
};

const uint32_t COTP_SWAP_EN_TABLE[] =
{
	0x55555555, // Check Code A
	0xAAAAAAAA, // Check Code B
	0xED810148, 0xAA910010, // CFMC->BCR = 0xAA910010, Memory Bank Selection Enable
	0xFFFFFFFF, 0xFFFFFFFF, // CFMC->BCR = 0xAA910011, Memory Bank Selection and Switching Bank	(Bank Switch occurrs after memory bank Selection Enable)
    0xFFFFFFFF, 0xFFFFFFFF, // CFMC->BSR = 0x00000100, Set Current Boot Bank Indicator Flag (This bit is set by user)
	0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, // < 0x0F010020UL
	0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, // < 0x0F010040UL
	0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, // < 0x0F010060UL
	0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, // < 0x0F010080UL
	0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, // < 0x0F0100A0UL
	0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, // < 0x0F0100C0UL
	0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, // < 0x0F0100E0UL
	0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, // < 0x0F010100UL
	0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, // < 0x0F010120UL
	0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, // < 0x0F010140UL
	0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, // < 0x0F010160UL
	0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, // < 0x0F010180UL
	0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, // < 0x0F0101A0UL
	0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, // < 0x0F0101C0UL
	0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, // < 0x0F0101E0UL
};

const char Memoryswap[] = 
"\r\n"
"    ____  ___    _   ____ __    ______       _____    ____  \r\n" 
"   / __ )/   |  / | / / //_/   / ___/ |     / /   |  / __ \\ \r\n"
"  / __  / /| | /  |/ / ,<      \\__ \\| | /| / / /| | / /_/ / \r\n"
" / /_/ / ___ |/ /|  / /| |    ___/ /| |/ |/ / ___ |/ ____/  \r\n"
"/_____/_/  |_/_/ |_/_/ |_|   /____/ |__/|__/_/  |_/_/     	 \r\n\r\n"
;                                                                  

const uint8_t SubMenu[] =
"************************************************************************\n\r"
" Code Flash - Memory Bank Swap Test\n\r"
"\t - MCU: A31G22x\n\r"
"\t - Core: ARM Cortex-M0+\n\r"
"\t - Communicate via: UART10 - 38400 bps\n\r"
"\t - 1. Enable Memory Bank Selection \n\r"
"\t - 2. Copies F/W from Active Bank and Pastes F/W to Non-active Bank \n\r"
"\t - 3. Swap Memory Banks\n\r"
"\t - 4. Erase Memory Bank Swap Option Area \n\r"
"\t - 5. Erase Non-Active Bank \n\r"
"\t - 6. Get checksum value of each bank \n\r"
"\t - 7. Memory Bank Swap - NMI Interrupt Test \n\r"
"\t - others. Print Menu \n\r"
"************************************************************************\n\r";

const uint8_t cmdm[] =
"SWAP> ";

void CFMC_PrintData(uint32_t addr, int size);
void CFMC_MemorySwapTest(void);
void CFMC_CheckMemorySwapStatus(void);
void CFMC_getChecksum(uint32_t startaddr, uint32_t size);
void CFMC_SwapNmiTest(void);
void CFMC_BankSwapAfterCopy(void);

void CFMC_SwapNmiTest(void);

void CFMC_getChecksum(uint32_t startaddr, uint32_t size) {
	int i;
	uint16_t checksum;
	uint32_t sum = 0;

	for(i = 0 ; i < size; i++) {
		sum += *((volatile unsigned char *)(startaddr+i));
	}

	checksum = (sum & 0xFFFF);
    cprintf("0x%08X ~ 0x%08X Checksum: 0x%04X\r\n", startaddr, (startaddr+size), checksum);
}


void CFMC_CheckMemorySwapStatus(void) {
	uint32_t Reg32;

	// View Bank Swap Control Register
	Reg32 = CFMC->BCR;
	cprintf("\r\nCFMC->BCR SFR = [0x%08X]\r\n", Reg32);

	if ((Reg32 & CFMC_BCR_BSE_Msk) == CFMC_BCR_BSE_Msk) {
		_DBG("Enable Bank Selection\r\n");
	} else {
		_DBG("Disable Bank Selection \r\n");
	}

	// View Bank Swap Status Register
	Reg32 = CFMC->BSR;
	cprintf("\r\nCFMC->BSR SFR = [0x%08X]\r\n", Reg32);

	if ((Reg32 & CFMC_BSR_CBF_Msk) == CFMC_BSR_CBF_Msk) {
		_DBG("Current Boot Area is Bank 1\r\n");
	} else {
		_DBG("Current Boot Area is Bank 0\r\n");
	}

	if ((Reg32 & CFMC_BSR_BSST_Msk) == CFMC_BSR_BSST_Msk) {
		_DBG("Enable Bank Selection\r\n");
	} else {
		_DBG("Disable Bank Selection\r\n");
	}

	if ((Reg32 & CFMC_BSR_BSWST_Msk) == CFMC_BSR_BSWST_Msk) {
		_DBG("Occurred bank swap!\r\n");
	} else {
		_DBG("Not occured bank swap.\r\n");
	}

	// Current Bank (Stamp Value)
	cprintf("Active Bank : "); CFMC_PrintData(((ACTIVE_BANK_START_ADDR+BANK_SIZE)-4) , 4);

	_DBG("\r\n\r\n");
}

void CFMC_PrintData(uint32_t DataAddr, int Size)
{
	int i;
	for(i = 0 ; i < Size ; i+= sizeof(uint32_t)) {
		if ((i > 0) && (i % 32 == 0)) {
			_DBG("\r\n");
		}
		cprintf("0x%08X ", *(volatile uint32_t *)(DataAddr + i));
	}
	_DBG("\n\r\n\r");
}

void Main_CFMC_MemorySwapTest(void) {
	uint8_t ch;
    int i;

	if ((CFMC->BSR & CFMC_BSR_BSWST_Msk) == CFMC_BSR_BSWST_Msk) {
        _DBG((char *)Memoryswap);
    }        
    
    CFMC_CheckMemorySwapStatus();

    cprintf("%s %s \r\n", __DATE__, __TIME__);
    _DBG((char*)SubMenu);
    _DBG((char*)cmdm);

    
	while(1) {
        
		ch = cgetchar();
        if ((ch > 0) && (ch <= 0x7E)) {
            cprintf("%c\r\n", ch);

            if (ch == '1') {
                _DBG("Enable Memory Bank Selection\n\r");
                __disable_irq();
                HAL_CFMC_EraseSector(OPTION_BANKSWAP, CFMC_OPTION_BANKSWAP_ADDR);
                HAL_CFMC_ProgramPage(OPTION_BANKSWAP, CFMC_OPTION_BANKSWAP_ADDR , PAGE_SIZE, (unsigned char *)COTP_SWAP_EN_TABLE);                
                __enable_irq();
                _DBG("<Memory Bank Swap Option Area ----------> \n\r");
                CFMC_PrintData(CFMC_OPTION_BANKSWAP_ADDR, PAGE_SIZE);
                
                break;
            }

            else if (ch == '2') {
                _DBG("Copies F/W from Active Bank and Pastes F/W to Non-active Bank \n\r");
                CFMC_BankSwapAfterCopy();
                
                break;                
            }            
            
            else if (ch == '3') {
                _DBG("Swap Memory Banks\n\r");
                __disable_irq();
                HAL_CFMC_ProgramPage(OPTION_BANKSWAP, CFMC_OPTION_BANKSWAP_ADDR , PAGE_SIZE, (unsigned char *)COTP_SWAP_TABLE);
                __enable_irq();

                _DBG("<Memory Bank Swap Option Area ----------> \n\r");
                CFMC_PrintData(CFMC_OPTION_BANKSWAP_ADDR, PAGE_SIZE);
                
                break;
            }            
            
            else if (ch == '4') {
                _DBG("Erase Memory Bank Swap Option Area\n\r");
                __disable_irq();
                HAL_CFMC_EraseSector(OPTION_BANKSWAP, CFMC_OPTION_BANKSWAP_ADDR);
                __enable_irq();
                _DBG("<Memory Bank Swap Option Area ----------> \n\r");
                CFMC_PrintData(CFMC_OPTION_BANKSWAP_ADDR, PAGE_SIZE);
                
                break;
            }                      
            
            else if (ch == '5') {
                _DBG("Erase Non-Active Bank \n\r");
                __disable_irq();
                for (i = NONACTIVE_BANK_START_ADDR; i < (NONACTIVE_BANK_START_ADDR+BANK_SIZE); i += PAGE_SIZE) {
                    HAL_CFMC_EraseSector(OPTION_NOT_USE, i);
                }
                __enable_irq();
            
                break;                
            }            

            else if (ch == '6') {
                _DBG("Get checksum value of each bank \r\n");
                CFMC_getChecksum(ACTIVE_BANK_START_ADDR, BANK_SIZE);                         // Active Area
                CFMC_getChecksum(NONACTIVE_BANK_START_ADDR, BANK_SIZE);                      // Non-Active Area                
                break;                
            } 
            
            else if (ch == '7') {
                _DBG("Memory Bank Swap - NMI Interrupt Test \n\r");
                CFMC_SwapNmiTest();
                
            }            
            
            else {                
                _DBG("!!Inserted wrong number!! type 1 to 6 \r\n");                
                
                CFMC_CheckMemorySwapStatus();

                cprintf("%c %c \r\n", __DATE__, __TIME__);
                _DBG((char*)SubMenu);
                _DBG((char*)cmdm);
                
                break;                
            }
        }
    }

}

void NMI_Handler_bankswap(void);
extern int CFMC_SelfProgram (unsigned long addr, unsigned long size, unsigned char *buf);
extern int CFMC_SelfErase (unsigned long addr, unsigned long size);

/*******************************************************************************
* @brief      Interrupt Handler of NMI
* @param      None
* @return     None
*******************************************************************************/
void NMI_Handler_bankswap(void)
{
	uint32_t NmiStatus;
	uint32_t SwapStatus;

    _DBG("\r\nBANKSWAPFAIL NMI Interrupt - Out of memory bank swap execution area.\r\n");
       
	SwapStatus = HAL_CFMC_GetBankSwapStatus();					// Get bank swap status
    if ((SwapStatus & CFMC_BSR_BSERR_Msk) == CFMC_BSR_BSERR_Msk) {
        _DBG("Bank Swap Error!\r\n");
    }
    
	NmiStatus = HAL_SCU_GetNMIStatus();							// Get NMI interrupt event
	if ((NmiStatus & SCU_NMIR_SWAPERRSTS_Msk) && (SwapStatus & CFMC_BSR_BSERR_Msk)) {
		CFMC->BSR |= (1 << CFMC_BSR_BSERR_Pos);			// Clear Bank swap error flag bit
        _DBG("Clear SWAPFAIL ERROR FLAG\r\n");
        
        SwapStatus = HAL_CFMC_GetBankSwapStatus();					// Get bank swap status
        if (!((SwapStatus & CFMC_BSR_BSERR_Msk) == CFMC_BSR_BSERR_Msk)) {
            _DBG("Clear Bank Swap Error!\r\n");
        }
	}
}

void CFMC_SwapNmiTest(void) {
	uint32_t Reg32;
    
    // !! NOTE !! This Test routine shourld be performed on Code Flash Memory.
    
	// NMI Handler Enable
	HAL_SCU_SetNMI(SCU_NMI_SWAPERR_ENABLE); // Enable Comparator NMI Interrupt
    
    __enable_irq();

    Reg32 = CFMC->BSR;  // get memory bank swap status
	if ((Reg32 & CFMC_BSR_CBF_Msk) == CFMC_BSR_CBF_Msk) {
        /* The memory bank is swapped. */
        
        // Swap Enable Switch in SRAM
        CFMC->BCR |= (0xAA91UL << CFMC_BCR_WTIDKY_Pos) | (0x0UL << CFMC_BCR_BSE_Pos);    // Tested on inactive bankswap
        // Switcing Bank in SRAM
        CFMC->BCR |= (0xAA91UL << CFMC_BCR_WTIDKY_Pos) | (0x0UL << CFMC_BCR_BSW_Pos);    // Tested on inactive bankswap             
	} else {
        /* The memory bank is NOT swapped. (normal mode)*/
        
        // Swap Enable Switch in SRAM
        CFMC->BCR |= (0xAA91UL << CFMC_BCR_WTIDKY_Pos) | (0x1UL << CFMC_BCR_BSE_Pos);    // Tested on inactive bankswap
        // Switcing Bank in SRAM
        CFMC->BCR |= (0xAA91UL << CFMC_BCR_WTIDKY_Pos) | (0x1UL << CFMC_BCR_BSW_Pos);    // Tested on inactive bankswap             
    }
      
    while(1) {}
}

void CFMC_BankSwapAfterCopy(void) {	
	unsigned long	*ptr;
	int i;
	int size = PAGE_SIZE;
	unsigned long B0[1] = {0xB0B0B0B0};
	unsigned long B1[1] = {0xB1B1B1B1};
	
	// Unprotected Non-active Bank Write Protection for data update
	CFMC->MR = 0x66;
	CFMC->MR = 0x99;
	CFMC->WPROT = (0x0 << 8); // The write protection of 0x20000~0x3FFFF (Non-active bank area) area is released.
	CFMC->NBWPROT = CFMC_NBWPROT_KEY | (0 << CFMC_NBWPROT_NB_WPROT_Pos); // The write protection of bootloader area in Nnn-active bank area is released.

	
	// Update the data to non-active bank area
	// Fill init value to buffer
	for (i = 0 ; i < TEST_CODE_SIZE; i += size) {    
        __disable_irq();        
		ptr = (unsigned long *)i;        
		HAL_CFMC_SelfErase((NONACTIVE_BANK_START_ADDR +i), size);
		HAL_CFMC_SelfProgram((NONACTIVE_BANK_START_ADDR+i), size, (unsigned char *)ptr);
        __enable_irq();
        cprintf("Copy 0x%08X and Paste 0x%08X OK\r\n", (ACTIVE_BANK_START_ADDR+i), (NONACTIVE_BANK_START_ADDR+i));
 	}

	// Save Stamp copy Data
	__disable_irq();
	HAL_CFMC_SelfErase(((ACTIVE_BANK_START_ADDR+BANK_SIZE)-PAGE_SIZE), 4);
	HAL_CFMC_SelfProgram((ACTIVE_BANK_START_ADDR+BANK_SIZE-4), 4, (unsigned char*)B0);
	 
	HAL_CFMC_SelfErase(((NONACTIVE_BANK_START_ADDR+BANK_SIZE)-PAGE_SIZE), 4);
	HAL_CFMC_SelfProgram((NONACTIVE_BANK_START_ADDR+BANK_SIZE-4), 4, (unsigned char*)B1);
	__enable_irq();
}

/* --------------------------------- End Of File ------------------------------ */
