/*****************************************************************************
* @file     TestCode_CMP_EdgePoleMonitoring.c
* @brief    Contains the external function prototypes for TestCode
* @author   AE Team, ABOV Semiconductor Co., Ltd.
* @version  V0.0.1
* @date     16. MAY. 2019
*
* Copyright(C) 2019, ABOV Semiconductor
* All rights reserved.
*
*
********************************************************************************
* DISCLAIMER
*
* THE PRESENT FIRMWARE WHICH IS FOR GUIDANCE ONLY AIMS AT PROVIDING CUSTOMERS
* WITH CODING INFORMATION REGARDING THEIR PRODUCTS IN ORDER FOR THEM TO SAVE
* TIME. AS A RESULT, ABOV SEMICONDUCTOR DISCLAIMS ALL LIABILITIES FROM ANY
* DIRECT, INDIRECT OR CONSEQUENTIAL DAMAGES WITH RESPECT TO ANY CLAIMS ARISING
* FROM THE CONTENT OF SUCH FIRMWARE AND/OR THE USE MADE BY CUSTOMERS OF THE
* CODING INFORMATION CONTAINED HEREIN IN CONNECTION WITH THEIR PRODUCTS.
*
*******************************************************************************/

/*******************************************************************************
* Pre-processor Definition & Macro
*******************************************************************************/
#include "TestCode_CMP_EdgePoleMonitoring.h"
#include "main_conf.h"

/*******************************************************************************
* Private Typedef
*******************************************************************************/
#define USED_CMP_N			(0)			// Select CMP_0(0), or CMP_1 (1)

/*******************************************************************************
* Private Variable
*******************************************************************************/
#if (USED_CMP_N == 0)
static const CMP_NUMBER_Type CMP_NUMBER = CMP_0;
static const uint32_t TEST_PIN_NUMBER = 0; // PE0
static const uint32_t SOURCE_PIN_NUMBER = 2; // PA2 (CP0 pin)
static const uint32_t REFER_PIN_NUMBER = 7; // PA7 (CREF0 pin)
#elif (USED_CMP_N == 1)
static const CMP_NUMBER_Type CMP_NUMBER = CMP_1;
static const uint32_t TEST_PIN_NUMBER = 1; // PE1
static const uint32_t SOURCE_PIN_NUMBER = 5; // PA5 (PC1A pin)
//static const uint32_t SOURCE_PIN_NUMBER = 4; // PA4 (PC1B pin)
//static const uint32_t SOURCE_PIN_NUMBER = 3; // PA3 (PC1C pin)
static const uint32_t REFER_PIN_NUMBER = 6; // PA6 (CREF1 pin)
#endif

static uint32_t TestPin_Output = 0;
static volatile uint32_t CmpCnt = 0;
static uint32_t MilliSec = 0;

/*******************************************************************************
* Private Function Prototype
*******************************************************************************/
void SysTick_Handler_EdgePoleMonitoring(void) {
    if (MilliSec) {
        MilliSec--;
    }
}

/*******************************************************************************
* Public Function
*******************************************************************************/

/*******************************************************************************
* @brief      Interrupt Handler of Comparator and CRC
* @param      None
* @return     None
*******************************************************************************/
void CMP_Interrupt_EdgePoleMonitoring(void) {
	uint32_t CmpStatus;

    cputs("CMP Interrupt Occurred!\r\n");    
    
	CmpStatus = HAL_CMP_GetMonitoringEdgeStatus();
    cprintf("[%d]:\r\n", CmpCnt);
	if (HAL_CMP_GetMonitoringStatus()) { // When Comparator Output Monitoring Function is Enabled, Outputs Output Status

		if (USED_CMP_N == CMP_0) {
			if ((CmpStatus & CMP0_OUTPUT_LEVEL) == CMP0_OUTPUT_LEVEL) {
				cputs("CMP0 Output is High\r\n");
			} else {
				cputs("CMP0 Output is Low\r\n");
			}

			if ((CmpStatus & CMP0_OUTPUT_RISING_EDGE) == CMP0_OUTPUT_RISING_EDGE) {
				cputs("--> CMP0 Rising\r\n");
			}
			if ((CmpStatus & CMP0_OUTPUT_FALLING_EDGE) == CMP0_OUTPUT_FALLING_EDGE) {
				cputs("--> CMP0 Falling\r\n");
			}
		}
		else {
			if ((CmpStatus & CMP1_OUTPUT_LEVEL) == CMP1_OUTPUT_LEVEL) {
				cputs("CMP1 Output is High\r\n");
			} else {
				cputs("CMP1 Output is Low\r\n");
			}

			if ((CmpStatus & CMP1_OUTPUT_RISING_EDGE) == CMP1_OUTPUT_RISING_EDGE) {
				cputs("--> CMP1 Rising\r\n");
			}
			if ((CmpStatus & CMP1_OUTPUT_FALLING_EDGE) == CMP1_OUTPUT_FALLING_EDGE) {
				cputs("--> CMP1 Falling\r\n");
			}
		}
		HAL_CMP_ClearMonitoringEdgeStatus();

	} else {	// When Comparator Output Monitoring Function is Disabled, LED Toggle and Clear
		if (HAL_CMP_GetInterruptStatus(CMP_NUMBER) == SET) {
            cprintf("Comparator %d Interrupt Occurred.\r\n", CMP_NUMBER);
			if (TestPin_Output == 1) { // toggle test pin
				HAL_GPIO_ClearPin(PE, _BIT(TEST_PIN_NUMBER));
			} else {
				HAL_GPIO_SetPin(PE, _BIT(TEST_PIN_NUMBER));
			}
			TestPin_Output ^= 1;
		}
	}

	HAL_CMP_ClearInterruptStatus(CMP_NUMBER);		// Clear Interrupt Status
	CmpCnt++;
}

/*******************************************************************************
* @brief      Main Comprator Polarity Monitoring Test
* @param      None
* @return     None
*******************************************************************************/
void Main_CmpEdgePolMonTest(void) {
    
	// Test pin setting
	HAL_GPIO_ConfigOutput(PE, TEST_PIN_NUMBER, PCU_MODE_PUSH_PULL);
	HAL_GPIO_ConfigPullup(PE, TEST_PIN_NUMBER, PCU_PUPD_DISABLE);
	HAL_GPIO_ClearPin(PE, _BIT(TEST_PIN_NUMBER));
	TestPin_Output = 0;

	// Reference pin setting
	// Reference : CMP0 -> CREF0(PA7), CMP1 -> CREF1(PA6)
	HAL_GPIO_ConfigOutput(PA, REFER_PIN_NUMBER, PCU_MODE_ALT_FUNC);    
	HAL_GPIO_ConfigFunction(PA, REFER_PIN_NUMBER, PCU_ALT_FUNCTION_3);
	HAL_GPIO_ConfigPullup(PA, REFER_PIN_NUMBER, PCU_PUPD_DISABLE);

	// Source pin setting
	// Source : CMP0 -> CP0(PA2), CMP1 -> CP1A(PA5)
	HAL_GPIO_ConfigOutput(PA, SOURCE_PIN_NUMBER, PCU_MODE_ALT_FUNC);
	HAL_GPIO_ConfigFunction(PA, SOURCE_PIN_NUMBER, PCU_ALT_FUNCTION_3);
	HAL_GPIO_ConfigPullup(PA, SOURCE_PIN_NUMBER, PCU_PUPD_DISABLE);

	HAL_CMP_Init(); // Initialize comparator

	HAL_CMP_ConfigureHysterisis(CMP_NUMBER, CMP_HYSSEL_5_MILLIVOLTS, DISABLE); // Configure comparator

	HAL_CMP_ConfigureInput(CMP_NUMBER, CMP_SOURCE_0, CMP_REFERENCE_1);

	HAL_CMP_ConfigureDBNCCounter(24);
	HAL_CMP_ConfigureDBNCNumber(CMP_NUMBER, 0);

	HAL_CMP_ConfigureTriggers(CMP_NUMBER, CMP_TRIGGER_POL_NORMAL, (CMP_INTERRUPT_POL_Type)(CMP_INTERRUPT_POL_HIGH | CMP_INTERRUPT_POL_LOW), CMP_INTERRUPT_MODE_BOTH_EDGE);   // When Rising or Falling edge of source is occurred above reference, Interrupt is occurred. (Tested)

	HAL_CMP_EnableInterrupt(CMP_NUMBER, ENABLE); // Comparator Interrupt Enable
	HAL_CMP_SetOutputMonitoring(ENABLE); // Enable Monitoring Setting

	// Enable interrupt
	NVIC_SetPriority(CMP_CRC_IRQn, 0);
	NVIC_EnableIRQ(CMP_CRC_IRQn);

    SysTick_Config(SystemPeriClock/1000);

	__enable_irq();

	HAL_CMP_Start(CMP_NUMBER);

	while(1) {
        if (TestPin_Output == 1) { // toggle test pin
            HAL_GPIO_ClearPin(PE, _BIT(TEST_PIN_NUMBER));
        } else {
            HAL_GPIO_SetPin(PE, _BIT(TEST_PIN_NUMBER));
        }
        TestPin_Output ^= 1;
        
        MilliSec=1000;
        while(MilliSec);
    }
}

/* --------------------------------- End Of File ------------------------------ */
