/*****************************************************************************
* @file     TestCode_CMP_Polling.c
* @brief    Contains the external function prototypes for TestCode
* @author   AE Team, ABOV Semiconductor Co., Ltd.
* @version  V0.0.1
* @date     16. MAY. 2019
*
* Copyright(C) 2019, ABOV Semiconductor
* All rights reserved.
*
*
********************************************************************************
* DISCLAIMER
*
* THE PRESENT FIRMWARE WHICH IS FOR GUIDANCE ONLY AIMS AT PROVIDING CUSTOMERS
* WITH CODING INFORMATION REGARDING THEIR PRODUCTS IN ORDER FOR THEM TO SAVE
* TIME. AS A RESULT, ABOV SEMICONDUCTOR DISCLAIMS ALL LIABILITIES FROM ANY
* DIRECT, INDIRECT OR CONSEQUENTIAL DAMAGES WITH RESPECT TO ANY CLAIMS ARISING
* FROM THE CONTENT OF SUCH FIRMWARE AND/OR THE USE MADE BY CUSTOMERS OF THE
* CODING INFORMATION CONTAINED HEREIN IN CONNECTION WITH THEIR PRODUCTS.
*
*******************************************************************************/

/*******************************************************************************
* Pre-processor Definition & Macro
*******************************************************************************/
#include "TestCode_CMP_Polling.h"
#include "main_conf.h"

/*******************************************************************************
* Private Typedef
*******************************************************************************/
#define USED_CMP_N			(1)			// Select CMP_0(0), or CMP_1 (1)

/*******************************************************************************
* Private Variable
*******************************************************************************/
#if (USED_CMP_N == 0)
static const CMP_NUMBER_Type CMP_NUMBER = CMP_0;
static const uint32_t TEST_PIN_NUMBER = 0; // PE0
static const uint32_t SOURCE_PIN_NUMBER = 2; // PA2 (CP0 pin)

#elif (USED_CMP_N == 1)
static const CMP_NUMBER_Type CMP_NUMBER = CMP_1;
static const uint8_t TEST_PIN_NUMBER = 1; // PE1
static const uint8_t SOURCE_PIN_NUMBER = 5; // PA5 (PC1A pin)
//static const uint32_t SOURCE_PIN_NUMBER = 4; // PA4 (PC1B pin)
//static const uint32_t SOURCE_PIN_NUMBER = 3; // PA3 (PC1C pin)
#endif

static const uint8_t FLAG_PIN_NUMBER = 3; // PE3

static uint32_t MilliSec = 0;
static uint8_t TestPin_Output = 0;
static uint8_t FlagPin_Output = 0;
/*******************************************************************************
* Private Function Prototype
*******************************************************************************/
void SysTick_Handler_CmpPolling(void) {
    if (TestPin_Output == 1) { // toggle test pin
        HAL_GPIO_ClearPin(PE, _BIT(TEST_PIN_NUMBER));
    } else {
        HAL_GPIO_SetPin(PE, _BIT(TEST_PIN_NUMBER));
    }
    TestPin_Output ^= 1;
    
    if (MilliSec) {
        MilliSec--;
    }
}
/*******************************************************************************
* Public Function
*******************************************************************************/

/*******************************************************************************
* @brief      Main Comprator Polling Test
* @param      None
* @return     None
*******************************************************************************/
void Main_CmpPollingTest(void) {
	uint32_t CmpStatus;
    

    
	// Test Pin setting (PE1 or PE2)
	HAL_GPIO_ConfigOutput(PE, TEST_PIN_NUMBER, PCU_MODE_PUSH_PULL);
	HAL_GPIO_ConfigPullup(PE, TEST_PIN_NUMBER, PCU_PUPD_DISABLE);
	HAL_GPIO_ClearPin(PE, _BIT(TEST_PIN_NUMBER));
    
    // Flag Pin setting (PE3)
	HAL_GPIO_ConfigOutput(PE, FLAG_PIN_NUMBER, PCU_MODE_PUSH_PULL);
	HAL_GPIO_ConfigPullup(PE, FLAG_PIN_NUMBER, PCU_PUPD_DISABLE);
	HAL_GPIO_ClearPin(PE, _BIT(FLAG_PIN_NUMBER));
    
	// Source Pin setting (PA2 or PA5/4/3)
	HAL_GPIO_ConfigOutput(PA, SOURCE_PIN_NUMBER, PCU_MODE_ALT_FUNC);    
	HAL_GPIO_ConfigFunction(PA, SOURCE_PIN_NUMBER, PCU_ALT_FUNCTION_3);
	HAL_GPIO_ConfigPullup(PA, SOURCE_PIN_NUMBER, PCU_PUPD_DISABLE);

	// Initialize comparator
	HAL_CMP_Init();

	// Configure comparator
	HAL_CMP_ConfigureHysterisis(CMP_NUMBER, CMP_HYSSEL_5_MILLIVOLTS, DISABLE);

    // Set source and reference to compare
    // SRC0 : PA2 @CMP0 (or PA5 @CMP1), SRC1: GND @CMP0 (or PA4 @CMP1 ) SRC2: GND@CMP0 (or PA3 @CMP1)
    // REF0 : PA7 @CMP0 (or PA6 @CMP1), REF1: BGR(1.0V),                REF2: DACOUT
	HAL_CMP_ConfigureInput(CMP_NUMBER, CMP_SOURCE_0, CMP_REFERENCE_1);		// Source and reference voltage settings for comparison. 
    
	HAL_CMP_ConfigureDBNCCounter(24);
	HAL_CMP_ConfigureDBNCNumber(CMP_NUMBER, 0);

//	HAL_CMP_ConfigureTriggers(CMP_NUMBER, CMP_TRIGGER_POL_NORMAL, CMP_INTERRUPT_POL_HIGH, CMP_INTERRUPT_MODE_LEVEL);   // When Comparision value is HIGH than reference, flag is occurred. (Tested)
//	HAL_CMP_ConfigureTriggers(CMP_NUMBER, CMP_TRIGGER_POL_NORMAL, CMP_INTERRUPT_POL_LOW, CMP_INTERRUPT_MODE_LEVEL);   // When Comparision value is LOW than reference, flag is occurred.   (Tested)
//	HAL_CMP_ConfigureTriggers(CMP_NUMBER, CMP_TRIGGER_POL_NORMAL, CMP_INTERRUPT_POL_HIGH, CMP_INTERRUPT_MODE_RISING_EDGE);   // When Rising edge of source is occurred above reference, flag is occurred. (Tested)
//	HAL_CMP_ConfigureTriggers(CMP_NUMBER, CMP_TRIGGER_POL_NORMAL, CMP_INTERRUPT_POL_HIGH, CMP_INTERRUPT_MODE_FALLING_EDGE);   // When Falling edge of source is occurred above reference, flag is occurred. (Tested)
//	HAL_CMP_ConfigureTriggers(CMP_NUMBER, CMP_TRIGGER_POL_NORMAL, CMP_INTERRUPT_POL_LOW, CMP_INTERRUPT_MODE_RISING_EDGE);   // When Rising edge of source is occurred above reference, flag is occurred.  (Tested)
//	HAL_CMP_ConfigureTriggers(CMP_NUMBER, CMP_TRIGGER_POL_NORMAL, CMP_INTERRUPT_POL_LOW, CMP_INTERRUPT_MODE_FALLING_EDGE);   // When Falling edge of source is occurred above reference, flag is occurred. (Tested)
	HAL_CMP_ConfigureTriggers(CMP_NUMBER, CMP_TRIGGER_POL_NORMAL, (CMP_INTERRUPT_POL_Type)(CMP_INTERRUPT_POL_HIGH | CMP_INTERRUPT_POL_LOW), CMP_INTERRUPT_MODE_BOTH_EDGE);   // When Rising or Falling edge of source is occurred above reference, Interrupt is occurred. (Tested)
	HAL_CMP_EnableInterrupt(CMP_NUMBER, DISABLE);	// Comparator Interrupt Disable (Polling)

    // Systick Interrnt Enable
    SysTick_Config(SystemCoreClock/10);   // 100ms Systick
    NVIC_SetPriority(SysTick_IRQn, 3);
    NVIC_EnableIRQ(SysTick_IRQn);

    __enable_irq();
    
	HAL_CMP_Start(CMP_NUMBER);						// Start Comparator Operation
  
	while(1) {
		CmpStatus = HAL_CMP_GetInterruptStatus(CMP_NUMBER); // Get Interrupt Status
		if (CmpStatus == SET) {
			HAL_CMP_ClearInterruptStatus(CMP_NUMBER);  // Clear CMP event flag
            
            if (FlagPin_Output == 1) { // toggle flag pin
                HAL_GPIO_ClearPin(PE, _BIT(FLAG_PIN_NUMBER));
            } else {
                HAL_GPIO_SetPin(PE, _BIT(FLAG_PIN_NUMBER));
            }
            FlagPin_Output ^= 1;
            
            cprintf("Comparator %d Flag Occurred.\r\n", CMP_NUMBER);
		}  else {      
            cprintf("Comparator %d Flag NOT Occurred.\r\n", CMP_NUMBER);
        }
        MilliSec = 1;   // 100ms period delay
        while(MilliSec);
	}
}

/* --------------------------------- End Of File ------------------------------ */
