/*****************************************************************************
* @file     TestCode_CORE_InterruptPriority.c
* @brief    Contains the external function prototypes for TestCode
* @author   AE Team, ABOV Semiconductor Co., Ltd.
* @version  V0.0.1
* @date     25. Aug. 2020
*
* Copyright(C) 2020, ABOV Semiconductor
* All rights reserved.
*
*
********************************************************************************
* DISCLAIMER
*
* THE PRESENT FIRMWARE WHICH IS FOR GUIDANCE ONLY AIMS AT PROVIDING CUSTOMERS
* WITH CODING INFORMATION REGARDING THEIR PRODUCTS IN ORDER FOR THEM TO SAVE
* TIME. AS A RESULT, ABOV SEMICONDUCTOR DISCLAIMS ALL LIABILITIES FROM ANY
* DIRECT, INDIRECT OR CONSEQUENTIAL DAMAGES WITH RESPECT TO ANY CLAIMS ARISING
* FROM THE CONTENT OF SUCH FIRMWARE AND/OR THE USE MADE BY CUSTOMERS OF THE
* CODING INFORMATION CONTAINED HEREIN IN CONNECTION WITH THEIR PRODUCTS.
*
*******************************************************************************/

/*******************************************************************************
* Pre-processor Definition & Macro
*******************************************************************************/
#include "TestCode_CORE_InterruptPriority.h"
#include "main_conf.h"

/* Definitions for VTOR TEST */
/* Referenced 4.3.4. Vector Table Offset Register - ARM Infocenter */
#define	VECTOR_TABLE_RELOCATION				// VTOR TEST (define : SRAM, undifine : Code Flash)

#define	SRAM_BASE               (0x20002000)	// SRAM BASE Address
#define	FLASH_BASE              (0x00000000)	// FLASH BASE Address
#define	VTOR_REG                (0xE000ED08)	// Vector Table Offset Register Address (
#define	VTOR_TBLBASE_Msk        (1 << 29)
#define	VTOR_TBLBASE_Pos        (29)
#define	VTOR_TBLOFF_Msk         (0x1FFFFF << 7)
#define	VTOR_TBLOFF_Pos         (7)

/*******************************************************************************
* Private Typedef
*******************************************************************************/

/*******************************************************************************
* Private Variable
*******************************************************************************/
volatile uint32_t intr_cnt;

volatile Bool systick_flag = FALSE;
volatile Bool timer10_flag = FALSE;
volatile Bool timer20_flag = FALSE;
volatile Bool timer30_flag = FALSE;
/*******************************************************************************
* Private Function Prototype
*******************************************************************************/

/*******************************************************************************
* Public Function
*******************************************************************************/
void TIMER30_IRQHandler_InterruptPriority(void) {
    cputs("\r\nTIMER30!");

    intr_cnt++;
    cprintf("\tINTR_CNT: %d\r\n", intr_cnt);

	// Period Match Interrupt
	if (HAL_TIMER3n_GetStatus(TIMER30) & TIMER3n_STATUS_PMATCH_INT) {
		if(timer30_flag == TRUE) {
			timer30_flag = FALSE;
			HAL_GPIO_ClearPin(PE, _BIT(5));
		} else {
			timer30_flag = TRUE;
			HAL_GPIO_SetPin(PE, _BIT(5));
		}
		HAL_TIMER3n_ClearStatus(TIMER30, TIMER3n_STATUS_PMATCH_INT);
	}
}

void TIMER20_IRQHandler_InterruptPriority(void) {
    cputs("\r\nTIMER20!");

    intr_cnt++;
    cprintf("\tINTR_CNT: %d\r\n", intr_cnt);

	if (HAL_TIMER2n_GetStatus(TIMER20) & TIMER2n_STATUS_MATCH_INT) {
        if (timer20_flag == TRUE) {
            HAL_GPIO_ClearPin(PE, _BIT(4));
            timer20_flag = FALSE;
        } else {
            HAL_GPIO_SetPin(PE, _BIT(4));
            timer20_flag = TRUE;
        }
        HAL_TIMER2n_ClearStatus(TIMER20, TIMER2n_STATUS_MATCH_INT);
	}
}

void TIMER10_IRQHandler_InterruptPriority(void) {
    cputs("\r\nTIMER10!");
    intr_cnt++;
    cprintf("\tINTR_CNT: %d\r\n", intr_cnt);

	if (HAL_TIMER1n_GetStatus(TIMER10) & TIMER1n_STATUS_MATCH_INT) {
        if (timer10_flag == TRUE) {
            HAL_GPIO_ClearPin(PE, _BIT(3));
            timer10_flag = FALSE;
        } else {
            HAL_GPIO_SetPin(PE, _BIT(3));
            timer10_flag = TRUE;
        }

        HAL_TIMER1n_ClearStatus(TIMER10, TIMER1n_STATUS_MATCH_INT);
	}
}

void Systick_Handler_InterruptPriority(void) {
    cputs("\r\nSYSTICK!");
    intr_cnt++;
    cprintf("\tINTR_CNT: %d\r\n", intr_cnt);

    if (timer10_flag == TRUE) {
        HAL_GPIO_ClearPin(PE, _BIT(2));
        timer10_flag = FALSE;
    } else {
        HAL_GPIO_SetPin(PE, _BIT(2));
        timer10_flag = TRUE;
    }
}

/**********************************************************************
 * @brief		Interrupt Priority Test
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void TestCode_CORE_InterruptPriority(void) {
    // Reference URL: https://www.keil.com/pack/doc/cmsis/Core/html/group__NVIC__gr.html

    TIMER1n_CFG_Type TIMER1n_Config;
    TIMER2n_CFG_Type TIMER2n_Config;
    TIMER3n_CFG_Type TIMER3n_Config;

    uint32_t irq_priority;
    int i;


    __disable_irq();    // Global Interrupt disable


    // TEST PIN (PE2 / PE3 / PE54 / PE5)
    HAL_GPIO_ConfigOutput(PE, 2, PCU_MODE_PUSH_PULL);
    HAL_GPIO_ConfigPullup(PE, 2, PCU_PUPD_DISABLE);
    HAL_GPIO_ClearPin(PE, _BIT(2));

    HAL_GPIO_ConfigOutput(PE, 3, PCU_MODE_PUSH_PULL);
    HAL_GPIO_ConfigPullup(PE, 3, PCU_PUPD_DISABLE);
    HAL_GPIO_ClearPin(PE, _BIT(3));

    HAL_GPIO_ConfigOutput(PE, 4, PCU_MODE_PUSH_PULL);
    HAL_GPIO_ConfigPullup(PE, 4, PCU_PUPD_DISABLE);
    HAL_GPIO_ClearPin(PE, _BIT(4));

    HAL_GPIO_ConfigOutput(PE, 5, PCU_MODE_PUSH_PULL);
    HAL_GPIO_ConfigPullup(PE, 5, PCU_PUPD_DISABLE);
    HAL_GPIO_ClearPin(PE, _BIT(5));

    // Systick ///////////////////////////////////////////////////////////
//    SYSTICK_IRQ_Call_F = Systick_Handler_Interrupt_Test;    // Systick interrupt function

    // TIMER10 ///////////////////////////////////////////////////////////
    HAL_SCU_MiscClockConfig(1, TIMER1n_TYPE, CLKSRC_MCLK, 48); // 48MHz/48 = 1MHz
    HAL_SCU_Timer1n_ClockConfig(T1NCLK_MCCR1);                // Use MCCR clock as timer10 clock

    TIMER1n_Config.ExtClock = DISABLE;
    TIMER1n_Config.PrescalerData = 1000;        // 1MHz / 1000 = 1ms
    TIMER1n_Config.StartSync = DISABLE;      // TIMER10 synchronized with TIMER30.
    TIMER1n_Config.ClearSync = DISABLE;
//    TIMER1n_Config.Mode = TIMER1n_PERIODIC_MODE;
    TIMER1n_Config.OutputPolarity = TIMER1n_OUTPUT_POLARITY_HIGH;
    TIMER1n_Config.MatchInterrupt = ENABLE;
    TIMER1n_Config.CaptureInterrupt = DISABLE;
    TIMER1n_Config.AData = (100-1);              // 1ms * 100 = 100ms
//    TIMER1n_Config.AData = (50-1);              // 1ms * 50 = 50ms
    HAL_TIMER1n_Init(TIMER10, TIMER1n_PERIODIC_MODE, &TIMER1n_Config);

//    TIMER10_IRQ_Call_F = TIMER10_IRQHandler_Interrupt;  // TIMER10 interrupt function
	HAL_TIMER1n_ClearCounter(TIMER10);

    // TIMER20 ///////////////////////////////////////////////////////////
    HAL_SCU_MiscClockConfig(2, TIMER2n_TYPE, CLKSRC_MCLK, 48);    // 48MHz/48 = 1MHz
    HAL_SCU_Timer20_ClockConfig(T20CLK_MCCR2);                   // TIMER2n MCCR2 source

    TIMER2n_Config.ExtClock = DISABLE; // MCCR 32MHz
    TIMER2n_Config.PrescalerData = 1000; // 1MHz / 1000 = 1ms
    TIMER2n_Config.ExtClockEdge = TIMER2n_EXT_CLOCK_EDGE_FALLING;
    TIMER2n_Config.Mode = TIMER2n_MODE_PERIODIC;
    TIMER2n_Config.OutputPolarity = TIMER2n_OUTPUT_POLARITY_LOW;
    TIMER2n_Config.MatchInterrupt = ENABLE;
    TIMER2n_Config.CaptureInterrupt = DISABLE;
//    TIMER2n_Config.AData = (25 - 1);     // 25ms
    TIMER2n_Config.AData = (100 - 1);     // 100ms
    HAL_TIMER2n_Init(TIMER20, &TIMER2n_Config);     // TIMER20 Init

//    TIMER20_IRQ_Call_F = TIMER20_IRQHandler_Interrupt;  // TIMER20 interrupt function
	HAL_TIMER2n_ClearCounter(TIMER20);

    // TIMER30 ///////////////////////////////////////////////////////////
    HAL_SCU_MiscClockConfig(2, TIMER3n_TYPE, CLKSRC_MCLK, 48);    // 48MHz/48 = 1MHz
    HAL_SCU_Timer30_ClockConfig(T30CLK_MCCR2);                   // TIMER3n MCCR2 source

	TIMER3n_Config.ExtClock = DISABLE;
	TIMER3n_Config.Mode = TIMER3n_MODE_INTERVAL;
	TIMER3n_Config.OutputMode = TIMER3n_OUTPUT_MODE_6_CHANNEL;
	TIMER3n_Config.DelayInsertion = DISABLE;
	TIMER3n_Config.DelayPosition = TIMER3n_DELAY_POSITION_FRONT;
	TIMER3n_Config.ReloadTime = TIMER3n_DATA_RELOAD_TIME_WRITING;
	TIMER3n_Config.MatchIntInterval = TIMER3n_MATCH_INT_INTERVAL_1;
	TIMER3n_Config.PData = 100; // 100ms (Interval)
//	TIMER3n_Config.AData = 750; // 75.0%
//	TIMER3n_Config.BData = 667; // 66.7%
//	TIMER3n_Config.CData = 500; // 50.0%
	TIMER3n_Config.PrescalerData = 1000;
	TIMER3n_Config.DelayData = 0; // (0 + 1) / 32MHz = 2us
	HAL_TIMER3n_Init(TIMER30, &TIMER3n_Config);

	HAL_TIMER3n_ConfigureInterrupt(TIMER30, TIMER3n_INTERRUPT_PMATCH_INT, ENABLE);	// Enable Period Match Interrupt

//    TIMER30_IRQ_Call_F = TIMER30_IRQHandler_Interrupt;  // TIMER30 interrupt function
	HAL_TIMER3n_ClearCounter(TIMER30);
    //////////////////////////////////////////////////////////////////////

    SCU->MCCR2 = 0x04300430;        // TIMER20 / TIMER30 Clock Source Selection

    // Set IRQ Priority and Enable IRQ
#if 0
    // No priority of Any IRQs (Default to 0 level)
    NVIC_SetPriority(SysTick_IRQn, 3);
    NVIC_SetPriority(TIMER10_IRQn, 3);
    NVIC_SetPriority(TIMER20_IRQn, 3);
	NVIC_SetPriority(TIMER30_IRQn, 3);
#endif
#if 0
    NVIC_SetPriority(SysTick_IRQn, 3);
    NVIC_SetPriority(TIMER10_IRQn, 2);
    NVIC_SetPriority(TIMER20_IRQn, 1);
	NVIC_SetPriority(TIMER30_IRQn, 0);
#endif
#if 0
    NVIC_SetPriority(SysTick_IRQn, 3);
    NVIC_SetPriority(TIMER10_IRQn, 2);
    NVIC_SetPriority(TIMER20_IRQn, 1);
	NVIC_SetPriority(TIMER30_IRQn, 0);
#endif
#if 0
    NVIC_SetPriority(TIMER10_IRQn, 0);
    NVIC_SetPriority(TIMER20_IRQn, 0);
	NVIC_SetPriority(TIMER30_IRQn, 0);
#endif
#if 1
    NVIC_SetPriority(TIMER10_IRQn, 2);
    NVIC_SetPriority(TIMER20_IRQn, 0);
	NVIC_SetPriority(TIMER30_IRQn, 1);
#endif

    // Get IRQ Priority
    cputs("IRQ PRIORITY ================================\r\n");
    irq_priority = NVIC_GetPriority(TIMER30_IRQn);
    cprintf("IRQ PRIORITY: TIMER30 %d\r\n", irq_priority);
//    _DBG("IRQ PRIORITY: "); _DBG("TIMER30 "); _DBD32(irq_priority); _DBG("\r\n");
    irq_priority = NVIC_GetPriority(TIMER20_IRQn);
    cprintf("IRQ PRIORITY: TIMER20 %d\r\n", irq_priority);    
    //_DBG("IRQ PRIORITY: "); _DBG("TIMER20 "); _DBD32(irq_priority); _DBG("\r\n");
    irq_priority = NVIC_GetPriority(TIMER10_IRQn);
    cprintf("IRQ PRIORITY: TIMER10 %d\r\n", irq_priority);    
    //_DBG("IRQ PRIORITY: "); _DBG("TIMER10 "); _DBD32(irq_priority); _DBG("\r\n");
    irq_priority = NVIC_GetPriority(SysTick_IRQn);
    cprintf("IRQ PRIORITY: SYSTICK %d\r\n", irq_priority);    
//    _DBG("IRQ PRIORITY: "); _DBG("SYSTICK "); _DBD32(irq_priority); _DBG("\r\n");

    // Enable IRQ
	NVIC_EnableIRQ(TIMER30_IRQn);
    NVIC_EnableIRQ(TIMER20_IRQn);
    NVIC_EnableIRQ(TIMER10_IRQn);
    NVIC_EnableIRQ(SysTick_IRQn);

    // Start Test

    for(i = 0; i < 0x100000; i++);  // delay
    cputs("TEST START ================================\r\n");

    __enable_irq();     // Global Interrupt enable

    intr_cnt = 0;

    for(i = 0; i < 0x100000; i++);  // delay

#if 1
    SysTick_Config(SystemCoreClock/10);   // 100ms
    HAL_TIMER1n_Start(TIMER10);         // Start TIMER10
    HAL_TIMER2n_Start(TIMER20);         // Start TIMER20
	HAL_TIMER3n_Start(TIMER30);         // Start TIMER30
#endif

#if 0
    HAL_TIMER3n_Start(TIMER30);                 // Start TIMER30
    HAL_TIMER2n_Start(TIMER20);                 // Start TIMER20
    HAL_TIMER1n_Start(TIMER10);                 // Start TIMER10
    SysTick_Config(SystemCoreClock/10);     // 100ms
#endif

#if 0
    HAL_TIMER1n_Start(TIMER10);                 // Start TIMER10
    HAL_TIMER2n_Start(TIMER20);                 // Start TIMER20
    HAL_TIMER3n_Start(TIMER30);                 // Start TIMER30
#endif

#if 0
    HAL_TIMER2n_Start(TIMER20);                 // Start TIMER20
    HAL_TIMER3n_Start(TIMER30);                 // Start TIMER30
    HAL_TIMER1n_Start(TIMER10);                 // Start TIMER10
#endif

    while(1) { }
}


/* --------------------------------- End Of File ------------------------------ */
