/*************************************************************************
* @file		A31G22x_it.c
* @brief	Contains all macro definitions and function prototypes
*           for system clock initialization
* @version	1.00
* @date		22 July 2020
* @author	ABOV PTS team
*
* Copyright(C) 2020, ABOV Semiconductor All rights reserved.
*
**************************************************************************/
#include "main_conf.h"

/* Private typedef -----------------------------------------------------------*/
/* Private define ------------------------------------------------------------*/
/* Private macro -------------------------------------------------------------*/
/* Private variables ---------------------------------------------------------*/
/* Private define ------------------------------------------------------------*/

#define USED_HSI            /* SystemClock HSI : 32MHz    */
/*#define USED_LSI */  		/* SystemClock LSI : 500KHz   */
/*#define USED_HSE */  		/* SystemClock HSE : 8KHz     */
/*#define USED_LSE */		/* SystemClock LSE : 32.768KHz*/
/*#define USED_HSEPLL */ 	/* SystemClock HSE PLL 	: 48MHz    */ 
/*#define USED_HSIPLL */	/* SystemClock HSI PLL	: 48MHz    */

/* System Clock source */
#define HSI_OSC (32000000)

/* System Clock source */
#define HSE_OSC  (8000000)

/* Clock Out Selection ( PF 4 Monitoring )*/
#define USED_CLKO


/* Private function prototypes -----------------------------------------------*/
void SystemClock_Config (void);
/* Private variables ---------------------------------------------------------*/


/**************************************************************************
 * @brief			Initialize default clock for A33M11x Board
 * @param[in]		None
 * @return			None
 **************************************************************************/
void SystemClock_Config (void)
{

#ifdef USED_LSI	    
    HAL_SCU_SetSystemClock(SYSCLK_LSI, SYSDIV_NONE);
#endif
    
#ifdef USED_LSE
    HAL_SCU_SetSystemClock(SYSCLK_LSE, SYSDIV_NONE);    
#endif

#ifdef USED_HSI
    HAL_SCU_SetSystemClock(SYSCLK_HSI, SYSDIV_NONE);    
#endif

#ifdef USED_HSE
    HAL_SCU_SetSystemClock(SYSCLK_HSE, SYSDIV_NONE);        
#endif

#ifdef USED_HSIPLL
    HAL_SCU_SetPLLFreq(IRC32MHz, PLL48MHz);    
#endif    
    
#ifdef USED_HSEPLL
    HAL_SCU_SetPLLFreq(XTAL8MHz, PLL48MHz);        
#endif
    
    
#ifdef USED_CLKO    
    HAL_SCU_ClockOutput(4, ENABLE);
#endif
}
