/**********************************************************************
* @file		main.c
* @version	1.00
* @date		
* @author	ABOV AE team
*
* Copyright(C) 2020, ABOV Semiconductor
* All rights reserved.
*
**********************************************************************/
#include "main_conf.h"

/* Private typedef -----------------------------------------------------------*/
/* Private define ------------------------------------------------------------*/
/* Private macro -------------------------------------------------------------*/
/* Private variables ---------------------------------------------------------*/
/* Private define ------------------------------------------------------------*/
/* Private function prototypes -----------------------------------------------*/
void DEBUG_MenuPrint(void);
void DEBUG_Init(void);
void GPIO_Configure(void);
void CLOCK_Configure(void);
void SysTick_Configure(void);
void SCU_SystemClockRun(void);
void mainloop(void);
int main (void);
void Error_Handler(void);

/* Private variables ---------------------------------------------------------*/
uint32_t MilliSec;
const uint8_t menu[] =
"\r\n************************************************\n\r"
" Clock change Test (LSI) \n\r"
"\t - MCU: A31G22x \n\r"
"\t - Core: ARM Cortex-M0+ \n\r"
"\t - Communicate via: UART1 - 38400 bps \n\r"
"************************************************\n\r";



/*********************************************************************//**
 * @brief		delay
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void delay(void) 
{
	uint32_t i;
	
	for (i = 0; i < 0x100 ; i++) ;
}


/**********************************************************************
 * @brief		menu Print
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void DEBUG_MenuPrint(void)
{
	uint32_t i;    
    
	#ifdef _DEBUG_MSG
	cputs((char *)menu);
	#endif
    
    for (i = 0; i <0x100000; i++);
}


/**********************************************************************
 * @brief		DEBUG_Init
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void DEBUG_Init(void)
{
	#ifdef _DEBUG_MSG
	debug_frmwrk_init();
	#endif
}


/**********************************************************************
 * @brief		GPIO_Configure
 * @param[in]	None
 * @return 	None
 **********************************************************************/
void GPIO_Configure(void)
{
	// Test Pin setting PE2
	HAL_GPIO_ConfigOutput(PE, 2, PCU_MODE_PUSH_PULL);
	HAL_GPIO_ConfigPullup(PE, 2, PCU_PUPD_DISABLE);	
	HAL_GPIO_ClearPin(PE, _BIT(2));

	// Test Pin setting PE1
	HAL_GPIO_ConfigOutput(PE, 1, PCU_MODE_PUSH_PULL);
	HAL_GPIO_ConfigPullup(PE, 1, PCU_PUPD_DISABLE);
	HAL_GPIO_ClearPin(PE, _BIT(1));
}

/**********************************************************************
 * @brief		CLOCK_Configure
 * @param[in]	None
 * @return 	None
 **********************************************************************/
void CLOCK_Configure(void)
{
    /* LSI Clock Setting 500kHz */
	HAL_SCU_LSI_ClockConfig(LSI_EN);            /*< Enable LSI Clock Source */
    	
	HAL_SCU_SystemClockChange(SCCR_LSI);        /*< Change System Clock to LSI */
	
	SystemCoreClock = 500000uL; 	            /*< Update System Core Clock (HCLK) 500kHz */
	SystemPeriClock = 500000uL; 	            /*< Update System Peri Clock (PCLK) 500kHz */
        
	HAL_GPIO_ConfigOutput((PORT_Type *)PF, 4, PCU_MODE_ALT_FUNC);         /*< PF4 port is used alternative function */    
    HAL_GPIO_ConfigFunction((PORT_Type *)PF, 4, PCU_ALT_FUNCTION_1);      /*< PF4 port is used to CLKO pin */
    
    HAL_SCU_ClockOutput(4, ENABLE);             /*< Enable Clock Output (CLKO, PF4) PCLK/{2*(1+4)} = 500kHz/10 = 50kHz */
}
	
/**********************************************************************
 * @brief		SysTick_Configure
 * @param[in]	None
 * @return 	None
 **********************************************************************/
void SysTick_Configure(void)
{
	 /*1MilliSec interrupt*/
	SysTick_Config(SystemCoreClock/1000);
}
/**********************************************************************
 * @brief		SCU_SystemClockRun
 * @param[in]	None
 * @return 	None
 **********************************************************************/
void SCU_SystemClockRun(void)
{	
	uint32_t i;
	
for (i = 0; i < 10; i++){
    if (i % 2){
        HAL_GPIO_SetPin(PE, _BIT(2));
        delay();
    }
    else {
        HAL_GPIO_ClearPin(PE, _BIT(2));
        delay();
    }
}
delay();
    
}


/**********************************************************************
 * @brief		Main loop
 * @param[in]	None
 * @return	None
 **********************************************************************/
void mainloop(void)
{
	/*Configure menu prinf*/
	DEBUG_MenuPrint();
	
	/*Configure port peripheral*/
	GPIO_Configure();
	
//	/*CLOCK Configure*/
	CLOCK_Configure();
	
	/*SysTick Configure*/
	SysTick_Configure();
  
	/* Enable IRQ Interrupts */
	__enable_irq();
	
	
	while(1)
	{
        /* Infinite loop */
        SCU_SystemClockRun();

	}
}

/**********************************************************************
 * @brief		Main program
 * @param[in]	None
 * @return	None
 **********************************************************************/
int main (void)
{
	 /* Initialize all port */
	Port_Init(); 

	/* Configure the system clock to HSI  32MHz */
	SystemClock_Config();
	
	/* Initialize Debug frame work through initializing USART port  */
	DEBUG_Init();		
	
	/* Infinite loop */
	mainloop();  

	return (0);
}

/**********************************************************************
  * @brief  Reports the name of the source file and the source line number
  *   where the check_param error has occurred.
  * @param file: pointer to the source file name
  * @param line: check_param error line source number
  * @retval : None
 **********************************************************************/
void Error_Handler(void)
{
    while (1)
    {
    }
}

#ifdef  USE_FULL_ASSERT
/**********************************************************************
  * @brief  Reports the name of the source file and the source line number
  *   where the assert_param error has occurred.
  * @param file: pointer to the source file name
  * @param line: assert_param error line source number
  * @retval : None
 **********************************************************************/
void check_failed(uint8_t* file, uint32_t line)
{ 
  /* User can add his own implementation to report the file name and line number,
     ex: printf("Wrong parameters value: file %s on line %d\r\n", file, line) */

   /* Infinite loop */
   while (1)
   {
   }
}
#endif
