/*****************************************************************************
* @file     TestCode_DAC_BufferClear.c
* @brief    Contains the external function prototypes for TestCode
* @author   AE Team, ABOV Semiconductor Co., Ltd.
* @version  V0.0.1
* @date     16. MAY. 2019
*
* Copyright(C) 2019, ABOV Semiconductor
* All rights reserved.
*
*
********************************************************************************
* DISCLAIMER
*
* THE PRESENT FIRMWARE WHICH IS FOR GUIDANCE ONLY AIMS AT PROVIDING CUSTOMERS
* WITH CODING INFORMATION REGARDING THEIR PRODUCTS IN ORDER FOR THEM TO SAVE
* TIME. AS A RESULT, ABOV SEMICONDUCTOR DISCLAIMS ALL LIABILITIES FROM ANY
* DIRECT, INDIRECT OR CONSEQUENTIAL DAMAGES WITH RESPECT TO ANY CLAIMS ARISING
* FROM THE CONTENT OF SUCH FIRMWARE AND/OR THE USE MADE BY CUSTOMERS OF THE
* CODING INFORMATION CONTAINED HEREIN IN CONNECTION WITH THEIR PRODUCTS.
*
*******************************************************************************/

/*******************************************************************************
* Pre-processor Definition & Macro
*******************************************************************************/
#include "TestCode_DAC_BufferClear.h"
#include "main_conf.h"

/*******************************************************************************
* Private Typedef
*******************************************************************************/
#define TIMER10_INDEX				(0)
#define TIMER11_INDEX				(1)

#if (TIMER11_INDEX == 1)
    #define TIMER_SOURCE				(TIMER11_INDEX) // TIMER10_INDEX ~ TIMER11_INDEX
#elif (TIMER10_INDEX == 1)
    #define TIMER_SOURCE    (TIMER10_INDEX)
#endif

/*******************************************************************************
* Private Variable
*******************************************************************************/
static DAC_CFG_Type DAC_Config;

#if (TIMER_SOURCE == TIMER10_INDEX)
static TIMER1n_Type * const pTIMER1x = TIMER10;
static const IRQn_Type TIMER1x_IRQ = TIMER10_IRQn;
static const DAC_RELOAD_SIGNAL_Type RELOAD_SIGNAL = DAC_RELOAD_SIGNAL_TIMER10;
#else
static TIMER1n_Type * const pTIMER1x = TIMER11;
static const IRQn_Type TIMER1x_IRQ = TIMER11_IRQn;
static const DAC_RELOAD_SIGNAL_Type RELOAD_SIGNAL = DAC_RELOAD_SIGNAL_TIMER11;
#endif
static TIMER1n_CFG_Type TIMER1n_Config;
static Bool SysTickFlag;
int Cnt;
/*******************************************************************************
* Private Function Prototype
*******************************************************************************/

/*******************************************************************************
* Public Function
*******************************************************************************/
/*******************************************************************************
* @brief      SysTick interrupt handler @ 1000Hz(1ms)
* @param      None
* @return     None
*******************************************************************************/
void SysTick_Handler_DAC_BufferClear(void) {
	SysTickFlag = TRUE;
}

/*******************************************************************************
* @brief      Timer10 interrupt handler
* @param      None
* @return     None
*******************************************************************************/

void TIMER10_IRQHandler_DAC_BufferClear(void) {
	cputs("TIMER10 Occurred\r\n");
	if (HAL_TIMER1n_GetStatus(TIMER10) & TIMER1n_STATUS_MATCH_INT) {
		HAL_TIMER1n_ClearStatus(TIMER10, TIMER1n_STATUS_MATCH_INT);
	}
    
	Cnt++;
	if(Cnt == 5) {
		HAL_DAC_ClearBuffer(DAC);
		Cnt = 0;
	}   
}

/*******************************************************************************
* @brief      Timer11 interrupt handler
* @param      None
* @return     None
*******************************************************************************/
void TIMER11_IRQHandler_DAC_BufferClear(void) {
	cputs("TIMER11 Occurred\r\n");
	if (HAL_TIMER1n_GetStatus(TIMER11) & TIMER1n_STATUS_MATCH_INT) {
		HAL_TIMER1n_ClearStatus(TIMER11, TIMER1n_STATUS_MATCH_INT);
	}
    
	Cnt++;
	if(Cnt == 5) {
        cputs("DAC BUffer Clear!\r\n");
		HAL_DAC_ClearBuffer(DAC);
		Cnt = 0;
	}  
}

/*******************************************************************************
* @brief      Main Timer Trigger Test
* @param      None
* @return     None
*******************************************************************************/
void Main_DacBufferClearTest(void) {
	uint32_t DAC_Data;
    
	// Initialize and configure timer1n for DAC trigger
	HAL_SCU_MiscClockConfig(1, TIMER1n_TYPE, CLKSRC_MCLK, 48);		// MCLK=48MHz, Divider=48, 48MHz/48 = 1MHz (TIMER10_CLK)
	HAL_SCU_Timer1n_ClockConfig(T1NCLK_MCCR1);					// Setting Timer1n External Clock source

	TIMER1n_Config.StartSync = DISABLE;
	TIMER1n_Config.ClearSync = DISABLE;
	TIMER1n_Config.ExtClock = DISABLE;
	TIMER1n_Config.MatchInterrupt = ENABLE;
	TIMER1n_Config.CaptureInterrupt = DISABLE;
	TIMER1n_Config.PrescalerData = 1000;				// 1MHz/1000 = 1kHz
	TIMER1n_Config.AData = 1000 - 1;					// 1kHz/(999+1) = 1Hz = 1s
	HAL_TIMER1n_Init(pTIMER1x, TIMER1n_PERIODIC_MODE, &TIMER1n_Config);			// Setting Timer1n Configuration

	// Timer10 Interrupt Setting
	NVIC_SetPriority(TIMER1x_IRQ, 3);
	NVIC_EnableIRQ(TIMER1x_IRQ);

	// DAO(PA6) setting
	HAL_GPIO_ConfigOutput(PA, 6, PCU_MODE_ALT_FUNC);
	HAL_GPIO_ConfigFunction(PA, 6, PCU_ALT_FUNCTION_3);
	HAL_GPIO_ConfigPullup(PA, 6, PCU_PUPD_DISABLE);

	// DAC setting
	DAC_Config.Stanby = ENABLE;						    // DAC normal mode
	DAC_Config.DAC_Out2 = DISABLE;						// Disconnect with comparator input
	DAC_Config.DAC_Out1 = DISABLE;						// Disconnect with ADC ch.20
	DAC_Config.DAC_Out0 = ENABLE;						// Connect with DAO(PA6)
	DAC_Config.OutBuffer = DISABLE;						// Bypass Output Buffer
	DAC_Config.ReloadSignal = RELOAD_SIGNAL;			// Setting Reload Timer
	HAL_DAC_Init(DAC, &DAC_Config);							// DAC Initialization

	// Initialize DAC data
	DAC_Data = 0x0;										// Update DAC Initialized Value
	HAL_DAC_SetData(DAC, (uint16_t)(DAC_Data << 4));		// Setting DAC value to DAC Data Register

	// Enable SysTick
	SysTick_Config(SystemCoreClock / 1000); 			// 1ms Systick

	HAL_TIMER1n_Start(pTIMER1x);							// Start TIMER10 Operation

	__enable_irq();

	HAL_DAC_Start(DAC);										// Start DAC Operation

	while (1) {
		HAL_DAC_SetData(DAC, 0xAAA << 4);					// Setting DAC Output value (Test value = 0xAAA)

		// After the 5 times of Timer10 tirgger event, DAC Buffer Clear will be operated. (Refed TIMER10_IRQHandler)

		if (SysTickFlag == TRUE) {
			DAC_Data++;												// Increased DAC Value
			DAC_Data &= 0x0FFFUL;									// Masked DAC value
			SysTickFlag = FALSE;
		}
	}
}

/* --------------------------------- End Of File ------------------------------ */
