/*****************************************************************************
* @file     TestCode_DAC_Polling.c
* @brief    Contains the external function prototypes for TestCode
* @author   AE Team, ABOV Semiconductor Co., Ltd.
* @version  V0.0.1
* @date     16. MAY. 2019
*
* Copyright(C) 2019, ABOV Semiconductor
* All rights reserved.
*
*
********************************************************************************
* DISCLAIMER
*
* THE PRESENT FIRMWARE WHICH IS FOR GUIDANCE ONLY AIMS AT PROVIDING CUSTOMERS
* WITH CODING INFORMATION REGARDING THEIR PRODUCTS IN ORDER FOR THEM TO SAVE
* TIME. AS A RESULT, ABOV SEMICONDUCTOR DISCLAIMS ALL LIABILITIES FROM ANY
* DIRECT, INDIRECT OR CONSEQUENTIAL DAMAGES WITH RESPECT TO ANY CLAIMS ARISING
* FROM THE CONTENT OF SUCH FIRMWARE AND/OR THE USE MADE BY CUSTOMERS OF THE
* CODING INFORMATION CONTAINED HEREIN IN CONNECTION WITH THEIR PRODUCTS.
*
*******************************************************************************/

/*******************************************************************************
* Pre-processor Definition & Macro
*******************************************************************************/
#include "TestCode_DAC_Polling.h"

// Peripherals Drivers
#include "main_conf.h"


/*******************************************************************************
* Private Typedef
*******************************************************************************/

/*******************************************************************************
* Private Variable
*******************************************************************************/
static DAC_CFG_Type DAC_Config;

/*******************************************************************************
* Private Function Prototype
*******************************************************************************/

/*******************************************************************************
* Public Function
*******************************************************************************/

/*******************************************************************************
* @brief      Main DAC Polling Test
* @param      None
* @return     None
*******************************************************************************/
void Main_DacPollingTest(void) {
	uint32_t DAC_Data;

	// DAO(PA6) setting
	HAL_GPIO_ConfigOutput(PA, 6, PCU_MODE_ALT_FUNC);
	HAL_GPIO_ConfigFunction(PA, 6, PCU_ALT_FUNCTION_3);
	HAL_GPIO_ConfigPullup(PA, 6, PCU_PUPD_DISABLE);

	// DAC setting
	DAC_Config.Stanby = DISABLE;									// DAC normal mode
	DAC_Config.DAC_Out2 = DISABLE;									// Disconnect with comparator input
	DAC_Config.DAC_Out1 = DISABLE;									// Disconnect with ADC ch.20
	DAC_Config.DAC_Out0 = ENABLE;									// Connect with DAO(PA6)
	DAC_Config.OutBuffer = DISABLE;									// Bypass Buffer
	DAC_Config.ReloadSignal = DAC_RELOAD_SIGNAL_ALWAYS;				// DAC reload signal
	HAL_DAC_Init(DAC, &DAC_Config);										// Initialized DAC Configuration

	// Initialize DAC data (Gain = 0DB, No Offset)
	DAC_Data = 0x00;
	HAL_DAC_SetData(DAC, (uint16_t)(DAC_Data << 4));									// Set DAC Data Value (0x0)
	HAL_DAC_Start(DAC);																	// Start DAC Operation

	// Make saw tooth waveform
    cprintf("DAC outputs the saw-tooth waveform...\r\n");
	while (1) {
		DAC_Data++;
		DAC_Data &= 0x0FFFUL;
		HAL_DAC_SetData(DAC, (uint16_t)(DAC_Data << 4));
        cprintf("DAC->DR: 0x%04X\r\n", HAL_DAC_GetData(DAC)); 
	}
}

/* --------------------------------- End Of File ------------------------------ */
