/*****************************************************************************
* @file     TestCode_DFMC_ReadProtection.c
* @brief    Contains the external function prototypes for TestCode
* @author   AE Team, ABOV Semiconductor Co., Ltd.
* @version  V0.0.1
* @date     16. MAY. 2019
*
* Copyright(C) 2019, ABOV Semiconductor
* All rights reserved.
*
*
********************************************************************************
* DISCLAIMER
*
* THE PRESENT FIRMWARE WHICH IS FOR GUIDANCE ONLY AIMS AT PROVIDING CUSTOMERS
* WITH CODING INFORMATION REGARDING THEIR PRODUCTS IN ORDER FOR THEM TO SAVE
* TIME. AS A RESULT, ABOV SEMICONDUCTOR DISCLAIMS ALL LIABILITIES FROM ANY
* DIRECT, INDIRECT OR CONSEQUENTIAL DAMAGES WITH RESPECT TO ANY CLAIMS ARISING
* FROM THE CONTENT OF SUCH FIRMWARE AND/OR THE USE MADE BY CUSTOMERS OF THE
* CODING INFORMATION CONTAINED HEREIN IN CONNECTION WITH THEIR PRODUCTS.
*
*******************************************************************************/

/*******************************************************************************
* Pre-processor Definition & Macro
*******************************************************************************/
#include "TestCode_DFMC_ReadProtection.h"
#include "main_conf.h"

/*******************************************************************************
* Private Typedef
*******************************************************************************/
// to test some area, you should be set the IROM1 Start address and the Startup in 'Options for Target(Alt+F7), Target tab.'
#define TEST_START_ADDR (0x0E000000UL)
#define TEST_END_ADDR   (0x0E008000UL)


/*******************************************************************************
* Private Variable
*******************************************************************************/

//static const uint32_t   BufferSize = 4096;	
#define BufferSize      (4096)
static unsigned char	Buffer[BufferSize];

static const uint8_t TEST_MENU[] =
"************************************************\n\r"
" Data Flash - Read Protection Test\n\r"
"\t - 1. View Read Protection OTP Area \n\r"
"\t - 2. PGM a LV1 data to Read Protection OTP Area \n\r"
"\t - 3. ERS a data from Read Protection OTP Area \n\r"
"\t - 4. Set Read Protection LV1 \n\r"
"\t - 5. Change Flash Access Timing Test \n\r"
"\t - 6. Data Flash Erase-Chip Test \n\r"
"\t - 7. Entered Password '12345678' to Preset Register \n\r"
"\t - 8. Entered Password '0x9AE0DAAF' to Preset Register \n\r"
"\t - 9. Insert Password '12345678' value \n\r"
"\t - 0. Insert Password '0x9AE0DAAF' value \n\r"
"\t - h. Print Menu \n\r"
"************************************************\n\r";

#if 1
const uint32_t DOTP_PROT0_TABLE_LV1[] =
{
	0x55555555, // Check Code A
	0xAAAAAAAA, // Check Code B
	0xFFFFFFFF, // Read Protection(Password : Level 1)
	
	0xEDD2023C, 0xFFFFFFAA,	// Save Read Protection     (DFMC->PWPRST, 0x40000244) with Value of 0x0E01_0008	
	0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,                             						        // < 0x0E010020UL
	0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, // < 0x0E010040UL
	0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, // < 0x0E010060UL
	0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, // < 0x0E010080UL
	0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, // < 0x0E0100A0UL
	0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, // < 0x0E0100C0UL
	0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, // < 0x0E0100E0UL
	0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, // < 0x0E010100UL
	0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, // < 0x0E010120UL
	0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, // < 0x0E010140UL
	0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, // < 0x0E010160UL
	0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, // < 0x0E010180UL
	0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, // < 0x0E0101A0UL
	0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, // < 0x0E0101C0UL
	0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, // < 0x0E0101E0UL
	0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, // < 0x0E0101F8UL
	0xFFFFFFFF, //Password Preset 1-Word(4-byte)(LSB) ; 0x0E0101F8, (Data : '1', '2', '3', '4', '5', '6', '7', '8', CRC32)
	0xFFFFFFFF  //Password Preset 1-Word(4-byte)(MSB) ; 0x0E0101FC, (Data : '1', '2', '3', '4', '5', '6', '7', '8', CRC32)
};
#else

static const uint32_t DOTP_PROT0_TABLE_LV1[] =
{
	0x55555555,					// Check code A
	0xAAAAAAAA,					// Check code B
	0xFFFFFFFF,

	0xEA2A4000,					// Set base address with 0x4000_0000	
	
	// <Set Password Preset>	4-word	
	0xED360244, 0x9AE0DAAF,		// Save 1st Password Preset (DFMC->PWPRST, 0x40000244) with Value of 0x0E01_01F8
	0xED360244, 0x9AE0DAAF,		// Save 2nd Password Preset (DFMC->PWPRST, 0x40000244) with Value of 0x0E01_01FC	

	// <Set Read Protection Level-1>	4-word
	0xEDD2023C, 0xFFFFFFAA,		// Save Read Protection     (DFMC->PWPRST, 0x40000244) with Value of 0x0E01_0008	
	

	0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, // < 0x0E010040UL
	0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, // < 0x0E010060UL
	0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, // < 0x0E010080UL
	0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, // < 0x0E0100A0UL
	0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, // < 0x0E0100C0UL
	0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, // < 0x0E0100E0UL
	0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, // < 0x0E010100UL
	0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, // < 0x0E010120UL
	0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, // < 0x0E010140UL
	0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, // < 0x0E010160UL
	0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, // < 0x0E010180UL
	0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, // < 0x0E0101A0UL
	0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, // < 0x0E0101C0UL
	0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, // < 0x0E0101E0UL
	0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, // < 0x0E0101F0UL
};
#endif

/*******************************************************************************
* Private Function Prototype
*******************************************************************************/
static void DFMC_PrintData(uint32_t DataAddr, int Size);
static void DFMC_FlashAccessTimingTest(void);
static void DFMC_FlashBytePgmTest(void);
static void DFMC_CheckReadProtectionStatus(void);

/*******************************************************************************
* Public Function
*******************************************************************************/
/*******************************************************************************
* @brief      DFMC Read Protection Test Code
* @param      None
* @return     None
*******************************************************************************/
void TestCode_DFMC_ReadProtection(void) {
	uint8_t cmd;


	DFMC_CheckReadProtectionStatus();
	cputs((char *)TEST_MENU);
	cputs(">");    
    
	while(1) {		
		
		cmd = cgetchar();
        
        if ((cmd > 0) && (cmd < 0x7F)) {

            if (cmd == '1') {
                cputs("1. View Read Protection OTP Area\n\r");
                cputs("<OTP - Read Protection Area ----------> \n\r");			
                DFMC_PrintData(DFMC_OPTION_READRPOT_ADDR, 512);
                DFMC_FlashBytePgmTest();
                
            } else if (cmd == '2') {
                cputs("2. PGM a LV1 data to Read Protection OTP Area \n\r");			
                HAL_DFMC_ByteProgram(DATA_OPTION_READPROT, DFMC_OPTION_READRPOT_ADDR, 512, (unsigned char *)DOTP_PROT0_TABLE_LV1);		
                cputs("\r\nOTP-READPROT ---------------------------------------\r\n");
                DFMC_PrintData(DFMC_OPTION_READRPOT_ADDR, 512);	
                
            } else if (cmd == '3') {
                cputs("3. ERS a data from Read Protection OTP Area \n\r");			
                HAL_DFMC_EraseSector(DATA_OPTION_READPROT, DFMC_OPTION_READRPOT_ADDR);
                cputs("\r\nOTP-READPROT ---------------------------------------\r\n");
                DFMC_PrintData(DFMC_OPTION_READRPOT_ADDR, 512);
                
            } else if (cmd == '4') {
                cputs("4. Set Read Protection LV1 \n\r");
                DFMC->MR = 0x66UL;
                DFMC->MR = 0x99UL;
                DFMC->RPROT = (0xB9 & DFMC_RPROT_RPROT_Msk); // Read Protection Level-1 with Password            
    //			DFMC->RPROT = (0xFF & DFMC_RPROT_RPROT_Msk); // Read Protection does not apply.
                DFMC->MR = 0x00UL;
                DFMC->MR = 0x00UL;

    			SCU->SCR = (SCU_SCR_WTIDKY_Msk | SCU_SCR_SWRST_Msk); // Internal System Reset (nRESET are not applied bootrom)			
                
            } else if (cmd == '5') {
                cputs("5. Change Flash Access Timing Test \n\r");
                DFMC_FlashAccessTimingTest();
                
            } else if (cmd == '6') {
                cputs("6. Data Flash Erase Chip & Read Protection OTP Erase\n\r");
//                __disable_irq();
                HAL_DFMC_EraseChip();
                HAL_DFMC_EraseSector(DATA_OPTION_READPROT, DFMC_OPTION_READRPOT_ADDR);
//                __enable_irq();
                
            } else if (cmd == '7') {
                cputs("7. Entered Password '12345678' to Preset Register \n\r");
                cputs("Entered Password '12345678' to Preset Register (12345678) ----------> \n\r\n\r");
                DFMC->PWPRST = 12345678;
                DFMC->PWPRST = 12345678;			

            } else if (cmd == '8') {
                cputs("Entered Password '0x9AE0DAAF' to Preset Register (0x9AE0DAAF) ----------> \n\r\n\r");
                DFMC->PWPRST = 0x9AE0DAAF;
                DFMC->PWPRST = 0x9AE0DAAF;			

            } else if (cmd == '9') {
                cputs("9. Insert Password '12345678' value \n\r");
                cputs("Entered Password '12345678' (12345678) ----------> \n\r\n\r");
                DFMC->PWIN = 12345678;
                DFMC->PWIN = 12345678;			
            
            } else if (cmd == '0') {
                cputs("0. Insert Password '0x9AE0DAAF' value \n\r");
                cputs("Entered Password '0x9AE0DAAF' (0x9AE0DAAF) ----------> \n\r\n\r");
                DFMC->PWIN = 0x9AE0DAAF;
                DFMC->PWIN = 0x9AE0DAAF;			
                
            } else if (cmd == 'h') {
                cputs((char *)TEST_MENU);
            } else if (cmd == 'x') {
                break;
            } else {
                cputs("\n\r");
            
            }
         
        }

	}
}

static void DFMC_PrintData(uint32_t DataAddr, int Size) {
	int i;
	for(i = 0 ; i < Size ; i+= sizeof(uint32_t)) {
		if ((i > 0) && (i % 32 == 0)) {
			cputs("\r\n");
		}		
		cprintf("0x%08X ", (*(volatile uint32_t *)(DataAddr + i)));
	}		
	cputs("\n\r\n\r");	
}


/*******************************************************************************
* @brief      DFMC_FlashAccessTimingTest
* @param      None
* @return     None
*******************************************************************************/
static void DFMC_FlashAccessTimingTest(void) {
	uint32_t Reg16;
	int i;

	// Check Current Status
	cputs("Current Flash Status ------------------------------------------\r\n");
	cprintf("Flash Access Timing : %d\r\n", HAL_DFMC_GetFlashAccessTiming());
	cprintf("System Core Clock : %d\r\n", SystemCoreClock);
	cprintf("Gets Flash Wait Time : %d\r\n\r\n", HAL_DFMC_GetFlashWaitValue());

	// Set Flash Access Timing Value
	cputs("Test Start ------------------------------------------\r\n");
	Reg16 = DFMC_WAIT_MAX_INDEX; // Max flash wait time value that can be set
	for (i = 0; i < Reg16 ; i++) {
		HAL_DFMC_SetflashAccesstiming(SystemCoreClock, i, ENABLE); // Setting Flash Wait Time from 0 ~ 32
		cprintf("System Core Clock : %d\r\n", SystemCoreClock);
		cprintf("Flash Access Timing : %d\r\n", HAL_DFMC_GetFlashAccessTiming());

		cprintf("Sets Flash Wait Time : %d\r\n", i);
		cprintf("Gets Flash Wait Time : %d\r\n", HAL_DFMC_GetFlashWaitValue());
		if (i != HAL_DFMC_GetFlashWaitValue()) { // If setting value and current value are not matched, print the error message.
			cputs("-->The setting value exceeds the specification!\r\n\r\n");
		}
	}
	cputs("\r\n");
}


/*******************************************************************************
* @brief      DFMC_FlashBytePgmTest
* @param      None
* @return     None
*******************************************************************************/
static void DFMC_FlashBytePgmTest(void) {
	int i;
	int result;
	int Valid;
	int byteSize = 512;
	
	// Fill a value to buffer 
	for(i = 0 ; i < byteSize ; i++) {
		Buffer[i] = (i & 0xff);
	}
	
	// DATA Area PROGRAM Test ///////////////////////////////////////////////////////	
	for(i = TEST_START_ADDR; i < TEST_END_ADDR; i += byteSize) {
		__disable_irq();
		result = HAL_DFMC_ByteProgram(DATA_OPTION_NOT_USE, i, byteSize, Buffer);
		__enable_irq();
		
		if (result) {
            cprintf("PGM Error! at 0x%08X\r\n", i);
			Valid = FALSE;
			break;
		} else {
			Valid = TRUE;
		}
		
	}
//	DFMC_PrintData(0X0E000000, 32);	
	
	if (Valid == TRUE) {
		cputs("PGM TEST OK\r\n");
	} else {
		cputs("PGM TEST FAIL\r\n");
	}	
	
	for(i = TEST_START_ADDR; i < TEST_END_ADDR; i+= byteSize) {
		result = HAL_DFMC_Verify(i, byteSize, Buffer);
		
		if (result != (i + byteSize)) {
			cprintf("PGM VERIFY Error! at 0x%08X\r\n", i);
			Valid = FALSE;
			break;
		} else {
			Valid = TRUE;
		}
	}
	
	if (Valid == TRUE) {
		cputs("PGM VERIFY OK\r\n");
	} else {
		cputs("PGM VERIFY FAIL\r\n");
	} 

}


/*******************************************************************************
* @brief      DFMC_CheckReadProtectionStatus
* @param      None
* @return     None
*******************************************************************************/
static void DFMC_CheckReadProtectionStatus(void) {
	uint32_t Reg32;

	// View Read Protection Status
	Reg32 = DFMC->RPROT;
	cprintf("\r\nDFMC->RPROT SFR = [0x%08X]\r\n");
	
	if ((Reg32 & DFMC_RPROT_DBGMOD_Msk) == DFMC_RPROT_DBGMOD_Msk) {
		cputs("Debug Operating Mode\r\n");
	} else {
		cputs("Debug NOT Working \r\n");
	}
	
	if ((Reg32 & DFMC_RPROT_SRBOOT_Msk) == DFMC_RPROT_SRBOOT_Msk) {
		cputs("SRAM Boot Mode \r\n");
	} else {
		cputs("Normal Mode \r\n");
	}
	
	if ((Reg32 & DFMC_RPROT_PWMATCH_Msk) == DFMC_RPROT_PWMATCH_Msk) {
		cputs("Matched Password!\r\n");
	} else {
		cputs("Wrong Password or Password has not been entered. \r\n");
	}

	if ((Reg32 & DFMC_RPROT_CERSD_Msk) == DFMC_RPROT_CERSD_Msk) {
		cputs("Chip Erased Done\r\n");
	}
	
	if ((Reg32 & DFMC_RPROT_RPBERSD_Msk) == DFMC_RPROT_RPBERSD_Msk) {
		cputs("Read Protection OTP Erase Done\r\n");
	}
	

	if ((Reg32 & CFMC_RPROT_LVL1_EN_Msk) == CFMC_RPROT_LVL1_EN_Msk) {
		cputs("Enable Read Protection Level 1 \r\n");
		
		if ((Reg32 & CFMC_RPROT_LVL1_STS_Msk) == CFMC_RPROT_LVL1_STS_Msk) {
			cputs("Read Protection Level 1 Status \r\n");
		} else {
			cputs("Read Protection Level 1 - Normal Status\r\n");
		}		
	}
	
	if ((Reg32 & CFMC_RPROT_RPROT_Msk) == 0xFF) {
		cputs("Unprotected! \r\n");
	}
	
	cputs("\r\n\r\n");
}

/* --------------------------------- End Of File ------------------------------ */
