/*****************************************************************************
* @file     TestCode_DFMC_Test.c
* @brief    Contains the external function prototypes for TestCode
* @author   AE Team, ABOV Semiconductor Co., Ltd.
* @version  V0.0.1
* @date     16. MAY. 2019
*
* Copyright(C) 2019, ABOV Semiconductor
* All rights reserved.
*
*
********************************************************************************
* DISCLAIMER
*
* THE PRESENT FIRMWARE WHICH IS FOR GUIDANCE ONLY AIMS AT PROVIDING CUSTOMERS
* WITH CODING INFORMATION REGARDING THEIR PRODUCTS IN ORDER FOR THEM TO SAVE
* TIME. AS A RESULT, ABOV SEMICONDUCTOR DISCLAIMS ALL LIABILITIES FROM ANY
* DIRECT, INDIRECT OR CONSEQUENTIAL DAMAGES WITH RESPECT TO ANY CLAIMS ARISING
* FROM THE CONTENT OF SUCH FIRMWARE AND/OR THE USE MADE BY CUSTOMERS OF THE
* CODING INFORMATION CONTAINED HEREIN IN CONNECTION WITH THEIR PRODUCTS.
*
*******************************************************************************/

/*******************************************************************************
* Pre-processor Definition & Macro
*******************************************************************************/
#include "TestCode_DFMC_Test.h"
#include "main_conf.h"

/*******************************************************************************
* Private Typedef
*******************************************************************************/
#define TEST_START_ADDR (0x0E000000UL)
#define TEST_END_ADDR   (0x0E008000UL)

/*******************************************************************************
* Private Variable
*******************************************************************************/
#define BufferSize (4096)
static unsigned char	Buffer[BufferSize];

static const uint8_t TEST_MENU[] =
"************************************************\n\r"
" Data Flash Test\n\r"
"\t - 1. 512Bytes Page Erase Test \n\r"
"\t - 2. 1KB Sector Erase Test \n\r"
"\t - 3. 4KB Sector Erase Test \n\r"
"\t - 4. Data Flash Erase-Chip Test \n\r"
"\t - 5. Data Flash Byte-Program Test \n\r"
"\t - 6. Data Flash Word-Program Test \n\r"
"\t - 7. Data Flash Option Area Access Test  \n\r"
"\t - 8. Change Flash Access Timing Test \n\r"
"\t - h. Print Menu \n\r"
"************************************************\n\r";

static const uint32_t DOTP_PROT0_TABLE_LV1[] =
{
	0x55555555, // Check Code A
	0xAAAAAAAA, // Check Code B
	0xFFFFFF00, // Read Protection(Password : Level 1)
	0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF,                                     // < 0x0F010020UL
	0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, // < 0x0F010040UL
	0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, // < 0x0F010060UL
	0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, // < 0x0F010080UL
	0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, // < 0x0F0100A0UL
	0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, // < 0x0F0100C0UL
	0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, // < 0x0F0100E0UL
	0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, // < 0x0F010100UL
	0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, // < 0x0F010120UL
	0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, // < 0x0F010140UL
	0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, // < 0x0F010160UL
	0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, // < 0x0F010180UL
	0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, // < 0x0F0101A0UL
	0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, // < 0x0F0101C0UL
	0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, // < 0x0F0101E0UL
	0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, 0xFFFFFFFF, // < 0x0F0101F8UL
	0x9AE0DAAF, //Password Preset 1-Word(4-byte)(LSB) ; 0x0F0101F8, (Data : '1', '2', '3', '4', '5', '6', '7', '8', CRC32)
	0x9AE0DAAF  //Password Preset 1-Word(4-byte)(MSB) ; 0x0F0101FC, (Data : '1', '2', '3', '4', '5', '6', '7', '8', CRC32)
};

/*******************************************************************************
* Private Function Prototype
*******************************************************************************/
static void DFMC_PrintData(uint32_t DataAddr, int Size);
static void DFMC_512SectorEraseTest(void);
static void DFMC_1kSectorEraseTest(void);
static void DFMC_4kSectorEraseTest(void);
static void DFMC_OptionAccesTest(void);
static void DFMC_FlashAccessTimingTest(void);
static void DFMC_EraseChipTest(void);
static void DFMC_FlashBytePgmTest(void);
static void DFMC_FlashWordPgmTest(void);

/*******************************************************************************
* Public Function
*******************************************************************************/
/*******************************************************************************
* @brief      DFMC Test Code
* @param      None
* @return     None
*******************************************************************************/
void TestCode_DFMC_Test(void) {
	uint8_t cmd;

	cputs((char *)TEST_MENU);
    cputs((char *)">");    
	while(1) {
		cmd = cgetchar();
        if ((cmd > 0) && (cmd <= 0x7F)) {
            if (cmd == '1') {
                cputs("1. 512Bytes Page Erase Test\n\r");
                DFMC_512SectorEraseTest();
            } else if (cmd == '2') {
                cputs("2. 1KB Erase Test \n\r");
                DFMC_1kSectorEraseTest();
            } else if (cmd == '3') {
                cputs("3. 4KB Erase Test \n\r");
                DFMC_4kSectorEraseTest();
            } else if (cmd == '4') {
                cputs("4. Data Flash Erase Chip \n\r");
                DFMC_EraseChipTest();	                
            } else if (cmd == '5') {
                cputs("5. Data Flash Byte-Program Test \n\r");
                DFMC_FlashBytePgmTest();
            } else if (cmd == '6') {
                cputs("6. Data Flash Word-Program Test \n\r");
                DFMC_FlashWordPgmTest();         
             } else if (cmd == '7') {
                cputs("7. Data Flash Option Area Access Test \n\r");
                DFMC_OptionAccesTest();
            } else if (cmd == '8') {
                cputs("8. Change Flash Access Timing Test \n\r");
                DFMC_FlashAccessTimingTest();               
            } else if (cmd == 'h') {
                cputs((char *)TEST_MENU);
            } else if (cmd == 'x') {
                break;
            } else {
                cputs("\n\r");
            }
        }
	}
}


/*******************************************************************************
* @brief      DFMC_PrintData
* @param      None
* @return     None
*******************************************************************************/
static void DFMC_PrintData(uint32_t DataAddr, int Size) {
	int i;
	for(i = 0 ; i < Size ; i+= sizeof(uint32_t)) {
		if ((i > 0) && (i % 32 == 0)) {
			cputs("\r\n");
		}		
        cprintf("0x%08X ", *(volatile uint32_t *)(DataAddr + i));
	}		
	cputs("\n\r\n\r");	
}

/*******************************************************************************
* @brief      DFMC_512SectorEraseTest
* @param      None
* @return     None
*******************************************************************************/
static void DFMC_512SectorEraseTest(void) {
	int	i;
	uint32_t SectorSize = 512;

	int result;
	Bool Valid;
	
	cprintf("Test Area: 0x%08X ~ 0x%08X\r\n", TEST_START_ADDR, TEST_END_ADDR-1);    
	// Fill init value to buffer 
	for(i = 0 ; i < SectorSize ; i++) {
		Buffer[i] = 0xFF;
	}

	// DATA Area ERASE Test ///////////////////////////////////////////////////////	
	for(i = TEST_START_ADDR; i < TEST_END_ADDR; i+= SectorSize) {
		result = HAL_DFMC_EraseSector(0, i); // 512B Sector Erase

		if (result) {
			cprintf("ERS Error! at 0x%08X\r\n", i);
			Valid = FALSE;
			break;
		} else {
			Valid = TRUE;
		}
	}
	
	if (Valid == TRUE) {
		cprintf("ERS TEST OK\r\n");
	} else {
		cprintf("ERS TEST FAIL\r\n");
	}
	
	
	for(i = TEST_START_ADDR; i < TEST_END_ADDR; i+= SectorSize) {
		result = HAL_DFMC_Verify(i, SectorSize, Buffer);
		
		if (result != (i + SectorSize)) {
			cprintf("ERS VERIFY Error! at 0x%08X\r\n", i);
			Valid = FALSE;
			break;
		} else {
			Valid = TRUE;
		}
	}
	if (Valid ==  TRUE) {
		cprintf("ERS VERIFY OK\r\n");
	} else {
		cprintf("ERS VERIFY FAIL\r\n");
	}

	cprintf("\r\n");
}

/*******************************************************************************
* @brief      DFMC_1kSectorEraseTest
* @param      None
* @return     None
*******************************************************************************/
static void DFMC_1kSectorEraseTest(void) {
	int	i;
	uint32_t SectorSize = 1024;	
	int result;
	Bool Valid;
	
	
	cprintf("Test Area: 0x%08X ~ 0x%08X\r\n", TEST_START_ADDR, TEST_END_ADDR-1);  
	// Fill init value to buffer 
	for(i = 0 ; i < SectorSize ; i++) {
		Buffer[i] = 0xFF;
	}
	
	// DATA Area ERASE Test ///////////////////////////////////////////////////////	
	for(i = TEST_START_ADDR; i < TEST_END_ADDR; i+= SectorSize) {
		result = HAL_DFMC_EraseSector1k(0, i); // 1KB Sector Erase

		if (result) {
			cprintf("ERS Error! at 0x%08X\r\n", i);
			Valid = FALSE;
			break;
		} else {
			Valid = TRUE;
		}
	}
	
	if (Valid == TRUE) {
		cprintf("ERS TEST OK\r\n");
	} else {
		cprintf("ERS TEST FAIL\r\n");
	}
	
	
	for(i = TEST_START_ADDR; i < TEST_END_ADDR; i+= SectorSize) {	
		result = HAL_DFMC_Verify(i, SectorSize, Buffer);
		
		if (result != (i + SectorSize)) {
			cprintf("ERS VERIFY Error! at 0x%08X\r\n", i);
			Valid = FALSE;
			break;
		} else {
			Valid = TRUE;
		}
	}
	if (Valid ==  TRUE) {
		cprintf("ERS VERIFY OK\r\n");
	} else {
		cprintf("ERS VERIFY FAIL\r\n");
	}

	cprintf("\r\n");
}

/*******************************************************************************
* @brief      DFMC_4kSectorEraseTest
* @param      None
* @return     None
*******************************************************************************/
static void DFMC_4kSectorEraseTest(void) {
	int	i;
	uint32_t SectorSize = 4096;	
	int result;
	Bool Valid;
	
	cprintf("Test Area: 0x%08X ~ 0x%08X\r\n", TEST_START_ADDR, TEST_END_ADDR-1);  
	// Fill init value to buffer 
	for(i = 0 ; i < SectorSize ; i++) {
		Buffer[i] = 0xFF;
	}
	
	// DATA Area ERASE Test ///////////////////////////////////////////////////////	
	for(i = TEST_START_ADDR; i < TEST_END_ADDR; i+= SectorSize) {
		result = HAL_DFMC_EraseSector4k(0, i); // 4K Sector Erase

		if (result) {
			cprintf("ERS Error! at 0x%08X\r\n", i);
			Valid = FALSE;
			break;
		} else {
			Valid = TRUE;
		}
	}
	
	if (Valid == TRUE) {
		cprintf("ERS TEST OK\r\n");
	} else {
		cprintf("ERS TEST FAIL\r\n");
	}
	
	
	for(i = TEST_START_ADDR; i < TEST_END_ADDR; i+= SectorSize) {
		result = HAL_DFMC_Verify(i, SectorSize, Buffer);
		
		if (result != (i + SectorSize)) {
			cprintf("ERS VERIFY Error! at 0x%08X\r\n", i);
			Valid = FALSE;
			break;
		} else {
			Valid = TRUE;
		}
	}
	if (Valid ==  TRUE) {
		cprintf("ERS VERIFY OK\r\n");
	} else {
		cprintf("ERS VERIFY FAIL\r\n");
	}

	cprintf("\r\n");	
}

/*******************************************************************************
* @brief      DFMC_OPTIONAccesTest
* @param      None
* @return     None
*******************************************************************************/
static void DFMC_OptionAccesTest(void)
{
	uint32_t SectorSize = 512;

	// Read Protection Area
	HAL_DFMC_EraseSector(DATA_OPTION_READPROT, DFMC_OPTION_READRPOT_ADDR);
	cprintf("\r\nOTP-READPROT 0x%08X---------------------------------------\r\n", DFMC_OPTION_READRPOT_ADDR);
	DFMC_PrintData(DFMC_OPTION_READRPOT_ADDR, SectorSize);

    HAL_DFMC_ByteProgram(DATA_OPTION_READPROT, DFMC_OPTION_READRPOT_ADDR, SectorSize, (unsigned char *)DOTP_PROT0_TABLE_LV1);
	cprintf("\r\nOTP-READPROT 0x%08X---------------------------------------\r\n", DFMC_OPTION_READRPOT_ADDR);
	DFMC_PrintData(DFMC_OPTION_READRPOT_ADDR, SectorSize);    	

    
#if 1     
	// OPTION Area Test ////////////////////////////////////////////////////////    
	HAL_DFMC_EraseSector(DATA_OPTION, DFMC_OPTION_USERDATA0_ADDR);
	HAL_DFMC_EraseSector(DATA_OPTION, DFMC_OPTION_USERDATA1_ADDR);
	HAL_DFMC_EraseSector(DATA_OPTION, DFMC_OPTION_USERDATA2_ADDR);    
	
	cprintf("\r\nOTP-OPTION0 0x%08X---------------------------------------\r\n", DFMC_OPTION_USERDATA0_ADDR);
	DFMC_PrintData(DFMC_OPTION_USERDATA0_ADDR, SectorSize);
	cprintf("\r\nOTP-OPTION1 0x%08X---------------------------------------\r\n", DFMC_OPTION_USERDATA1_ADDR);
	DFMC_PrintData(DFMC_OPTION_USERDATA1_ADDR, SectorSize);
	cprintf("\r\nOTP-OPTION2 0x%08X---------------------------------------\r\n", DFMC_OPTION_USERDATA2_ADDR);
	DFMC_PrintData(DFMC_OPTION_USERDATA2_ADDR, SectorSize);
           
    HAL_DFMC_ByteProgram(DATA_OPTION, DFMC_OPTION_USERDATA0_ADDR, SectorSize, (unsigned char *)DOTP_PROT0_TABLE_LV1);
    HAL_DFMC_ByteProgram(DATA_OPTION, DFMC_OPTION_USERDATA1_ADDR, SectorSize, (unsigned char *)DOTP_PROT0_TABLE_LV1);
    HAL_DFMC_ByteProgram(DATA_OPTION, DFMC_OPTION_USERDATA2_ADDR, SectorSize, (unsigned char *)DOTP_PROT0_TABLE_LV1);
	
	cprintf("\r\nOTP-OPTION0 0x%08X---------------------------------------\r\n", DFMC_OPTION_USERDATA0_ADDR);
	DFMC_PrintData(DFMC_OPTION_USERDATA0_ADDR, SectorSize);	
	cprintf("\r\nOTP-OPTION1 0x%08X---------------------------------------\r\n", DFMC_OPTION_USERDATA1_ADDR);
	DFMC_PrintData(DFMC_OPTION_USERDATA1_ADDR, SectorSize);
	cprintf("\r\nOTP-OPTION2 0x%08X---------------------------------------\r\n", DFMC_OPTION_USERDATA2_ADDR);
	DFMC_PrintData(DFMC_OPTION_USERDATA2_ADDR, SectorSize);
#endif    
          
    
	/////////////////////////////////////////////////////////////////////////

}

/*******************************************************************************
* @brief      DFMC_FlashAccessTimingTest
* @param      None
* @return     None
*******************************************************************************/
static void DFMC_FlashAccessTimingTest(void) {
	uint32_t Reg16;
	int i;

	// Check Current Status
	cprintf("Current Flash Status ------------------------------------------\r\n");    
    cprintf("Flash Access Speed [Hz]: %d\r\n", HAL_DFMC_GetFlashAccessTiming());
    cprintf("System Core Clock [Hz]: %d\r\n", SystemCoreClock);
	cprintf("Gets Flash Wait Time: %d\r\n\r\n", HAL_DFMC_GetFlashWaitValue());    

	// Set Flash Access Timing Value
	cprintf("Test Start ------------------------------------------\r\n");
	Reg16 = DFMC_WAIT_MAX_INDEX; // Max flash wait time value that can be set
	for (i = 0; i < Reg16 ; i++) {
		HAL_DFMC_SetflashAccesstiming(SystemCoreClock, i, ENABLE); // Setting Flash Wait Time from 0 ~ 5       
        cprintf("System Core Clock [Hz]: %d\r\n", SystemCoreClock);
        cprintf("Flash Access Timing [Hz]: %d\r\n", HAL_DFMC_GetFlashAccessTiming());        

		cprintf("Sets Flash Wait Time : %d\r\n", i);
		cprintf("Gets Flash Wait Time : %d\r\n", HAL_DFMC_GetFlashWaitValue());
		if (i != HAL_DFMC_GetFlashWaitValue()) { // If setting value and current value are not matched, print the error message.
			cprintf("-->The setting value exceeds the specification!\r\n\r\n");
		}
	}
	cprintf("\r\n");
}


/*******************************************************************************
* @brief      DFMC_EraseChip
* @param      None
* @return     None
*******************************************************************************/
static void DFMC_EraseChipTest(void) {
	int i;
	uint32_t SectorSize = 512;
	int result;
	Bool Valid;

	// Fill init value to buffer 
	for(i = 0 ; i < SectorSize ; i++) {
		Buffer[i] = 0xFF;
	}
	
	// Check Current Status
	cprintf("Erase Chip \r\n");
	HAL_DFMC_EraseChip();
	
	for(i = TEST_START_ADDR; i < TEST_END_ADDR; i+= SectorSize)
	{
		result = HAL_DFMC_Verify(i, SectorSize, Buffer);
		
		if (result != (i + SectorSize)) {
			cprintf("PGM VERIFY Error! at 0x%08X\r\n", i);
			Valid = FALSE;
			break;
		} else {
			Valid = TRUE;
		}
	}
	
	if (Valid == TRUE) {
		cprintf("PGM VERIFY OK\r\n");
	} else {
		cprintf("PGM VERIFY FAIL\r\n");
	}	
}


/*******************************************************************************
* @brief      DFMC_FlashBytePgmTest
* @param      None
* @return     None
*******************************************************************************/
static void DFMC_FlashBytePgmTest(void) {
	int i;
	int result;
	int Valid;
	int byteSize = 128;
    int pageSize = 512;
	
	// Fill init value to buffer 
	for(i = 0 ; i < byteSize ; i++) {
		Buffer[i] = 0xFF;
	}
        
	// DATA Area ERASE Test ///////////////////////////////////////////////////////	
    // Erase 512 bytes
	for(i = TEST_START_ADDR; i < (TEST_START_ADDR + pageSize); i += pageSize) {
		result = HAL_DFMC_EraseSector(DATA_OPTION_NOT_USE, i);
		
		if (result) {
			cprintf("ERS Error! at 0x%08X\r\n", i);
			Valid = FALSE;
			break;
		} else {
			Valid = TRUE;
		}		
	}    
	DFMC_PrintData(0X0E000000, byteSize);
    
	if (Valid == TRUE) {
		cprintf("ERS TEST OK\r\n");
	} else {
		cprintf("ERS TEST FAIL\r\n");
	}
	
	for(i = TEST_START_ADDR; i < (TEST_START_ADDR + pageSize); i+= pageSize) {
		result = HAL_DFMC_Verify(i, byteSize, Buffer);
		
		if (result != (i + byteSize)) {
			cprintf("ERS VERIFY Error! at 0x%08X\r\n", i);
			Valid = FALSE;
			break;
		} else {
			Valid = TRUE;
		}
	}    

	if (Valid == TRUE) {
		cprintf("ERS VERIFY OK\r\n");
	} else {
		cprintf("ERS VERIFY FAIL\r\n");
	}   
    
	// Fill a value to buffer 
	for(i = 0 ; i < byteSize ; i++) {
		Buffer[i] = ((0xAA+i) & 0xff);
	}    
    
	// DATA Area PROGRAM Test ///////////////////////////////////////////////////////
    // Program 128 bytes per Byte
	for(i = TEST_START_ADDR; i < (TEST_START_ADDR + byteSize); i += byteSize) {
		result = HAL_DFMC_ByteProgram(DATA_OPTION_NOT_USE, i, byteSize, Buffer);
		
		if (result) {
			cprintf("PGM Error! at 0x%08X\r\n", i);
			Valid = FALSE;
			break;
		} else {
			Valid = TRUE;
		}		
	}
    
	DFMC_PrintData(0X0E000000, byteSize);	
	
	if (Valid == TRUE) {
		cprintf("PGM TEST OK\r\n");
	} else {
		cprintf("PGM TEST FAIL\r\n");
	}	
	
	for(i = TEST_START_ADDR; i < (TEST_START_ADDR + byteSize); i+= byteSize) {
		result = HAL_DFMC_Verify(i, byteSize, Buffer);
		
		if (result != (i + byteSize)) {
			cprintf("PGM VERIFY Error! at 0x%08X\r\n", i);
			Valid = FALSE;
			break;
		} else {
			Valid = TRUE;
		}
	}
	
	if (Valid == TRUE) {
		cprintf("PGM VERIFY OK\r\n");
	} else {
		cprintf("PGM VERIFY FAIL\r\n");
	}
}

/*******************************************************************************
* @brief      DFMC_FlashWordPgmTest
* @param      None
* @return     None
*******************************************************************************/
static void DFMC_FlashWordPgmTest(void) {
	int i;
	int result;
	int Valid;
	int byteSize = 512;
	
	// Fill init value to buffer 
	for(i = 0 ; i < byteSize ; i++) {
		Buffer[i] = 0xFF;
	}
        
	// DATA Area ERASE Test ///////////////////////////////////////////////////////	
	for(i = TEST_START_ADDR; i < (TEST_START_ADDR + byteSize); i += byteSize) {
        result = HAL_DFMC_EraseSector(DATA_OPTION_NOT_USE, i);
		
		if (result) {
			cprintf("ERS Error! at 0x%08X\r\n", i);
			Valid = FALSE;
			break;
		} else {
			Valid = TRUE;
		}		
	}    
	DFMC_PrintData(0X0E000000, byteSize);
    
	if (Valid == TRUE) {
		cprintf("ERS TEST OK\r\n");
	} else {
		cprintf("ERS TEST FAIL\r\n");
	}
	
	for(i = TEST_START_ADDR; i < (TEST_START_ADDR + byteSize); i+= byteSize) {
		result = HAL_DFMC_Verify(i, byteSize, Buffer);
		
		if (result != (i + byteSize)) {
			cprintf("ERS VERIFY Error! at 0x%08X\r\n", i);
			Valid = FALSE;
			break;
		} else {
			Valid = TRUE;
		}
	}    

	if (Valid == TRUE) {
		cprintf("ERS VERIFY OK\r\n");
	} else {
		cprintf("ERS VERIFY FAIL\r\n");
	}   
    
	// Fill a value to buffer 
	for(i = 0 ; i < byteSize ; i++) {
		Buffer[i] = ((0xAA+i) & 0xff);
	}    
    
	// DATA Area PROGRAM Test ///////////////////////////////////////////////////////	
	for(i = TEST_START_ADDR; i < (TEST_START_ADDR + byteSize); i += byteSize) {
		result = HAL_DFMC_WordProgram(DATA_OPTION_NOT_USE, i, byteSize, Buffer);
		
		if (result) {
			cprintf("PGM Error! at 0x%08X\r\n", i);
			Valid = FALSE;
			break;
		} else {
			Valid = TRUE;
		}
	}
    
	DFMC_PrintData(0X0E000000, byteSize);	
	
	if (Valid == TRUE) {
		cprintf("PGM TEST OK\r\n");
	} else {
		cprintf("PGM TEST FAIL\r\n");
	}	
	
	for(i = TEST_START_ADDR; i < (TEST_START_ADDR + byteSize); i+= byteSize) {
		result = HAL_DFMC_Verify(i, byteSize, Buffer);
		
		if (result != (i + byteSize)) {
			cprintf("PGM VERIFY Error! at 0x%08X\r\n", i);
			Valid = FALSE;
			break;
		} else {
			Valid = TRUE;
		}
	}
	
	if (Valid == TRUE) {
		cprintf("PGM VERIFY OK\r\n");
	} else {
		cprintf("PGM VERIFY FAIL\r\n");
	}
}

/* --------------------------------- End Of File ------------------------------ */
