/*****************************************************************************
* @file     TestCode_GPIO_ISEG.c
* @brief    Contains the external function prototypes for TestCode
* @author   AE Team, ABOV Semiconductor Co., Ltd.
* @version  V0.0.1
* @date     27. MAR. 2020
*
* Copyright(C) 2019, ABOV Semiconductor
* All rights reserved.
*
*
********************************************************************************
* DISCLAIMER
*
* THE PRESENT FIRMWARE WHICH IS FOR GUIDANCE ONLY AIMS AT PROVIDING CUSTOMERS
* WITH CODING INFORMATION REGARDING THEIR PRODUCTS IN ORDER FOR THEM TO SAVE
* TIME. AS A RESULT, ABOV SEMICONDUCTOR DISCLAIMS ALL LIABILITIES FROM ANY
* DIRECT, INDIRECT OR CONSEQUENTIAL DAMAGES WITH RESPECT TO ANY CLAIMS ARISING
* FROM THE CONTENT OF SUCH FIRMWARE AND/OR THE USE MADE BY CUSTOMERS OF THE
* CODING INFORMATION CONTAINED HEREIN IN CONNECTION WITH THEIR PRODUCTS.
*
*******************************************************************************/

/*******************************************************************************
* Pre-processor Definition & Macro
*******************************************************************************/
#include "TestCode_GPIO_ISEG.h"
#include "main_conf.h"

#define ISEG_CHANNEL_MAX       (10)
/*******************************************************************************
* Private Typedef
*******************************************************************************/
typedef struct {
    PORT_Type * port;
    uint32_t    pin_num;
    char        alias[8];
} TEST_PORT_Type;

TEST_PORT_Type ISEG[ISEG_CHANNEL_MAX] = {
  {PE, 2, "ISEG0"},
  {PE, 3, "ISEG1"},
  {PE, 4, "ISEG2"},
  {PE, 5, "ISEG3"},
  {PE, 6, "ISEG4"},
  {PE, 7, "ISEG5"},
  {PD, 5, "ISEG6"},
  {PD, 4, "ISEG7"},
  {PD, 3, "ISEG8"},
  {PD, 2, "ISEG9"},
};


/*******************************************************************************
* Private Function Prototype
*******************************************************************************/

/*******************************************************************************
* Private Variable
*******************************************************************************/
uint32_t MilliSec;

/*******************************************************************************
* Public Function
*******************************************************************************/
void SysTick_Handler_GpioIseg(void) {
    if (MilliSec)
        MilliSec--;
}

/**********************************************************************
 * @brief		TestCode_GPIO_ISEG
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void TestCode_GPIO_ISEG(void) {
    int i;

    cputs("\r\n");

	/* ISEG Channel Init PD2~PD5,PE2~PE7) */
    for (i = 0; i < ISEG_CHANNEL_MAX; i++) {
        HAL_GPIO_ConfigFunction(ISEG[i].port, ISEG[i].pin_num, PCU_ALT_FUNCTION_4);
        HAL_GPIO_ConfigOutput(ISEG[i].port, ISEG[i].pin_num, PCU_MODE_ALT_FUNC);
        cprintf("ISEG %d Channel is initialized.\r\n", i);
    }

    /* ISEG Function Enable */
	HAL_PCU_UseISEG(ENABLE);
    cprintf("ISEG function is Enabled \r\n");

    SysTick_Config(SystemCoreClock/1000);
    __enable_irq();

    cprintf("\r\n\r\n-------------------ISEG Control Test\r\n");
    /* ISEG Port ON/OFF Test */
    for (i = 0; i < ISEG_CHANNEL_MAX; i++) {
        MilliSec = 500;
        while(MilliSec);
        cprintf("\r\n%s: ON\r\n", ISEG[i].alias);
        HAL_PCU_ConfigISEG(i, PCU_ISEG_STATUS_HIGH_CURRENT_LOW);
        cprintf("PCU2->ISEGR: 0x%08X\r\n", HAL_PCU_GetISEGStatus());
        cprintf("PCU2->ISEGIR: 0x%08X\r\n", HAL_PCU_GetISEGInvStatus());

        MilliSec = 500;
        while(MilliSec);

        cprintf("%s: OFF\r\n", ISEG[i].alias);
        HAL_PCU_ConfigISEG(i, PCU_ISEG_STATUS_FLOATING);
        cprintf("PCU2->ISEGR: 0x%08X\r\n", HAL_PCU_GetISEGStatus());
        cprintf("PCU2->ISEGIR: 0x%08X\r\n", HAL_PCU_GetISEGInvStatus());
    }

    cprintf("\r\n\r\n-------------------ISEG Inversion Control Test\r\n");
    /* ISEG Inversion Port ON/OFF Test */
    for (i = 0; i < ISEG_CHANNEL_MAX; i++) {
        MilliSec = 500;
        while(MilliSec);
        cprintf("\r\n%s: ON\r\n", ISEG[i].alias);
        HAL_PCU_ConfigISEGInv(i, PCU_ISEGINV_STATUS_HIGH_CURRENT_LOW);
        cprintf("PCU2->ISEGR: 0x%08X\r\n", HAL_PCU_GetISEGStatus());
        cprintf("PCU2->ISEGIR: 0x%08X\r\n", HAL_PCU_GetISEGInvStatus());

        MilliSec = 500;
        while(MilliSec);

        cprintf("%s: OFF\r\n", ISEG[i].alias);
        HAL_PCU_ConfigISEGInv(i, PCU_ISEGINV_STATUS_FLOATING);
        cprintf("PCU2->ISEGR: 0x%08X\r\n", HAL_PCU_GetISEGStatus());
        cprintf("PCU2->ISEGIR: 0x%08X\r\n", HAL_PCU_GetISEGInvStatus());
    }

    cprintf("\r\n\r\n-------------------ISEGx Control Test\r\n");
    /* ISEG Port ON/OFF Test */

    MilliSec = 500;
    while(MilliSec);

    HAL_PCU_ConfigISEGx(0x2AA, PCU_ISEG_STATUS_HIGH_CURRENT_LOW);
    cprintf("PCU2->ISEGR: 0x%08X\r\n", HAL_PCU_GetISEGStatus());
    cprintf("PCU2->ISEGIR: 0x%08X\r\n", HAL_PCU_GetISEGInvStatus());

    MilliSec = 500;
    while(MilliSec);

    HAL_PCU_ConfigISEGx(0x155, PCU_ISEG_STATUS_FLOATING);
    cprintf("PCU2->ISEGR: 0x%08X\r\n", HAL_PCU_GetISEGStatus());
    cprintf("PCU2->ISEGIR: 0x%08X\r\n", HAL_PCU_GetISEGInvStatus());


    cprintf("\r\n\r\n-------------------ISEGx Inversion Control Test\r\n");
    /* ISEG Port ON/OFF Test */

    MilliSec = 500;
    while(MilliSec);
    HAL_PCU_ConfigISEGxInv(0x2AA, PCU_ISEGINV_STATUS_HIGH_CURRENT_LOW);
    cprintf("PCU2->ISEGR: 0x%08X\r\n", HAL_PCU_GetISEGStatus());
    cprintf("PCU2->ISEGIR: 0x%08X\r\n", HAL_PCU_GetISEGInvStatus());

    MilliSec = 500;
    while(MilliSec);

    HAL_PCU_ConfigISEGxInv(0x155, PCU_ISEGINV_STATUS_FLOATING);
    cprintf("PCU2->ISEGR: 0x%08X\r\n", HAL_PCU_GetISEGStatus());
    cprintf("PCU2->ISEGIR: 0x%08X\r\n", HAL_PCU_GetISEGInvStatus());

    HAL_PCU_UseISEG(DISABLE);
    cprintf("\r\n\r\nCompleted ISEG pin Test! \r\n\r\n");
}

/* --------------------------------- End Of File ------------------------------ */
