/*****************************************************************************
* @file     TestCode_GPIO_InputPDebounce.c
* @brief    Contains the external function prototypes for TestCode
* @author   AE Team, ABOV Semiconductor Co., Ltd.
* @version  V0.0.1
* @date     27. MAR. 2020
*
* Copyright(C) 2019, ABOV Semiconductor
* All rights reserved.
*
*
********************************************************************************
* DISCLAIMER
*
* THE PRESENT FIRMWARE WHICH IS FOR GUIDANCE ONLY AIMS AT PROVIDING CUSTOMERS
* WITH CODING INFORMATION REGARDING THEIR PRODUCTS IN ORDER FOR THEM TO SAVE
* TIME. AS A RESULT, ABOV SEMICONDUCTOR DISCLAIMS ALL LIABILITIES FROM ANY
* DIRECT, INDIRECT OR CONSEQUENTIAL DAMAGES WITH RESPECT TO ANY CLAIMS ARISING
* FROM THE CONTENT OF SUCH FIRMWARE AND/OR THE USE MADE BY CUSTOMERS OF THE
* CODING INFORMATION CONTAINED HEREIN IN CONNECTION WITH THEIR PRODUCTS.
*
*******************************************************************************/

/*******************************************************************************
* Pre-processor Definition & Macro
*******************************************************************************/
#include "TestCode_GPIO_InputDebounce.h"
#include "main_conf.h"

/*******************************************************************************
* Private Typedef
*******************************************************************************/
typedef struct {
    PORT_Type * test_port;
    uint32_t    port_max_nums;
    char        alias[8];
} TEST_PORT_Type;

TEST_PORT_Type PORT_A = {PA, 16, "GPIO A"};
TEST_PORT_Type PORT_B = {PB, 16, "GPIO B"};
TEST_PORT_Type PORT_C = {PC, 16, "GPIO C"};
TEST_PORT_Type PORT_D = {PD, 16, "GPIO D"};
TEST_PORT_Type PORT_E = {PE, 16, "GPIO E"};
TEST_PORT_Type PORT_F = {(PORT_Type *)PF, 16,  "GPIO F"};

/*******************************************************************************
* Private Function Prototype
*******************************************************************************/
void TestCode_GPIO_InputDebounceTest(void);

static void gpio_irq_common(TEST_PORT_Type * port);
void GPIOAB_IRQHandler_Input(void);

/*******************************************************************************
* Private Variable
*******************************************************************************/

/*******************************************************************************
* Public Function
*******************************************************************************/
static void gpio_irq_common(TEST_PORT_Type * port) {
    int i;
    uint32_t Status;
    Status = HAL_GPIO_EXTI_GetState(port->test_port);

    cprintf("%s IER: 0x%08X\r\n", port->test_port->IER);
    cprintf("%s ICR: 0x%08X\r\n", port->test_port->ICR);
    cprintf("%s ISR: 0x%08X\r\n", port->test_port->ISR);

    for(i = 0; i < 16 ; i++) {

        // Rising Edge or High Level
        if ((Status & (2 << (i*2))) == (2 << (i*2))) {
            cprintf("%d Rising Edge Interrupt\r\n", i);
            PB->OUTDR ^= 6;
            HAL_GPIO_EXTI_ClearPin((PORT_Type *)port->test_port, Status);
        }
    }
}

void GPIOAB_IRQHandler_InputDebounce(void) {
    gpio_irq_common(&PORT_A);

}

void TestCode_GPIO_InputDebounceTest(void) {

    TIMER1n_CFG_Type TIMER1n_Config;

    // Timer for Debounce Test
    HAL_SCU_HSE_ClockConfig(HSE_EN);

    HAL_SCU_MiscClockConfig(1, TIMER1n_TYPE, CLKSRC_HSE, 4);       // 8MHz/4 = 2MHz TCLK
	HAL_SCU_Timer1n_ClockConfig(T1NCLK_MCCR1);
	TIMER1n_Config.ExtClock = DISABLE;                  // PCLK 1MHz
	TIMER1n_Config.PrescalerData = (2 - 1);             // 2MHz/(1 + 1) = 1MHz (1us)
	TIMER1n_Config.StartSync = DISABLE;
	TIMER1n_Config.ClearSync = DISABLE;
	TIMER1n_Config.OutputPolarity = TIMER1n_OUTPUT_POLARITY_HIGH;
	TIMER1n_Config.MatchInterrupt = DISABLE;
	TIMER1n_Config.CaptureInterrupt = DISABLE;

//    TIMER1n_Config.AData = 35-1;                          // 1us * 35 = 35us, T10O Period = 70us
    TIMER1n_Config.AData = 40-1;                          // 1us * 40 = 40us, T10O Period = 80us

    HAL_TIMER1n_Init(TIMER10, TIMER1n_PERIODIC_MODE, &TIMER1n_Config);
    HAL_TIMER1n_Start(TIMER10);

    // Timer10 T10O output
    HAL_GPIO_ConfigOutput(PE, 6, PCU_MODE_ALT_FUNC);
    HAL_GPIO_ConfigFunction(PE, 6, PCU_ALT_FUNCTION_1);

    // GPIO Setting
    SCU->PER1 |= (SCU_PER1_GPIOA_Msk | SCU_PER1_GPIOB_Msk);
    SCU->PCER1 |= (SCU_PCER1_GPIOA_Msk | SCU_PCER1_GPIOB_Msk);

    // PA/PB/PC port groups debounce
    HAL_SCU_MiscClockConfig(4, PD0_TYPE, CLKSRC_HSE, 80);   // F_debounce = 8MHz/80 = 100kHz (10us)


    // Interrupt Flag Check (PB6 Output)
    HAL_GPIO_ConfigOutput(PB, 6, PCU_MODE_PUSH_PULL);
    HAL_GPIO_ConfigPullup(PB, 6, PCU_PUPD_DISABLE);

    // Debounce Port
    HAL_GPIO_ConfigOutput(PA, 0, PCU_MODE_INPUT);
    HAL_GPIO_EXTI_Config((PORT_Type *)PA, 0, PCU_INTERRUPT_MODE_EDGE, PCU_INTERRUPT_CTRL_EDGE_RISING);       // Rising Edge interrupt
    HAL_GPIO_SetDebouncePin((PORT_Type *)PA, 0, ENABLE);                                                          // Debounce Enable
    HAL_GPIO_EXTI_ClearPin((PORT_Type *)PA, HAL_GPIO_EXTI_GetState((PORT_Type *)PA));

    // PORT A Test
    NVIC_EnableIRQ(GPIOAB_IRQn);
    NVIC_SetPriority(GPIOAB_IRQn, 3);
    __enable_irq();

    cputs("PORT A GPIO Interrupt Test\r\n");

    while(1) { }

}


/* --------------------------------- End Of File ------------------------------ */
