/*****************************************************************************
* @file     TestCode_GPIO_InputInterrupt.c
* @brief    Contains the external function prototypes for TestCode
* @author   AE Team, ABOV Semiconductor Co., Ltd.
* @version  V0.0.1
* @date     27. MAR. 2020
*
* Copyright(C) 2019, ABOV Semiconductor
* All rights reserved.
*
*
********************************************************************************
* DISCLAIMER
*
* THE PRESENT FIRMWARE WHICH IS FOR GUIDANCE ONLY AIMS AT PROVIDING CUSTOMERS
* WITH CODING INFORMATION REGARDING THEIR PRODUCTS IN ORDER FOR THEM TO SAVE
* TIME. AS A RESULT, ABOV SEMICONDUCTOR DISCLAIMS ALL LIABILITIES FROM ANY
* DIRECT, INDIRECT OR CONSEQUENTIAL DAMAGES WITH RESPECT TO ANY CLAIMS ARISING
* FROM THE CONTENT OF SUCH FIRMWARE AND/OR THE USE MADE BY CUSTOMERS OF THE
* CODING INFORMATION CONTAINED HEREIN IN CONNECTION WITH THEIR PRODUCTS.
*
*******************************************************************************/

/*******************************************************************************
* Pre-processor Definition & Macro
*******************************************************************************/
#include "TestCode_GPIO_InputInterrupt.h"
#include "main_conf.h"

/*******************************************************************************
* Private Typedef
*******************************************************************************/
typedef struct {
    PORT_Type * test_port;
    uint32_t    port_max_nums;
    char        alias[8];
    IRQn_Type   gpio_irq;
} TEST_PORT_Type;


TEST_PORT_Type PORT_A = {PA, 16, "GPIO A", GPIOAB_IRQn};
TEST_PORT_Type PORT_B = {PB, 16, "GPIO B", GPIOAB_IRQn};
TEST_PORT_Type PORT_C = {PC, 16, "GPIO C", GPIOCD_IRQn};
TEST_PORT_Type PORT_D = {PD, 16, "GPIO D", GPIOCD_IRQn};
TEST_PORT_Type PORT_E = {PE, 16, "GPIO E", GPIOE_IRQn};
TEST_PORT_Type PORT_F = {(PORT_Type *)PF, 16,  "GPIO F", GPIOF_IRQn};

/*******************************************************************************
* Private Function Prototype
*******************************************************************************/
static void gpio_init(TEST_PORT_Type port, PCU_MODE_Type testType, PCU_PUPD_Type pupd);

/*******************************************************************************
* Private Variable
*******************************************************************************/
uint32_t MilliSec;

/*******************************************************************************
* Public Function
*******************************************************************************/

static void gpio_irq_common(TEST_PORT_Type * port) {
    int i;
    uint32_t Status;
    Status = HAL_GPIO_EXTI_GetState(port->test_port);
    cprintf("%s IER: 0x%08X\r\n", port->alias, port->test_port->IER);
    cprintf("%s ICR: 0x%08X\r\n", port->alias, port->test_port->ICR);
    cprintf("%s ISR: 0x%08X\r\n", port->alias, port->test_port->ISR);

    for(i = 0; i < 16 ; i++) {

    #if 1
        // Falling Edge or Low Level
        if ((Status & (1 << (i*2))) == (1 << (i*2))) {
            cprintf("%d Falling or Low Level Interrupt\r\n", i);
            HAL_GPIO_EXTI_ClearPin((PORT_Type *)port->test_port, Status);
        }
    #endif
    #if 1
        // Rising Edge or High Level
        if ((Status & (2 << (i*2))) == (2 << (i*2))) {
            cprintf("%d Rising or High Level Interrupt\r\n", i);
            HAL_GPIO_EXTI_ClearPin((PORT_Type *)port->test_port, Status);
        }
    #endif
    #if 0
        // Both Edge
        if ((Status & (3 << (i*2))) == (3 << (i*2))) {
            cprintf("%d Both edge Interrupt\r\n", i);
            HAL_GPIO_EXTI_ClearPin((PORT_Type *)port->test_port, Status);
        }
    #endif
    }
}

void GPIOAB_IRQHandler_Input(void) {
    gpio_irq_common(&PORT_A);
//    gpio_irq_common(&PORT_B);
}

void GPIOCD_IRQHandler_Input(void) {
    gpio_irq_common(&PORT_C);
//    gpio_irq_common(&PORT_D);
}

void GPIOE_IRQHandler_Input(void) {
    gpio_irq_common(&PORT_E);
}

void GPIOF_IRQHandler_Input(void) {
    gpio_irq_common(&PORT_F);
}

/**********************************************************************
 * @brief		TestCode_GPIO_StregnthTest
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void TestCode_GPIO_InputInterrupt(void) {
    int i;
#if 1

    // PORT A Test
    NVIC_EnableIRQ(GPIOAB_IRQn);
    NVIC_SetPriority(GPIOAB_IRQn, 3);
    __enable_irq();

    gpio_init(PORT_A, PCU_MODE_INPUT, PCU_PUPD_DISABLE);
    HAL_GPIO_EXTI_ClearPin((PORT_Type *)PA, HAL_GPIO_EXTI_GetState((PORT_Type *)PA));

    _DBG("PORT A"); _DBG(" GPIO Interrupt Test\r\n");

    for (i = 0; i < 16; i ++) {
//        HAL_GPIO_EXTI_Config((PORT_Type *)PA, i, PCU_INTERRUPT_MODE_EDGE, PCU_INTERRUPT_CTRL_EDGE_FALLING);
        HAL_GPIO_EXTI_Config((PORT_Type *)PA, i, PCU_INTERRUPT_MODE_EDGE, PCU_INTERRUPT_CTRL_EDGE_RISING);
//        HAL_GPIO_EXTI_Config((PORT_Type *)PA, i, PCU_INTERRUPT_MODE_EDGE, PCU_INTERRUPT_CTRL_EDGE_BOTH);
//        HAL_GPIO_EXTI_Config((PORT_Type *)PA, i, PCU_INTERRUPT_MODE_LEVEL, PCU_INTERRUPT_CTRL_LEVEL_LOW);
//        HAL_GPIO_EXTI_Config((PORT_Type *)PA, i, PCU_INTERRUPT_MODE_LEVEL, PCU_INTERRUPT_CTRL_LEVEL_HIGH);

    }
#endif

#if 0

    // PORT B Test
    NVIC_EnableIRQ(GPIOAB_IRQn);
    NVIC_SetPriority(GPIOAB_IRQn, 3);
    __enable_irq();

    gpio_init(PORT_B, PCU_MODE_INPUT, PCU_PUPD_DISABLE);
    HAL_GPIO_EXTI_ClearPin((PORT_Type *)PB, HAL_GPIO_EXTI_GetState((PORT_Type *)PB));

    _DBG("PORT B"); _DBG(" GPIO Interrupt Test\r\n");

    for (i = 0; i < 16; i ++) {
        if ((i != 3) && (i != 4) && (i != 5)) {
//        HAL_GPIO_EXTI_Config((PORT_Type *)PB, i, PCU_INTERRUPT_MODE_EDGE, PCU_INTERRUPT_CTRL_EDGE_FALLING);
//        HAL_GPIO_EXTI_Config((PORT_Type *)PB, i, PCU_INTERRUPT_MODE_EDGE, PCU_INTERRUPT_CTRL_EDGE_RISING);
//        HAL_GPIO_EXTI_Config((PORT_Type *)PB, i, PCU_INTERRUPT_MODE_EDGE, PCU_INTERRUPT_CTRL_EDGE_BOTH);
        HAL_GPIO_EXTI_Config((PORT_Type *)PB, i, PCU_INTERRUPT_MODE_LEVEL, PCU_INTERRUPT_CTRL_LEVEL_LOW);
//        HAL_GPIO_EXTI_Config((PORT_Type *)PB, i, PCU_INTERRUPT_MODE_LEVEL, PCU_INTERRUPT_CTRL_LEVEL_HIGH);

        }
    }
#endif

 #if 0
    // PORT C Test
    NVIC_EnableIRQ(GPIOCD_IRQn);
    NVIC_SetPriority(GPIOCD_IRQn, 3);
    __enable_irq();

    gpio_init(PORT_C, PCU_MODE_INPUT, PCU_PUPD_DISABLE);
    HAL_GPIO_EXTI_ClearPin((PORT_Type *)PC, HAL_GPIO_EXTI_GetState((PORT_Type *)PC));

    _DBG("PORT C"); _DBG(" GPIO Interrupt Test\r\n");

    for (i = 0; i < 16; i ++) {
//      HAL_GPIO_EXTI_Config((PORT_Type *)PC, i, PCU_INTERRUPT_MODE_EDGE, PCU_INTERRUPT_CTRL_EDGE_FALLING);
//      HAL_GPIO_EXTI_Config((PORT_Type *)PC, i, PCU_INTERRUPT_MODE_EDGE, PCU_INTERRUPT_CTRL_EDGE_RISING);
//      HAL_GPIO_EXTI_Config((PORT_Type *)PC, i, PCU_INTERRUPT_MODE_EDGE, PCU_INTERRUPT_CTRL_EDGE_BOTH);
//      HAL_GPIO_EXTI_Config((PORT_Type *)PC, i, PCU_INTERRUPT_MODE_LEVEL, PCU_INTERRUPT_CTRL_LEVEL_LOW);
//      HAL_GPIO_EXTI_Config((PORT_Type *)PC, i, PCU_INTERRUPT_MODE_LEVEL, PCU_INTERRUPT_CTRL_LEVEL_HIGH);

    }
#endif

#if 0

    // PORT D Test
    NVIC_EnableIRQ(GPIOCD_IRQn);
    NVIC_SetPriority(GPIOCD_IRQn, 3);
    __enable_irq();

    gpio_init(PORT_D, PCU_MODE_INPUT, PCU_PUPD_DISABLE);
    HAL_GPIO_EXTI_ClearPin((PORT_Type *)PD, HAL_GPIO_EXTI_GetState((PORT_Type *)PD));

    _DBG("PORT D"); _DBG(" GPIO Interrupt Test\r\n");

    for (i = 0; i < 16; i ++) {
//      HAL_GPIO_EXTI_Config((PORT_Type *)PD, i, PCU_INTERRUPT_MODE_EDGE, PCU_INTERRUPT_CTRL_EDGE_FALLING);
//      HAL_GPIO_EXTI_Config((PORT_Type *)PD, i, PCU_INTERRUPT_MODE_EDGE, PCU_INTERRUPT_CTRL_EDGE_RISING);
//      HAL_GPIO_EXTI_Config((PORT_Type *)PD, i, PCU_INTERRUPT_MODE_EDGE, PCU_INTERRUPT_CTRL_EDGE_BOTH);
//      HAL_GPIO_EXTI_Config((PORT_Type *)PD, i, PCU_INTERRUPT_MODE_LEVEL, PCU_INTERRUPT_CTRL_LEVEL_LOW);
//      HAL_GPIO_EXTI_Config((PORT_Type *)PD, i, PCU_INTERRUPT_MODE_LEVEL, PCU_INTERRUPT_CTRL_LEVEL_HIGH);
    }
#endif

 #if 0

    // PORT E Test
    NVIC_EnableIRQ(GPIOE_IRQn);
    NVIC_SetPriority(GPIOE_IRQn, 3);
    __enable_irq();

    gpio_init(PORT_E, PCU_MODE_INPUT, PCU_PUPD_DISABLE);
    HAL_GPIO_EXTI_ClearPin((PORT_Type *)PE, HAL_GPIO_EXTI_GetState((PORT_Type *)PE));

    _DBG("PORT E"); _DBG("  GPIO Interrupt Test\r\n");

    for (i = 0; i < 16; i ++) {
//      HAL_GPIO_EXTI_Config((PORT_Type *)PE, i, PCU_INTERRUPT_MODE_EDGE, PCU_INTERRUPT_CTRL_EDGE_FALLING);
//      HAL_GPIO_EXTI_Config((PORT_Type *)PE, i, PCU_INTERRUPT_MODE_EDGE, PCU_INTERRUPT_CTRL_EDGE_RISING);
//      HAL_GPIO_EXTI_Config((PORT_Type *)PE, i, PCU_INTERRUPT_MODE_EDGE, PCU_INTERRUPT_CTRL_EDGE_BOTH);
//      HAL_GPIO_EXTI_Config((PORT_Type *)PE, i, PCU_INTERRUPT_MODE_LEVEL, PCU_INTERRUPT_CTRL_LEVEL_LOW);
//      HAL_GPIO_EXTI_Config((PORT_Type *)PE, i, PCU_INTERRUPT_MODE_LEVEL, PCU_INTERRUPT_CTRL_LEVEL_HIGH);
    }
#endif

 #if 0

    // PORT F Test
    NVIC_EnableIRQ(GPIOF_IRQn);
    NVIC_SetPriority(GPIOF_IRQn, 3);
    __enable_irq();

    gpio_init(PORT_F, PCU_MODE_INPUT, PCU_PUPD_DISABLE);
    HAL_GPIO_EXTI_ClearPin((PORT_Type *)PF, HAL_GPIO_EXTI_GetState((PORT_Type *)PF));

    _DBG("PORT F"); _DBG(" GPIO Interrupt Test\r\n");

    for (i = 0; i < 16; i ++) {
//      HAL_GPIO_EXTI_Config((PORT_Type *)PF, i, PCU_INTERRUPT_MODE_EDGE, PCU_INTERRUPT_CTRL_EDGE_FALLING);
//      HAL_GPIO_EXTI_Config((PORT_Type *)PF, i, PCU_INTERRUPT_MODE_EDGE, PCU_INTERRUPT_CTRL_EDGE_RISING);
//      HAL_GPIO_EXTI_Config((PORT_Type *)PF, i, PCU_INTERRUPT_MODE_EDGE, PCU_INTERRUPT_CTRL_EDGE_BOTH);
//      HAL_GPIO_EXTI_Config((PORT_Type *)PF, i, PCU_INTERRUPT_MODE_LEVEL, PCU_INTERRUPT_CTRL_LEVEL_LOW);
      HAL_GPIO_EXTI_Config((PORT_Type *)PF, i, PCU_INTERRUPT_MODE_LEVEL, PCU_INTERRUPT_CTRL_LEVEL_HIGH);
    }
#endif
    while(1) { }
}


static void gpio_init(TEST_PORT_Type port, PCU_MODE_Type testType, PCU_PUPD_Type pupd) {
    int i;

    // Port peri and clock enable
    if (port.test_port == PA) {
        SCU->PER1 |= (SCU_PER1_GPIOA_Msk);
        SCU->PCER1 |= (SCU_PCER1_GPIOA_Msk);
    } else if (port.test_port == PB) {
        SCU->PER1 |= (SCU_PER1_GPIOB_Msk);
        SCU->PCER1 |= (SCU_PCER1_GPIOB_Msk);
    } else if (port.test_port == PC) {
        SCU->PER1 |= (SCU_PER1_GPIOC_Msk);
        SCU->PCER1 |= (SCU_PCER1_GPIOC_Msk);
    } else if (port.test_port == PD) {
        SCU->PER1 |= (SCU_PER1_GPIOD_Msk);
        SCU->PCER1 |= (SCU_PCER1_GPIOD_Msk);
    } else if (port.test_port == PE) {
        SCU->PER1 |= (SCU_PER1_GPIOE_Msk);
        SCU->PCER1 |= (SCU_PCER1_GPIOE_Msk);
    } else if (port.test_port == (PORT_Type *)PF) {
        SCU->PER1 |= (SCU_PER1_GPIOF_Msk);
        SCU->PCER1 |= (SCU_PCER1_GPIOF_Msk);
    }

    for (i = 0 ; i < port.port_max_nums; i++) {
        // Port Direction

        // Excluded PB3(BOOT), SWDIO(PB4), SWDCLK(PB5) to debug and to test
        if (!(((port.test_port == PB) && (i == 3)) || ((port.test_port == PB) && (i == 4)) || ((port.test_port == PB) && (i == 5)))) {

            if (testType == PCU_MODE_PUSH_PULL) {
            HAL_GPIO_ConfigOutput((PORT_Type *)port.test_port, i, PCU_MODE_PUSH_PULL);
            } else if (testType == PCU_MODE_INPUT) {
                HAL_GPIO_ConfigOutput((PORT_Type *)port.test_port, i, PCU_MODE_INPUT);
            } else if (testType == PCU_MODE_OPEN_DRAIN) {
                HAL_GPIO_ConfigOutput((PORT_Type *)port.test_port, i, PCU_MODE_OPEN_DRAIN);
            }

            // Pinternal port Pull-up/down resistor
            if (pupd == PCU_PUPD_DISABLE) {
                HAL_GPIO_ConfigPullup((PORT_Type *)port.test_port, i, PCU_PUPD_DISABLE);
            } else if (pupd == PCU_PUPD_PULL_UP) {
                HAL_GPIO_ConfigPullup((PORT_Type *)port.test_port, i, PCU_PUPD_PULL_UP);
            } else if (pupd == PCU_PUPD_PULL_DOWN) {
                HAL_GPIO_ConfigPullup((PORT_Type *)port.test_port, i, PCU_PUPD_PULL_DOWN);
            }
        }
    }
}


/* --------------------------------- End Of File ------------------------------ */
