/*****************************************************************************
* @file     TestCode_GPIO_Output.c
* @brief    Contains the external function prototypes for TestCode
* @author   AE Team, ABOV Semiconductor Co., Ltd.
* @version  V0.0.1
* @date     27. MAR. 2020
*
* Copyright(C) 2019, ABOV Semiconductor
* All rights reserved.
*
*
********************************************************************************
* DISCLAIMER
*
* THE PRESENT FIRMWARE WHICH IS FOR GUIDANCE ONLY AIMS AT PROVIDING CUSTOMERS
* WITH CODING INFORMATION REGARDING THEIR PRODUCTS IN ORDER FOR THEM TO SAVE
* TIME. AS A RESULT, ABOV SEMICONDUCTOR DISCLAIMS ALL LIABILITIES FROM ANY
* DIRECT, INDIRECT OR CONSEQUENTIAL DAMAGES WITH RESPECT TO ANY CLAIMS ARISING
* FROM THE CONTENT OF SUCH FIRMWARE AND/OR THE USE MADE BY CUSTOMERS OF THE
* CODING INFORMATION CONTAINED HEREIN IN CONNECTION WITH THEIR PRODUCTS.
*
*******************************************************************************/

/*******************************************************************************
* Pre-processor Definition & Macro
*******************************************************************************/
#include "TestCode_GPIO_Output.h"
#include "main_conf.h"

/*******************************************************************************
* Private Typedef
*******************************************************************************/
typedef struct {
    PORT_Type * test_port;
    uint32_t    port_max_nums;
    char        alias[8];
} TEST_PORT_Type;

TEST_PORT_Type PORT_A = {PA, 16, "GPIO A"};
TEST_PORT_Type PORT_B = {PB, 16, "GPIO B"};
TEST_PORT_Type PORT_C = {PC, 16, "GPIO C"};
TEST_PORT_Type PORT_D = {PD, 16, "GPIO D"};
TEST_PORT_Type PORT_E = {PE, 16, "GPIO E"};
TEST_PORT_Type PORT_F = {(PORT_Type *)PF, 16,  "GPIO F"};

/*******************************************************************************
* Private Function Prototype
*******************************************************************************/
static void gpio_init(TEST_PORT_Type port, PCU_MODE_Type testType, PCU_PUPD_Type pupd);
static void gpio_getOutputStatus(PORT_Type *port, uint32_t port_num);

/*******************************************************************************
* Private Variable
*******************************************************************************/
uint32_t MilliSec;

/*******************************************************************************
* Public Function
*******************************************************************************/
void SysTick_Handler_GpioOutput(void) {
    if (MilliSec)
        MilliSec--;
}

/**********************************************************************
 * @brief		TestCode_GPIO_StregnthTest
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void TestCode_GPIO_Output(void) {
    int i;

    // GPIO_A Output Init
    gpio_init(PORT_A, PCU_MODE_PUSH_PULL, PCU_PUPD_DISABLE);

    cputs("\r\nGPIO Output High Test\r\n");
    for (i = 0 ; i < PORT_A.port_max_nums; i++)  {
        HAL_GPIO_SetPin((PORT_Type *)PORT_A.test_port, _BIT(i));
        // Print Output Status
        gpio_getOutputStatus(PORT_A.test_port, i);
    }

    cputs("\r\nGPIO Output Low Test\r\n");
    for (i = 0 ; i < PORT_A.port_max_nums; i++) {
        // Output Low
        HAL_GPIO_ClearPin((PORT_Type *)PORT_A.test_port, _BIT(i));
        // Print Output Status
        gpio_getOutputStatus(PORT_A.test_port, i);
    }

#if ((DEBUG_PORT_NUMBER ==11) || (DEBUG_PORT_NUMBER == 12) || (DEBUG_PORT_NUMBER == 13))
    // GPIO_B Output Init
    gpio_init(PORT_B, PCU_MODE_PUSH_PULL, PCU_PUPD_DISABLE);

    cputs("\r\nGPIO Output High Test\r\n");
    for (i = 0 ; i < PORT_B.port_max_nums; i++)  {
        HAL_GPIO_SetPin((PORT_Type *)PORT_B.test_port, _BIT(i));
        // Print Output Status
        gpio_getOutputStatus(PORT_B.test_port, i);
    }

    cputs("\r\nGPIO Output Low Test\r\n");
    for (i = 0 ; i < PORT_B.port_max_nums; i++) {
        // Output Low
        HAL_GPIO_ClearPin((PORT_Type *)PORT_B.test_port, _BIT(i));
        // Print Output Status
        gpio_getOutputStatus(PORT_B.test_port, i);
    }
#endif

    // GPIO_C Output Init
    gpio_init(PORT_C, PCU_MODE_PUSH_PULL, PCU_PUPD_DISABLE);

    cputs("\r\nGPIO Output High Test\r\n");
    for (i = 0 ; i < PORT_C.port_max_nums; i++)  {
        HAL_GPIO_SetPin((PORT_Type *)PORT_C.test_port, _BIT(i));
        // Print Output Status
        gpio_getOutputStatus(PORT_C.test_port, i);
    }

    cputs("\r\nGPIO Output Low Test\r\n");
    for (i = 0 ; i < PORT_C.port_max_nums; i++) {
        // Output Low
        HAL_GPIO_ClearPin((PORT_Type *)PORT_C.test_port, _BIT(i));
        // Print Output Status
        gpio_getOutputStatus(PORT_C.test_port, i);
    }

#if ((DEBUG_PORT_NUMBER ==10) || (DEBUG_PORT_NUMBER == 12) || (DEBUG_PORT_NUMBER == 13) || (DEBUG_PORT_NUMBER == 0) || (DEBUG_PORT_NUMBER == 1))
    // GPIO_D Output Init
    gpio_init(PORT_D, PCU_MODE_PUSH_PULL, PCU_PUPD_DISABLE);

    cputs("\r\nGPIO Output High Test\r\n");
    for (i = 0 ; i < PORT_D.port_max_nums; i++)  {
        HAL_GPIO_SetPin((PORT_Type *)PORT_D.test_port, _BIT(i));
        // Print Output Status
        gpio_getOutputStatus(PORT_D.test_port, i);
    }

    cputs("\r\nGPIO Output Low Test\r\n");
    for (i = 0 ; i < PORT_D.port_max_nums; i++) {
        // Output Low
        HAL_GPIO_ClearPin((PORT_Type *)PORT_D.test_port, _BIT(i));
        // Print Output Status
        gpio_getOutputStatus(PORT_D.test_port, i);
    }
#endif

#if ((DEBUG_PORT_NUMBER ==10) || (DEBUG_PORT_NUMBER == 11) || (DEBUG_PORT_NUMBER == 0) || (DEBUG_PORT_NUMBER == 1))
    // GPIO_E Output Init
    gpio_init(PORT_E, PCU_MODE_PUSH_PULL, PCU_PUPD_DISABLE);

    cputs("\r\nGPIO Output High Test\r\n");
    for (i = 0 ; i < PORT_E.port_max_nums; i++) {
        HAL_GPIO_SetPin((PORT_Type *)PORT_E.test_port, _BIT(i));
        // Print Output Status
        gpio_getOutputStatus(PORT_E.test_port, i);
    }

    cputs("\r\nGPIO Output Low Test\r\n");
    for (i = 0 ; i < PORT_E.port_max_nums; i++) {
        // Output Low
        HAL_GPIO_ClearPin((PORT_Type *)PORT_E.test_port, _BIT(i));
        // Print Output Status
        gpio_getOutputStatus(PORT_E.test_port, i);
    }
#endif

    // GPIO_F Output Init
    gpio_init(PORT_F, PCU_MODE_PUSH_PULL, PCU_PUPD_DISABLE);

    cputs("\r\nGPIO Output High Test\r\n");
    for (i = 0 ; i < PORT_F.port_max_nums; i++) {
        HAL_GPIO_SetPin((PORT_Type *)PORT_F.test_port, _BIT(i));
        // Print Output Status
        gpio_getOutputStatus(PORT_F.test_port, i);
    }

    cputs("\r\nGPIO Output Low Test\r\n");
    for (i = 0 ; i < PORT_F.port_max_nums; i++) {
        // Output Low
        HAL_GPIO_ClearPin((PORT_Type *)PORT_F.test_port, _BIT(i));
        // Print Output Status
        gpio_getOutputStatus(PORT_F.test_port, i);
    }

    while(1) {}
}


static void gpio_init(TEST_PORT_Type port, PCU_MODE_Type testType, PCU_PUPD_Type pupd) {
    int i;

    // Port peri and clock enable
    if (port.test_port == PA) {
        SCU->PER1 |= (SCU_PER1_GPIOA_Msk);
        SCU->PCER1 |= (SCU_PCER1_GPIOA_Msk);
    } else if (port.test_port == PB) {
        SCU->PER1 |= (SCU_PER1_GPIOB_Msk);
        SCU->PCER1 |= (SCU_PCER1_GPIOB_Msk);
    } else if (port.test_port == PC) {
        SCU->PER1 |= (SCU_PER1_GPIOC_Msk);
        SCU->PCER1 |= (SCU_PCER1_GPIOC_Msk);
    } else if (port.test_port == PD) {
        SCU->PER1 |= (SCU_PER1_GPIOD_Msk);
        SCU->PCER1 |= (SCU_PCER1_GPIOD_Msk);
    } else if (port.test_port == PE) {
        SCU->PER1 |= (SCU_PER1_GPIOE_Msk);
        SCU->PCER1 |= (SCU_PCER1_GPIOE_Msk);
    } else if (port.test_port == (PORT_Type *)PF) {
        SCU->PER1 |= (SCU_PER1_GPIOF_Msk);
        SCU->PCER1 |= (SCU_PCER1_GPIOF_Msk);
    }

    for (i = 0 ; i < port.port_max_nums; i++) {
        // Port Direction

        // Excluded PB3(BOOT), SWDIO(PB4), SWDCLK(PB5) to debug and to test
        if (!(((port.test_port == PB) && (i == 3)) || ((port.test_port == PB) && (i == 4)) || ((port.test_port == PB) && (i == 5)))) {

            if (testType == PCU_MODE_PUSH_PULL) {
                HAL_GPIO_ConfigOutput((PORT_Type *)port.test_port, i, PCU_MODE_PUSH_PULL);
            } else if (testType == PCU_MODE_INPUT) {
                HAL_GPIO_ConfigOutput((PORT_Type *)port.test_port, i, PCU_MODE_INPUT);
            } else if (testType == PCU_MODE_OPEN_DRAIN) {
                HAL_GPIO_ConfigOutput((PORT_Type *)port.test_port, i, PCU_MODE_OPEN_DRAIN);
            }

            // Pinternal port Pull-up/down resistor
            if (pupd == PCU_PUPD_DISABLE) {
                HAL_GPIO_ConfigPullup((PORT_Type *)port.test_port, i, PCU_PUPD_DISABLE);
            } else if (pupd == PCU_PUPD_PULL_UP) {
                HAL_GPIO_ConfigPullup((PORT_Type *)port.test_port, i, PCU_PUPD_PULL_UP);
            } else if (pupd == PCU_PUPD_PULL_DOWN) {
                HAL_GPIO_ConfigPullup((PORT_Type *)port.test_port, i, PCU_PUPD_PULL_DOWN);
            }
        }
    }
}

static void gpio_getOutputStatus(PORT_Type *port, uint32_t port_num) {
    uint32_t Reg32;

    if (port == PA)                       cputs("PA");
    else if (port == PB)                  cputs("PB");
    else if (port == PC)                  cputs("PC");
    else if (port == PD)                  cputs("PD");
    else if (port == PE)                  cputs("PE");
    else if (port == (PORT_Type*)PF)      cputs("PF");

    cprintf("%d", port_num);

    Reg32 = ((port->OUTDR & (1 << port_num))  >> port_num);
    if (Reg32 == 1) {
        cputs(" High\r\n");
    } else {
        cputs(" Low\r\n");
    }
}

/* --------------------------------- End Of File ------------------------------ */
