/*****************************************************************************
* @file     TestCode_GPIO_STRENGTH.c
* @brief    Contains the external function prototypes for TestCode
* @author   AE Team, ABOV Semiconductor Co., Ltd.
* @version  V0.0.1
* @date     27. MAR. 2020
*
* Copyright(C) 2019, ABOV Semiconductor
* All rights reserved.
*
*
********************************************************************************
* DISCLAIMER
*
* THE PRESENT FIRMWARE WHICH IS FOR GUIDANCE ONLY AIMS AT PROVIDING CUSTOMERS
* WITH CODING INFORMATION REGARDING THEIR PRODUCTS IN ORDER FOR THEM TO SAVE
* TIME. AS A RESULT, ABOV SEMICONDUCTOR DISCLAIMS ALL LIABILITIES FROM ANY
* DIRECT, INDIRECT OR CONSEQUENTIAL DAMAGES WITH RESPECT TO ANY CLAIMS ARISING
* FROM THE CONTENT OF SUCH FIRMWARE AND/OR THE USE MADE BY CUSTOMERS OF THE
* CODING INFORMATION CONTAINED HEREIN IN CONNECTION WITH THEIR PRODUCTS.
*
*******************************************************************************/

/*******************************************************************************
* Pre-processor Definition & Macro
*******************************************************************************/
#include "TestCode_GPIO_Strength.h"
#include "main_conf.h"

#define STRENGTH_CHANNEL_MAX    (6)
/*******************************************************************************
* Private Typedef
*******************************************************************************/
typedef struct {
    PORT_Type * port;
    uint32_t    pin_num;
    char        alias[8];
} TEST_PORT_Type;


TEST_PORT_Type STR[STRENGTH_CHANNEL_MAX] = {
  {PB, 0, "STR0"},
  {PB, 1, "STR1"},
  {PB, 2, "STR2"},
  {PC, 2, "STR3"},
  {PC, 3, "STR4"},
  {PC, 4, "STR5"},
};

/*******************************************************************************
* Private Function Prototype
*******************************************************************************/

/*******************************************************************************
* Private Variable
*******************************************************************************/
uint32_t MilliSec;

/*******************************************************************************
* Public Function
*******************************************************************************/
void SysTick_Handler_GpioIStrTest(void) {
    if (MilliSec)
        MilliSec--;
}

/**********************************************************************
 * @brief		TestCode_GPIO_StregnthTest
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void TestCode_GPIO_Strength(void) {
    int i;

    cputs("\r\n");

	// Stregnth Channel Init
    for (i = 0; i < STRENGTH_CHANNEL_MAX; i++) {
        HAL_GPIO_ConfigOutput(STR[i].port, STR[i].pin_num, PCU_MODE_PUSH_PULL);
        HAL_GPIO_ConfigPullup(STR[i].port, STR[i].pin_num, PCU_PUPD_DISABLE);
        cprintf("STR %d Channel is initialized.\r\n", i);
    }

    SysTick_Config(SystemCoreClock/1000);
    __enable_irq();

    // Stregnth Port ON/OFF Test
    for (i = 0; i < STRENGTH_CHANNEL_MAX; i++) {
        MilliSec = 500;
        while(MilliSec);
        cputs("\r\n");
        cprintf("%s: ENABLE\r\n", STR[i].alias);

        HAL_PCU_SetStrength(STR[i].port, STR[i].pin_num, ENABLE);
        cprintf("0x%08X: 0x%08X\r\n", (uint32_t)&(STR[i].port->STR), STR[i].port->STR);

        MilliSec = 500;
        while(MilliSec);

        cprintf("%s: DISABLE\r\n", STR[i].alias);
        HAL_PCU_SetStrength(STR[i].port, STR[i].pin_num, DISABLE);
        cprintf("0x%08X: 0x%08X\r\n", (uint32_t)&(STR[i].port->STR), STR[i].port->STR);
    }
    cputs("\r\n");

    while(1) {}
}

/* --------------------------------- End Of File ------------------------------ */
