/*****************************************************************************
* @file     TestCode_define.h
* @brief    Contains the external function prototypes for TestCode
* @author   AE Team, ABOV Semiconductor Co., Ltd.
* @version  V0.0.1
* @date     23. MAR. 2020
*
* Copyright(C) 2019, ABOV Semiconductor
* All rights reserved.
*
*
********************************************************************************
* DISCLAIMER
*
* THE PRESENT FIRMWARE WHICH IS FOR GUIDANCE ONLY AIMS AT PROVIDING CUSTOMERS
* WITH CODING INFORMATION REGARDING THEIR PRODUCTS IN ORDER FOR THEM TO SAVE
* TIME. AS A RESULT, ABOV SEMICONDUCTOR DISCLAIMS ALL LIABILITIES FROM ANY
* DIRECT, INDIRECT OR CONSEQUENTIAL DAMAGES WITH RESPECT TO ANY CLAIMS ARISING
* FROM THE CONTENT OF SUCH FIRMWARE AND/OR THE USE MADE BY CUSTOMERS OF THE
* CODING INFORMATION CONTAINED HEREIN IN CONNECTION WITH THEIR PRODUCTS.
*
*******************************************************************************/

/*******************************************************************************
* Pre-processor Definition & Macro
*******************************************************************************/
#include "TestCode_LCD_Polling.h"
#include "main_conf.h"

#define MAX_LCD_BUF_SIZE     (44)
#define LCD_DR_OFFSET_ADDR   (0x40005010UL)     // LCD->DR0
/*******************************************************************************
* Private Typedef
*******************************************************************************/
uint8_t * DATA_BUFFER = (uint8_t *)LCD_DR_OFFSET_ADDR;      // LCD DR BUFFER to display

LCD_PORT_Type LCD_SEG_PORT[LCD_IP_SEG_INDEX_MAX] = {
    {PE,  3,  "SEG0"},
    {PE,  4,  "SEG1"},
    {PE,  5,  "SEG2"},
    {PE,  6,  "SEG3"},
    {PE,  7,  "SEG4"},
    {PE, 14,  "SEG5"},
    {PE, 15,  "SEG6"},
    {PD,  5,  "SEG7"},
    {PD,  4,  "SEG8"},
    {PD,  3,  "SEG9"},
    {PD,  2, "SEG10"},
    {PD,  1, "SEG11"},
    {PD,  0, "SEG12"},
    {PC, 12, "SEG13"},
    {PC, 11, "SEG14"},
    {PC, 10, "SEG15"},
    {PC,  9, "SEG16"},
    {PC,  8, "SEG17"},
    {PC,  7, "SEG18"},
    {PC,  6, "SEG19"},
    {PC,  5, "SEG20"},
    {PC,  4, "SEG21"},
    {PC,  3, "SEG22"},
    {PC,  2, "SEG23"},
    {PC,  1, "SEG24"},
    {PC,  0, "SEG25"},
    {PB, 15, "SEG26"},
    {PB, 14, "SEG27"},
    {PB, 13, "SEG28"},
    {PB, 12, "SEG29"},
    {PB, 11, "SEG30"},
    {PB, 10, "SEG31"},
    {PB,  9, "SEG32"},
    {PB,  8, "SEG33"},
    {PB,  7, "SEG34"},
    {PB,  6, "SEG35"},
    {PB,  5, "SEG36"},
    {PB,  4, "SEG37"},
    {PB,  3, "SEG38"},
    {PB,  2, "SEG39"},
    {PB,  1, "SEG40"},
    {PB,  0, "SEG41"},
    {PA,  7, "SEG42"},
    {PA,  6, "SEG43"}
};

LCD_PORT_Type LCD_COM_PORT[LCD_IP_COM_INDEX_MAX] = {
    {PE,  0, "COM0"},
    {PE,  1, "COM1"},
    {PE,  2, "COM2"},
    {PE,  3, "COM3"},
    {PE,  4, "COM4"},
    {PE,  5, "COM5"},
    {PE,  6, "COM6"},
    {PE,  7, "COM7"},
};

LCD_PORT_Type LCD_VLC_PORT[LCD_IP_VLC_INDEX_MAX] = {
    {PE,   8, "VLC0"},
    {PE,   9, "VLC1"},
    {PE,  10, "VLC2"},
    {PE,  11, "VLC3"},
};

/*******************************************************************************
* Private Variable
*******************************************************************************/

// for Blind PKG
//static const LCD_PORT_Type * SEG7 = &LCD_SEG_PORT[7];
//static const LCD_PORT_Type * SEG8 = &LCD_SEG_PORT[8];
//static const LCD_PORT_Type * SEG9 = &LCD_SEG_PORT[9];
//static const LCD_PORT_Type * SEG10 = &LCD_SEG_PORT[10];
//static const LCD_PORT_Type * SEG11 = &LCD_SEG_PORT[11];
//static const LCD_PORT_Type * SEG12 = &LCD_SEG_PORT[12];
//static const LCD_PORT_Type * SEG13 = &LCD_SEG_PORT[13];
//static const LCD_PORT_Type * SEG14 = &LCD_SEG_PORT[14];
//static const LCD_PORT_Type * SEG19 = &LCD_SEG_PORT[19];


// for Standard PKG
static const LCD_PORT_Type * SEG1 = &LCD_SEG_PORT[1];
static const LCD_PORT_Type * SEG2 = &LCD_SEG_PORT[2];
static const LCD_PORT_Type * SEG3 = &LCD_SEG_PORT[3];
static const LCD_PORT_Type * SEG4 = &LCD_SEG_PORT[4];
static const LCD_PORT_Type * SEG7 = &LCD_SEG_PORT[7];
static const LCD_PORT_Type * SEG8 = &LCD_SEG_PORT[8];
static const LCD_PORT_Type * SEG9 = &LCD_SEG_PORT[9];
static const LCD_PORT_Type * SEG10 = &LCD_SEG_PORT[10];
static const LCD_PORT_Type * SEG11 = &LCD_SEG_PORT[11];

static const LCD_PORT_Type * COM0 = &LCD_COM_PORT[0];
static const LCD_PORT_Type * COM1 = &LCD_COM_PORT[1];
static const LCD_PORT_Type * COM2 = &LCD_COM_PORT[2];
static const LCD_PORT_Type * COM3 = &LCD_COM_PORT[3];

static const LCD_PORT_Type * VLC0 = &LCD_VLC_PORT[0];
static const LCD_PORT_Type * VLC1 = &LCD_VLC_PORT[1];
static const LCD_PORT_Type * VLC2 = &LCD_VLC_PORT[2];
static const LCD_PORT_Type * VLC3 = &LCD_VLC_PORT[3];

uint32_t MilliSec;
/*******************************************************************************
* Private Function Prototype
*******************************************************************************/

void SysTick_Handler_LCD_Polling(void) {
    if (MilliSec) {
        MilliSec--;
    }
}
/*******************************************************************************
* Public Function
*******************************************************************************/
void LCD_Init(void);
void LCD_Disp(uint32_t disp);
void LCD_Test(void);
void LCD_BufferAllClear(void);
void LCD_BufferAllDisp(void);

/*******************************************************************************
* @brief      Main_LcdPollingTest
* @param      None
* @return     None
*******************************************************************************/
void Main_LcdPollingTest(void) {

	char cmd;

    cputs("\n\rLCD test Program\n\r");
    cputs("0. LCD INIT\r\n");
    cputs("1. LCD ON\n\r");
    cputs("2. LCD OFF\n\r");
    cputs("3. LCD Display All Clear\n\r");
    cputs("4. LCD Display All\n\r");
    cputs("\r\n");

    cputs("LCD> ");

	while(1) {
		cmd = cgetchar();

		if ((cmd > 0x0) && (cmd <= 0x7F)) {
            cprintf("%c\r\n", cmd);

			switch(cmd) {
				case '0':
					LCD_Init();
					break;
				case '1':
                    _DBG("LCD Display On\r\n");
					HAL_LCD_Display(ENABLE);
					break;
				case '2':
                    _DBG("LCD Display Off\r\n");
					HAL_LCD_Display(DISABLE);
					break;
				case '3':
                    _DBG("LCD Display All Clear\r\n");
                    LCD_BufferAllClear();
					break;
 				case '4':
                    _DBG("LCD Display All\r\n");
					LCD_BufferAllDisp();
					break;
			}
		}
	}   // Loop

}


void LCD_Init(void) {
    LCD_CFG_Type          lcdConfig;

    __disable_irq();

    LCD_BufferAllClear();

    HAL_LCD_Module(ENABLE);                         // LCD Peri and Clock enable

    // LCD CLock Setting
#if 1
    HAL_SCU_LCD_ClockConfig(LCDCLK_MCCR5); //LCDCLK_MCCR5
    HAL_SCU_MiscClockConfig(5, LCD_TYPE, CLKSRC_LSI, 5);
#endif

#if 0
	HAL_SCU_LSE_ClockConfig(LSE_EN);
	HAL_SCU_LSE_ClockMonitoring();
	HAL_SCU_LCD_ClockConfig(LCDCLK_LSE);
#endif

#if 0
	HAL_SCU_LCD_ClockConfig(LCDCLK_WDTRC);
#endif

    HAL_LCD_Display(DISABLE);

    ////////////////////////////////////////////
    // LCD display part (GDC8310D)
    // Voltage : 3.3V, 1/4 Duty, 1/3 Bias.
    ////////////////////////////////////////////

    // LCD Port Init
    HAL_LCD_PortInit((LCD_PORT_Type *)SEG1);
    HAL_LCD_PortInit((LCD_PORT_Type *)SEG2);
    HAL_LCD_PortInit((LCD_PORT_Type *)SEG3);
    HAL_LCD_PortInit((LCD_PORT_Type *)SEG4);
    HAL_LCD_PortInit((LCD_PORT_Type *)SEG7);
    HAL_LCD_PortInit((LCD_PORT_Type *)SEG8);
    HAL_LCD_PortInit((LCD_PORT_Type *)SEG9);
    HAL_LCD_PortInit((LCD_PORT_Type *)SEG10);
    HAL_LCD_PortInit((LCD_PORT_Type *)SEG11);

    HAL_LCD_PortInit((LCD_PORT_Type *)COM0);
    HAL_LCD_PortInit((LCD_PORT_Type *)COM1);
    HAL_LCD_PortInit((LCD_PORT_Type *)COM2);
    HAL_LCD_PortInit((LCD_PORT_Type *)COM3);

    HAL_LCD_PortInit((LCD_PORT_Type *)VLC0);
    HAL_LCD_PortInit((LCD_PORT_Type *)VLC1);
    HAL_LCD_PortInit((LCD_PORT_Type *)VLC2);
    HAL_LCD_PortInit((LCD_PORT_Type *)VLC3);
    cprintf("\r\nLCD Port Initialized\r\n");

    lcdConfig.InternalBiasResistor    = RLCD2_66_66_50;
    lcdConfig.LcdDutyandBias          = DUTY1_4_BIAS1_3;
    lcdConfig.LcdClock                = LCDCLK_128Hz;
    lcdConfig.LcdAutoBias             = ENABLE;
    lcdConfig.BiasModeClk             = fLCD_4CLK;
    lcdConfig.LcdContrast             = DISABLE;
    lcdConfig.VlcVoltage              = VDD16_31_STEP;
#if 0
    lcdConfig.LcdExtBias              = DISABLE;           // Use at a system operating voltage of 3.3V.
    lcdConfig.LcdExtBiasPath          = DISABLE;
#else
    lcdConfig.LcdExtBias              = ENABLE;            // If a 3.3V LCD is used in a system with a 5.0V voltage, the 3.3V voltage must be set using a VCL external bias circuit.
    lcdConfig.LcdExtBiasPath          = ENABLE;
#endif
    HAL_LCD_Configuration(&lcdConfig);      // update LCD Setting

    if (lcdConfig.LcdExtBias == ENABLE) {
        HAL_LCD_SetVLC(LCD_VLC0, ENABLE);
        HAL_LCD_SetVLC(LCD_VLC1, ENABLE);
        HAL_LCD_SetVLC(LCD_VLC2, ENABLE);
        HAL_LCD_SetVLC(LCD_VLC3, ENABLE);
    } else {
        HAL_LCD_SetVLC(LCD_VLC0, DISABLE);
        HAL_LCD_SetVLC(LCD_VLC1, DISABLE);
        HAL_LCD_SetVLC(LCD_VLC2, DISABLE);
        HAL_LCD_SetVLC(LCD_VLC3, DISABLE);
    }

    cprintf("\r\nInitialized LCD Driver\r\n");

#if 0
    SysTick_Config(SystemCoreClock/1000);   // 1ms Systick Timer
    __enable_irq();
#endif
}

void LCD_BufferAllClear(void) {
    int i;

    for (i = 0; i < MAX_LCD_BUF_SIZE; i++) {
        DATA_BUFFER[i] = 0x00;
    }
}

void LCD_BufferAllDisp(void) {
#if 0
    int i;
    int j;
#endif
    
    SysTick_Config(SystemCoreClock/1000);   // 1ms Systick Timer
    __enable_irq();

    DATA_BUFFER[1] = 0xF;
    DATA_BUFFER[2] = 0xF;
    DATA_BUFFER[3] = 0xF;
    DATA_BUFFER[4] = 0xF;
//    DATA_BUFFER[7] = 0xF;
//    DATA_BUFFER[8] = 0xF;
//    DATA_BUFFER[9] = 0xF;
//    DATA_BUFFER[10] = 0xF;
//    DATA_BUFFER[11] = 0xF;

#if 0
    // Update a data All LCD Data Register
    for (i = 0; i < MAX_LCD_BUF_SIZE; i++) {
        for(j = 0; j < 0x10; j++) {
            DATA_BUFFER[i] = j;
            MilliSec=100;
            while(MilliSec);
        }
    }
#endif
}


/* --------------------------------- End Of File ------------------------------ */
