/*****************************************************************************
* @file     TestCode_SCU_ClkMonitor.c
* @brief    Contains the external function prototypes for TestCode
* @author   AE Team, ABOV Semiconductor Co., Ltd.
* @version  V0.0.1
* @date     4. Sep. 2020
*
* Copyright(C) 2020, ABOV Semiconductor
* All rights reserved.
*
*
********************************************************************************
* DISCLAIMER
*
* THE PRESENT FIRMWARE WHICH IS FOR GUIDANCE ONLY AIMS AT PROVIDING CUSTOMERS
* WITH CODING INFORMATION REGARDING THEIR PRODUCTS IN ORDER FOR THEM TO SAVE
* TIME. AS A RESULT, ABOV SEMICONDUCTOR DISCLAIMS ALL LIABILITIES FROM ANY
* DIRECT, INDIRECT OR CONSEQUENTIAL DAMAGES WITH RESPECT TO ANY CLAIMS ARISING
* FROM THE CONTENT OF SUCH FIRMWARE AND/OR THE USE MADE BY CUSTOMERS OF THE
* CODING INFORMATION CONTAINED HEREIN IN CONNECTION WITH THEIR PRODUCTS.
*
*******************************************************************************/

/*******************************************************************************
* Pre-processor Definition & Macro
*******************************************************************************/
#include "TestCode_SCU_ClkMonitor.h"
#include "main_conf.h"

/*******************************************************************************
* Private Typedef
*******************************************************************************/

/*******************************************************************************
* Private Variable
*******************************************************************************/

/*******************************************************************************
* Private Function Prototype
*******************************************************************************/

/*******************************************************************************
* Public Function
*******************************************************************************/

/*******************************************************************************
* Private Function
*******************************************************************************/

/*******************************************************************************
* Pre-processor Definition & Macro
*******************************************************************************/
/*******************************************************************************
* @brief      SYSCLKFAIL interrupt handler
* @param      None
* @return     None
*******************************************************************************/
void SYSCLKFAIL_IRQHandler_ScuTest(void) {
	uint32_t Reg32;

	Reg32 = SCU->CMR;

    cputs("SYSCLKFAIL - Interrupt Occurred!\r\n");


	// HSE FAIL occured, Reset HSE Oscillation
	if((Reg32 & SCU_CMR_HSEFAIL_Msk) >> SCU_CMR_HSEFAIL_Pos)
	{
		cputs("HSE Oscillation is FAIL!\r\n");

		HAL_SCU_HSE_ClockConfig(HSE_EN); 						// Run HSE Oscillation
		HAL_SCU_HSE_ClockMonitoring();					// Stabilization of HSE

		SCU->CMR &= ~SCU_CMR_HSEFAIL_Msk;	// Clear HSEFAIL Flag
	}

	// LSE FAIL occured, Reset LSE Oscillation
	if((Reg32 & SCU_CMR_LSEFAIL_Msk) >> SCU_CMR_LSEFAIL_Pos)
	{
        cputs("LSE Oscillation is FAIL!\r\n");

		HAL_SCU_LSE_ClockConfig(LSE_EN); 						// Run LSE Oscillation
		HAL_SCU_LSE_ClockMonitoring();					// Stabilization of LSE

		SCU->CMR &= ~SCU_CMR_HSEFAIL_Msk;	        // Clear LSEFAIL Flag
	}

	// MCLK FAIL occured, Reset MCLK Oscillation
	if((Reg32 & SCU_CMR_MCLKFAIL_Msk) >> SCU_CMR_MCLKFAIL_Pos)
	{
		HAL_SCU_HSE_ClockConfig(HSE_EN); 					// Run HSE Oscillation
		HAL_SCU_HSE_ClockMonitoring();				// Stabilization of HSE
		HAL_SCU_SystemClockChange(SCCR_HSE);				// Select HSE clock as MCLK (abnormal operation for test)

        HAL_SCU_SetPLLFreq(IRC32MHz, PLL32MHz);
        debug_frmwrk_init();

        cputs("MCLK FAIL is Occurred\r\n");

		SCU->CMR &= ~SCU_CMR_MCLKFAIL_Msk;	// Clear MCLKFAIL Flag
	}
}


/*******************************************************************************
* @brief      Clock Monitoring Test
* @param      None
* @return     None
*******************************************************************************/
void TestCode_SCU_ClkMonitor(void)
{
	uint32_t	Reg32;
	int i;
	volatile Status t_Status;

	SCU->CMR = 0;		// Clear Flags

	// HSE Monitoring Test (Abnormal)
#if 0
	HAL_SCU_HSE_ClockConfig(HSE_EN); 									// Enable HSE
	HAL_SCU_HSE_ClockMonitoring();								// Monitoring HSE Clock

	SCU->CMR |= SCU_CMR_HSEMNT_Msk| SCU_CMR_HSEIE_Msk;		// ENABLE HSE Monitoring and HSE Interrupt
	_DBG("HSE Oscillation Interrupt Enabled\r\n");

	// Interrupt Setting
	#if 1
	NVIC_SetPriority(SYSCLKFAIL_IRQn, 3);		// Set SYSCLKFAIL Interrupt
	NVIC_EnableIRQ(SYSCLKFAIL_IRQn);			// Enable SYSCLKFAIL Interrupt
	__enable_irq();
	#endif

	for (i = 0; i < 0x100000; i++)				// Delay to check flag
	{
		__NOP();
	}
	HAL_SCU_HSE_ClockConfig(HSE_STOP);					    // HSE has been forced to stop for checking HSE Monitoring
	for (i = 0; i < 0x100000; i++)				// Delay to check flag
	{
		__NOP();
	}
	while(1)
	{
		Reg32 = SCU->CMR;
		if(Reg32 & SCU_CMR_HSESTS_Msk)
		{
			_DBG("HSE Oscillation is OK\r\n");
			break;
		}
	}
#endif

	// LSE Monitoring Test (Abnormal)
#if 0
	HAL_SCU_LSE_ClockConfig(LSE_EN); 					// Stop HSE Oscillation
    HAL_SCU_LSE_ClockMonitoring();                // Monitoring LSE clock

    SCU->CMR |= SCU_CMR_LSEMNT_Msk| SCU_CMR_LSEIE_Msk;		// ENABLE LSE Monitoring and LSE Interrupt
	_DBG("LSE Oscillation Interrupt Enabled\r\n");

    // Interrupt Setting
	#if 1
	NVIC_SetPriority(SYSCLKFAIL_IRQn, 3);		// Set SYSCLKFAIL Interrupt
	NVIC_EnableIRQ(SYSCLKFAIL_IRQn);			// Enable SYSCLKFAIL Interrupt
	__enable_irq();
	#endif

	for (i = 0; i < 0x100000; i++)				// Delay to check flag
	{
		__NOP();
	}
//	HAL_SCU_LSE_ClockConfig(LSE_STOP);						// LSE has been forced to stop for checking LSE Monitoring
	for (i = 0; i < 0x100000; i++)				// Forces the delay to increase to check for the LSEFAIL flag.
	{
		__NOP();
	}
	while(1)
	{
		Reg32 = SCU->CMR;
		if(Reg32 & SCU_CMR_LSESTS_Msk)
		{
			_DBG("LSE Oscillation is OK\r\n");
			break;
		}
	}
#endif

	// MCLK Monitoring Test (Abnormal)
#if 1
	HAL_SCU_LSI_ClockConfig(LSI_EN);		// LSI (500kHz) clock is always on system.

    SCU->CMR = 0;                                                                   // Init SCU-CMR register

#if 0
    // ENABLE MCLK Monitoring, MCLK Interrupt and MCLK Auto Recovery
	SCU->CMR |= SCU_CMR_MCLKMNT_Msk| SCU_CMR_MCLKIE_Msk;
#else
    // ENABLE MCLK Monitoring and MCLK Interrupt, and DISABLE MCLK Auto Recovery
	SCU->CMR |= SCU_CMR_MCLKMNT_Msk| SCU_CMR_MCLKIE_Msk | SCU_CMR_MCLKREC_Msk;
#endif

    // Check MCLK Auto Recovery function
    _DBG("ENABLED MCLK Oscillation Monitoring\r\n");
    if (SCU->CMR & SCU_CMR_MCLKREC_Msk) {
        _DBG("DISABLED MCLK Auto recovery function.\r\n");
    } else {
        _DBG("ENABLED MCLK Auto recovery function.\r\n");
    }

    HAL_SCU_HSE_ClockConfig(HSE_EN);
	HAL_SCU_HSE_ClockMonitoring();								// Monitoring HSE Clock
	HAL_SCU_SystemClockChange(SCCR_HSE);								// Select HSE clock as MCLK (abnormal operation for test)


    #if 1
    // MCLKFAIL NMI Interrupt
	HAL_SCU_SetNMI(SCU_NMI_MCLKFAIL_ENABLE); 					// Disable MCLKFAIL NMI Interrupt
	NVIC_SetPriority(NonMaskableInt_IRQn, 3);		        // Set NMI Interrupt
	NVIC_EnableIRQ(NonMaskableInt_IRQn);			        // Enable NMI Interrupt
    #endif

	// Interrupt Setting
	#if 1
	NVIC_SetPriority(SYSCLKFAIL_IRQn, 3);		// Set SYSCLKFAIL Interrupt
	NVIC_EnableIRQ(SYSCLKFAIL_IRQn);			// Enable SYSCLKFAIL Interrupt
	__enable_irq();
	#endif


    // MCLK FAIL Test #1 : Disconnect logically the HSE clock source in SCU_CSCR register.
	for (i = 0; i < 0x100000; i++)				            // Forces the delay to increase to check for the MCLKFAIL flag.
	{
		__NOP();
	}
	HAL_SCU_HSE_ClockConfig(HSE_STOP);									// HSE has been forced to stop for checking MCLK Monitoring

    // MCLK FAIL Test #2 : Disconnect physically X-TAL from test board.
#endif

	// If MCLK FAIL is occured, SYSCLKFAIL Interrupt will be happend
	while(1) {}

}

/* --------------------------------- End Of File ------------------------------ */
