/*****************************************************************************
* @file     TestCode_SCU_lvdTest.c
* @brief    Contains the external function prototypes for TestCode
* @author   AE Team, ABOV Semiconductor Co., Ltd.
* @version  V0.0.1
* @date     4. Sep. 2020
*
* Copyright(C) 2020, ABOV Semiconductor
* All rights reserved.
*
*
********************************************************************************
* DISCLAIMER
*
* THE PRESENT FIRMWARE WHICH IS FOR GUIDANCE ONLY AIMS AT PROVIDING CUSTOMERS
* WITH CODING INFORMATION REGARDING THEIR PRODUCTS IN ORDER FOR THEM TO SAVE
* TIME. AS A RESULT, ABOV SEMICONDUCTOR DISCLAIMS ALL LIABILITIES FROM ANY
* DIRECT, INDIRECT OR CONSEQUENTIAL DAMAGES WITH RESPECT TO ANY CLAIMS ARISING
* FROM THE CONTENT OF SUCH FIRMWARE AND/OR THE USE MADE BY CUSTOMERS OF THE
* CODING INFORMATION CONTAINED HEREIN IN CONNECTION WITH THEIR PRODUCTS.
*
*******************************************************************************/

/*******************************************************************************
* Pre-processor Definition & Macro
*******************************************************************************/
#include "TestCode_SCU_LvdTest.h"
#include "main_conf.h"

/*******************************************************************************
* Private Typedef
*******************************************************************************/

/*******************************************************************************
* Private Variable
*******************************************************************************/

/*******************************************************************************
* Private Function Prototype
*******************************************************************************/

/*******************************************************************************
* Public Function
*******************************************************************************/

/*******************************************************************************
* Private Function
*******************************************************************************/

/*******************************************************************************
* Pre-processor Definition & Macro
*******************************************************************************/
const char menu[] =
"\r\n------------------------------------\r\n"
"\r\nSelect LVD Level\r\n"
"0 : 4.50\r\n"
"1 : 4.22\r\n"
"2 : 4.04\r\n"
"3 : 3.74\r\n"
"4 : 3.60\r\n"
"5 : 3.19\r\n"
"6 : 3.05\r\n"
"7 : 2.68\r\n"
"8 : 2.48\r\n"
"9 : 2.31\r\n"
"a : 2.13\r\n"
"b : 2.00\r\n"
"c : 1.91\r\n"
"d : 1.79\r\n"
"e : 1.70\r\n"
"f : 1.56\r\n"
"------------------------------------\r\n";


void TestCode_LVD_Test(void) {
    char ch;

    // PE0 GPIO
    HAL_GPIO_ConfigOutput(PC, 0, PCU_MODE_PUSH_PULL);
    HAL_GPIO_ConfigPullup(PC, 0, PCU_PUPD_DISABLE);
    HAL_GPIO_SetPin(PC, _BIT(0));                             // PC0 set 'H" during system on

    HAL_SCU_ClockOutput(4, ENABLE);                                  // Check System Clock (PF4)

    LvdResetStatus();
    LvdLevelStatus();

    _DBG(menu);
    cputs(">");

    while(1) {
        ch = cgetchar();
        if ((ch > 0) && (ch <= 0x7F)) {
            if (ch == '0') {
                cputs("0. 4.50\r\n");
                LVD_Test(LVR_VOLTAGE_450);
            } else if (ch == '1') {
                cputs("1. 4.22\r\n");
                LVD_Test(LVR_VOLTAGE_422);
            } else if (ch == '2') {
                cputs("2. 4.04\r\n");
                LVD_Test(LVR_VOLTAGE_404);
            } else if (ch == '3') {
                cputs("3. 3.74\r\n");
                LVD_Test(LVR_VOLTAGE_374);
            } else if (ch == '4') {
                cputs("4. 3.60\r\n");
                LVD_Test(LVR_VOLTAGE_360);
            } else if (ch == '5') {
                cputs("5. 3.19\r\n");
                LVD_Test(LVR_VOLTAGE_319);
            } else if (ch == '6') {
                cputs("6. 3.05\r\n");
                LVD_Test(LVR_VOLTAGE_305);
            } else if (ch == '7') {
                cputs("7. 2.68\r\n");
                LVD_Test(LVR_VOLTAGE_268);
            } else if (ch == '8') {
                cputs("8. 2.48\r\n");
                LVD_Test(LVR_VOLTAGE_248);
            } else if (ch == '9') {
                cputs("9. 2.31\r\n");
                LVD_Test(LVR_VOLTAGE_231);
            } else if (ch == 'a') {
                cputs("a. 2.13\r\n");
                LVD_Test(LVR_VOLTAGE_213);
            } else if (ch == 'b') {
                cputs("b. 2.00\r\n");
                LVD_Test(LVR_VOLTAGE_200);
            } else if (ch == 'c') {
                cputs("c. 1.91\r\n");
                LVD_Test(LVR_VOLTAGE_191);
            } else if (ch == 'd') {
                cputs("d. 1.79\r\n");
                LVD_Test(LVR_VOLTAGE_179);
            } else if (ch == 'e') {
                cputs("e. 1.70\r\n");
                LVD_Test(LVR_VOLTAGE_170);
            } else if (ch == 'f') {
                cputs("f. 1.56\r\n");
                LVD_Test(LVR_VOLTAGE_156);
            } else {
                cputs("Wrong Number! \r\n");
            }
        }
    }

}

static void LVD_Test(LVR_VOLTAGE_Type lvdLevel) {

    LVR_CFG_Type LvrCfg;

    HAL_SCU_SetResetSrc(RST_LVDRST, DISABLE);       // LVD Reset Disable (Master Setting)

    LvrCfg.EnableOperation = LVR_CONFIG_MASTER;     //  Controlled by SCU->RSER register
    LvrCfg.EnableReset = LVR_ENABLE;                // LVR Enable

    LvrCfg.LVR_Voltage = lvdLevel;                  // Select LVD Level

    HAL_LVR_Init(&LvrCfg);
    HAL_SCU_ClearResetStatus(RST_LVDRST);  // LVD Reset Flag Clear 

    while(1) { }

}

void LvdLevelStatus(void) {
    uint8_t Reg8;

    Reg8 =  SCULV->LVRCNFIG;

    cputs("\r\n");
    switch(Reg8) {
        case 0:
            cputs("Current LVD Level: 4.50\r\n");
        break;
        case 1:
            cputs("Current LVD Level: 4.22\r\n");
        break;
        case 2:
            cputs("Current LVD Level: 4.04\r\n");
        break;
        case 3:
            cputs("Current LVD Level: 3.74\r\n");
        break;
        case 4:
            cputs("Current LVD Level: 3.60\r\n");
        break;
        case 5:
            cputs("Current LVD Level: 3.19\r\n");
        break;
        case 6:
            cputs("Current LVD Level: 3.05\r\n");
        break;
        case 7:
            cputs("Current LVD Level: 2.68\r\n");
        break;
        case 8:
            cputs("Current LVD Level: 2.48\r\n");
        break;
        case 9:
            cputs("Current LVD Level: 2.31\r\n");
        break;
        case 10:
            cputs("Current LVD Level: 2.13\r\n");
        break;
        case 11:
            cputs("Current LVD Level: 2.00\r\n");
        break;
        case 12:
            cputs("Current LVD Level: 1.91\r\n");
        break;
        case 13:
            cputs("Current LVD Level: 1.79\r\n");
        break;
        case 14:
            cputs("Current LVD Level: 1.70\r\n");
        break;
        case 15:
            cputs("Current LVD Level: 1.56\r\n");
        break;

    }
}

static void LvdResetStatus(void) {
    uint8_t Reg8;

    Reg8 = ((SCU->RSSR & SCU_RSSR_PORST_Msk)>>SCU_RSSR_PORST_Pos);
    if (Reg8) {
        cputs("\r\n! System Reset by >POR< !\r\n");
    }

    Reg8 = ((SCU->RSSR & SCU_RSSR_LVDRST_Msk)>>SCU_RSSR_LVDRST_Pos);
    if (Reg8) {
        cputs("\r\n! System Reset by >LVD< !\r\n");
    }
}


/* --------------------------------- End Of File ------------------------------ */
