/*****************************************************************************
* @file     TestCode_SCU_PllTimerClk.h
* @brief    Contains the external function prototypes for TestCode
* @author   AE Team, ABOV Semiconductor Co., Ltd.
* @version  V0.0.1
* @date     17. Sep. 2020
*
* Copyright(C) 2020, ABOV Semiconductor
* All rights reserved.
*
*
********************************************************************************
* DISCLAIMER
*
* THE PRESENT FIRMWARE WHICH IS FOR GUIDANCE ONLY AIMS AT PROVIDING CUSTOMERS
* WITH CODING INFORMATION REGARDING THEIR PRODUCTS IN ORDER FOR THEM TO SAVE
* TIME. AS A RESULT, ABOV SEMICONDUCTOR DISCLAIMS ALL LIABILITIES FROM ANY
* DIRECT, INDIRECT OR CONSEQUENTIAL DAMAGES WITH RESPECT TO ANY CLAIMS ARISING
* FROM THE CONTENT OF SUCH FIRMWARE AND/OR THE USE MADE BY CUSTOMERS OF THE
* CODING INFORMATION CONTAINED HEREIN IN CONNECTION WITH THEIR PRODUCTS.
*
*******************************************************************************/

/*******************************************************************************
* Pre-processor Definition & Macro
*******************************************************************************/
#include "TestCode_SCU_LviTest.h"
#include "main_conf.h"

/*******************************************************************************
* Private Typedef
*******************************************************************************/

/*******************************************************************************
* Private Variable
*******************************************************************************/
uint32_t MilliSec;
/*******************************************************************************
* Private Function Prototype
*******************************************************************************/
void GetLviStatus(void);
void LVI_Test(uint8_t lvi_level);
void TestCode_SCU_TestLvi(void);

/*******************************************************************************
* Public Function
*******************************************************************************/
/*******************************************************************************
* @brief      Systick interrupt handler
* @param      None
* @return     None
*******************************************************************************/
void Systick_Handler_LviTest(void) {
    if(MilliSec) {
        MilliSec--;
    }
}

/*******************************************************************************
* @brief      LVI interrupt handler
* @param      None
* @return     None
*******************************************************************************/

void LVI_IRQHandler_LviTest(void)
{
    cprintf("\tOccured LVI Interrupt! - SCULV->LVICR: 0x%08X\r\n", SCULV->LVICR);
}


const char menu[] =
"\r\n------------------------------------\r\n"
"\r\nSelect LVD Level\r\n"
"f : 4.50\r\n"
"e : 4.22\r\n"
"d : 4.04\r\n"
"c : 3.74\r\n"
"b : 3.60\r\n"
"a : 3.19\r\n"
"9 : 3.05\r\n"
"8 : 2.68\r\n"
"7 : 2.48\r\n"
"6 : 2.31\r\n"
"5 : 2.13\r\n"
"4 : 2.00\r\n"
"3 : 1.91\r\n"
"2 : 1.79\r\n"
"1 : 1.70\r\n"
"0 : 1.56\r\n"
"------------------------------------\r\n";


void TestCode_SCU_LviTest(void) {
    char ch;

    HAL_SCU_SetResetSrc(RST_LVDRST, DISABLE);

    GetLviStatus();

    _DBG(menu);
    cputs(">");

    while(1) {
        ch = cgetchar();
        if ((ch > 0) && (ch <= 0x7F)) {
            if (ch == 'f') {
                cputs("f. 4.50\r\n");
                LVI_Test(LVI_VOLTAGE_450);
            } else if (ch == 'e') {
                cputs("e. 4.22\r\n");
                LVI_Test(LVI_VOLTAGE_422);
            } else if (ch == 'd') {
                cputs("d. 4.04\r\n");
                LVI_Test(LVI_VOLTAGE_404);
            } else if (ch == 'c') {
                cputs("c. 3.74\r\n");
                LVI_Test(LVI_VOLTAGE_374);
            } else if (ch == 'b') {
                cputs("b. 3.60\r\n");
                LVI_Test(LVI_VOLTAGE_360);
            } else if (ch == 'a') {
                cputs("a. 3.19\r\n");
                LVI_Test(LVI_VOLTAGE_319);
            } else if (ch == '9') {
                cputs("9. 3.05\r\n");
                LVI_Test(LVI_VOLTAGE_305);
            } else if (ch == '8') {
                cputs("8. 2.68\r\n");
                LVI_Test(LVI_VOLTAGE_268);
            } else if (ch == '7') {
                cputs("7. 2.48\r\n");
                LVI_Test(LVI_VOLTAGE_248);
            } else if (ch == '6') {
                cputs("6. 2.31\r\n");
                LVI_Test(LVI_VOLTAGE_231);
            } else if (ch == '5') {
                cputs("5. 2.13\r\n");
                LVI_Test(LVI_VOLTAGE_213);
            } else if (ch == '4') {
                cputs("4. 2.00\r\n");
                LVI_Test(LVI_VOLTAGE_200);
            } else if (ch == '3') {
                cputs("3. 1.91\r\n");
                LVI_Test(LVI_VOLTAGE_191);
            } else if (ch == '2') {
                cputs("2. 1.79\r\n");
                LVI_Test(LVI_VOLTAGE_179);
            } else if (ch == '1') {
                cputs("1. 1.70\r\n");
                LVI_Test(LVI_VOLTAGE_170);
            } else if (ch == '0') {
                cputs("0. 1.56\r\n");
                LVI_Test(LVI_VOLTAGE_156);
            } else {
                cputs("Wrong Number! \r\n");
            }
        }
    }

}

/*******************************************************************************
* @brief      GetLviStatus
* @param      None
* @return     None
*******************************************************************************/
static void GetLviStatus(void)
{
	uint32_t Reg32;
	uint16_t Reg8;

	Reg32 = SCULV->LVICR;
	cprintf("SCULV->LVICR = 0x%08X\r\n", Reg32);

	if(Reg32 & (1 << SCULV_LVICR_LVIEN_Pos))
		cputs("Enable LVI\r\n");
	else
		cputs("Disable LVI\r\n");

	if(Reg32 & (1 << SCULV_LVICR_LVINTEN_Pos))
		cputs("Enable LVI Interrupt\r\n");
	else
		cputs("Disable LVI Interrupt\r\n");

	cputs("LVI Level: ");
	Reg8 = ( (Reg32 & SCULV_LVICR_LVIVS_Msk) >> SCULV_LVICR_LVIVS_Pos ) & 0xFF;
	switch(Reg8)
	{
		case LVI_VOLTAGE_156:		cputs("1.56V\r\n");		break;
		case LVI_VOLTAGE_170:		cputs("1.70V\r\n");		break;
		case LVI_VOLTAGE_179:		cputs("1.79V\r\n");		break;
		case LVI_VOLTAGE_191:		cputs("1.91V\r\n");		break;
		case LVI_VOLTAGE_200:		cputs("2.00V\r\n");		break;
		case LVI_VOLTAGE_213:		cputs("2.13V\r\n");		break;
		case LVI_VOLTAGE_231:		cputs("2.31V\r\n");		break;
		case LVI_VOLTAGE_248:		cputs("2.48V\r\n");		break;
		case LVI_VOLTAGE_268:		cputs("2.68V\r\n");		break;
		case LVI_VOLTAGE_305:		cputs("3.05V\r\n");		break;
		case LVI_VOLTAGE_319:		cputs("3.19V\r\n");		break;
		case LVI_VOLTAGE_360:		cputs("3.60V\r\n");		break;
		case LVI_VOLTAGE_374:		cputs("3.74V\r\n");		break;
		case LVI_VOLTAGE_404:		cputs("4.04V\r\n");		break;
		case LVI_VOLTAGE_422:		cputs("4.22V\r\n");		break;
		case LVI_VOLTAGE_450:		cputs("4.50V\r\n");		break;
		default: 											break;
	}
}

/*******************************************************************************
* @brief      Main_TestLvi
* @param      None
* @return     None
*******************************************************************************/

void LVI_Test(uint8_t lvi_level) {
	LVI_CFG_Type LviCfg;

    SCU->RSER &= ~(SCU_RSER_LVDRST_Msk);    // Disable LVD

    // Enable LVI
	LviCfg.EnableIndicator = LVI_ENABLE;
    LviCfg.EnableInterrupt = LVI_INTR_ENABLE;
    LviCfg.LVI_Voltage     = lvi_level;

    NVIC_ClearPendingIRQ(LVI_IRQn);
    NVIC_SetPriority(LVI_IRQn, 3);			// Set LVI Interrupt
    NVIC_EnableIRQ(LVI_IRQn);				// Enable LVI Interrupt
    __enable_irq();

	HAL_LVI_Init(&LviCfg);				        // Initialized LVI

    GetLviStatus();					    // Print LVI Status
    cprintf("LVI_CR(0x4000_5100): 0x%08X\r\n\r\n", (*(volatile uint32_t*)0x40005100));

    SysTick_Config(SystemCoreClock/1000);   // 1ms Systick

    while(1) {
        MilliSec = 500;
        while(MilliSec);
        cputs("*");
    }

}


/* --------------------------------- End Of File ------------------------------ */
