/*****************************************************************************
* @file     TestCode_SCU_PllTimerClk.h
* @brief    Contains the external function prototypes for TestCode
* @author   AE Team, ABOV Semiconductor Co., Ltd.
* @version  V0.0.1
* @date     17. Sep. 2020
*
* Copyright(C) 2020, ABOV Semiconductor
* All rights reserved.
*
*
********************************************************************************
* DISCLAIMER
*
* THE PRESENT FIRMWARE WHICH IS FOR GUIDANCE ONLY AIMS AT PROVIDING CUSTOMERS
* WITH CODING INFORMATION REGARDING THEIR PRODUCTS IN ORDER FOR THEM TO SAVE
* TIME. AS A RESULT, ABOV SEMICONDUCTOR DISCLAIMS ALL LIABILITIES FROM ANY
* DIRECT, INDIRECT OR CONSEQUENTIAL DAMAGES WITH RESPECT TO ANY CLAIMS ARISING
* FROM THE CONTENT OF SUCH FIRMWARE AND/OR THE USE MADE BY CUSTOMERS OF THE
* CODING INFORMATION CONTAINED HEREIN IN CONNECTION WITH THEIR PRODUCTS.
*
*******************************************************************************/

/*******************************************************************************
* Pre-processor Definition & Macro
*******************************************************************************/
#include "TestCode_SCU_PllTimerClk.h"
#include "main_conf.h"

/*******************************************************************************
* Private Typedef
*******************************************************************************/

/*******************************************************************************
* Private Variable
*******************************************************************************/
uint32_t MilliSec;

/*******************************************************************************
* Private Function Prototype
*******************************************************************************/

/*******************************************************************************
* Public Function
*******************************************************************************/
/*******************************************************************************
* @brief      SysTick interrupt handler @ 1000Hz(1ms)
* @param      None
* @return     None
*******************************************************************************/
void SysTick_Handler_PllTimerClk(void) {
	if (MilliSec) {
		MilliSec--;
	}
}

void TestCode_SCU_PllTimerCLK(void) {
	Bool Flag;
    TIMER1n_CFG_Type TIMER1n_Config;

	// Test port(PE2) setting
	HAL_GPIO_ConfigOutput(PE, 2, PCU_MODE_PUSH_PULL);
	HAL_GPIO_ConfigPullup(PE, 2, PCU_PUPD_DISABLE);
	HAL_GPIO_ClearPin(PE, _BIT(2));

    // Timer1n output port setting (PE6, T10O)
    HAL_GPIO_ConfigOutput(PE, 6, PCU_MODE_ALT_FUNC);
    HAL_GPIO_ConfigFunction(PE, 6, PCU_ALT_FUNCTION_1);

#if 1
    // Use PLL with HSI Clock
    HAL_SCU_SetSystemClock(SYSCLK_HSI, SYSDIV_4);       // HSI/4 = 32MHz/4 = 8MHz
    debug_frmwrk_init();                                // Init Debug interface

#else
    // Use PLL with HSE Clock
    HAL_SCU_SetSystemClock(SYSCLK_HSE, SYSDIV_NONE);    // HSE/1 = 8MHz
    debug_frmwrk_init();                                // Init Debug interface

#endif
    // Configuration PLL VCO (Refer SCU_PLLCON registers)
    HAL_SCU_PLL_ClockConfig(ENABLE, PLLCON_BYPASS_PLL, 0,  3, 71, 0, 0); // PLL VCO = 144MHz

    // Timer1n and clock source setting (PLL@ MCCR1)
    HAL_SCU_MiscClockConfig(1, TIMER1n_TYPE, CLKSRC_PLL, 144);  //  PLL144MHz/144 = 1MHz
	HAL_SCU_Timer1n_ClockConfig(T1NCLK_MCCR1);

    TIMER1n_Config.ExtClock = DISABLE;
	TIMER1n_Config.PrescalerData = 1 - 1; // 1MHz/1 = 1MHz -> 1us
	TIMER1n_Config.StartSync = DISABLE;
	TIMER1n_Config.ClearSync = DISABLE;
	TIMER1n_Config.OutputPolarity = TIMER1n_OUTPUT_POLARITY_HIGH;
	TIMER1n_Config.MatchInterrupt = DISABLE;
	TIMER1n_Config.CaptureInterrupt = DISABLE;

    TIMER1n_Config.AData = (1000 - 1);  // 1MHz/1000 = 1kHz

    HAL_TIMER1n_Init(TIMER10, TIMER1n_PERIODIC_MODE, &TIMER1n_Config);

    HAL_TIMER1n_Start(TIMER10);
    cprintf("Test Start\r\n");

    while (1) {
        while ((HAL_TIMER1n_GetStatus(TIMER10) & TIMER1n_STATUS_MATCH_INT) == 0);
        HAL_TIMER1n_ClearStatus(TIMER10, TIMER1n_STATUS_MATCH_INT);

        if (Flag == TRUE) {
            Flag = FALSE;
            cprintf("Off\r\n");
            HAL_GPIO_ClearPin(PE, _BIT(2));
        } else {
            Flag = TRUE;
            cprintf("On\r\n");
            HAL_GPIO_SetPin(PE, _BIT(2));
        }
    }
}

/* --------------------------------- End Of File ------------------------------ */
