/*****************************************************************************
* @file     TestCode_SCU_SysClk.c
* @brief    Contains the external function prototypes for TestCode
* @author   AE Team, ABOV Semiconductor Co., Ltd.
* @version  V0.0.1
* @date     13. FEB. 2020
*
* Copyright(C) 2019, ABOV Semiconductor
* All rights reserved.
*
*
********************************************************************************
* DISCLAIMER
*
* THE PRESENT FIRMWARE WHICH IS FOR GUIDANCE ONLY AIMS AT PROVIDING CUSTOMERS
* WITH CODING INFORMATION REGARDING THEIR PRODUCTS IN ORDER FOR THEM TO SAVE
* TIME. AS A RESULT, ABOV SEMICONDUCTOR DISCLAIMS ALL LIABILITIES FROM ANY
* DIRECT, INDIRECT OR CONSEQUENTIAL DAMAGES WITH RESPECT TO ANY CLAIMS ARISING
* FROM THE CONTENT OF SUCH FIRMWARE AND/OR THE USE MADE BY CUSTOMERS OF THE
* CODING INFORMATION CONTAINED HEREIN IN CONNECTION WITH THEIR PRODUCTS.
*
*******************************************************************************/

/*******************************************************************************
* Pre-processor Definition & Macro
*******************************************************************************/
#include "TestCode_SCU_SysClk.h"
#include "main_conf.h"
#define TEST_MAX_FREQ   (200)
/*******************************************************************************
* Private Typedef
*******************************************************************************/
Bool result[TEST_MAX_FREQ];
/*******************************************************************************
* Private Variable
*******************************************************************************/

/*******************************************************************************
* Private Function Prototype
*******************************************************************************/

/*******************************************************************************
* Public Function
*******************************************************************************/
void delay(void) {
    int i;

    for(i = 0; i < 500000; i++) {
        __NOP();
    }
}
/*******************************************************************************
* Private Function
*******************************************************************************/
void pllTest(void);
void SCU_PrintPllDividers(void);
void systemClockChangeTest(void);

/*
***********************************************************************
 * @brief		PLL Freq Output Test
 * @param[in]	None
 * @return 		None
 **********************************************************************/

#define SYSCLK_PLL_HSI_TEST
//#define SYSCLK_PLL_HSE_TEST

void pllTest(void) {

    int i;
    
    // To print PLL Result from 1 MHz to 48MHz, select the baud rate of debug channel to 1200 bps.
    uint8_t cmd;
    uint8_t pllOutfreq = PLL48MHz;  // Init PLL Freq 48MHz

    HAL_SCU_ClockOutput(4, ENABLE);  // CLKO(PF4) is out 1/10 scale of systemclock (MCLK/(2*(4+1)) = CLKO Freq)

    cputs("(=) Increase PLL Freq, (-) Decrease PLL Freq, (s) Test PLL Test\r\n");
    cputs("\r\nPLL>");

    // Manual Test
	while(1) {
		cmd = cgetchar();
        if ((cmd > 0) && (cmd <= 0x7F)) {
            // Increase PLL Out Frequency
            if (cmd == '=') {
                if (pllOutfreq >= PLL48MHz) {
                   pllOutfreq = PLL48MHz;
                } else {
                    pllOutfreq++;
                }
                cprintf("\r\npllOutFreq: %d", pllOutfreq);
            // Decrease PLL Out Frequency
            } else if (cmd == '-') {
                if (pllOutfreq <= PLL1MHz) {
                   pllOutfreq = PLL1MHz;
                } else {
                    pllOutfreq--;
                }
                cprintf("\r\npllOutFreq: %d", pllOutfreq);
            // Output PLL Frequency
            } else if (cmd == 's') {
                cputs("\r\nPLL Test Start\r\n\r\n");

                // PLL Test from 1MHz to 48MHz

#ifdef SYSCLK_PLL_HSE_TEST
                if (HAL_SCU_SetPLLFreq(XTAL8MHz, (PLLFREQ_Type)pllOutfreq) == PLL_OK) {
#else
                 if (HAL_SCU_SetPLLFreq(IRC32MHz, (PLLFREQ_Type)pllOutfreq) == PLL_OK) {
#endif

                    debug_frmwrk_init();
                     
                    for (i = 0; i < 0x10000; i++);
                    SCU_PrintPllDividers();
                    cprintf("\r\n%d MHz OK\r\n", pllOutfreq);

                }
            }
        }
    }

}

void systemClockChangeTest(void) {

    HAL_SCU_ClockOutput(4, ENABLE);

    HAL_SCU_SetSystemClock(SYSCLK_LSI, SYSDIV_NONE);     // 500kHz
//    HAL_SCU_SetSystemClock(SYSCLK_LSI, SYSDIV_2);        // 250kHz
//    HAL_SCU_SetSystemClock(SYSCLK_LSI, SYSDIV_4);        // 125kHz
//
//    HAL_SCU_SetSystemClock(SYSCLK_LSE, SYSDIV_NONE);     // 32768Hz
//    HAL_SCU_SetSystemClock(SYSCLK_LSE, SYSDIV_2);        // 16384Hz
//    HAL_SCU_SetSystemClock(SYSCLK_LSE, SYSDIV_4);        // 8192Hz

//    HAL_SCU_SetSystemClock(SYSCLK_HSE, SYSDIV_NONE);     // 8MHz (Use XTAL8MHz)
//    HAL_SCU_SetSystemClock(SYSCLK_HSE, SYSDIV_2);        // 4MHz
//    HAL_SCU_SetSystemClock(SYSCLK_HSE, SYSDIV_4);        // 2MHz

//    HAL_SCU_SetSystemClock(SYSCLK_HSI, SYSDIV_NONE);     // 32MHz
//    HAL_SCU_SetSystemClock(SYSCLK_HSI, SYSDIV_2);        // 16MHz
//    HAL_SCU_SetSystemClock(SYSCLK_HSI, SYSDIV_4);        // 8MHz
//    HAL_SCU_SetSystemClock(SYSCLK_HSI, SYSDIV_8);        // 4MHz
//    HAL_SCU_SetSystemClock(SYSCLK_HSI, SYSDIV_16);       // 2MHz
//    HAL_SCU_SetSystemClock(SYSCLK_HSI, SYSDIV_32);       // 1MHz

    while(1);

}

/*******************************************************************************
* @brief      TestCode_SCU_PLLTest
* @param      None
* @return     None
*******************************************************************************/
void TestCode_SCU_SystemClockTest(void) {
	uint8_t cmd;

    cputs("\r\n1. PLL Test\r\n2. System Main Clock Test\r\n\r\n");

    cputs(">");
	while(1) {
		cmd = cgetchar();
        if ((cmd > 0x0) && (cmd <= 0x7F)) {
            if (cmd == '1') {
                cputs("1. PLL Test \n\r\r\n");
                pllTest();
            } else if (cmd == '2') {
                cputs("2. MCLK System Clock change Test \n\r");
                systemClockChangeTest();
            }
        }
	}
}

/*******************************************************************************
* @brief      SCU_PrintPllDividers
* @param      None
* @return     None
*******************************************************************************/
void SCU_PrintPllDividers(void) {
    uint32_t Reg32;

    Reg32 = SCU->PLLCON;

    cprintf("R : %d, ", (Reg32 & SCU_PLLCON_PREDIV_Msk) >> SCU_PLLCON_PREDIV_Pos);
    cprintf("N1: %d, ", (Reg32 & SCU_PLLCON_POSTDIV1_Msk) >> SCU_PLLCON_POSTDIV1_Pos);
    cprintf("N2: %d, ", (Reg32 & SCU_PLLCON_POSTDIV2_Msk) >> SCU_PLLCON_POSTDIV2_Pos);
    cprintf("P : %d, ", (Reg32 & SCU_PLLCON_OUTDIV_Msk) >> SCU_PLLCON_OUTDIV_Pos);
    cprintf("D : %d\r\n\r\n", (Reg32 & SCU_PLLCON_PLLMODE_Msk) >> SCU_PLLCON_PLLMODE_Pos);
}

/* --------------------------------- End Of File ------------------------------ */
