/*****************************************************************************
* @file     TestCode_SPI_DMA.c
* @brief    Contains the external function prototypes for TestCode
* @author   AE Team, ABOV Semiconductor Co., Ltd.
* @version  V0.0.1
* @date     14. JUN. 2019
*
* Copyright(C) 2019, ABOV Semiconductor
* All rights reserved.
*
*
********************************************************************************
* DISCLAIMER
*
* THE PRESENT FIRMWARE WHICH IS FOR GUIDANCE ONLY AIMS AT PROVIDING CUSTOMERS
* WITH CODING INFORMATION REGARDING THEIR PRODUCTS IN ORDER FOR THEM TO SAVE
* TIME. AS A RESULT, ABOV SEMICONDUCTOR DISCLAIMS ALL LIABILITIES FROM ANY
* DIRECT, INDIRECT OR CONSEQUENTIAL DAMAGES WITH RESPECT TO ANY CLAIMS ARISING
* FROM THE CONTENT OF SUCH FIRMWARE AND/OR THE USE MADE BY CUSTOMERS OF THE
* CODING INFORMATION CONTAINED HEREIN IN CONNECTION WITH THEIR PRODUCTS.
*
*******************************************************************************/

/*******************************************************************************
* Pre-processor Definition & Macro
*******************************************************************************/
#include "TestCode_SPI_DMA.h"
#include "main_conf.h"


/*******************************************************************************
* Private Typedef
*******************************************************************************/
// #define PIN_REMAP

/*******************************************************************************
* Private Variable
*******************************************************************************/

uint8_t Loop_Data_C = 0xAB;

uint32_t i;
SPI_CFG_Type 	SPS_config; // for Slave
SPI_CFG_Type 	SPM_config; // for Master
DMAC_CFG_Type	DMA_Config_rx;
DMAC_CFG_Type	DMA_Config_tx;

Bool SPI_DMA_TX_Done = FALSE;
Bool SPI_DMA_RX_Done = FALSE;

#define BUFFER_SIZE             (32)
uint8_t SPI_rx_b[BUFFER_SIZE] = {0,};
uint8_t SPI_tx_b[BUFFER_SIZE] = {0,};
#define SPI_BORATE_set          (470) // PCLK/(3+1) = 48MHz/(47+1)= 1MHz

/*******************************************************************************
* Private Function Prototype
*******************************************************************************/
static uint8_t * Init_Buffer(uint8_t *Buf, uint32_t Size);

/*******************************************************************************
* Public Function
*******************************************************************************/
/*******************************************************************************
* @brief         Init_Buffer
* @param[in]     buf
* @param[in]     size
* @return     None
*******************************************************************************/
static uint8_t * Init_Buffer(uint8_t *Buf, uint32_t Size) {
	for(i = 0; i < Size;  i++) {
		Buf[i] = 0;
	}
	return Buf;
}
/*******************************************************************************
* @brief     SPI20_IRQHandler
* @param      None
* @return     None
*******************************************************************************/
void I2C2_SPI20_IRQHandler_DmaTest(void) {
	uint32_t status;
	status = HAL_SPI_GetStatus(SPI20); // Get SPI20 Status

	// DMA RX Done Interrupt
	if ( (status & SPI_STAT_RXDMA_DONE) == SPI_STAT_RXDMA_DONE) {
		HAL_SPI_ClearPending(SPI20, SPI_STAT_RXDMA_DONE);
		SPI_DMA_RX_Done  = TRUE;
	} else {
		SPI_DMA_RX_Done = FALSE;
	}

}
/*******************************************************************************
* @brief     SPI21_IRQHandler
* @param      None
* @return     None
*******************************************************************************/
void USART12_13_SPI21_IRQHandler_DmaTest(void) {
	uint32_t status;
	status = HAL_SPI_GetStatus(SPI21); // Get SPI21 Status

	// DMA TX Done Interrupt
	if ((status & SPI_STAT_TXDMA_DONE) == SPI_STAT_TXDMA_DONE) {
		HAL_SPI_ClearPending(SPI21, SPI_STAT_TXDMA_DONE);
		SPI_DMA_TX_Done = TRUE;
	} else {
		SPI_DMA_TX_Done = FALSE;
	}

}

/**********************************************************************
 * @brief		Main_SpiDmaTest
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void Main_SpiDmaTest(void) {
   
#ifndef PIN_REMAP
	// SLAVE : SPI20(MOSI20, MISO20, SCK20, SS20) PIN FUNCTION SETTING
	HAL_SPI_PortInit(SPI20);
#else

	HAL_SPI_PortRemap(REMAP_USART10, ENABLE);
	cputs("Replaced SPI20 with USART10 channel.\r\n");
#endif

	SPS_config.Databit = SPI_DS_8BITS;
	SPS_config.CPHA = SPI_CPHA_LO;
	SPS_config.CPOL = SPI_CPOL_LO;
	SPS_config.DataDir = SPI_MSB_FIRST;
	SPS_config.Mode = SPI_SLAVE_MODE;
	SPS_config.BaudRate = SPI_BORATE_set;

	// SPI0 EANBLE
	HAL_SPI_Init(SPI20, &SPS_config);
	HAL_SPI_IntConfig(SPI20, SPI_INTCFG_RXDIE , ENABLE); 	 // SPI20 rx interrupt enable

	NVIC_SetPriority(I2C2_SPI20_IRQn, 3);
	NVIC_EnableIRQ(I2C2_SPI20_IRQn);

	HAL_SPI_Cmd(SPI20, ENABLE); 							 // SPI20 enable

	cputs("SPI20 initialized.\r\n");

#ifndef PIN_REMAP
	// MASTER : SPI12(SS21, SCK21, MOSI21, MISO21) PIN FUNCTION SETTING
	HAL_SPI_PortInit(SPI21);
#else
	// MASTER : USART11(SS11, SCK11, MOSI11, MISO11) PIN FUNCTION SETTING
	SPI_PortRemap(REMAP_USART11, ENABLE);
	cputs("Replaced SPI21 with USART11 channel.\r\n");
#endif

	SPM_config.Databit = SPI_DS_8BITS;
	SPM_config.CPHA = SPI_CPHA_LO;
	SPM_config.CPOL = SPI_CPOL_LO;
	SPM_config.DataDir = SPI_MSB_FIRST;
	SPM_config.Mode = SPI_MASTER_MODE;
	SPM_config.BaudRate = SPI_BORATE_set;

	// SPI1 ENABLE
	HAL_SPI_Init(SPI21, &SPM_config);
	HAL_SPI_Cmd(SPI21, ENABLE);
	HAL_SPI_IntConfig(SPI21, SPI_INTCFG_TXDIE , ENABLE);

	/* preemption = 1, sub-priority = 1 */
	NVIC_SetPriority(USART12_13_SPI21_IRQn, 3);
	NVIC_EnableIRQ(USART12_13_SPI21_IRQn);

	cputs("SPI21 initialized.\r\n");


	// DMAC0 Configuration : SPI20 Rx
	DMA_Config_rx.Peripheral = DMAC_PERIPHERAL_SPI20_RX;			// Selected DMA SPI20 RX
	DMA_Config_rx.Width = DMAC_WIDTH_BYTE;							// Half-Word byte transfer (16-bit)
	DMA_Config_rx.Direction = DMAC_DIRECTION_RX;					// Peripheral to designated address
	DMA_Config_rx.PeripheralAddress = (uint32_t)&SPI20->RDR;		// SPI20->RDR register
	HAL_DMAC_Init(DMAC0, &DMA_Config_rx);							// DMA Channel Initization
	HAL_DMAC_SetMemoryAddress(DMAC0, (uint32_t)&SPI_rx_b);
	HAL_DMAC_SetTransferCount(DMAC0, BUFFER_SIZE);					// Setting DMA Transfer Count Value

	cputs("DMAC0 (SPI20_RX) Initialized.\r\n");

	// DMAC1 Configuration : SPI21 Tx
	DMA_Config_tx.Peripheral = DMAC_PERIPHERAL_SPI21_TX;		// Selected DMA SPI20 RX
	DMA_Config_tx.Width = DMAC_WIDTH_BYTE;						// Half-Word byte transfer (16-bit)
	DMA_Config_tx.Direction = DMAC_DIRECTION_TX;				// Peripheral to designated address
	DMA_Config_tx.PeripheralAddress = (uint32_t)&SPI21->TDR;	// SPI21->TDR register
	HAL_DMAC_Init(DMAC1, &DMA_Config_tx);						// DMA Channel Initization
	HAL_DMAC_SetMemoryAddress(DMAC1, (uint32_t)&SPI_tx_b);
	HAL_DMAC_SetTransferCount(DMAC1, BUFFER_SIZE);				// Setting DMA Transfer Count Value
	cputs("DMAC1 (SPI21_TX) Initialized.\r\n");

    cprintf("\r\n\r\n Tx Num: %d, Data: %02X\r\n", BUFFER_SIZE, Loop_Data_C);

	Init_Buffer(SPI_tx_b, sizeof(SPI_tx_b));
	Init_Buffer(SPI_rx_b, sizeof(SPI_rx_b));

	if(SPI_DMA_RX_Done == FALSE) {
		HAL_DMAC_SetMemoryAddress(DMAC0, (uint32_t)&(SPI_rx_b));
		HAL_DMAC_SetTransferCount(DMAC0, BUFFER_SIZE);						// Setting DMA Transfer Count Value
		HAL_DMAC_Start(DMAC0); // START DMA Receive
	}

	i = 0;
	while(1) {
		if (SPI_DMA_TX_Done == FALSE) {
			SPI_tx_b[i++] = Loop_Data_C; // Fill the value to DMA TX memory

			if(i >= BUFFER_SIZE) {
				HAL_DMAC_SetMemoryAddress(DMAC1, (uint32_t)&SPI_tx_b);		// Re-init Memory Address
				HAL_DMAC_SetTransferCount(DMAC1, BUFFER_SIZE);					// Re-init DMA Transfer Count Value

				i=0;
				HAL_DMAC_Start(DMAC1); // Start DMA Transmit
			}
		} else if (SPI_DMA_TX_Done == TRUE) {
			HAL_DMAC_Stop(DMAC1);

			cputs("DMA TX= ");
			for(i = 0; i < sizeof(SPI_tx_b); i++) {
                cprintf("%02X ", SPI_tx_b[i]);
//				_DBH8(SPI_tx_b[i]); _DBG(" ");
			}
			cputs("\r\n");

			Init_Buffer(SPI_tx_b, sizeof(SPI_tx_b)); // Init Tx Buffer
			break;
		}
	}

	if (SPI_DMA_RX_Done == TRUE) {
		HAL_DMAC_Stop(DMAC0);

		cputs("DMA RX= ");

		for(i = 0 ; i<BUFFER_SIZE ; i++) {
			if(SPI_rx_b[i] != Loop_Data_C) {
				cputs("\r\nFail!!\r\n");
			}
//			_DBH8(SPI_rx_b[i]); _DBG(" ");
            cprintf("%02X ", SPI_rx_b[i]);
		}

		if(i == BUFFER_SIZE ) {
			cputs(" -SUCCESS!!\r\n");
		}
		Init_Buffer(SPI_rx_b, sizeof(SPI_rx_b)); // Init Tx Buffer
	}
}

/* --------------------------------- End Of File ------------------------------ */
