/*****************************************************************************
* @file     TestCode_SPI_Interrupt.c
* @brief    Contains the external function prototypes for TestCode
* @author   AE Team, ABOV Semiconductor Co., Ltd.
* @version  V0.0.1
* @date     14. JUN. 2019
*
* Copyright(C) 2019, ABOV Semiconductor
* All rights reserved.
*
*
********************************************************************************
* DISCLAIMER
*
* THE PRESENT FIRMWARE WHICH IS FOR GUIDANCE ONLY AIMS AT PROVIDING CUSTOMERS
* WITH CODING INFORMATION REGARDING THEIR PRODUCTS IN ORDER FOR THEM TO SAVE
* TIME. AS A RESULT, ABOV SEMICONDUCTOR DISCLAIMS ALL LIABILITIES FROM ANY
* DIRECT, INDIRECT OR CONSEQUENTIAL DAMAGES WITH RESPECT TO ANY CLAIMS ARISING
* FROM THE CONTENT OF SUCH FIRMWARE AND/OR THE USE MADE BY CUSTOMERS OF THE
* CODING INFORMATION CONTAINED HEREIN IN CONNECTION WITH THEIR PRODUCTS.
*
*******************************************************************************/

/*******************************************************************************
* Pre-processor Definition & Macro
*******************************************************************************/
#include "TestCode_SPI_InterruptwithStrength.h"
#include "main_conf.h"
#define PIN_STREGNTH

//#define PIN_REMAP_MASTER
//#define PIN_REMAP_SLAVE

#define USE_DELAY


/*******************************************************************************
* Private Typedef
*******************************************************************************/

/*******************************************************************************
* Private Variable
*******************************************************************************/
static const uint8_t cmdm[] = "SPI2n> ";

// Variables for Interrupt Test
static uint32_t fflag;

static uint32_t rxdata[10];
static uint32_t txdata[10];
static uint32_t cnt, cntrx;

/*******************************************************************************
* Private Function Prototype
*******************************************************************************/

/*******************************************************************************
* Public Function
*******************************************************************************/
void USART12_13_SPI21_IRQHandler_InterruptTest(void) {
	uint32_t status;
	status = HAL_SPI_GetStatus(SPI21); // Get SPI20 Status

	// RXBUF Ready Interrupt
	if ( (status & SPI_STAT_RXBUF_READY) && (HAL_SPI_GetControl(SPI21) & SPI_INTCFG_RXIE)) { // rx buffer full
		rxdata[cntrx] = SPI21->RDR;
		cntrx++;
		if (cntrx==10) {
			fflag=1;
		}
	}
}
void I2C2_SPI20_IRQHandler_InterruptTest(void) {
	uint32_t status;
	status = HAL_SPI_GetStatus(SPI20); // Get SPI21 Status

	// TX Buffer Empty Interrupt
	if ( (status & SPI_STAT_TXBUF_EMPTY) && (HAL_SPI_GetControl(SPI20) & SPI_INTCFG_TXIE)) {
		if ((status & SPI_STAT_IDLE)) {
			HAL_SPI_Cmd(SPI20, DISABLE);  // SPI disable
			HAL_SPI_SendData(SPI20, txdata[cnt]); cnt++;
			HAL_SPI_Cmd(SPI20, ENABLE);  // SPI enable
		} else {
			HAL_SPI_SendData(SPI20, txdata[cnt]); cnt++;
		}

		if (cnt==10){
			HAL_SPI_IntConfig(SPI20, SPI_INTCFG_TXIE, DISABLE); //TXIE disable
		}
	}
}

/**********************************************************************
 * @brief		Main_SpiInterruptTest
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void TestCode_SPI_InterruptwithStrength(void) {
	uint32_t tmp;
	uint32_t i,j;

	SPI_CFG_Type SP_config;
	uint8_t		cmdn_cnt=0, ch_rtn, mod_check[4] = {0x30, 0x31, 0x32, 0x78};

    // Interrupt Setting
//    I2C2_SPI20_Call_F = I2C2_SPI20_IRQHandler_InterruptTest;
//    USART12_13_SPI21_IRQ_Call_F = USART12_13_SPI21_IRQHandler_InterruptTest;
    
	while(1) {
		if (cmdn_cnt == 0) {
			cputs((char *)cmdm);
			cmdn_cnt = 1;
			cputs("...Please input character 1:Init, 2:Test!, x:Exit \n\r");
		}

		ch_rtn = cgetchar();

        if ((ch_rtn > 0) && (ch_rtn <= 0x7F)) {
            cputc(ch_rtn);
            cputs("\r\n");


            if (ch_rtn == mod_check[1]) {
    #ifndef PIN_REMAP_SLAVE
                // SLAVE : SPI21(MOSI21, MISO21, SCK21, SS21) PIN FUNCTION SETTING
                HAL_SPI_PortInit(SPI21);
    #else
                // SLAVE : USART10(MOSI10, MISO10, SCK10, SS10) PIN FUNCTION SETTING
                HAL_SPI_PortRemap(REMAP_USART10, ENABLE);
                cputs("Replaced SPI20 with USART11 channel.\r\n");
    #endif

                SP_config.Databit = SPI_DS_8BITS;
                SP_config.CPHA = SPI_CPHA_LO;
                SP_config.CPOL = SPI_CPOL_LO;
                SP_config.DataDir = SPI_MSB_FIRST;
                SP_config.Mode = SPI_SLAVE_MODE;
                SP_config.BaudRate = 47; // 1MHz
                
                HAL_SPI_Init(SPI21, &SP_config);

                // SPI0 rx interrupt enable
                HAL_SPI_IntConfig(SPI21, SPI_INTCFG_RXIE, ENABLE);
                 // SPI0 enable
                HAL_SPI_Cmd(SPI21, ENABLE);

                /* preemption = 1, sub-priority = 1 */
                NVIC_SetPriority(USART12_13_SPI21_IRQn, 3);
                NVIC_EnableIRQ(USART12_13_SPI21_IRQn);

    #ifndef PIN_REMAP_MASTER
                // MASTER : SPI20(SS20, SCK20, MOSI20, MISO20) PIN FUNCTION SETTING
                HAL_SPI_PortInit(SPI20);
    #else
                // MASTER : USART11(SS11, SCK1, MOSI11, MISO11) PIN FUNCTION SETTING
                HAL_SPI_PortRemap(REMAP_USART11, ENABLE);
                cputs("Replaced SPI21 with USART11 channel.\r\n");
    #endif

                SP_config.Databit = SPI_DS_8BITS;
                SP_config.CPHA = SPI_CPHA_LO;
                SP_config.CPOL = SPI_CPOL_LO;
                SP_config.DataDir = SPI_MSB_FIRST;
                SP_config.Mode = SPI_MASTER_MODE;
                SP_config.BaudRate = 47; // 1MHz
                HAL_SPI_Init(SPI20, &SP_config);

    #ifdef  USE_DELAY
                HAL_SPI_DelayConfig(SPI20, 0x1, 0x1, 0x1);   // Set Start/Burst/Stop Delay of SPI Channel (Refer SPIn_LR)
    //            SPI_DelayConfig(SPI20, 0x1, 0xFF, 0x1);   // Set Start/Burst/Stop Delay of SPI Channel (Refer SPIn_LR)
    #endif            
                cprintf("Master SPI->LR: 0x%08X\r\n", SPI20->LR);

                // SPI1 enable
                HAL_SPI_Cmd(SPI20, ENABLE);

                /* preemption = 1, sub-priority = 1 */
                NVIC_SetPriority(I2C2_SPI20_IRQn, 3);
                NVIC_EnableIRQ(I2C2_SPI20_IRQn);

                __enable_irq();

                cmdn_cnt=0;

    #ifdef PIN_STREGNTH
            HAL_PCU_SetStrength(PB, 0, ENABLE);
            HAL_PCU_SetStrength(PB, 1, ENABLE);
            HAL_PCU_SetStrength(PB, 2, ENABLE);
            HAL_PCU_SetStrength(PC, 2, ENABLE);
            HAL_PCU_SetStrength(PC, 3, ENABLE);
            HAL_PCU_SetStrength(PC, 4, ENABLE);
    #endif

            } else if(ch_rtn == mod_check[2]) {

                while(1) {
                    for(i = 0; i < 10; i++) {
                        txdata[i] = i+0x30;
                        rxdata[i] = 0;
                    }
                    cnt = 0; cntrx = 0;
                    fflag = 0;

                    // Tx Interrupt Enable
                    HAL_SPI_IntConfig(SPI20, SPI_INTCFG_TXIE, ENABLE);

                    // check rx buffer full
                    while(fflag == 0);

                    for (i = 0; i < 10; i++) {
                        cprintf(" tx= %02X  rx= %02X\r\n", txdata[i], rxdata[i]);
                    }

                    j++;

                    for (tmp = 0; tmp < 1000000; tmp++) {}

                    if (j==0x01) {
                        j=0;
                        break;
                    }
                }

                cmdn_cnt=0;
            } else if (ch_rtn == mod_check[3]) {
                break;
            } else {
                cputs("...Please input character 1:Init, 2:Test!  \n\r");
                cmdn_cnt=0;
            }
        }
    }
}

/* --------------------------------- End Of File ------------------------------ */
