/*****************************************************************************
* @file     TestCode_SPI_Polling.c
* @brief    Contains the external function prototypes for TestCode
* @author   AE Team, ABOV Semiconductor Co., Ltd.
* @version  V0.0.1
* @date     14. JUN. 2019
*
* Copyright(C) 2019, ABOV Semiconductor
* All rights reserved.
*
*
********************************************************************************
* DISCLAIMER
*
* THE PRESENT FIRMWARE WHICH IS FOR GUIDANCE ONLY AIMS AT PROVIDING CUSTOMERS
* WITH CODING INFORMATION REGARDING THEIR PRODUCTS IN ORDER FOR THEM TO SAVE
* TIME. AS A RESULT, ABOV SEMICONDUCTOR DISCLAIMS ALL LIABILITIES FROM ANY
* DIRECT, INDIRECT OR CONSEQUENTIAL DAMAGES WITH RESPECT TO ANY CLAIMS ARISING
* FROM THE CONTENT OF SUCH FIRMWARE AND/OR THE USE MADE BY CUSTOMERS OF THE
* CODING INFORMATION CONTAINED HEREIN IN CONNECTION WITH THEIR PRODUCTS.
*
*******************************************************************************/

/*******************************************************************************
* Pre-processor Definition & Macro
*******************************************************************************/
#include "TestCode_SPI_Polling.h"
#include "main_conf.h"

/*******************************************************************************
* Private Typedef
*******************************************************************************/
// #define PIN_REMAP

/*******************************************************************************
* Private Variable
*******************************************************************************/
static const uint8_t cmdm[] = "SPI2n> ";

/*******************************************************************************
* Public Function Prototype
*******************************************************************************/

/*******************************************************************************
* Public Function
*******************************************************************************/
/**********************************************************************
 * @brief		Main_SpiPollingTest
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void Main_SpiPollingTest(void) {
	uint32_t rxdata;
	uint32_t tmp;
	uint32_t i, j;
	SPI_CFG_Type SPS_config;	// Slave
	SPI_CFG_Type SPM_config;	// Master

	uint8_t		cmdn_cnt=0, ch_rtn, mod_check[4] = {0x30, 0x31, 0x32, 0x78};


	while(1) {
		if (cmdn_cnt == 0) {
			cputs((char *)cmdm);
			cmdn_cnt=1;
			cputs("...Please input character 1:Init, 2:Test!, x:Exit  \n\r");
		}

		ch_rtn = cgetchar();
        
        if ((ch_rtn > 0) && (ch_rtn <= 0x7F)) { 
        cprintf("%c\r\n", ch_rtn);

            if (ch_rtn == mod_check[1]) {
    #ifndef PIN_REMAP
                // SLAVE : SPI20(MOSI20, MISO20, SCK20, SS20) PIN FUNCTION SETTING
                HAL_SPI_PortInit(SPI20);
    #else
                // SLAVE : USART10(MOSI10, MISO10, SCK10, SS10) PIN FUNCTION SETTING
                HAL_SPI_PortRemap(REMAP_USART10, ENABLE);
                cputs("Replaced SPI20 with USART11 channel.\r\n");
    #endif
                SPS_config.Databit = SPI_DS_8BITS;
                SPS_config.CPHA = SPI_CPHA_LO;
                SPS_config.CPOL = SPI_CPOL_LO;
                SPS_config.DataDir = SPI_MSB_FIRST;
                SPS_config.Mode = SPI_SLAVE_MODE;
                SPS_config.BaudRate = 47; // PCLK / (47+1)  = 48MHz/48 = 1MHz

                // SPI0 EANBLE
                HAL_SPI_Init(SPI20, &SPS_config);
                HAL_SPI_Cmd(SPI20, ENABLE);
                cputs("Initialized SPI20 as slave\r\n");

    #ifndef PIN_REMAP
                // MASTER : SPI21(SS1, SCK1, MOSI1, MISO1) PIN FUNCTION SETTING
                HAL_SPI_PortInit(SPI21);
    #else
                // MASTER : USART11(SS11, SCK1, MOSI11, MISO11) PIN FUNCTION SETTING
                HAL_SPI_PortRemap(REMAP_USART11, ENABLE);
                cputs("Replaced SPI21 with USART11 channel.\r\n");
    #endif
                SPM_config.Databit = SPI_DS_8BITS;
                SPM_config.CPHA = SPI_CPHA_LO;
                SPM_config.CPOL = SPI_CPOL_LO;
                SPM_config.DataDir = SPI_MSB_FIRST;
                SPM_config.Mode = SPI_MASTER_MODE;
                SPM_config.BaudRate = 47; // PCLK / (47+1)  = 48MHz/48 = 1MHz

                // SPI1 ENABLE
                HAL_SPI_Init(SPI21, &SPM_config);
                HAL_SPI_Cmd(SPI21, ENABLE);

                cputs("Initialized SPI21 as master\r\n");

                cmdn_cnt=0;

            } else if (ch_rtn == mod_check[2]) {

                cputs(" ");

                j=0;
                i=0x30;
                cprintf(" init1 tx= %02X ", i);
                while(1){

                    // CHECK TX BUFFER EMPTY
                    while((HAL_SPI_GetStatus(SPI21) & SPI_STAT_TXBUF_EMPTY) != SPI_STAT_TXBUF_EMPTY);
                    HAL_SPI_Cmd(SPI21, DISABLE);  // SPI disable
                    HAL_SPI_SendData(SPI21, i);
                    HAL_SPI_Cmd(SPI21, ENABLE);  // SPI enable

                    // CHECK RX BUFFER FULL
                    while((HAL_SPI_GetStatus(SPI20) & SPI_STAT_RXBUF_READY) != SPI_STAT_RXBUF_READY);
                    rxdata = HAL_SPI_ReceiveData(SPI20);
                    cprintf(" tx= %02X rx= %02X\r\n", i, rxdata);
     

                    i++;
                    if (i==0x3A) {
                        i=0x30;
                        j++;
                    }

                    if (j==0x01) {
                        cmdn_cnt=0;
                        break;
                    }

                    for (tmp=0;tmp <1000000; tmp++);	// Delay for test
                }
            } else if (ch_rtn == mod_check[3]) {
                break;
            } else {
                cputs("...Please input character 1:Init, 2:Test!  \n\r");
                cmdn_cnt=0;
            }
        }
	}
}

/* --------------------------------- End Of File ------------------------------ */
