/*****************************************************************************
* @file     TestCode_SRAM_Test.c
* @brief    Contains the external function prototypes for TestCode
* @author   AE Team, ABOV Semiconductor Co., Ltd.
* @version  V0.0.1
* @date     31. MAY. 2019
*
* Copyright(C) 2019, ABOV Semiconductor
* All rights reserved.
*
*
********************************************************************************
* DISCLAIMER
*
* THE PRESENT FIRMWARE WHICH IS FOR GUIDANCE ONLY AIMS AT PROVIDING CUSTOMERS
* WITH CODING INFORMATION REGARDING THEIR PRODUCTS IN ORDER FOR THEM TO SAVE
* TIME. AS A RESULT, ABOV SEMICONDUCTOR DISCLAIMS ALL LIABILITIES FROM ANY
* DIRECT, INDIRECT OR CONSEQUENTIAL DAMAGES WITH RESPECT TO ANY CLAIMS ARISING
* FROM THE CONTENT OF SUCH FIRMWARE AND/OR THE USE MADE BY CUSTOMERS OF THE
* CODING INFORMATION CONTAINED HEREIN IN CONNECTION WITH THEIR PRODUCTS.
*
*******************************************************************************/

/*******************************************************************************
* Pre-processor Definition & Macro
*******************************************************************************/
#include "TestCode_SRAM_Test.h"
#include "main_conf.h"

/* (NOTICE!!!)
For SRAM test, Have to set 'Options - IRAM Region and Size' and Build this target.
Because some user code data can exist the region of SRAM test, It maybe occurs fail while test.

start_offset = 0x20000000 --> IRAM1 Offset : 0x20002000, Size : 0x00003000 (0x20003000~0x20004FFF)
  or
start_offset = 0x20002000 --> IRAM2 Offset : 0x20000000, Size : 0x00002000 (0x20000000~0x20001FFF)
*/

//#define START_OFFSET    (0x20000000UL)			// SRAM Start Address for Test
//#define SIZE 			(8*1024)				// SIZE : 0x3000

#define START_OFFSET    (0x20002000UL)			// SRAM Start Address for Test
#define SIZE 			(12*1024)				// SIZE : 0x3000



#define DEBUF_PRINT

/*******************************************************************************
* Private Typedef
*******************************************************************************/

/*******************************************************************************
* Private Variable
*******************************************************************************/

/*******************************************************************************
* Private Function Prototype
*******************************************************************************/

/*******************************************************************************
* Public Function
*******************************************************************************/

/**********************************************************************
 * @brief		SRAM Test
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void Main_SramTest(void) {

	int i;
	uint32_t ucnt		= 0;

	// WRITE DATA TO SRAM
   for(i = 0; i < SIZE; i += 4) {
        (*(volatile unsigned int *)(START_OFFSET + i)) = (START_OFFSET + i);
   }

    // READ DATA FROM SRAM AND CHECK MATCHED?
    for(i = 0; i < SIZE; i += 4) {
        cprintf("0x%08X: 0x%08X", (START_OFFSET + i), *(volatile unsigned int *)(START_OFFSET + i));

        if( (START_OFFSET + i) != (*(volatile unsigned int *)(START_OFFSET + i)) ) {
            cprintf("\r\n FAIL @ 0x%08X \r\n", *(volatile unsigned int *)(START_OFFSET + i));

            ucnt++;
        }
        cputs("\r\n");
    }

    if (ucnt > 0) {
        cprintf("%d", ucnt);

        cputs("SRAM FAIL\r\n");
    } else {
        cputs("SRAM OK\r\n");
    }
}

/* --------------------------------- End Of File ------------------------------ */
