/*****************************************************************************
* @file     TestCode_TIMER1n_Capture.c
* @brief    Contains the external function prototypes for TestCode
* @author   AE Team, ABOV Semiconductor Co., Ltd.
* @version  V0.0.1
* @date     16. JUL. 2019
*
* Copyright(C) 2019, ABOV Semiconductor
* All rights reserved.
*
*
********************************************************************************
* DISCLAIMER
*
* THE PRESENT FIRMWARE WHICH IS FOR GUIDANCE ONLY AIMS AT PROVIDING CUSTOMERS
* WITH CODING INFORMATION REGARDING THEIR PRODUCTS IN ORDER FOR THEM TO SAVE
* TIME. AS A RESULT, ABOV SEMICONDUCTOR DISCLAIMS ALL LIABILITIES FROM ANY
* DIRECT, INDIRECT OR CONSEQUENTIAL DAMAGES WITH RESPECT TO ANY CLAIMS ARISING
* FROM THE CONTENT OF SUCH FIRMWARE AND/OR THE USE MADE BY CUSTOMERS OF THE
* CODING INFORMATION CONTAINED HEREIN IN CONNECTION WITH THEIR PRODUCTS.
*
*******************************************************************************/

/*******************************************************************************
* Pre-processor Definition & Macro
*******************************************************************************/
#include "TestCode_TIMER1n_Capture.h"
#include "main_conf.h"


#define MAX_TEST_STRING				(1)
#define MAX_BUFFER_LENGTH			(10)

#define ASCII_CODE_BS				(0x08) // Back space
#define ASCII_CODE_LF				(0x0A) // '\n'
#define ASCII_CODE_CR				(0x0D) // '\r'
#define ASCII_CODE_SPACE			(0x20) // Space
#define ASCII_CODE_DEL				(0x7F) // Delete

/*******************************************************************************
* Private Typedef
*******************************************************************************/
/**
 * @brief Test Time
 */
typedef enum {
	TIMER1n_TEST_TIME_010MS,
	TIMER1n_TEST_TIME_050MS,
	TIMER1n_TEST_TIME_100MS,
	TIMER1n_TEST_TIME_200MS,
	TIMER1n_TEST_TIME_400MS,
	TIMER1n_TEST_TIME_800MS,
	TIMER1n_TEST_TIME_333MS,
	TIMER1n_TEST_TIME_777MS,
	TIMER1n_TEST_TIME_968MS,
	TIMER1n_TEST_TIME_MAX
} TIMER1n_TEST_TIME_Type;

// Timer1n capture demo configuration structure definition
typedef struct {
	// base address
	TIMER1n_Type *pTIMER1x;
	// IRQ number
	IRQn_Type TIMER1x_IRQ;
	// Capture Port
	PORT_Type *pCapturePort_Group;
	uint32_t CapturePort_Number;
	PCU_ALT_FUNCTION_Type CapturePort_AltFunc;
} TIMER1n_CAPTURE_Type;

TIMER1n_CFG_Type TIMER1n_Config;

/*******************************************************************************
* Private Variable
*******************************************************************************/
static const uint8_t TEST_MENU[] =
"\t - 0: timer10, 1: timer11 ... 6: timer16\r\n";

static const uint8_t PROMPT_STRING[] =
"TIMER1n>";
static const uint8_t INVALID_STRING[] =
"Invaild";

static const TIMER1n_CAPTURE_Type TIMER1n_CAPTURE_CFG[10] = {
	{TIMER10, TIMER10_IRQn,              PE,  6, PCU_ALT_FUNCTION_2},   // 0
	{TIMER11, TIMER11_IRQn,              PA,  6, PCU_ALT_FUNCTION_2},   // 1
	{TIMER11, TIMER11_IRQn,              PE,  7, PCU_ALT_FUNCTION_2},   // 2
	{TIMER12, TIMER12_IRQn,              PA,  5, PCU_ALT_FUNCTION_2},   // 3
	{TIMER13, TIMER13_IRQn, (PORT_Type *)PF, 10, PCU_ALT_FUNCTION_2},   // 4
	{TIMER14, TIMER14_IRQn, (PORT_Type *)PF, 11, PCU_ALT_FUNCTION_2},   // 5
	{TIMER15, TIMER15_IRQn,              PB,  8, PCU_ALT_FUNCTION_2},   // 6
	{TIMER15, TIMER15_IRQn,              PB, 11, PCU_ALT_FUNCTION_1},   // 7
	{TIMER16, TIMER16_IRQn,              PB,  9, PCU_ALT_FUNCTION_2},   // 8
	{TIMER16, TIMER16_IRQn,              PB, 10, PCU_ALT_FUNCTION_1}    // 9
};
static const uint32_t TIMER1n_TEST_TIME[TIMER1n_TEST_TIME_MAX] = {
	10, 50, 100, 200, 400, 800, 333, 777, 968
};

static Bool Timer1nFlag;
static uint32_t MilliSec;
static TIMER1n_CFG_Type TIMER1n_Config;


/*******************************************************************************
* Private Function Prototype
*******************************************************************************/
static void Main_Timer1nIRQHandler(TIMER1n_Type *pTIMER1x);
static uint32_t Main_GetString(uint8_t *pBuffer, uint32_t MaxLength);

/*******************************************************************************
* Public Function
*******************************************************************************/

/*******************************************************************************
* @brief      SysTick interrupt handler @ 1000Hz(1ms)
* @param      None
* @return     None
*******************************************************************************/
void SysTick_Handler_Capture(void) {
	if (MilliSec) {
		MilliSec--;
	}
}

/*******************************************************************************
* @brief      Timer10 interrupt handler
* @param      None
* @return     None
*******************************************************************************/
void TIMER10_IRQHandler_Capture(void) {
	Main_Timer1nIRQHandler(TIMER10);
}

/*******************************************************************************
* @brief      Timer11 interrupt handler
* @param      None
* @return     None
*******************************************************************************/
void TIMER11_IRQHandler_Capture(void) {
	Main_Timer1nIRQHandler(TIMER11);
}

/*******************************************************************************
* @brief      Timer12 interrupt handler
* @param      None
* @return     None
*******************************************************************************/
void TIMER12_IRQHandler_Capture(void) {
	Main_Timer1nIRQHandler(TIMER12);
}

/*******************************************************************************
* @brief      Timer13 interrupt handler
* @param      None
* @return     None
*******************************************************************************/
void TIMER13_IRQHandler_Capture(void) {
	Main_Timer1nIRQHandler(TIMER13);
}

/*******************************************************************************
* @brief      Timer14 interrupt handler
* @param      None
* @return     None
*******************************************************************************/
void TIMER14_IRQHandler_Capture(void) {
	Main_Timer1nIRQHandler(TIMER14);
}

/*******************************************************************************
* @brief      Timer15 interrupt handler
* @param      None
* @return     None
*******************************************************************************/
void TIMER15_IRQHandler_Capture(void) {
   Main_Timer1nIRQHandler(TIMER15);
}

/*******************************************************************************
* @brief      Timer16 interrupt handler
* @param      None
* @return     None
*******************************************************************************/
void TIMER16_IRQHandler_Capture(void) {
	Main_Timer1nIRQHandler(TIMER16);
}


/*******************************************************************************
* @brief      Timer1n Interrupt Handler
* @param      None
* @return     None
*******************************************************************************/
static void Main_Timer1nIRQHandler(TIMER1n_Type *pTIMER1x) {
	if (HAL_TIMER1n_GetStatus(pTIMER1x) & TIMER1n_STATUS_CAPTURE_INT) {
		if (Timer1nFlag == FALSE) {
			Timer1nFlag = TRUE;
		}

		HAL_TIMER1n_ClearStatus(pTIMER1x, TIMER1n_STATUS_CAPTURE_INT);
	}
}



/*******************************************************************************
* @brief      Get String
* @param      pBuffer : Pointer of string Buffer
* @param      MaxLength : Max length of buffer
* @return     String lenth
*******************************************************************************/
static uint32_t Main_GetString(uint8_t *pBuffer, uint32_t MaxLength) {
	uint8_t InputData;
	uint32_t InputCount;
	static uint8_t PreInputData = 0x00;

	InputCount = 0;
	while (1) {
		InputData = cgetchar();
		if((InputData >= 0x20) && (InputData <= 0x7E)) {
			PreInputData = InputData;
			if (InputCount < MaxLength) {
				cputc(InputData);
				pBuffer[InputCount++] = InputData;
			}
		} else if ((InputData == ASCII_CODE_LF) || (InputData == ASCII_CODE_CR)) {
			if (((InputData == ASCII_CODE_LF) && (PreInputData != ASCII_CODE_CR))
				|| ((InputData == ASCII_CODE_CR) && (PreInputData != ASCII_CODE_LF))) {
				cprintf("\r\n");
				PreInputData = InputData;
                break;
			} else {
				PreInputData = InputData;
			}
		} else if ((InputData == ASCII_CODE_DEL) || (InputData == ASCII_CODE_BS)) {
			PreInputData = InputData;
			if (InputCount) {
				cputc(ASCII_CODE_BS);
				cputc(ASCII_CODE_SPACE);
				cputc(ASCII_CODE_BS);
				InputCount--;
			}
		}
	}

	return InputCount;
}

/*******************************************************************************
* @brief      TestCode_TIMER1n_Pwm
* @param      None
* @return     None
*******************************************************************************/
void TestCode_TIMER1n_Capture(void) {
	uint32_t Index;
	uint32_t Count;
	uint32_t StringLength;
	uint8_t StringBuffer[MAX_BUFFER_LENGTH];
	TIMER1n_TEST_TIME_Type TestTime;
	const TIMER1n_CAPTURE_Type * pConfig;

    cputs((char *)TEST_MENU);

	// Test port(PA0) setting
	HAL_GPIO_ConfigOutput(PA, 0, PCU_MODE_PUSH_PULL);
	HAL_GPIO_ConfigPullup(PA, 0, PCU_PUPD_DISABLE);
	HAL_GPIO_SetPin(PA, _BIT(0));

	// Capture port setting
	HAL_SCU_MiscClockConfig(4, PD0_TYPE, CLKSRC_LSI, 5); // 1 clock = 10us
	HAL_SCU_MiscClockConfig(4, PD1_TYPE, CLKSRC_LSI, 5); // 1 clock = 10us

	// Timer1n and clock source setting
	HAL_SCU_Timer1n_ClockConfig(T1NCLK_PCLK);

	TIMER1n_Config.StartSync = DISABLE;
	TIMER1n_Config.ClearSync = DISABLE;
	TIMER1n_Config.ExtClock = DISABLE; // PCLK
	TIMER1n_Config.CapturePolarity = TIMER1n_CAPTURE_POLARITY_FALLING;
	TIMER1n_Config.MatchInterrupt = DISABLE;
	TIMER1n_Config.CaptureInterrupt = ENABLE;

	TIMER1n_Config.AData = 0xFFFF;

	TIMER1n_Config.PrescalerData = 3200 - 1; // 32MHz / 3200 = 100kHz -> 0.1ms

	SysTick_Config(SystemCoreClock / 1000);

	__enable_irq();

	cputs((char *)PROMPT_STRING);

	while (1) {
		StringLength = Main_GetString(StringBuffer, MAX_BUFFER_LENGTH);
		if (StringLength == MAX_TEST_STRING) {
			Index = (uint32_t)(StringBuffer[0] - '0');

			pConfig = &(TIMER1n_CAPTURE_CFG[Index]);

			HAL_GPIO_ConfigOutput(pConfig->pCapturePort_Group, pConfig->CapturePort_Number, PCU_MODE_ALT_FUNC);
			HAL_GPIO_ConfigFunction(pConfig->pCapturePort_Group, pConfig->CapturePort_Number, pConfig->CapturePort_AltFunc);
			HAL_GPIO_SetDebouncePin(pConfig->pCapturePort_Group, pConfig->CapturePort_Number, ENABLE);
			HAL_GPIO_SetPin(PA, _BIT(0));
			TIMER1n_Config.CapturePolarity = TIMER1n_CAPTURE_POLARITY_FALLING;
			MilliSec = 100;
			while(MilliSec);

			NVIC_SetPriority(pConfig->TIMER1x_IRQ, 3);

			NVIC_EnableIRQ(pConfig->TIMER1x_IRQ);

			HAL_TIMER1n_Init(pConfig->pTIMER1x, TIMER1n_CAPTURE_MODE, &TIMER1n_Config);

			for (TestTime = TIMER1n_TEST_TIME_010MS; TestTime < TIMER1n_TEST_TIME_MAX; TestTime++) {
				HAL_TIMER1n_ClearCounter(pConfig->pTIMER1x);
				HAL_TIMER1n_Start(pConfig->pTIMER1x);
				MilliSec = TIMER1n_TEST_TIME[TestTime];
				while(MilliSec);
				HAL_GPIO_ClearPin(PA, _BIT(0));

                cprintf("%c ,", ('0' + (uint8_t)TestTime));
				while (Timer1nFlag == FALSE);
				Timer1nFlag = FALSE;
				Count = HAL_TIMER1n_GetCaptureData(pConfig->pTIMER1x);
				cprintf("%d,", Count);

				HAL_TIMER1n_Stop(pConfig->pTIMER1x);
				HAL_GPIO_SetPin(PA, _BIT(0));
				MilliSec = 100;
				while(MilliSec);
			}

			HAL_TIMER1n_DeInit(pConfig->pTIMER1x);
			HAL_GPIO_ClearPin(PA, _BIT(0));

			TIMER1n_Config.CapturePolarity = TIMER1n_CAPTURE_POLARITY_RISING;
			MilliSec = 100;
			while(MilliSec);

			HAL_TIMER1n_Init(pConfig->pTIMER1x, TIMER1n_CAPTURE_MODE, &TIMER1n_Config);

			for (TestTime = TIMER1n_TEST_TIME_010MS; TestTime < TIMER1n_TEST_TIME_MAX; TestTime++) {
				HAL_TIMER1n_ClearCounter(pConfig->pTIMER1x);
				HAL_TIMER1n_Start(pConfig->pTIMER1x);
				MilliSec = TIMER1n_TEST_TIME[TestTime];
				while(MilliSec);
				HAL_GPIO_SetPin(PA, _BIT(0));


                cprintf("%c ,", ('0' + (uint8_t)TestTime));
				while (Timer1nFlag == FALSE);
				Timer1nFlag = FALSE;
				Count = HAL_TIMER1n_GetCaptureData(pConfig->pTIMER1x);
				HAL_TIMER1n_ClearCounter(pConfig->pTIMER1x);
				cprintf("%d,", Count);

				HAL_TIMER1n_Stop(pConfig->pTIMER1x);
				HAL_GPIO_ClearPin(PA, _BIT(0));
				MilliSec = 100;
				while(MilliSec);
			}

			cputs(" ");
			HAL_TIMER1n_DeInit(pConfig->pTIMER1x);
			cputs((char *)PROMPT_STRING);
		} else {
			cprintf("%s\r\n", INVALID_STRING);
			cputs((char *)PROMPT_STRING);
		}
	}
}

/* --------------------------------- End Of File ------------------------------ */
