/*****************************************************************************
* @file     TestCode_TIMER1n_Interrupt.c
* @brief    Contains the external function prototypes for TestCode
* @author   AE Team, ABOV Semiconductor Co., Ltd.
* @version  V0.0.1
* @date     15. JUL. 2019
*
* Copyright(C) 2019, ABOV Semiconductor
* All rights reserved.
*
*
********************************************************************************
* DISCLAIMER
*
* THE PRESENT FIRMWARE WHICH IS FOR GUIDANCE ONLY AIMS AT PROVIDING CUSTOMERS
* WITH CODING INFORMATION REGARDING THEIR PRODUCTS IN ORDER FOR THEM TO SAVE
* TIME. AS A RESULT, ABOV SEMICONDUCTOR DISCLAIMS ALL LIABILITIES FROM ANY
* DIRECT, INDIRECT OR CONSEQUENTIAL DAMAGES WITH RESPECT TO ANY CLAIMS ARISING
* FROM THE CONTENT OF SUCH FIRMWARE AND/OR THE USE MADE BY CUSTOMERS OF THE
* CODING INFORMATION CONTAINED HEREIN IN CONNECTION WITH THEIR PRODUCTS.
*
*******************************************************************************/

/*******************************************************************************
* Pre-processor Definition & Macro
*******************************************************************************/
#include "TestCode_TIMER1n_Interrupt.h"
#include "main_conf.h"


#define MAX_TEST_STRING				(3)
#define MAX_BUFFER_LENGTH			(10)

#define ASCII_CODE_BS				(0x08) // Back space
#define ASCII_CODE_LF				(0x0A) // '\n'
#define ASCII_CODE_CR				(0x0D) // '\r'
#define ASCII_CODE_SPACE			(0x20) // Space
#define ASCII_CODE_DEL				(0x7F) // Delete

/*******************************************************************************
* Private Typedef
*******************************************************************************/
// Timer1n match demo configuration structure definition
typedef struct {
	// base address
	TIMER1n_Type *pTIMER1x;
	// IRQ number
	IRQn_Type TIMER1x_IRQ;
	// Output port
	PORT_Type *pOutputPort_Group;
	uint32_t OutputPort_Number;
	PCU_ALT_FUNCTION_Type OutputPort_AltFunc;
	// External clock port
	PORT_Type *pExtClockPort_Group;
	uint32_t ExtClockPort_Number;
	PCU_ALT_FUNCTION_Type ExtClockPort_AltFunc;
	uint8_t Timer1nName[7];
} TIMER1n_MATCH_TYPE;


/*******************************************************************************
* Private Variable
*******************************************************************************/
static const uint8_t PROMPT_STRING[] =
"TIMER1n>";
static const uint8_t INVALID_STRING[] =
"Invaild";

static const TIMER1n_MATCH_TYPE TIMER1n_MATCH_CFG[17] = {
	{TIMER10, TIMER10_IRQn,              PE,  6, PCU_ALT_FUNCTION_1,              PC, 11, PCU_ALT_FUNCTION_1, "TIMER10"}, // 00
	{TIMER10, TIMER10_IRQn,              PE,  6, PCU_ALT_FUNCTION_1,              PD,  1, PCU_ALT_FUNCTION_2, "TIMER10"}, // 01
	{TIMER11, TIMER11_IRQn,              PA,  6, PCU_ALT_FUNCTION_1,              PB,  6, PCU_ALT_FUNCTION_2, "TIMER11"}, // 02
	{TIMER11, TIMER11_IRQn,              PA,  6, PCU_ALT_FUNCTION_1,              PC, 12, PCU_ALT_FUNCTION_1, "TIMER11"}, // 03
	{TIMER11, TIMER11_IRQn,              PE,  7, PCU_ALT_FUNCTION_1,              PB,  6, PCU_ALT_FUNCTION_2, "TIMER11"}, // 04
	{TIMER11, TIMER11_IRQn,              PE,  7, PCU_ALT_FUNCTION_1,              PC, 12, PCU_ALT_FUNCTION_1, "TIMER11"}, // 05
	{TIMER12, TIMER12_IRQn,              PA,  5, PCU_ALT_FUNCTION_1,              PA,  2, PCU_ALT_FUNCTION_1, "TIMER12"}, // 06
	{TIMER13, TIMER13_IRQn, (PORT_Type *)PF, 10, PCU_ALT_FUNCTION_1, (PORT_Type *)PF,  8, PCU_ALT_FUNCTION_1, "TIMER13"}, // 07
	{TIMER14, TIMER14_IRQn, (PORT_Type *)PF, 11, PCU_ALT_FUNCTION_1, (PORT_Type *)PF,  9, PCU_ALT_FUNCTION_1, "TIMER14"}, // 08
	{TIMER15, TIMER15_IRQn,              PB,  8, PCU_ALT_FUNCTION_1,              PB,  9, PCU_ALT_FUNCTION_3, "TIMER15"}, // 09
	{TIMER15, TIMER15_IRQn,              PB,  8, PCU_ALT_FUNCTION_1,              PB, 10, PCU_ALT_FUNCTION_2, "TIMER15"}, // 10
	{TIMER15, TIMER15_IRQn,              PB, 11, PCU_ALT_FUNCTION_3,              PB,  9, PCU_ALT_FUNCTION_3, "TIMER15"}, // 11
	{TIMER15, TIMER15_IRQn,              PB, 11, PCU_ALT_FUNCTION_3,              PB, 10, PCU_ALT_FUNCTION_2, "TIMER15"}, // 12
	{TIMER16, TIMER16_IRQn,              PB,  9, PCU_ALT_FUNCTION_1,              PB,  8, PCU_ALT_FUNCTION_3, "TIMER16"}, // 13
	{TIMER16, TIMER16_IRQn,              PB,  9, PCU_ALT_FUNCTION_1,              PB, 11, PCU_ALT_FUNCTION_2, "TIMER16"}, // 14
	{TIMER16, TIMER16_IRQn,              PB, 10, PCU_ALT_FUNCTION_3,              PB,  8, PCU_ALT_FUNCTION_3, "TIMER16"}, // 15
	{TIMER16, TIMER16_IRQn,              PB, 10, PCU_ALT_FUNCTION_3,              PB, 11, PCU_ALT_FUNCTION_2, "TIMER16"}  // 16
};

static Bool Timer1nFlag;
static uint32_t Timer1nCount;

static TIMER1n_CFG_Type TIMER1n_Config;
static TIMER2n_CFG_Type TIMER2n_Config;

/*******************************************************************************
* Private Function Prototype
*******************************************************************************/
static void Main_Timer1nIRQHandler(TIMER1n_Type *pTIMER1x);
static uint32_t Main_GetString(uint8_t *pBuffer, uint32_t MaxLength);
void Main_InitializeTimer2n(void);

/*******************************************************************************
* Public Function
*******************************************************************************/
/*******************************************************************************
* @brief      Timer10 interrupt handler
* @param      None
* @return     None
*******************************************************************************/
void TIMER10_IRQHandler_Interrupt(void) {
	Main_Timer1nIRQHandler(TIMER10);
}

/*******************************************************************************
* @brief      Timer11 interrupt handler
* @param      None
* @return     None
*******************************************************************************/
void TIMER11_IRQHandler_Interrupt(void) {
	Main_Timer1nIRQHandler(TIMER11);
}

/*******************************************************************************
* @brief      Timer12 interrupt handler
* @param      None
* @return     None
*******************************************************************************/
void TIMER12_IRQHandler_Interrupt(void) {
	Main_Timer1nIRQHandler(TIMER12);
}

/*******************************************************************************
* @brief      Timer13 interrupt handler
* @param      None
* @return     None
*******************************************************************************/
void TIMER13_IRQHandler_Interrupt(void) {
	Main_Timer1nIRQHandler(TIMER13);
}

/*******************************************************************************
* @brief      Timer14 interrupt handler
* @param      None
* @return     None
*******************************************************************************/
void TIMER14_IRQHandler_Interrupt(void) {
	Main_Timer1nIRQHandler(TIMER14);
}

/*******************************************************************************
* @brief      Timer15 interrupt handler
* @param      None
* @return     None
*******************************************************************************/
void TIMER15_IRQHandler_Interrupt(void) {
   Main_Timer1nIRQHandler(TIMER15);
}

/*******************************************************************************
* @brief      Timer16 interrupt handler
* @param      None
* @return     None
*******************************************************************************/
void TIMER16_IRQHandler_Interrupt(void) {
	Main_Timer1nIRQHandler(TIMER16);
}

/*******************************************************************************
* @brief      Timer20 interrupt handler
* @param      None
* @return     None
*******************************************************************************/
void TIMER20_IRQHandler_Interrupt(void) {
	if (HAL_TIMER2n_GetStatus(TIMER20) & TIMER2n_STATUS_MATCH_INT) {
		HAL_TIMER2n_ClearStatus(TIMER20, TIMER2n_STATUS_MATCH_INT);
	}
}


/*******************************************************************************
* @brief      Timer1n Interrupt Handler
* @param      None
* @return     None
*******************************************************************************/
static void Main_Timer1nIRQHandler(TIMER1n_Type *pTIMER1x) {

	if (HAL_TIMER1n_GetStatus(pTIMER1x) & TIMER1n_STATUS_MATCH_INT) {
		Timer1nCount++;
		if (Timer1nCount >= 200) {
			Timer1nCount = 0;

			if (Timer1nFlag == TRUE) {
				Timer1nFlag = FALSE;
				HAL_GPIO_ClearPin(PE, _BIT(2));
			} else {
				Timer1nFlag = TRUE;
				HAL_GPIO_SetPin(PE, _BIT(2));
			}
		}
		HAL_TIMER1n_ClearStatus(pTIMER1x, TIMER1n_STATUS_MATCH_INT);
	}
}

/*******************************************************************************
* @brief      Get String
* @param      pBuffer : Pointer of string Buffer
* @param      MaxLength : Max length of buffer
* @return     String lenth
*******************************************************************************/
static uint32_t Main_GetString(uint8_t *pBuffer, uint32_t MaxLength) {
	uint8_t InputData;
	uint32_t InputCount;
	static uint8_t PreInputData = 0x00;

	InputCount = 0;
	while (1) {
		InputData = cgetchar();
		if((InputData >= 0x20) && (InputData <= 0x7E)) {
			PreInputData = InputData;
			if (InputCount < MaxLength) {
				cputc(InputData);
				pBuffer[InputCount++] = InputData;
			}
		} else if ((InputData == ASCII_CODE_LF) || (InputData == ASCII_CODE_CR)) {
			if (((InputData == ASCII_CODE_LF) && (PreInputData != ASCII_CODE_CR))
				|| ((InputData == ASCII_CODE_CR) && (PreInputData != ASCII_CODE_LF))) {
				cprintf("\r\n");
				PreInputData = InputData;
                break;
			} else {
				PreInputData = InputData;
			}
		} else if ((InputData == ASCII_CODE_DEL) || (InputData == ASCII_CODE_BS)) {
			PreInputData = InputData;
			if (InputCount) {
				cputc(ASCII_CODE_BS);
				cputc(ASCII_CODE_SPACE);
				cputc(ASCII_CODE_BS);
				InputCount--;
			}
		}
	}

	return InputCount;
}

/*******************************************************************************
* @brief      Initialize Timer2n for External Clock Source
* @param      None
* @return     None
*******************************************************************************/
static void Main_InitializeTimer2n(void) {
	// Timer2n output pin setting
	HAL_GPIO_ConfigOutput(PC, 0, PCU_MODE_ALT_FUNC);
	HAL_GPIO_ConfigFunction(PC, 0, PCU_ALT_FUNCTION_1);

	HAL_SCU_Timer20_ClockConfig(T20CLK_PCLK);

	TIMER2n_Config.ExtClock = DISABLE; // PCLK 32MHz
	TIMER2n_Config.PrescalerData = 32 - 1; // 32MHz / 32 = 1MHz -> 1us
	TIMER2n_Config.Mode = TIMER2n_MODE_PERIODIC;
	TIMER2n_Config.OutputPolarity = TIMER2n_OUTPUT_POLARITY_LOW;
	TIMER2n_Config.MatchInterrupt = ENABLE;
	TIMER2n_Config.CaptureInterrupt = DISABLE;

	TIMER2n_Config.AData = 1000 - 1; // 1ms

	HAL_TIMER2n_Init(TIMER20, &TIMER2n_Config);

	NVIC_SetPriority(TIMER20_IRQn, 3);
	NVIC_EnableIRQ(TIMER20_IRQn);

	HAL_TIMER2n_Start(TIMER20);
}


/**********************************************************************
 * @brief		TestCode TIMER1n Interrupt
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void TestCode_TIMER1n_Interrupt(void) {
	Bool StartedTest;
	uint32_t Index;
	uint32_t StringLength;
	const TIMER1n_MATCH_TYPE * pConfig;
	uint8_t StringBuffer[MAX_BUFFER_LENGTH];

	Timer1nFlag = FALSE;
	Timer1nCount = 0;

	// Test port(PE0) setting
	HAL_GPIO_ConfigOutput(PE, 2, PCU_MODE_PUSH_PULL);
	HAL_GPIO_ConfigPullup(PE, 2, PCU_PUPD_DISABLE);
	HAL_GPIO_ClearPin(PE, _BIT(2));

	__enable_irq();

	Main_InitializeTimer2n();

	cputs((char *)PROMPT_STRING);
	StartedTest = FALSE;
	while (1) {
		StringLength = Main_GetString(StringBuffer, MAX_BUFFER_LENGTH);
		if (StringLength == MAX_TEST_STRING) {
			if (StartedTest == TRUE) {
				pConfig = &(TIMER1n_MATCH_CFG[Index]);
				HAL_TIMER1n_Stop(pConfig->pTIMER1x);
				NVIC_DisableIRQ(pConfig->TIMER1x_IRQ);
				HAL_TIMER1n_DeInit(pConfig->pTIMER1x);
			} else {
				StartedTest = TRUE;
			}

			Index = (uint32_t)((StringBuffer[0] - '0') * 10) + (uint32_t)(StringBuffer[1] - '0');
			pConfig = &(TIMER1n_MATCH_CFG[Index]);

			// Timer1n output port setting
			HAL_GPIO_ConfigOutput(pConfig->pOutputPort_Group, pConfig->OutputPort_Number, PCU_MODE_ALT_FUNC);
			HAL_GPIO_ConfigFunction(pConfig->pOutputPort_Group, pConfig->OutputPort_Number, pConfig->OutputPort_AltFunc);

			// Timer1n and clock source setting (MCCR1 / PCLK / ECn)
			if (StringBuffer[2] == 'M') {
				HAL_SCU_MiscClockConfig(1, TIMER1n_TYPE, CLKSRC_MCLK, 2);
				HAL_SCU_Timer1n_ClockConfig(T1NCLK_MCCR1);

				TIMER1n_Config.ExtClock = DISABLE; // MCCR 16MHz
				TIMER1n_Config.PrescalerData = 16 - 1; // 16MHz / 16 = 1MHz -> 1us
			} else if (StringBuffer[2] == 'P') {
				HAL_SCU_Timer1n_ClockConfig(T1NCLK_PCLK);

				TIMER1n_Config.ExtClock = DISABLE; // PCLK 32MHz
				TIMER1n_Config.PrescalerData = 64 - 1; // 32MHz / 64 = 0.5MHz -> 2us
			} else {
				HAL_GPIO_ConfigOutput(pConfig->pExtClockPort_Group, pConfig->ExtClockPort_Number, PCU_MODE_ALT_FUNC);
				HAL_GPIO_ConfigFunction(pConfig->pExtClockPort_Group, pConfig->ExtClockPort_Number, pConfig->ExtClockPort_AltFunc);

				TIMER1n_Config.ExtClock = ENABLE; // EC1n External Pin
				TIMER1n_Config.ExtClockEdge = TIMER1n_EXT_CLOCK_EDGE_FALLING;
				TIMER1n_Config.PrescalerData = 0; // Bypass, 4us
			}

            cprintf("%s Channel Selected. \r\n", (pConfig->Timer1nName));

			TIMER1n_Config.StartSync = DISABLE;
			TIMER1n_Config.ClearSync = DISABLE;
			TIMER1n_Config.OutputPolarity = TIMER1n_OUTPUT_POLARITY_HIGH;
			TIMER1n_Config.MatchInterrupt = ENABLE;
			TIMER1n_Config.CaptureInterrupt = DISABLE;

			TIMER1n_Config.AData = 1000 - 1; // 1ms

			HAL_TIMER1n_Init(pConfig->pTIMER1x, TIMER1n_PERIODIC_MODE, &TIMER1n_Config);

			NVIC_SetPriority(pConfig->TIMER1x_IRQ, 3);

			NVIC_EnableIRQ(pConfig->TIMER1x_IRQ);

			HAL_TIMER1n_Start(pConfig->pTIMER1x);

            Main_InitializeTimer2n();
		} else {
			cprintf((const char *)INVALID_STRING);
			cprintf((const char *)PROMPT_STRING);
        }
	}

}

/* --------------------------------- End Of File ------------------------------ */
