/*****************************************************************************
* @file     TestCode_TIMER1n_Polling.c
* @brief    Contains the external function prototypes for TestCode
* @author   AE Team, ABOV Semiconductor Co., Ltd.
* @version  V0.0.1
* @date     15. JUL. 2019
*
* Copyright(C) 2019, ABOV Semiconductor
* All rights reserved.
*
*
********************************************************************************
* DISCLAIMER
*
* THE PRESENT FIRMWARE WHICH IS FOR GUIDANCE ONLY AIMS AT PROVIDING CUSTOMERS
* WITH CODING INFORMATION REGARDING THEIR PRODUCTS IN ORDER FOR THEM TO SAVE
* TIME. AS A RESULT, ABOV SEMICONDUCTOR DISCLAIMS ALL LIABILITIES FROM ANY
* DIRECT, INDIRECT OR CONSEQUENTIAL DAMAGES WITH RESPECT TO ANY CLAIMS ARISING
* FROM THE CONTENT OF SUCH FIRMWARE AND/OR THE USE MADE BY CUSTOMERS OF THE
* CODING INFORMATION CONTAINED HEREIN IN CONNECTION WITH THEIR PRODUCTS.
*
*******************************************************************************/

/*******************************************************************************
* Pre-processor Definition & Macro
*******************************************************************************/
#include "TestCode_TIMER1n_Polling.h"
#include "main_conf.h"


#define MAX_TEST_STRING				(1)
#define MAX_BUFFER_LENGTH			(10)

#define ASCII_CODE_BS				(0x08) // Back space
#define ASCII_CODE_LF				(0x0A) // '\n'
#define ASCII_CODE_CR				(0x0D) // '\r'
#define ASCII_CODE_SPACE			(0x20) // Space
#define ASCII_CODE_DEL				(0x7F) // Delete
/*******************************************************************************
* Private Typedef
*******************************************************************************/
// Timer1n match demo configuration structure definition
typedef struct {
	// base address
	TIMER1n_Type *pTIMER1x;
	// IRQ number
	IRQn_Type TIMER1x_IRQ;
	// Output port
	PORT_Type *pOutputPort_Group;
	uint32_t OutputPort_Number;
	PCU_ALT_FUNCTION_Type OutputPort_AltFunc;
} TIMER1n_MATCH_Type;

/*******************************************************************************
* Private Variable
**************************************.*****************************************/
static const uint8_t TEST_MENU[] =
"\t - Select a TIMER1n channel \r\n"
"\t - 0: timer10\r\n"
"\t - 1: timer11\r\n"
"\t - 2: timer12\r\n"
"\t - 3: timer13\r\n"
"\t - 4: timer14\r\n"
"\t - 5: timer15\r\n"
"\t - 6: timer16\r\n";

static const uint8_t PROMPT_STRING[] =
"TIMER1n>";
static const uint8_t INVALID_STRING[] =
"Invaild\r\n";


static const TIMER1n_MATCH_Type TIMER1n_MATCH_CFG[TIMER1n_IP_INDEX_MAX] = {
	{TIMER10, TIMER10_IRQn,              PE,  6, PCU_ALT_FUNCTION_1}, // 0
	{TIMER11, TIMER11_IRQn,              PE,  7, PCU_ALT_FUNCTION_1}, // 1
	{TIMER12, TIMER12_IRQn,              PA,  5, PCU_ALT_FUNCTION_1}, // 2
	{TIMER13, TIMER13_IRQn, (PORT_Type *)PF, 10, PCU_ALT_FUNCTION_1}, // 3
	{TIMER14, TIMER14_IRQn, (PORT_Type *)PF, 11, PCU_ALT_FUNCTION_1}, // 4
	{TIMER15, TIMER15_IRQn,              PB,  8, PCU_ALT_FUNCTION_1}, // 5
	{TIMER16, TIMER16_IRQn,              PB,  9, PCU_ALT_FUNCTION_1}  // 6
};

static TIMER1n_CFG_Type TIMER1n_Config;

/*******************************************************************************
* Private Function Prototype
*******************************************************************************/
static uint32_t Main_GetString(uint8_t *pBuffer, uint32_t MaxLength);
/*******************************************************************************
* Public Function
*******************************************************************************/


/*******************************************************************************
* @brief      Get String
* @param      pBuffer : Pointer of string Buffer
* @param      MaxLength : Max length of buffer
* @return     String lenth
*******************************************************************************/
static uint32_t Main_GetString(uint8_t *pBuffer, uint32_t MaxLength) {
	uint8_t InputData;
	uint32_t InputCount;
	static uint8_t PreInputData = 0x00;

	InputCount = 0;
	while (1) {
		InputData = cgetchar();
		if((InputData >= 0x20) && (InputData <= 0x7E)) {
			PreInputData = InputData;
			if (InputCount < MaxLength) {
				cputc(InputData);
				pBuffer[InputCount++] = InputData;
			}
		} else if ((InputData == ASCII_CODE_LF) || (InputData == ASCII_CODE_CR)) {
			if (((InputData == ASCII_CODE_LF) && (PreInputData != ASCII_CODE_CR))
				|| ((InputData == ASCII_CODE_CR) && (PreInputData != ASCII_CODE_LF))) {
				cprintf("\r\n");
				PreInputData = InputData;
                break;
			} else {
				PreInputData = InputData;
			}
		} else if ((InputData == ASCII_CODE_DEL) || (InputData == ASCII_CODE_BS)) {
			PreInputData = InputData;
			if (InputCount) {
				cputc(ASCII_CODE_BS);
				cputc(ASCII_CODE_SPACE);
				cputc(ASCII_CODE_BS);
				InputCount--;
			}
		}
	}

	return InputCount;
}

/**********************************************************************
 * @brief		TestCode TIMER1n Polling Test
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void TestCode_TIMER1n_Polling(void) {
	Bool Flag;
	uint32_t Index;
	uint32_t Count;
	uint32_t StringLength;
	const TIMER1n_MATCH_Type *pConfig;
	uint8_t StringBuffer[MAX_BUFFER_LENGTH];

	cputs((char *)TEST_MENU);

	// Test port(PE0) setting
	HAL_GPIO_ConfigOutput(PE, 0, PCU_MODE_PUSH_PULL);
	HAL_GPIO_ConfigPullup(PE, 0, PCU_PUPD_DISABLE);
	HAL_GPIO_ClearPin(PE, _BIT(0));

#if 1
	// Timer1n and clock source setting (PCLK)
	HAL_SCU_Timer1n_ClockConfig(T1NCLK_PCLK);
	TIMER1n_Config.ExtClock = DISABLE; // PCLK 48MHz
#endif
#if 0
    // Timer1n and clock source setting (PLL@ MCCR1)
	HAL_SCU_Timer1n_ClockConfig(T1NCLK_MCCR1);
    HAL_SCU_MiscClockConfig(1, TIMER1n_TYPE, CLKSRC_PLL, 1);
	TIMER1n_Config.ExtClock = DISABLE; // PCLK 48MHz
#endif
#if 0
    // Timer1n and clock source setting (EC1)
    HAL_GPIO_ConfigFunction(PC, 11, PCU_ALT_FUNCTION_1);
    HAL_GPIO_ConfigOutput(PC, 11, PCU_MODE_ALT_FUNC);
	TIMER1n_Config.ExtClock = ENABLE; // PCLK 48MHz
#endif

	TIMER1n_Config.PrescalerData = 48 - 1; // 48MHz / 48 = 1MHz -> 1us
	TIMER1n_Config.StartSync = DISABLE;
	TIMER1n_Config.ClearSync = DISABLE;
	TIMER1n_Config.OutputPolarity = TIMER1n_OUTPUT_POLARITY_HIGH;
	TIMER1n_Config.MatchInterrupt = DISABLE;
	TIMER1n_Config.CaptureInterrupt = DISABLE;

	Flag = FALSE;
	Count = 0;

	cprintf((const char *)PROMPT_STRING);

	while (1) {
		StringLength = Main_GetString(StringBuffer, MAX_BUFFER_LENGTH);
		if (StringLength == MAX_TEST_STRING) {
			Index = (uint32_t)(StringBuffer[0] - '0');

			pConfig = &(TIMER1n_MATCH_CFG[Index]);

			// Timer1n output port setting
			HAL_GPIO_ConfigOutput(pConfig->pOutputPort_Group, pConfig->OutputPort_Number, PCU_MODE_ALT_FUNC);
			HAL_GPIO_ConfigFunction(pConfig->pOutputPort_Group, pConfig->OutputPort_Number, pConfig->OutputPort_AltFunc);

			TIMER1n_Config.AData = (1000 * (Index + 1)) - 1;

			HAL_TIMER1n_Init(pConfig->pTIMER1x, TIMER1n_PERIODIC_MODE, &TIMER1n_Config);

			HAL_TIMER1n_Start(pConfig->pTIMER1x);
			cprintf("Test Start\r\n");

			while (1) {
				while ((HAL_TIMER1n_GetStatus(pConfig->pTIMER1x) & TIMER1n_STATUS_MATCH_INT) == 0);
				HAL_TIMER1n_ClearStatus(pConfig->pTIMER1x, TIMER1n_STATUS_MATCH_INT);

				Count++;
				if (Count >= 200) {
					Count = 0;

					if (Flag == TRUE) {
						Flag = FALSE;
						cprintf("Off\r\n");
						HAL_GPIO_ClearPin(PE, _BIT(0));
					} else {
						Flag = TRUE;
						cprintf("On\r\n");
						HAL_GPIO_SetPin(PE, _BIT(0));
					}
				}
			}
		} else {
			cprintf((const char *)INVALID_STRING);
			cprintf((const char *)PROMPT_STRING);
		}
	}
}

/* --------------------------------- End Of File ------------------------------ */
