/*****************************************************************************
* @file     TestCode_TIMER1n_Pwm.c
* @brief    Contains the external function prototypes for TestCode
* @author   AE Team, ABOV Semiconductor Co., Ltd.
* @version  V0.0.1
* @date     15. JUL. 2019
*
* Copyright(C) 2019, ABOV Semiconductor
* All rights reserved.
*
*
********************************************************************************
* DISCLAIMER
*
* THE PRESENT FIRMWARE WHICH IS FOR GUIDANCE ONLY AIMS AT PROVIDING CUSTOMERS
* WITH CODING INFORMATION REGARDING THEIR PRODUCTS IN ORDER FOR THEM TO SAVE
* TIME. AS A RESULT, ABOV SEMICONDUCTOR DISCLAIMS ALL LIABILITIES FROM ANY
* DIRECT, INDIRECT OR CONSEQUENTIAL DAMAGES WITH RESPECT TO ANY CLAIMS ARISING
* FROM THE CONTENT OF SUCH FIRMWARE AND/OR THE USE MADE BY CUSTOMERS OF THE
* CODING INFORMATION CONTAINED HEREIN IN CONNECTION WITH THEIR PRODUCTS.
*
*******************************************************************************/

/*******************************************************************************
* Pre-processor Definition & Macro
*******************************************************************************/
#include "TestCode_TIMER1n_Pwm.h"
#include "main_conf.h"


#define MAX_TEST_STRING				(1)
#define MAX_BUFFER_LENGTH			(10)

#define ASCII_CODE_BS				(0x08) // Back space
#define ASCII_CODE_LF				(0x0A) // '\n'
#define ASCII_CODE_CR				(0x0D) // '\r'
#define ASCII_CODE_SPACE			(0x20) // Space
#define ASCII_CODE_DEL				(0x7F) // Delete

/*******************************************************************************
* Private Typedef
*******************************************************************************/
// Timer1n PWM demo configuration structure definition
typedef struct {
	// base address
	TIMER1n_Type *pTIMER1x;
	// IRQ number
	IRQn_Type TIMER1x_IRQ;
 	// Output port
	PORT_Type *pOutputPort_Group;
	uint32_t OutputPort_Number;
	PCU_ALT_FUNCTION_Type OutputPort_AltFunc;
    uint8_t timer_name[7];
} TIMER1n_PWM_Type;

/*******************************************************************************
* Private Variable
*******************************************************************************/
static const uint8_t TEST_MENU[] =
"\t - 0: timer10, 1: timer11, ... 6: timer16\r\n";

static const uint8_t PROMPT_STRING[] =
"TIMER1n >";
static const uint8_t INVALID_STRING[] =
"Invaild";

static const TIMER1n_PWM_Type TIMER1n_PWM_CFG[TIMER1n_IP_INDEX_MAX] = {
	{TIMER10, TIMER10_IRQn,              PE,  6, PCU_ALT_FUNCTION_1, "TIMER10"},     // 0
	{TIMER11, TIMER11_IRQn,              PE,  7, PCU_ALT_FUNCTION_1, "TIMER11"},     // 1
	{TIMER12, TIMER12_IRQn,              PA,  5, PCU_ALT_FUNCTION_1, "TIMER12"},     // 2
	{TIMER13, TIMER13_IRQn, (PORT_Type *)PF, 10, PCU_ALT_FUNCTION_1, "TIMER13"},     // 3
	{TIMER14, TIMER14_IRQn, (PORT_Type *)PF, 11, PCU_ALT_FUNCTION_1, "TIMER14"},     // 4
	{TIMER15, TIMER15_IRQn,              PB,  8, PCU_ALT_FUNCTION_1, "TIMER15"},     // 5
	{TIMER16, TIMER16_IRQn,              PB,  9, PCU_ALT_FUNCTION_1, "TIMER16"}      // 6
};

static Bool Timer1nFlag;
static uint32_t Timer1nCount;
static TIMER1n_CFG_Type TIMER1n_Config;

/*******************************************************************************
* Private Function Prototype
*******************************************************************************/
static void Main_Timer1nIRQHandler(TIMER1n_Type *pTIMER1x);
static uint32_t Main_GetString(uint8_t *pBuffer, uint32_t MaxLength);

/*******************************************************************************
* Public Function
*******************************************************************************/
/*******************************************************************************
* @brief      Timer10 interrupt handler
* @param      None
* @return     None
*******************************************************************************/
void TIMER10_IRQHandler_Pwm(void) {
	Main_Timer1nIRQHandler(TIMER10);
}

/*******************************************************************************
* @brief      Timer11 interrupt handler
* @param      None
* @return     None
*******************************************************************************/
void TIMER11_IRQHandler_Pwm(void) {
	Main_Timer1nIRQHandler(TIMER11);
}

/*******************************************************************************
* @brief      Timer12 interrupt handler
* @param      None
* @return     None
*******************************************************************************/
void TIMER12_IRQHandler_Pwm(void) {
	Main_Timer1nIRQHandler(TIMER12);
}

/*******************************************************************************
* @brief      Timer13 interrupt handler
* @param      None
* @return     None
*******************************************************************************/
void TIMER13_IRQHandler_Pwm(void) {
	Main_Timer1nIRQHandler(TIMER13);
}

/*******************************************************************************
* @brief      Timer14 interrupt handler
* @param      None
* @return     None
*******************************************************************************/
void TIMER14_IRQHandler_Pwm(void) {
	Main_Timer1nIRQHandler(TIMER14);
}

/*******************************************************************************
* @brief      Timer15 interrupt handler
* @param      None
* @return     None
*******************************************************************************/
void TIMER15_IRQHandler_Pwm(void) {
   Main_Timer1nIRQHandler(TIMER15);
}

/*******************************************************************************
* @brief      Timer16 interrupt handler
* @param      None
* @return     None
*******************************************************************************/
void TIMER16_IRQHandler_Pwm(void) {
	Main_Timer1nIRQHandler(TIMER16);
}


/*******************************************************************************
* @brief      Timer1n Interrupt Handler
* @param      None
* @return     None
*******************************************************************************/
static void Main_Timer1nIRQHandler(TIMER1n_Type *pTIMER1x) {
	if (HAL_TIMER1n_GetStatus(pTIMER1x) & TIMER1n_STATUS_MATCH_INT) {
		Timer1nCount++;
		if (Timer1nCount >= 200) {
			Timer1nCount = 0;

			if (Timer1nFlag == TRUE) {
				Timer1nFlag = FALSE;
				HAL_GPIO_ClearPin(PE, _BIT(2));
			} else {
				Timer1nFlag = TRUE;
				HAL_GPIO_SetPin(PE, _BIT(2));
			}
		}

		HAL_TIMER1n_ClearStatus(pTIMER1x, TIMER1n_STATUS_MATCH_INT);
	}
}



/*******************************************************************************
* @brief      Get String
* @param      pBuffer : Pointer of string Buffer
* @param      MaxLength : Max length of buffer
* @return     String lenth
*******************************************************************************/
static uint32_t Main_GetString(uint8_t *pBuffer, uint32_t MaxLength) {
	uint8_t InputData;
	uint32_t InputCount;
	static uint8_t PreInputData = 0x00;

	InputCount = 0;
	while (1) {
		InputData = cgetchar();
		if((InputData >= 0x20) && (InputData <= 0x7E)) {
			PreInputData = InputData;
			if (InputCount < MaxLength) {
				cputc(InputData);
				pBuffer[InputCount++] = InputData;
			}
		} else if ((InputData == ASCII_CODE_LF) || (InputData == ASCII_CODE_CR)) {
			if (((InputData == ASCII_CODE_LF) && (PreInputData != ASCII_CODE_CR))
				|| ((InputData == ASCII_CODE_CR) && (PreInputData != ASCII_CODE_LF))) {
				cprintf("\r\n");
				PreInputData = InputData;
                break;
			} else {
				PreInputData = InputData;
			}
		} else if ((InputData == ASCII_CODE_DEL) || (InputData == ASCII_CODE_BS)) {
			PreInputData = InputData;
			if (InputCount) {
				cputc(ASCII_CODE_BS);
				cputc(ASCII_CODE_SPACE);
				cputc(ASCII_CODE_BS);
				InputCount--;
			}
		}
	}

	return InputCount;
}

/*******************************************************************************
* @brief      TestCode_TIMER1n_Pwm
* @param      None
* @return     None
*******************************************************************************/
void TestCode_TIMER1n_Pwm(void) {
	uint32_t Index;
	uint32_t StringLength;
	uint8_t StringBuffer[MAX_BUFFER_LENGTH];
	const TIMER1n_PWM_Type * pConfig;

	Timer1nFlag = FALSE;
	Timer1nCount = 0;

    cputs((char *)TEST_MENU);

	// Test port(PE0) setting
	HAL_GPIO_ConfigOutput(PE, 2, PCU_MODE_PUSH_PULL);
	HAL_GPIO_ConfigPullup(PE, 2, PCU_PUPD_DISABLE);
	HAL_GPIO_ClearPin(PE, _BIT(2));

	// Timer1n and clock source setting
	HAL_SCU_Timer1n_ClockConfig(T1NCLK_PCLK);

	TIMER1n_Config.StartSync = DISABLE;
	TIMER1n_Config.ClearSync = DISABLE;
	TIMER1n_Config.ExtClock = DISABLE; // PCLK 32MHz
	TIMER1n_Config.OutputPolarity = TIMER1n_OUTPUT_POLARITY_HIGH;
	TIMER1n_Config.MatchInterrupt = ENABLE;
	TIMER1n_Config.CaptureInterrupt = DISABLE;

	TIMER1n_Config.PrescalerData = 32 - 1; // 32MHz / 32 = 1MHz -> 1us

	cputs((char *)PROMPT_STRING);

	while (1) {
		StringLength = Main_GetString(StringBuffer, MAX_BUFFER_LENGTH);
		if (StringLength == MAX_TEST_STRING) {
			Index = (uint32_t)(StringBuffer[0] - '0');

			pConfig = &(TIMER1n_PWM_CFG[Index]);

            HAL_TIMER1n_Stop(pConfig->pTIMER1x);
            NVIC_DisableIRQ(pConfig->TIMER1x_IRQ);
            HAL_TIMER1n_DeInit(pConfig->pTIMER1x);

			// Timer1n output port setting
			HAL_GPIO_ConfigOutput(pConfig->pOutputPort_Group, pConfig->OutputPort_Number, PCU_MODE_ALT_FUNC);
			HAL_GPIO_ConfigFunction(pConfig->pOutputPort_Group, pConfig->OutputPort_Number, pConfig->OutputPort_AltFunc);

			TIMER1n_Config.AData = 150 * (Index + 1);

			TIMER1n_Config.BData = 100 * (Index + 1);

			cprintf("%s Start Test\r\n", pConfig->timer_name);
			HAL_TIMER1n_Init(pConfig->pTIMER1x, TIMER1n_PWM_MODE, &TIMER1n_Config);

			NVIC_SetPriority(pConfig->TIMER1x_IRQ, 3);
			NVIC_EnableIRQ(pConfig->TIMER1x_IRQ);

			HAL_TIMER1n_Start(pConfig->pTIMER1x);
		} else {
			cprintf("%s\r\n", INVALID_STRING);
			cputs((char *)PROMPT_STRING);
		}
	}
}

/* --------------------------------- End Of File ------------------------------ */
