/*****************************************************************************
* @file     TestCode_TIMER2n_CaptureWdt.c
* @brief    Contains the external function prototypes for TestCode
* @author   AE Team, ABOV Semiconductor Co., Ltd.
* @version  V0.0.1
* @date     16. JUL. 2019
*
* Copyright(C) 2019, ABOV Semiconductor
* All rights reserved.
*
*
********************************************************************************
* DISCLAIMER
*
* THE PRESENT FIRMWARE WHICH IS FOR GUIDANCE ONLY AIMS AT PROVIDING CUSTOMERS
* WITH CODING INFORMATION REGARDING THEIR PRODUCTS IN ORDER FOR THEM TO SAVE
* TIME. AS A RESULT, ABOV SEMICONDUCTOR DISCLAIMS ALL LIABILITIES FROM ANY
* DIRECT, INDIRECT OR CONSEQUENTIAL DAMAGES WITH RESPECT TO ANY CLAIMS ARISING
* FROM THE CONTENT OF SUCH FIRMWARE AND/OR THE USE MADE BY CUSTOMERS OF THE
* CODING INFORMATION CONTAINED HEREIN IN CONNECTION WITH THEIR PRODUCTS.
*
*******************************************************************************/

/*******************************************************************************
* Pre-processor Definition & Macro
*******************************************************************************/
#include "TestCode_TIMER2n_CaptureWdt.h"
#include "main_conf.h"


//#define USED_CAPTURE_LSE
#define USED_CAPTURE_WDTRC

#define MAX_COUNT_INDEX				(20)

#define TARGET_TIMER				(TIMER2n_IP_INDEX_TIMER20) // TIMER2n_INDEX_TIMER20 ~ TIMER2n_INDEX_TIMER21

/*******************************************************************************
* Private Typedef
*******************************************************************************/
// Timer2n capture demo configuration structure definition
typedef struct {
	// base address
	TIMER2n_Type *pTIMER2x;
	// IRQ number
	IRQn_Type TIMER2x_IRQ;
	// Capture port
	PORT_Type *pCapturePort_Group;
	uint32_t CapturePort_Number;
	PCU_ALT_FUNCTION_Type CapturePort_AltFunc;
} TIMER2n_CAPTURE_Type;


/*******************************************************************************
* Private Variable
*******************************************************************************/
static const TIMER2n_CAPTURE_Type TIMER2n_CAPTURE_CFG[TIMER2n_IP_INDEX_MAX] = {
	{TIMER20, TIMER20_IRQn, PC, 0, PCU_ALT_FUNCTION_2},
	{TIMER21, TIMER21_IRQn, PC, 1, PCU_ALT_FUNCTION_2}
};

static uint32_t Timer2nIndex;
static uint32_t Timer2nCount[MAX_COUNT_INDEX];

static TIMER2n_CFG_Type TIMER2n_Config;


/*******************************************************************************
* Private Function Prototype
*******************************************************************************/
static void Main_Timer2nIRQHandler(TIMER2n_Type *pTIMER2x);

/*******************************************************************************
* Public Function
*******************************************************************************/

/*******************************************************************************
* @brief      Timer20 interrupt handler
* @param      None
* @return     None
*******************************************************************************/
void TIMER20_IRQHandler_CaptureWdt(void) {
	Main_Timer2nIRQHandler(TIMER20);
}

/*******************************************************************************
* @brief      Timer21 interrupt handler
* @param      None
* @return     None
*******************************************************************************/
void TIMER21_IRQHandler_CaptureWdt(void) {
	Main_Timer2nIRQHandler(TIMER21);
}

/*******************************************************************************
* @brief      Timer2n Interrupt Handler
* @param      None
* @return     None
*******************************************************************************/
static void Main_Timer2nIRQHandler(TIMER2n_Type *pTIMER2x) {
	if (HAL_TIMER2n_GetStatus(pTIMER2x) & TIMER2n_STATUS_CAPTURE_INT) {
		if (Timer2nIndex < MAX_COUNT_INDEX) {
			Timer2nCount[Timer2nIndex] = HAL_TIMER2n_GetCaptureData(pTIMER2x);
			HAL_TIMER2n_ClearCounter(pTIMER2x);
			Timer2nIndex++;
		}

		HAL_TIMER2n_ClearStatus(pTIMER2x, TIMER2n_STATUS_CAPTURE_INT);
	}
}

/**********************************************************************
 * @brief		TestCode TIMER2n Capture WDT Test
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void TestCode_TIMER2n_CaptureWdt(void) {
	volatile uint32_t i;
	const TIMER2n_CAPTURE_Type * pConfig;

	pConfig = &(TIMER2n_CAPTURE_CFG[TARGET_TIMER]);

	// Capture port setting
	HAL_SCU_MiscClockConfig(4, PD0_TYPE, CLKSRC_LSI, 5); // 1 clock = 10us

	HAL_GPIO_ConfigOutput(pConfig->pCapturePort_Group, pConfig->CapturePort_Number, PCU_MODE_ALT_FUNC);
	HAL_GPIO_ConfigFunction(pConfig->pCapturePort_Group, pConfig->CapturePort_Number, pConfig->CapturePort_AltFunc);
	HAL_GPIO_SetDebouncePin(pConfig->pCapturePort_Group, pConfig->CapturePort_Number, ENABLE);

#if defined(USED_CAPTURE_WDTRC)
    SCU->PPCLKSR |= (SCU_PPCLKSR_WDTCLK_Msk);
#endif


	// Timer2n and clock source setting
	HAL_SCU_Timer20_ClockConfig(T20CLK_PCLK);

	TIMER2n_Config.ExtClock = DISABLE; // PCLK
	TIMER2n_Config.Mode = TIMER2n_MODE_CAPTURE;
#if defined(USED_CAPTURE_LSE)
	SCU_SetLSE(LSE_EN); // 32kHz
	SCU_WaitForLSEStartUp();

	TIMER2n_Config.CaptureSignal = TIMER2n_CAPTURE_SIGNAL_LSE;
	_DBG("LSE,");
#elif defined(USED_CAPTURE_WDTRC)
	TIMER2n_Config.CaptureSignal = TIMER2n_CAPTURE_SIGNAL_WDTRC;
	_DBG("WDTRC,");
#else
	TIMER2n_Config.CaptureSignal = TIMER2n_CAPTURE_SIGNAL_EXTERNAL;
	_DBG("EXT,");
#endif
	TIMER2n_Config.CapturePolarity = TIMER2n_CAPTURE_POLARITY_RISING;
	TIMER2n_Config.MatchInterrupt = DISABLE;
	TIMER2n_Config.CaptureInterrupt = ENABLE;

	TIMER2n_Config.AData = 0xFFFFFFFF;

	TIMER2n_Config.PrescalerData = 4 - 1; // 32MHz / 4 = 8MHz -> 0.125us

	HAL_TIMER2n_Init(pConfig->pTIMER2x, &TIMER2n_Config);
	NVIC_SetPriority(pConfig->TIMER2x_IRQ, 3);

	NVIC_EnableIRQ(pConfig->TIMER2x_IRQ);

	__enable_irq();

//	Timer2nIndex = 0;
	HAL_TIMER2n_ClearCounter(pConfig->pTIMER2x);
	HAL_TIMER2n_Start(pConfig->pTIMER2x);

	for (i = 0; i < 10000; i++);

	i = 0;
	while (1) {
		if (Timer2nIndex == MAX_COUNT_INDEX) {
			for (i = 0; i < MAX_COUNT_INDEX; i++) {
                cprintf("%d, ", Timer2nCount[i]);
			}
			Timer2nIndex++;
		}
	}

}

/* --------------------------------- End Of File ------------------------------ */
