/*****************************************************************************
* @file     TestCode_TIMER3n_AdcTrigger.c
* @brief    Contains the external function prototypes for TestCode
* @author   AE Team, ABOV Semiconductor Co., Ltd.
* @version  V0.0.1
* @date     16. JUL. 2019
*
* Copyright(C) 2019, ABOV Semiconductor
* All rights reserved.
*
*
********************************************************************************
* DISCLAIMER
*
* THE PRESENT FIRMWARE WHICH IS FOR GUIDANCE ONLY AIMS AT PROVIDING CUSTOMERS
* WITH CODING INFORMATION REGARDING THEIR PRODUCTS IN ORDER FOR THEM TO SAVE
* TIME. AS A RESULT, ABOV SEMICONDUCTOR DISCLAIMS ALL LIABILITIES FROM ANY
* DIRECT, INDIRECT OR CONSEQUENTIAL DAMAGES WITH RESPECT TO ANY CLAIMS ARISING
* FROM THE CONTENT OF SUCH FIRMWARE AND/OR THE USE MADE BY CUSTOMERS OF THE
* CODING INFORMATION CONTAINED HEREIN IN CONNECTION WITH THEIR PRODUCTS.
*
*******************************************************************************/

/*******************************************************************************
* Pre-processor Definition & Macro
*******************************************************************************/
#include "TestCode_TIMER3n_AdcTrigger.h"
#include "main_conf.h"

/*******************************************************************************
* Private Typedef
*******************************************************************************/

/*******************************************************************************
* Private Variable
*******************************************************************************/
static Bool Timer3nPeriodFlag;
static Bool Timer3nBottomFlag;
static Bool AdcMultiSamplingFlag;

static TIMER3n_CFG_Type TIMER3n_Config;
static TIMER3n_OUTPUT_CFG_Type TIMER3n_OutputA_Config;
static TIMER3n_OUTPUT_CFG_Type TIMER3n_OutputB_Config;
static TIMER3n_HiZ_CFG_Type TIMER3n_HiZ_Config;

static TIMER3n_ADC_TRIGGER_CFG_Type TIMER3n_AdcTrigger_Config;
static ADC_CFG_Type ADC_Config;

static uint32_t Data[4];
static ADC_TRIGGER_SOURCE_Type Source[4];
static ADC_CHANNEL_Type Channel[4];

static int AdcTestFlag;
/*******************************************************************************
* Public Function Prototype
*******************************************************************************/


/*******************************************************************************
* Public Function
*******************************************************************************/

/*******************************************************************************
* @brief      Timer30 interrupt handler
* @param      None
* @return     None
*******************************************************************************/
void TIMER30_IRQHandler_AdcTrigger(void) {
	static uint32_t Status;

	Status = HAL_TIMER3n_GetStatus(TIMER30);

//	TIMER3n_ClearStatus(TIMER30, (TIMER3n_STATUS_Type)TIMER3n_INTFLAG_STATUS_CLEAR_MASK);	 // Clear all TIMER30 flags

	// TIMER30 C channel Match Interrupt
	if (Status & TIMER3n_STATUS_CMATCH_INT) {
		HAL_TIMER3n_ClearStatus(TIMER30, TIMER3n_STATUS_CMATCH_INT);

		AdcTestFlag = ~AdcTestFlag;
		if (AdcTestFlag) {
			HAL_GPIO_ClearPin(PC, _BIT(1));
        } else {
			HAL_GPIO_SetPin(PC, _BIT(1));
        }
	}

	// TIMER30 B channel Match Interrupt
	if (Status & TIMER3n_STATUS_BMATCH_INT) {
		HAL_TIMER3n_ClearStatus(TIMER30, TIMER3n_STATUS_BMATCH_INT);

		AdcTestFlag = ~AdcTestFlag;
		if (AdcTestFlag) {
			HAL_GPIO_ClearPin(PC, _BIT(1));
        } else {
			HAL_GPIO_SetPin(PC, _BIT(1));
        }
	}

	// TIMER30 A channel Match Interrupt
	if (Status & TIMER3n_STATUS_AMATCH_INT) {
		HAL_TIMER3n_ClearStatus(TIMER30, TIMER3n_STATUS_AMATCH_INT);

		AdcTestFlag = ~AdcTestFlag;
		if (AdcTestFlag) {
			HAL_GPIO_ClearPin(PC, _BIT(1));
        } else {
			HAL_GPIO_SetPin(PC, _BIT(1));
        }
	}

	// Period Match Interrupt
	if (Status & TIMER3n_STATUS_PMATCH_INT) {
		if(Timer3nPeriodFlag == TRUE) {
			Timer3nPeriodFlag = FALSE;
			HAL_GPIO_ClearPin(PC, _BIT(0));
		} else {
			Timer3nPeriodFlag = TRUE;
			HAL_GPIO_SetPin(PC, _BIT(0));
		}
		HAL_TIMER3n_ClearStatus(TIMER30, TIMER3n_STATUS_PMATCH_INT);
	}

	// TIMER30 Bottom Match Interrupt
	if (Status & TIMER3n_STATUS_BOTTOM_INT) {
		if(Timer3nBottomFlag == TRUE) {
			Timer3nBottomFlag = FALSE;
			HAL_GPIO_ClearPin(PC, _BIT(1));
		} else {
			Timer3nBottomFlag = TRUE;
			HAL_GPIO_SetPin(PC, _BIT(1));
		}
		HAL_TIMER3n_ClearStatus(TIMER30, TIMER3n_STATUS_BOTTOM_INT);
	}

	// TIMER30 High-Impedence Interrupt
	if (Status & TIMER3n_STATUS_HIZ_INT) {
		HAL_TIMER3n_ClearStatus(TIMER30, TIMER3n_STATUS_HIZ_INT);
	}
}

/*******************************************************************************
* @brief      ADC interrupt handler
* @param      None
* @return     None
*******************************************************************************/
void ADC_IRQHandler_AdcTrigger(void) {
	ADC_STATUS_Type status; // check interrupt flag

	status = HAL_ADC_GetStatus(ADC);

	// EOC Flag
	if (status & ADC_STATUS_SINGLE_INT) {
		HAL_ADC_ClearStatus(ADC, ADC_STATUS_SINGLE_INT);
	}
	// EOS Flag
	if (status & ADC_STATUS_SEQUENCE_INT) {
		HAL_ADC_ClearStatus(ADC, ADC_STATUS_SEQUENCE_INT);
		AdcMultiSamplingFlag = TRUE;
	}
	// Trigger Flag
	if (status & ADC_STATUS_TRIGGER_INT) {
		HAL_ADC_ClearStatus(ADC, ADC_STATUS_TRIGGER_INT);
	}
	// DMA Overrun Flag
	if (status & ADC_STATUS_DMA_OVER) {
		HAL_ADC_ClearStatus(ADC, ADC_STATUS_DMA_OVER);
	}
}

/*******************************************************************************
* @brief      ADC Config Init
* @param      None
* @return     None
*******************************************************************************/
static void Main_AdcConfigInit(void) {
	// Set ADC input pin
	HAL_ADC_PortInit(0);
	HAL_ADC_PortInit(1);
	HAL_ADC_PortInit(2);
	HAL_ADC_PortInit(3);

	// Set ADC external clock
	HAL_SCU_HSI_ClockConfig(HSI_EN);	// HSI (32MHz)
	HAL_SCU_MiscClockConfig(7, ADC_TYPE, CLKSRC_HSI, 8);

	// Initialize and configure ADC
	ADC_Config.TriggerInfo = ENABLE;
	ADC_Config.ChannelInfo = ENABLE;
	ADC_Config.Dma = DISABLE;
	ADC_Config.Restart = ENABLE;	// Restart after a sequence sampling
	ADC_Config.Mode = ADC_MODE_BURST;
	ADC_Config.TriggerSelection = ADC_TRIGGER_SEL_TIMER3n;	// TIMER_30
	ADC_Config.SamplingTime = 0x1F;
	ADC_Config.SequenceCounter = ADC_SEQ_NUM_4;
	ADC_Config.ExternalClock = ENABLE;
	ADC_Config.InternalClockDivider = 1;
	HAL_ADC_Init(ADC, &ADC_Config);

	// Setting trigger source and channel
	Source[0] = ADC_TRIGGER_SOURCE_0;
	Source[1] = ADC_TRIGGER_SOURCE_0;
	Source[2] = ADC_TRIGGER_SOURCE_0;
	Source[3] = ADC_TRIGGER_SOURCE_0;
	HAL_ADC_ConfigureAllTrigger(ADC, 4, Source);
	Channel[0] = ADC_CHANNEL_0;
	Channel[1] = ADC_CHANNEL_1;
	Channel[2] = ADC_CHANNEL_2;
	Channel[3] = ADC_CHANNEL_3;
	HAL_ADC_ConfigureAllSequence(ADC, 4, Channel);

	// ADC Interrupt Enable
	HAL_ADC_ConfigureInterrupt(ADC, (ADC_INTERRUPT_Type)(ADC_INTERRUPT_SINGLE | ADC_INTERRUPT_SEQUENCE | ADC_INTERRUPT_TRIGGER), ENABLE);
	NVIC_SetPriority(ADC_IRQn, 3);
	NVIC_EnableIRQ(ADC_IRQn);

	HAL_ADC_ClearStatus(ADC, (ADC_STATUS_Type)ADC_SR_STATUS_CLEAR_MASK);
}


/**********************************************************************
 * @brief		TestCode TIMER3n Capture Test
 * @param[in]	None
 * @return 		None
 **********************************************************************/
#if 1
// Period Match Timer Trigger
void TestCode_TIMER3n_AdcTrigger(void) {

	// Deinitialization of TIMER30
	HAL_TIMER3n_DeInit(TIMER30);

	// Set test output pin (PC0) - for Period Match
	HAL_GPIO_ConfigOutput(PC, 0, PCU_MODE_PUSH_PULL);
	HAL_GPIO_ConfigPullup(PC, 0, PCU_PUPD_DISABLE);
	HAL_GPIO_ClearPin(PC, _BIT(0));

	// Set test output pin (PC1) - for Bottom Match
	HAL_GPIO_ConfigOutput(PC, 1, PCU_MODE_PUSH_PULL);
	HAL_GPIO_ConfigPullup(PC, 1, PCU_PUPD_DISABLE);
	HAL_GPIO_ClearPin(PC, _BIT(1));

	// Set PWM pin
	// PWM30AA(PE0)
	HAL_GPIO_ConfigOutput(PE, 0, PCU_MODE_ALT_FUNC);
	HAL_GPIO_ConfigFunction(PE, 0, PCU_ALT_FUNCTION_1);
	// PWM30AB(PE1)
	HAL_GPIO_ConfigOutput(PE, 1, PCU_MODE_ALT_FUNC);
	HAL_GPIO_ConfigFunction(PE, 1, PCU_ALT_FUNCTION_1);
	// PWM30BA(PE2)
	HAL_GPIO_ConfigOutput(PE, 2, PCU_MODE_ALT_FUNC);
	HAL_GPIO_ConfigFunction(PE, 2, PCU_ALT_FUNCTION_1);
	// PWM30BB(PE3)
	HAL_GPIO_ConfigOutput(PE, 3, PCU_MODE_ALT_FUNC);
	HAL_GPIO_ConfigFunction(PE, 3, PCU_ALT_FUNCTION_1);
	// PWM30CA(PE4)
	HAL_GPIO_ConfigOutput(PE, 4, PCU_MODE_ALT_FUNC);
	HAL_GPIO_ConfigFunction(PE, 4, PCU_ALT_FUNCTION_1);
	// PWM30CB(PE5)
	HAL_GPIO_ConfigOutput(PE, 5, PCU_MODE_ALT_FUNC);
	HAL_GPIO_ConfigFunction(PE, 5, PCU_ALT_FUNCTION_1);

	// Initialize Motor PWM
	HAL_SCU_Timer30_ClockConfig(T30CLK_PCLK);

	TIMER3n_Config.ExtClock = DISABLE;
	TIMER3n_Config.Mode = TIMER3n_MODE_INTERVAL; // Set TIMER30 Operationg Mode
	TIMER3n_Config.OutputMode = TIMER3n_OUTPUT_MODE_6_CHANNEL;
	TIMER3n_Config.DelayInsertion = DISABLE;
	TIMER3n_Config.DelayPosition = TIMER3n_DELAY_POSITION_FRONT;
	TIMER3n_Config.ReloadTime = TIMER3n_DATA_RELOAD_TIME_MATCH;
	TIMER3n_Config.MatchIntInterval = TIMER3n_MATCH_INT_INTERVAL_1;
	TIMER3n_Config.PrescalerData = 3199; // 32MHz / (31999 + 1) = 10kHz = 0.1ms
	TIMER3n_Config.PData = 10000; // 0.1ms * 10000 = 1s
	TIMER3n_Config.AData = 3000; // 30.0%
	TIMER3n_Config.BData = 5000; // 50.0%
	TIMER3n_Config.CData = 7000; // 70.0%
	TIMER3n_Config.DelayData = 0; // (0 + 1) / 1MHz = 1us
	HAL_TIMER3n_Init(TIMER30, &TIMER3n_Config);

	// Setting High Impedence control
	TIMER3n_OutputA_Config.StartLevel = TIMER3n_LEVEL_LOW;
	TIMER3n_OutputA_Config.Output_PWM30Ax = DISABLE;
	TIMER3n_OutputA_Config.Output_PWM30Bx = DISABLE;
	TIMER3n_OutputA_Config.Output_PWM30Cx = DISABLE;
	TIMER3n_OutputA_Config.DisableLevel_PWM30Ax = TIMER3n_LEVEL_LOW;
	TIMER3n_OutputA_Config.DisableLevel_PWM30Bx = TIMER3n_LEVEL_LOW;
	TIMER3n_OutputA_Config.DisableLevel_PWM30Cx = TIMER3n_LEVEL_LOW;
	TIMER3n_OutputB_Config.StartLevel = TIMER3n_LEVEL_LOW;
	TIMER3n_OutputB_Config.Output_PWM30Ax = DISABLE;
	TIMER3n_OutputB_Config.Output_PWM30Bx = DISABLE;
	TIMER3n_OutputB_Config.Output_PWM30Cx = DISABLE;
	TIMER3n_OutputB_Config.DisableLevel_PWM30Ax = TIMER3n_LEVEL_HIGH;
	TIMER3n_OutputB_Config.DisableLevel_PWM30Bx = TIMER3n_LEVEL_HIGH;
	TIMER3n_OutputB_Config.DisableLevel_PWM30Cx = TIMER3n_LEVEL_HIGH;
	HAL_TIMER3n_ConfigureOutput(TIMER30, TIMER3n_OUTPUT_CTRL_ALL, &TIMER3n_OutputA_Config, &TIMER3n_OutputB_Config);

	TIMER3n_HiZ_Config.HiZ = DISABLE;
	TIMER3n_HiZ_Config.BLNK_Edge = TIMER3n_BLNK_EDGE_FALLING;
	HAL_TIMER3n_ConfigureHiZ(TIMER30, &TIMER3n_HiZ_Config);
	HAL_TIMER3n_SetHiZOutput(TIMER30, RESET);

	// ADC Trigger with Period Match Interrupt
	TIMER3n_AdcTrigger_Config.Trigger_Bottom = DISABLE;
	TIMER3n_AdcTrigger_Config.Trigger_PMatch = ENABLE;
	TIMER3n_AdcTrigger_Config.Trigger_AMatch = DISABLE;
	TIMER3n_AdcTrigger_Config.Trigger_BMatch = DISABLE;
	TIMER3n_AdcTrigger_Config.Trigger_CMatch = DISABLE;
	HAL_TIMER3n_ConfigureADCTrigger(TIMER30, &TIMER3n_AdcTrigger_Config);

	Main_AdcConfigInit();	// ADC Initialization (Port, Configuration, Interrupt)

	// TIMER Interrupt
	HAL_TIMER3n_ConfigureInterrupt(TIMER30, TIMER3n_INTERRUPT_PMATCH_INT, ENABLE);
	NVIC_SetPriority(TIMER30_IRQn, 3);
	NVIC_EnableIRQ(TIMER30_IRQn);
	__enable_irq();

	// Clear Counter value, Flag Status and Start PWM
	HAL_TIMER3n_ClearStatus(TIMER30, (TIMER3n_STATUS_Type)TIMER3n_INTFLAG_STATUS_CLEAR_MASK);	// Clear Interrupt Flag
	HAL_TIMER3n_ClearCounter(TIMER30);
	HAL_TIMER3n_Start(TIMER30);
	HAL_ADC_Start(ADC);

	// Start timer
	AdcMultiSamplingFlag = FALSE;
	while (1) {
		if (AdcMultiSamplingFlag == TRUE) {
			Data[0] = HAL_ADC_GetConversionData(ADC, ADC_DATA_SEL_SEQ0);
			Data[1] = HAL_ADC_GetConversionData(ADC, ADC_DATA_SEL_SEQ1);
			Data[2] = HAL_ADC_GetConversionData(ADC, ADC_DATA_SEL_SEQ2);
			Data[3] = HAL_ADC_GetConversionData(ADC, ADC_DATA_SEL_SEQ3);

            cprintf("AN0 = %04X ", Data[0]);
            cprintf("AN1 = %04X ", Data[1]);
            cprintf("AN2 = %04X ", Data[2]);
            cprintf("AN3 = %04X ", Data[3]);
            cputs("\r\n");
			AdcMultiSamplingFlag = FALSE;
		}
	}
}
#endif

#if 0
// PWM Back-to-Back Bottom ADC Trigger
void TestCode_TIMER3n_AdcTrigger(void) {

	// Deinitialization of TIMER30
	HAL_TIMER3n_DeInit(TIMER30);

	// Set test output pin (PC0) - for Period Match
	HAL_GPIO_ConfigOutput(PC, 0, PCU_MODE_PUSH_PULL);
	HAL_GPIO_ConfigPullup(PC, 0, PCU_PUPD_DISABLE);
	HAL_GPIO_ClearPin(PC, _BIT(0));

	// Set test output pin (PC1) - for Bottom Match
	HAL_GPIO_ConfigOutput(PC, 1, PCU_MODE_PUSH_PULL);
	HAL_GPIO_ConfigPullup(PC, 1, PCU_PUPD_DISABLE);
	HAL_GPIO_ClearPin(PC, _BIT(1));

	// Set PWM pin
	// PWM30AA(PE0)
	HAL_GPIO_ConfigOutput(PE, 0, PCU_MODE_ALT_FUNC);
	HAL_GPIO_ConfigFunction(PE, 0, PCU_ALT_FUNCTION_1);
	// PWM30AB(PE1)
	HAL_GPIO_ConfigOutput(PE, 1, PCU_MODE_ALT_FUNC);
	HAL_GPIO_ConfigFunction(PE, 1, PCU_ALT_FUNCTION_1);
	// PWM30BA(PE2)
	HAL_GPIO_ConfigOutput(PE, 2, PCU_MODE_ALT_FUNC);
	HAL_GPIO_ConfigFunction(PE, 2, PCU_ALT_FUNCTION_1);
	// PWM30BB(PE3)
	HAL_GPIO_ConfigOutput(PE, 3, PCU_MODE_ALT_FUNC);
	HAL_GPIO_ConfigFunction(PE, 3, PCU_ALT_FUNCTION_1);
	// PWM30CA(PE4)
	HAL_GPIO_ConfigOutput(PE, 4, PCU_MODE_ALT_FUNC);
	HAL_GPIO_ConfigFunction(PE, 4, PCU_ALT_FUNCTION_1);
	// PWM30CB(PE5)
	HAL_GPIO_ConfigOutput(PE, 5, PCU_MODE_ALT_FUNC);
	HAL_GPIO_ConfigFunction(PE, 5, PCU_ALT_FUNCTION_1);

	// Initialize Motor PWM
	HAL_SCU_Timer30_ClockConfig(T30CLK_PCLK);

	TIMER3n_Config.ExtClock = DISABLE;
	TIMER3n_Config.Mode = TIMER3n_MODE_BACKTOBACK; // Set TIMER30 Operationg Mode
	TIMER3n_Config.OutputMode = TIMER3n_OUTPUT_MODE_6_CHANNEL;
	TIMER3n_Config.DelayInsertion = DISABLE;
	TIMER3n_Config.DelayPosition = TIMER3n_DELAY_POSITION_FRONT;
	TIMER3n_Config.ReloadTime = TIMER3n_DATA_RELOAD_TIME_MATCH;
	TIMER3n_Config.MatchIntInterval = TIMER3n_MATCH_INT_INTERVAL_1;
	TIMER3n_Config.PrescalerData = 31999; // 32MHz / (31999 + 1) = 10kHz = 0.1ms
	TIMER3n_Config.PData = 10000; // 0.1ms * 10000 = 1s
	TIMER3n_Config.AData = 3000; // 30.0%
	TIMER3n_Config.BData = 5000; // 50.0%
	TIMER3n_Config.CData = 7000; // 70.0%
	TIMER3n_Config.DelayData = 0; // (0 + 1) / 1MHz = 1us
	HAL_TIMER3n_Init(TIMER30, &TIMER3n_Config);

	// Setting High Impedence control
	TIMER3n_OutputA_Config.StartLevel = TIMER3n_LEVEL_LOW;
	TIMER3n_OutputA_Config.Output_PWM30Ax = DISABLE;
	TIMER3n_OutputA_Config.Output_PWM30Bx = DISABLE;
	TIMER3n_OutputA_Config.Output_PWM30Cx = DISABLE;
	TIMER3n_OutputA_Config.DisableLevel_PWM30Ax = TIMER3n_LEVEL_LOW;
	TIMER3n_OutputA_Config.DisableLevel_PWM30Bx = TIMER3n_LEVEL_LOW;
	TIMER3n_OutputA_Config.DisableLevel_PWM30Cx = TIMER3n_LEVEL_LOW;
	TIMER3n_OutputB_Config.StartLevel = TIMER3n_LEVEL_LOW;
	TIMER3n_OutputB_Config.Output_PWM30Ax = DISABLE;
	TIMER3n_OutputB_Config.Output_PWM30Bx = DISABLE;
	TIMER3n_OutputB_Config.Output_PWM30Cx = DISABLE;
	TIMER3n_OutputB_Config.DisableLevel_PWM30Ax = TIMER3n_LEVEL_HIGH;
	TIMER3n_OutputB_Config.DisableLevel_PWM30Bx = TIMER3n_LEVEL_HIGH;
	TIMER3n_OutputB_Config.DisableLevel_PWM30Cx = TIMER3n_LEVEL_HIGH;
	HAL_TIMER3n_ConfigureOutput(TIMER30, TIMER3n_OUTPUT_CTRL_ALL, &TIMER3n_OutputA_Config, &TIMER3n_OutputB_Config);

	TIMER3n_HiZ_Config.HiZ = DISABLE;
	TIMER3n_HiZ_Config.BLNK_Edge = TIMER3n_BLNK_EDGE_FALLING;
	HAL_TIMER3n_ConfigureHiZ(TIMER30, &TIMER3n_HiZ_Config);
	HAL_TIMER3n_SetHiZOutput(TIMER30, RESET);

	// ADC Trigger with Period Match Interrupt
	TIMER3n_AdcTrigger_Config.Trigger_Bottom = ENABLE;    // Bottom Match Trigger
	TIMER3n_AdcTrigger_Config.Trigger_PMatch = DISABLE;
	TIMER3n_AdcTrigger_Config.Trigger_AMatch = DISABLE;
	TIMER3n_AdcTrigger_Config.Trigger_BMatch = DISABLE;
	TIMER3n_AdcTrigger_Config.Trigger_CMatch = DISABLE;
	HAL_TIMER3n_ConfigureADCTrigger(TIMER30, &TIMER3n_AdcTrigger_Config);

	Main_AdcConfigInit();	// ADC Initialization (Port, Configuration, Interrupt)

	// TIMER Interrupt
	HAL_TIMER3n_ConfigureInterrupt(TIMER30, TIMER3n_INTERRUPT_BOTTOM_INT, ENABLE);	// Bottom Match Interrupt
	NVIC_SetPriority(TIMER30_IRQn, 3);
	NVIC_EnableIRQ(TIMER30_IRQn);
	__enable_irq();

	// Clear Counter value, Flag Status and Start PWM
	HAL_TIMER3n_ClearStatus(TIMER30, (TIMER3n_STATUS_Type)TIMER3n_INTFLAG_STATUS_CLEAR_MASK);	// Clear Interrupt Flag
	HAL_TIMER3n_ClearCounter(TIMER30);
	HAL_TIMER3n_Start(TIMER30);
	HAL_ADC_Start(ADC);

	// Start timer
	AdcMultiSamplingFlag = FALSE;
	while (1) {
		if (AdcMultiSamplingFlag == TRUE) {

			Data[0] = HAL_ADC_GetConversionData(ADC, ADC_DATA_SEL_SEQ0);
			Data[1] = HAL_ADC_GetConversionData(ADC, ADC_DATA_SEL_SEQ1);
			Data[2] = HAL_ADC_GetConversionData(ADC, ADC_DATA_SEL_SEQ2);
			Data[3] = HAL_ADC_GetConversionData(ADC, ADC_DATA_SEL_SEQ3);

            cprintf("AN0 = %04X ", Data[0]);
            cprintf("AN1 = %04X ", Data[1]);
            cprintf("AN2 = %04X ", Data[2]);
            cprintf("AN3 = %04X ", Data[3]);
            cputs("\r\n");

   			AdcMultiSamplingFlag = FALSE;   // Flag status update
		}
	}
}
#endif
/* --------------------------------- End Of File ------------------------------ */
