/*****************************************************************************
* @file     TestCode_TIMER3n_Capture.c
* @brief    Contains the external function prototypes for TestCode
* @author   AE Team, ABOV Semiconductor Co., Ltd.
* @version  V0.0.1
* @date     16. JUL. 2019
*
* Copyright(C) 2019, ABOV Semiconductor
* All rights reserved.
*
*
********************************************************************************
* DISCLAIMER
*
* THE PRESENT FIRMWARE WHICH IS FOR GUIDANCE ONLY AIMS AT PROVIDING CUSTOMERS
* WITH CODING INFORMATION REGARDING THEIR PRODUCTS IN ORDER FOR THEM TO SAVE
* TIME. AS A RESULT, ABOV SEMICONDUCTOR DISCLAIMS ALL LIABILITIES FROM ANY
* DIRECT, INDIRECT OR CONSEQUENTIAL DAMAGES WITH RESPECT TO ANY CLAIMS ARISING
* FROM THE CONTENT OF SUCH FIRMWARE AND/OR THE USE MADE BY CUSTOMERS OF THE
* CODING INFORMATION CONTAINED HEREIN IN CONNECTION WITH THEIR PRODUCTS.
*
*******************************************************************************/

/*******************************************************************************
* Pre-processor Definition & Macro
*******************************************************************************/
#include "TestCode_TIMER3n_Capture.h"
#include "main_conf.h"

/*******************************************************************************
* Private Typedef
*******************************************************************************/

/*******************************************************************************
* Private Variable
*******************************************************************************/

/*******************************************************************************
* Private Function Prototype
*******************************************************************************/
static Bool Timer3nPeriodFlag;
static Bool Timer3nBottomFlag;
static uint32_t Timer3nCaptureFlag;
static uint32_t Timer3nCaptureCount[2];
static uint32_t Timer3nCaptureNum;	// Capture Number
static uint32_t Timer3nCaptureTotalNum;	// Capture Totoal Number

static TIMER3n_CFG_Type TIMER3n_Config;
static TIMER3n_OUTPUT_CFG_Type TIMER3n_OutputA_Config;
static TIMER3n_OUTPUT_CFG_Type TIMER3n_OutputB_Config;
static TIMER3n_HiZ_CFG_Type TIMER3n_HiZ_Config;

static uint32_t MilliSec;

/*******************************************************************************
* Public Function
*******************************************************************************/
/*******************************************************************************
* @brief      SysTick interrupt handler @ 1000Hz(1ms)
* @param      None
* @return     None
*******************************************************************************/
void Systick_Handler_Capture(void) {
	if (MilliSec) {
		MilliSec--;
	}
}


/*******************************************************************************
* @brief      Timer30 interrupt handler
* @param      None
* @return     None
*******************************************************************************/
void TIMER30_IRQHandler_Capture(void) {
	static uint32_t Status;

	Status = HAL_TIMER3n_GetStatus(TIMER30);

	// TIMER30 C channel Match Interrupt
	if (Status & TIMER3n_STATUS_CMATCH_INT) {
		HAL_TIMER3n_ClearStatus(TIMER30, TIMER3n_STATUS_CMATCH_INT);
	}

	// TIMER30 B channel Match Interrupt
	if (Status & TIMER3n_STATUS_BMATCH_INT) {
		HAL_TIMER3n_ClearStatus(TIMER30, TIMER3n_STATUS_BMATCH_INT);
	}

	// TIMER30 A channel Match Interrupt
	if (Status & TIMER3n_STATUS_AMATCH_INT) {
		HAL_TIMER3n_ClearStatus(TIMER30, TIMER3n_STATUS_AMATCH_INT);
	}

	// Period Match Interrupt
	if (Status & TIMER3n_STATUS_PMATCH_INT) {
		if(Timer3nPeriodFlag == TRUE) {
			Timer3nPeriodFlag = FALSE;
			HAL_GPIO_ClearPin(PC, _BIT(0));
		} else {
			Timer3nPeriodFlag = TRUE;
			HAL_GPIO_SetPin(PC, _BIT(0));
		}
		HAL_TIMER3n_ClearStatus(TIMER30, TIMER3n_STATUS_PMATCH_INT);
	}

	// TIMER30 Bottom Match Interrupt
	if (Status & TIMER3n_STATUS_BOTTOM_INT) {
		if(Timer3nBottomFlag == TRUE) {
			Timer3nBottomFlag = FALSE;
//			HAL_GPIO_ClearPin(PC, _BIT(1));
		} else {
			Timer3nBottomFlag = TRUE;
//			HAL_GPIO_SetPin(PC, _BIT(1));
		}
		HAL_TIMER3n_ClearStatus(TIMER30, TIMER3n_STATUS_BOTTOM_INT);
	}

	// TIMER30 Capture Match Interrupt
	if (Status & TIMER3n_STATUS_CAPTURE_INT) {
		HAL_GPIO_SetPin(PC, _BIT(1));

//		if(Timer3nCaptureFlag == TRUE) {
//			Timer3nCaptureFlag = FALSE;
//			HAL_GPIO_ClearPin(PC, _BIT(1));
//		} else {
//			Timer3nCaptureFlag = TRUE;
//			HAL_GPIO_SetPin(PC, _BIT(1));
//		}

		Timer3nCaptureNum++;
		Timer3nCaptureTotalNum++;

		switch(Timer3nCaptureNum) {
			case 0:
				break;
			case 1:		// 1st Capture
				Timer3nCaptureCount[0] = HAL_TIMER3n_GetCaptureData(TIMER30);
				break;
			case 2:		// 2nd Capture
				// Capture counter value = The interval of capture timing
				Timer3nCaptureCount[1] = HAL_TIMER3n_GetCaptureData(TIMER30);
				Timer3nCaptureNum = 0;
				break;
		}
        Timer3nCaptureFlag = TRUE;
		HAL_TIMER3n_ClearStatus(TIMER30, TIMER3n_STATUS_CAPTURE_INT);
	}

	// TIMER30 High-Impedence Interrupt
	if (Status & TIMER3n_STATUS_HIZ_INT) {
		HAL_TIMER3n_ClearStatus(TIMER30, TIMER3n_STATUS_HIZ_INT);
	}
}


/**********************************************************************
 * @brief		TestCode TIMER3n Capture Test
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void TestCode_TIMER3n_Capture(void) {

	// Deinitialization of TIMER30
	HAL_TIMER3n_DeInit(TIMER30);

	// Set test output pin (PC0) - for Period Match
	HAL_GPIO_ConfigOutput(PC, 0, PCU_MODE_PUSH_PULL);
	HAL_GPIO_ConfigPullup(PC, 0, PCU_PUPD_DISABLE);
	HAL_GPIO_ClearPin(PC, _BIT(0));

	// Set test output pin (PC1) - for Capture Signal
	HAL_GPIO_ConfigOutput(PC, 1, PCU_MODE_PUSH_PULL);
	HAL_GPIO_ConfigPullup(PC, 1, PCU_PUPD_DISABLE);
	HAL_GPIO_ClearPin(PC, _BIT(1));

	// Set test output pin (PC2) - for Test Pulse Signal
	HAL_GPIO_ConfigOutput(PC, 2, PCU_MODE_PUSH_PULL);
	HAL_GPIO_ConfigPullup(PC, 2, PCU_PUPD_DISABLE);
	HAL_GPIO_ClearPin(PC, _BIT(2));

	// Set PWM pin
	// PWM30AA(PE0)
	HAL_GPIO_ConfigOutput(PE, 0, PCU_MODE_ALT_FUNC);
	HAL_GPIO_ConfigFunction(PE, 0, PCU_ALT_FUNCTION_1);
	// PWM30AB(PE1)
	HAL_GPIO_ConfigOutput(PE, 1, PCU_MODE_ALT_FUNC);
	HAL_GPIO_ConfigFunction(PE, 1, PCU_ALT_FUNCTION_1);
	// PWM30BA(PE2)
	HAL_GPIO_ConfigOutput(PE, 2, PCU_MODE_ALT_FUNC);
	HAL_GPIO_ConfigFunction(PE, 2, PCU_ALT_FUNCTION_1);
	// PWM30BB(PE3)
	HAL_GPIO_ConfigOutput(PE, 3, PCU_MODE_ALT_FUNC);
	HAL_GPIO_ConfigFunction(PE, 3, PCU_ALT_FUNCTION_1);
	// PWM30CA(PE4)
	HAL_GPIO_ConfigOutput(PE, 4, PCU_MODE_ALT_FUNC);
	HAL_GPIO_ConfigFunction(PE, 4, PCU_ALT_FUNCTION_1);
	// PWM30CB(PE5)
	HAL_GPIO_ConfigOutput(PE, 5, PCU_MODE_ALT_FUNC);
	HAL_GPIO_ConfigFunction(PE, 5, PCU_ALT_FUNCTION_1);

	// T30C pin config set if need
	HAL_GPIO_ConfigOutput((PORT_Type *)PF, 7, PCU_MODE_ALT_FUNC);
	HAL_GPIO_ConfigFunction((PORT_Type *)PF, 7, PCU_ALT_FUNCTION_1);

	// Initialize Motor PWM
	HAL_SCU_Timer30_ClockConfig(T30CLK_PCLK);

	TIMER3n_Config.ExtClock = DISABLE;
	TIMER3n_Config.Mode = TIMER3n_MODE_CAPTURE; // Set Capture Mode
	TIMER3n_Config.OutputMode = TIMER3n_OUTPUT_MODE_6_CHANNEL;
	TIMER3n_Config.DelayInsertion = DISABLE;
	TIMER3n_Config.DelayPosition = TIMER3n_DELAY_POSITION_FRONT;
	TIMER3n_Config.CapturePolarity = TIMER3n_CAPTURE_POLARITY_FALLING;	 // Setting Capture Timing (FALLING/RISING/BOTH EDGE)
//	TIMER3n_Config.CapturePolarity = TIMER3n_CAPTURE_POLARITY_RISING;	 // Setting Capture Timing
//	TIMER3n_Config.CapturePolarity = TIMER3n_CAPTURE_POLARITY_BOTH;	 // Setting Capture Timing
	TIMER3n_Config.ReloadTime = TIMER3n_DATA_RELOAD_TIME_WRITING;
	TIMER3n_Config.MatchIntInterval = TIMER3n_MATCH_INT_INTERVAL_1;
	TIMER3n_Config.PrescalerData = 31; // 32MHz / (31 + 1) = 1MHz = 1us
	TIMER3n_Config.PData = 65535; // TIMER30 Max. Period (65.355ms)
	TIMER3n_Config.AData = 0;
	TIMER3n_Config.BData = 0;
	TIMER3n_Config.CData = 0;
	TIMER3n_Config.DelayData = 0; // (0 + 1) / 1MHz = 1us
	HAL_TIMER3n_Init(TIMER30, &TIMER3n_Config);

	// Setting High Impedence control
	TIMER3n_OutputA_Config.StartLevel = TIMER3n_LEVEL_LOW;
	TIMER3n_OutputA_Config.Output_PWM30Ax = DISABLE;
	TIMER3n_OutputA_Config.Output_PWM30Bx = DISABLE;
	TIMER3n_OutputA_Config.Output_PWM30Cx = DISABLE;
	TIMER3n_OutputA_Config.DisableLevel_PWM30Ax = TIMER3n_LEVEL_LOW;
	TIMER3n_OutputA_Config.DisableLevel_PWM30Bx = TIMER3n_LEVEL_LOW;
	TIMER3n_OutputA_Config.DisableLevel_PWM30Cx = TIMER3n_LEVEL_LOW;
	TIMER3n_OutputB_Config.StartLevel = TIMER3n_LEVEL_LOW;
	TIMER3n_OutputB_Config.Output_PWM30Ax = DISABLE;
	TIMER3n_OutputB_Config.Output_PWM30Bx = DISABLE;
	TIMER3n_OutputB_Config.Output_PWM30Cx = DISABLE;
	TIMER3n_OutputB_Config.DisableLevel_PWM30Ax = TIMER3n_LEVEL_LOW;
	TIMER3n_OutputB_Config.DisableLevel_PWM30Bx = TIMER3n_LEVEL_LOW;
	TIMER3n_OutputB_Config.DisableLevel_PWM30Cx = TIMER3n_LEVEL_LOW;
	HAL_TIMER3n_ConfigureOutput(TIMER30, TIMER3n_OUTPUT_CTRL_ALL, &TIMER3n_OutputA_Config, &TIMER3n_OutputB_Config);

	TIMER3n_HiZ_Config.HiZ = DISABLE;
	TIMER3n_HiZ_Config.BLNK_Edge = TIMER3n_BLNK_EDGE_FALLING;
	HAL_TIMER3n_ConfigureHiZ(TIMER30, &TIMER3n_HiZ_Config);
	HAL_TIMER3n_SetHiZOutput(TIMER30, RESET);

	SysTick_Config(SystemCoreClock / 1000);	//  1ms systick timer
	HAL_TIMER3n_ConfigureInterrupt(TIMER30, (TIMER3n_INTERRUPT_Type)(TIMER3n_INTERRUPT_PMATCH_INT | TIMER3n_INTERRUPT_CAPTURE_INT), ENABLE);

	NVIC_SetPriority(TIMER30_IRQn, 3);
	NVIC_EnableIRQ(TIMER30_IRQn);
	__enable_irq();

	HAL_TIMER3n_ClearStatus(TIMER30, (TIMER3n_STATUS_Type)TIMER3n_INTFLAG_STATUS_CLEAR_MASK);	// Clear Interrupt Flag

	// Clear Counter value and Start Motor PWM
	HAL_TIMER3n_ClearCounter(TIMER30);
	HAL_TIMER3n_Start(TIMER30);

	cputs("\n\rWait Pulse!\n\r");

	// Make Pulse (PC2 --> T30C (PF7) pins are connected together.)
	HAL_GPIO_ClearPin(PC, _BIT(2));	// Low Singnal of PC2
	MilliSec = 10;		// 10ms
	while(MilliSec);
	HAL_GPIO_SetPin(PC, _BIT(2));		// High signal of PC2
	MilliSec = 10;		// 10ms
	while(MilliSec);
	HAL_GPIO_ClearPin(PC, _BIT(2));	// Low signal of PC2
	MilliSec = 10;		// 10ms
	while(MilliSec);
	HAL_GPIO_SetPin(PC, _BIT(2));		// High signal of PC2
	MilliSec = 10;		// 10ms
	while(MilliSec);
	HAL_GPIO_ClearPin(PC, _BIT(2));	// Low signal of PC2

	while(1) {
		if (Timer3nCaptureFlag) {
			HAL_GPIO_ClearPin(PC, _BIT(1));
			cputs("capture value! \n\r");
			cprintf("Capture Total Num = %d ", Timer3nCaptureTotalNum);
			// Captured at Falling Edge : 20000*0.000001 = 0.02 (20ms) will be captured
			// Captured at Rising Edge : 20000*0.000001 = 0.02 (20ms) will be captured
			// Captured at Both Edge : 10000*0.000001 = 0.01 (10ms) will be cpatured
			cprintf("Capture Val0 = %d ", Timer3nCaptureCount[0]);
			cprintf("Capture Val1 = %d\r\n", Timer3nCaptureCount[1]);

			Timer3nCaptureFlag=0;
			Timer3nCaptureTotalNum = 0;
			Timer3nCaptureCount[0] = 0;
			Timer3nCaptureCount[1] = 0;
//			break;
		}
	}
}

/* --------------------------------- End Of File ------------------------------ */
