/*****************************************************************************
* @file     TestCode_TIMER3n_ExtInputSignal.c
* @brief    Contains the external function prototypes for TestCode
* @author   AE Team, ABOV Semiconductor Co., Ltd.
* @version  V0.0.1
* @date     16. JUL. 2019
*
* Copyright(C) 2019, ABOV Semiconductor
* All rights reserved.
*
*
********************************************************************************
* DISCLAIMER
*
* THE PRESENT FIRMWARE WHICH IS FOR GUIDANCE ONLY AIMS AT PROVIDING CUSTOMERS
* WITH CODING INFORMATION REGARDING THEIR PRODUCTS IN ORDER FOR THEM TO SAVE
* TIME. AS A RESULT, ABOV SEMICONDUCTOR DISCLAIMS ALL LIABILITIES FROM ANY
* DIRECT, INDIRECT OR CONSEQUENTIAL DAMAGES WITH RESPECT TO ANY CLAIMS ARISING
* FROM THE CONTENT OF SUCH FIRMWARE AND/OR THE USE MADE BY CUSTOMERS OF THE
* CODING INFORMATION CONTAINED HEREIN IN CONNECTION WITH THEIR PRODUCTS.
*
*******************************************************************************/

/*******************************************************************************
* Pre-processor Definition & Macro
*******************************************************************************/
#include "TestCode_TIMER3n_ExtInputSignal.h"
#include "main_conf.h"

/*******************************************************************************
* Private Typedef
*******************************************************************************/

/*******************************************************************************
* Private Variable
*******************************************************************************/
static uint32_t MilliSec;

static Bool Timer3nPeriodFlag;
static Bool Timer1nMatchFlag;

static TIMER3n_CFG_Type TIMER3n_Config;
static TIMER3n_OUTPUT_CFG_Type TIMER3n_OutputA_Config;
static TIMER3n_OUTPUT_CFG_Type TIMER3n_OutputB_Config;
static TIMER3n_HiZ_CFG_Type TIMER3n_HiZ_Config;

static TIMER1n_CFG_Type TIMER1n_Config;

/*******************************************************************************
* Private Function Prototype
*******************************************************************************/

/*******************************************************************************
* Public Function
*******************************************************************************/
/*******************************************************************************
* @brief      SysTick interrupt handler @ 1000Hz(1ms)
* @param      None
* @return     None
*******************************************************************************/
void Systick_Handler_ExtInputSignal(void) {
	if (MilliSec) {
		MilliSec--;
	}
}

/*******************************************************************************
* @brief      Timer30 interrupt handler
* @param      None
* @return     None
*******************************************************************************/
void TIMER30_IRQHandler_ExtInputSignal(void) {
	static uint32_t Status;

	Status = HAL_TIMER3n_GetStatus(TIMER30);

	// TIMER30 C channel Match Interrupt
	if (Status & TIMER3n_STATUS_CMATCH_INT) {
		HAL_TIMER3n_ClearStatus(TIMER30, TIMER3n_STATUS_CMATCH_INT);
	}

	// TIMER30 B channel Match Interrupt
	if (Status & TIMER3n_STATUS_BMATCH_INT) {
		HAL_TIMER3n_ClearStatus(TIMER30, TIMER3n_STATUS_BMATCH_INT);
	}

	// TIMER30 A channel Match Interrupt
	if (Status & TIMER3n_STATUS_AMATCH_INT) {
		HAL_TIMER3n_ClearStatus(TIMER30, TIMER3n_STATUS_AMATCH_INT);
	}

	// Period Match Interrupt
	if (Status & TIMER3n_STATUS_PMATCH_INT) {
		if(Timer3nPeriodFlag == TRUE) {
			Timer3nPeriodFlag = FALSE;
			HAL_GPIO_ClearPin(PC, _BIT(0));
		} else {
			Timer3nPeriodFlag = TRUE;
			HAL_GPIO_SetPin(PC, _BIT(0));
		}
		HAL_TIMER3n_ClearStatus(TIMER30, TIMER3n_STATUS_PMATCH_INT);
	}

	// TIMER30 Bottom Match Interrupt
	if (Status & TIMER3n_STATUS_BOTTOM_INT) {
		HAL_TIMER3n_ClearStatus(TIMER30, TIMER3n_STATUS_BOTTOM_INT);
	}

	// TIMER30 Capture Match Interrupt
	if (Status & TIMER3n_STATUS_CAPTURE_INT) {
		HAL_TIMER3n_ClearStatus(TIMER30, TIMER3n_STATUS_CAPTURE_INT);
	}

	// TIMER30 High-Impedence Interrupt
	if (Status & TIMER3n_STATUS_HIZ_INT) {
		HAL_TIMER3n_ClearStatus(TIMER30, TIMER3n_STATUS_HIZ_INT); // Clear High-Impedance Interrupt
	}
}

/*******************************************************************************
* @brief      TIMER10_IRQHandler_ExtInputSignal
* @param      None
* @return     None
*******************************************************************************/
void TIMER10_IRQHandler_ExtInputSignal(void)
{
	if (HAL_TIMER1n_GetStatus(TIMER10) & TIMER1n_STATUS_MATCH_INT) {
		if(Timer1nMatchFlag == TRUE) {
			Timer1nMatchFlag = FALSE;
			HAL_GPIO_ClearPin(PC, _BIT(2));
		} else {
			Timer1nMatchFlag = TRUE;
			HAL_GPIO_SetPin(PC, _BIT(2));
		}
		HAL_TIMER1n_ClearStatus(TIMER10, TIMER1n_STATUS_MATCH_INT);
	}
}

/**********************************************************************
 * @brief		TestCode TIMER3n Extneral Input Signal Test
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void TestCode_TIMER3n_ExtInputSignal(void) {

	//////////////// TIMER1n ///////////////////////////////////////////////////////
	HAL_TIMER1n_DeInit(TIMER10); // Deinitialization of TIMER1n

	// TIMER1n Output port
	HAL_GPIO_ConfigOutput(PE, 6, PCU_MODE_ALT_FUNC);
	HAL_GPIO_ConfigFunction(PE, 6, PCU_ALT_FUNCTION_1);

	// Initialize and configure timer1n for ADC trigger
	HAL_SCU_MiscClockConfig(1, TIMER1n_TYPE, CLKSRC_MCLK, 32);
	HAL_SCU_Timer1n_ClockConfig(T1NCLK_MCCR1);

	TIMER1n_Config.StartSync = DISABLE; // Start Sync
	TIMER1n_Config.ClearSync = DISABLE; // Clear Sync
	TIMER1n_Config.ExtClock = DISABLE;
	TIMER1n_Config.MatchInterrupt = ENABLE; // Enable Match Interrupt
	TIMER1n_Config.CaptureInterrupt = DISABLE;
	TIMER1n_Config.PrescalerData = 0; // 1MHz / (0 + 1) = 1MHz -> 1us
	TIMER1n_Config.AData = 500 - 1; // 1us * 500 = 500us
	TIMER1n_Config.BData = 250 - 1;  // 1us * 0250 = 250us // When TIMER1n runs in REPEAT Mode, This value is applied to Timer1n Output Signal.
	HAL_TIMER1n_Init(TIMER10, TIMER1n_PERIODIC_MODE, &TIMER1n_Config);

	//////////////// TIMER30 ///////////////////////////////////////////////////////
	HAL_TIMER3n_DeInit(TIMER30); // Deinitialization of TIMER30

	// Set test output pin (PC0) - for TIMER30 Period Match
	HAL_GPIO_ConfigOutput(PC, 0, PCU_MODE_PUSH_PULL);
	HAL_GPIO_ConfigPullup(PC, 0, PCU_PUPD_DISABLE);
	HAL_GPIO_ClearPin(PC, _BIT(0));

	// Set PWM pin
	// PWM30AA(PE0)
	HAL_GPIO_ConfigOutput(PE, 0, PCU_MODE_ALT_FUNC);
	HAL_GPIO_ConfigFunction(PE, 0, PCU_ALT_FUNCTION_1);
	// PWM30AB(PE1)
	HAL_GPIO_ConfigOutput(PE, 1, PCU_MODE_ALT_FUNC);
	HAL_GPIO_ConfigFunction(PE, 1, PCU_ALT_FUNCTION_1);
	// PWM30BA(PE2)
	HAL_GPIO_ConfigOutput(PE, 2, PCU_MODE_ALT_FUNC);
	HAL_GPIO_ConfigFunction(PE, 2, PCU_ALT_FUNCTION_1);
	// PWM30BB(PE3)
	HAL_GPIO_ConfigOutput(PE, 3, PCU_MODE_ALT_FUNC);
	HAL_GPIO_ConfigFunction(PE, 3, PCU_ALT_FUNCTION_1);
	// PWM30CA(PE4)
	HAL_GPIO_ConfigOutput(PE, 4, PCU_MODE_ALT_FUNC);
	HAL_GPIO_ConfigFunction(PE, 4, PCU_ALT_FUNCTION_1);
	// PWM30CB(PE5)
	HAL_GPIO_ConfigOutput(PE, 5, PCU_MODE_ALT_FUNC);
	HAL_GPIO_ConfigFunction(PE, 5, PCU_ALT_FUNCTION_1);

	// EC30(PF6) (TIMER30 External Clock Source Port)
	// PE6(T10O) connected to PF6(EC30) pin for TIMER30 External Clock Source
	HAL_GPIO_ConfigOutput((PORT_Type *)PF, 6, PCU_MODE_ALT_FUNC);
	HAL_GPIO_ConfigFunction((PORT_Type *)PF, 6, PCU_ALT_FUNCTION_1);

//	SCU_SetTimer30Clk(T30CLK_PCLK); // TIMER10 TIMER Clock is source clock of TIMER30

	TIMER3n_Config.ExtClock = ENABLE;
	TIMER3n_Config.Mode = TIMER3n_MODE_INTERVAL; // Set TIMER30 Operationg Mode
	TIMER3n_Config.OutputMode = TIMER3n_OUTPUT_MODE_6_CHANNEL;
	TIMER3n_Config.DelayInsertion = DISABLE;
	TIMER3n_Config.DelayPosition = TIMER3n_DELAY_POSITION_FRONT;
	TIMER3n_Config.ReloadTime = TIMER3n_DATA_RELOAD_TIME_MATCH;
	TIMER3n_Config.MatchIntInterval = TIMER3n_MATCH_INT_INTERVAL_1;
	TIMER3n_Config.PrescalerData = 0; // 1kHz/(0 + 1) = 1kHz
	TIMER3n_Config.PData = 50; // 50ms * 1ms = 50ms
	TIMER3n_Config.AData = 0; // 30.0%
	TIMER3n_Config.BData = 0; // 50.0%
	TIMER3n_Config.CData = 0; // 70.0%
	TIMER3n_Config.DelayData = 0; // (0 + 1) / 1MHz = 1us
	HAL_TIMER3n_Init(TIMER30, &TIMER3n_Config);

	// Setting High Impedence control
	TIMER3n_OutputA_Config.StartLevel = TIMER3n_LEVEL_LOW;
	TIMER3n_OutputA_Config.Output_PWM30Ax = DISABLE;
	TIMER3n_OutputA_Config.Output_PWM30Bx = DISABLE;
	TIMER3n_OutputA_Config.Output_PWM30Cx = ENABLE;
	TIMER3n_OutputA_Config.DisableLevel_PWM30Ax = TIMER3n_LEVEL_LOW;
	TIMER3n_OutputA_Config.DisableLevel_PWM30Bx = TIMER3n_LEVEL_LOW;
	TIMER3n_OutputA_Config.DisableLevel_PWM30Cx = TIMER3n_LEVEL_LOW;
	TIMER3n_OutputB_Config.StartLevel = TIMER3n_LEVEL_LOW;
	TIMER3n_OutputB_Config.Output_PWM30Ax = DISABLE;
	TIMER3n_OutputB_Config.Output_PWM30Bx = DISABLE;
	TIMER3n_OutputB_Config.Output_PWM30Cx = DISABLE;
	TIMER3n_OutputB_Config.DisableLevel_PWM30Ax = TIMER3n_LEVEL_HIGH;
	TIMER3n_OutputB_Config.DisableLevel_PWM30Bx = TIMER3n_LEVEL_HIGH;
	TIMER3n_OutputB_Config.DisableLevel_PWM30Cx = TIMER3n_LEVEL_HIGH;
	HAL_TIMER3n_ConfigureOutput(TIMER30, TIMER3n_OUTPUT_CTRL_ALL, &TIMER3n_OutputA_Config, &TIMER3n_OutputB_Config);

	TIMER3n_HiZ_Config.HiZ = DISABLE;
	TIMER3n_HiZ_Config.BLNK_Edge = TIMER3n_BLNK_EDGE_FALLING;
	HAL_TIMER3n_ConfigureHiZ(TIMER30, &TIMER3n_HiZ_Config);

	// Systick interrupt
	SysTick_Config(SystemCoreClock / 1000);	//  1ms systick timer

	// TIMER1n Interrupt
	NVIC_SetPriority(TIMER10_IRQn, 3);
	NVIC_EnableIRQ(TIMER10_IRQn);

	// TIMER30 Interrupt
	HAL_TIMER3n_ConfigureInterrupt(TIMER30, TIMER3n_INTERRUPT_PMATCH_INT, ENABLE);
	NVIC_SetPriority(TIMER30_IRQn, 3);
	NVIC_EnableIRQ(TIMER30_IRQn);

	// Enable Interrupt Vector
	__enable_irq();

	// Clear Counter value, Flag Status of TIMER1n
	HAL_TIMER1n_ClearStatus(TIMER10, TIMER1n_STATUS_MATCH_INT);
	HAL_TIMER1n_ClearCounter(TIMER10);

	// Clear Counter value, Flag Status of TIMER30
	HAL_TIMER3n_ClearStatus(TIMER30, (TIMER3n_STATUS_Type)TIMER3n_INTFLAG_STATUS_CLEAR_MASK);
	HAL_TIMER3n_ClearCounter(TIMER30);

	HAL_TIMER1n_Start(TIMER10);	// Run TIMER1n
	cputs("TIMER10 Runs...\r\n");

	MilliSec = 200;
	while(MilliSec);

	HAL_TIMER3n_Start(TIMER30);	// Run TIMER30
	cputs("TIMER30 Runs with TIMER10 clock source\r\n");

	while(1);
}

/* --------------------------------- End Of File ------------------------------ */
