/*****************************************************************************
* @file     TestCode_TIMER3n_Nmi.c
* @brief    Contains the external function prototypes for TestCode
* @author   AE Team, ABOV Semiconductor Co., Ltd.
* @version  V0.0.1
* @date     16. JUL. 2019
*
* Copyright(C) 2019, ABOV Semiconductor
* All rights reserved.
*
*
********************************************************************************
* DISCLAIMER
*
* THE PRESENT FIRMWARE WHICH IS FOR GUIDANCE ONLY AIMS AT PROVIDING CUSTOMERS
* WITH CODING INFORMATION REGARDING THEIR PRODUCTS IN ORDER FOR THEM TO SAVE
* TIME. AS A RESULT, ABOV SEMICONDUCTOR DISCLAIMS ALL LIABILITIES FROM ANY
* DIRECT, INDIRECT OR CONSEQUENTIAL DAMAGES WITH RESPECT TO ANY CLAIMS ARISING
* FROM THE CONTENT OF SUCH FIRMWARE AND/OR THE USE MADE BY CUSTOMERS OF THE
* CODING INFORMATION CONTAINED HEREIN IN CONNECTION WITH THEIR PRODUCTS.
*
*******************************************************************************/

/*******************************************************************************
* Pre-processor Definition & Macro
*******************************************************************************/
#include "TestCode_TIMER3n_Nmi.h"
#include "main_conf.h"

/*******************************************************************************
* Private Typedef
*******************************************************************************/

/*******************************************************************************
* Private Variable
*******************************************************************************/
static uint32_t MilliSec;

static Bool Timer3nPeriodFlag;

static TIMER3n_CFG_Type TIMER3n_Config;
static TIMER3n_OUTPUT_CFG_Type TIMER3n_OutputA_Config;
static TIMER3n_OUTPUT_CFG_Type TIMER3n_OutputB_Config;
static TIMER3n_HiZ_CFG_Type TIMER3n_HiZ_Config;

/*******************************************************************************
* Private Function Prototype
*******************************************************************************/

/*******************************************************************************
* Public Function
*******************************************************************************/
/*******************************************************************************
* @brief      SysTick interrupt handler @ 1000Hz(1ms)
* @param      None
* @return     None
*******************************************************************************/
void Systick_Handler_Nmi(void) {
	if (MilliSec) {
		MilliSec--;
	}
}

/*******************************************************************************
* @brief      Interrupt Handler of NMI
* @param      None
* @return     None
*******************************************************************************/
void NMI_Handler_Nmi(void) {
	uint32_t NmiStatus;
	uint32_t T30Status;

	NmiStatus = HAL_SCU_GetNMIStatus();	// Get NMI interrupt event
	_DBG("NMI - T30 NMI Interrupt Occurred!\r\n");

	if (NmiStatus & SCU_NMI_T30INT_STATUS) {
		T30Status = HAL_TIMER3n_GetStatus(TIMER30); // Get Timer30 interrupt event flag

		if ((T30Status & TIMER3n_STATUS_HIZ_INT) == TIMER3n_STATUS_HIZ_INT) {
			HAL_TIMER3n_ClearStatus(TIMER30, TIMER3n_STATUS_HIZ_INT); // Clear Hi-Z Interrupt flag of TIMER30
		}
	}
}

/*******************************************************************************
* @brief      Timer30 interrupt handler
* @param      None
* @return     None
*******************************************************************************/
void TIMER30_IRQHandler_Nmi(void) {
	static uint32_t Status;

	Status = HAL_TIMER3n_GetStatus(TIMER30);

	// TIMER30 C channel Match Interrupt
	if (Status & TIMER3n_STATUS_CMATCH_INT) {
		HAL_TIMER3n_ClearStatus(TIMER30, TIMER3n_STATUS_CMATCH_INT);
	}

	// TIMER30 B channel Match Interrupt
	if (Status & TIMER3n_STATUS_BMATCH_INT) {
		HAL_TIMER3n_ClearStatus(TIMER30, TIMER3n_STATUS_BMATCH_INT);
	}

	// TIMER30 A channel Match Interrupt
	if (Status & TIMER3n_STATUS_AMATCH_INT) {
		HAL_TIMER3n_ClearStatus(TIMER30, TIMER3n_STATUS_AMATCH_INT);
	}

	// Period Match Interrupt
	if (Status & TIMER3n_STATUS_PMATCH_INT) {
		if(Timer3nPeriodFlag == TRUE) {
			Timer3nPeriodFlag = FALSE;
			HAL_GPIO_ClearPin(PC, _BIT(0));
		} else {
			Timer3nPeriodFlag = TRUE;
			HAL_GPIO_SetPin(PC, _BIT(0));
		}
		HAL_TIMER3n_ClearStatus(TIMER30, TIMER3n_STATUS_PMATCH_INT);
	}

	// TIMER30 Bottom Match Interrupt
	if (Status & TIMER3n_STATUS_BOTTOM_INT) {
		HAL_TIMER3n_ClearStatus(TIMER30, TIMER3n_STATUS_BOTTOM_INT);
	}

	// TIMER30 Capture Match Interrupt
	if (Status & TIMER3n_STATUS_CAPTURE_INT) {
		HAL_TIMER3n_ClearStatus(TIMER30, TIMER3n_STATUS_CAPTURE_INT);
	}

	// TIMER30 High-Impedence Interrupt
	if (Status & TIMER3n_STATUS_HIZ_INT) {
		HAL_TIMER3n_ClearStatus(TIMER30, TIMER3n_STATUS_HIZ_INT); // Clear High-Impedance Interrupt
	}
}

/**********************************************************************
 * @brief		TestCode TIMER3n Nmi Interrupt Test
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void TestCode_TIMER3n_Nmi(void) {

 	// Deinitialization of TIMER30
	HAL_TIMER3n_DeInit(TIMER30);

	// Set test output pin (PC0)
	HAL_GPIO_ConfigOutput(PC, 0, PCU_MODE_PUSH_PULL);
	HAL_GPIO_ConfigPullup(PC, 0, PCU_PUPD_DISABLE);
	HAL_GPIO_ClearPin(PC, _BIT(0));

	// Set Hi-Z Interrupt pin (PC1)
	HAL_GPIO_ConfigOutput(PC, 1, PCU_MODE_PUSH_PULL);
	HAL_GPIO_ConfigPullup(PC, 1, PCU_PUPD_DISABLE);
	HAL_GPIO_ClearPin(PC, _BIT(1));

	// Set test output pin (PC2) - for Test Pulse Signal
	HAL_GPIO_ConfigOutput(PC, 2, PCU_MODE_PUSH_PULL);
	HAL_GPIO_ConfigPullup(PC, 2, PCU_PUPD_DISABLE);
	HAL_GPIO_ClearPin(PC, _BIT(2));

	// Set BLNK Pin (Open-Drain) (PF5)
	HAL_GPIO_ConfigOutput((PORT_Type *)PF, 5, PCU_MODE_ALT_FUNC);
	HAL_GPIO_ConfigFunction((PORT_Type *)PF, 5, PCU_ALT_FUNCTION_1);

	// Set PWM pin
	// PWM30AA(PE0)
	HAL_GPIO_ConfigOutput(PE, 0, PCU_MODE_ALT_FUNC);
	HAL_GPIO_ConfigFunction(PE, 0, PCU_ALT_FUNCTION_1);
	// PWM30AB(PE1)
	HAL_GPIO_ConfigOutput(PE, 1, PCU_MODE_ALT_FUNC);
	HAL_GPIO_ConfigFunction(PE, 1, PCU_ALT_FUNCTION_1);
	// PWM30BA(PE2)
	HAL_GPIO_ConfigOutput(PE, 2, PCU_MODE_ALT_FUNC);
	HAL_GPIO_ConfigFunction(PE, 2, PCU_ALT_FUNCTION_1);
	// PWM30BB(PE3)
	HAL_GPIO_ConfigOutput(PE, 3, PCU_MODE_ALT_FUNC);
	HAL_GPIO_ConfigFunction(PE, 3, PCU_ALT_FUNCTION_1);
	// PWM30CA(PE4)
	HAL_GPIO_ConfigOutput(PE, 4, PCU_MODE_ALT_FUNC);
	HAL_GPIO_ConfigFunction(PE, 4, PCU_ALT_FUNCTION_1);
	// PWM30CB(PE5)
	HAL_GPIO_ConfigOutput(PE, 5, PCU_MODE_ALT_FUNC);
	HAL_GPIO_ConfigFunction(PE, 5, PCU_ALT_FUNCTION_1);

	// Initialize Motor PWM
	HAL_SCU_Timer30_ClockConfig(T30CLK_PCLK);

	TIMER3n_Config.ExtClock = DISABLE;
	TIMER3n_Config.Mode = TIMER3n_MODE_INTERVAL;
	TIMER3n_Config.OutputMode = TIMER3n_OUTPUT_MODE_6_CHANNEL;
	TIMER3n_Config.DelayInsertion = DISABLE;
	TIMER3n_Config.DelayPosition = TIMER3n_DELAY_POSITION_FRONT;
	TIMER3n_Config.ReloadTime = TIMER3n_DATA_RELOAD_TIME_WRITING;
	TIMER3n_Config.MatchIntInterval = TIMER3n_MATCH_INT_INTERVAL_1;
	TIMER3n_Config.PData = 100; // 1kHz, 1ms(Interval)
	TIMER3n_Config.AData = 75; // 75.0%
	TIMER3n_Config.BData = 66; // 66.7%
	TIMER3n_Config.CData = 50; // 50.0%
	TIMER3n_Config.PrescalerData = 0; // 32MHz / (31 + 1) = 1MHz
	TIMER3n_Config.DelayData = 0; // (0 + 1) / 32MHz = 2us
	HAL_TIMER3n_Init(TIMER30, &TIMER3n_Config);

	TIMER3n_OutputA_Config.StartLevel = TIMER3n_LEVEL_LOW;
	TIMER3n_OutputA_Config.Output_PWM30Ax = DISABLE;
	TIMER3n_OutputA_Config.Output_PWM30Bx = DISABLE;
	TIMER3n_OutputA_Config.Output_PWM30Cx = DISABLE;
	TIMER3n_OutputA_Config.DisableLevel_PWM30Ax = TIMER3n_LEVEL_LOW;
	TIMER3n_OutputA_Config.DisableLevel_PWM30Bx = TIMER3n_LEVEL_LOW;
	TIMER3n_OutputA_Config.DisableLevel_PWM30Cx = TIMER3n_LEVEL_LOW;
	TIMER3n_OutputB_Config.StartLevel = TIMER3n_LEVEL_LOW;
	TIMER3n_OutputB_Config.Output_PWM30Ax = DISABLE;
	TIMER3n_OutputB_Config.Output_PWM30Bx = DISABLE;
	TIMER3n_OutputB_Config.Output_PWM30Cx = DISABLE;
	TIMER3n_OutputB_Config.DisableLevel_PWM30Ax = TIMER3n_LEVEL_LOW;
	TIMER3n_OutputB_Config.DisableLevel_PWM30Bx = TIMER3n_LEVEL_LOW;
	TIMER3n_OutputB_Config.DisableLevel_PWM30Cx = TIMER3n_LEVEL_LOW;
	HAL_TIMER3n_ConfigureOutput(TIMER30, TIMER3n_OUTPUT_CTRL_ALL, &TIMER3n_OutputA_Config, &TIMER3n_OutputB_Config);

	TIMER3n_HiZ_Config.HiZ = ENABLE; // Enable High-Impedance PWM30Ax, PWM30Bx Output
	TIMER3n_HiZ_Config.BLNK_Edge = TIMER3n_BLNK_EDGE_FALLING;
	HAL_TIMER3n_ConfigureHiZ(TIMER30, &TIMER3n_HiZ_Config);
	HAL_TIMER3n_SetHiZOutput(TIMER30, RESET); // High-Impedance Output (PWMAx, PWMBx) Clear bit

	SysTick_Config(SystemCoreClock / 1000);	//  1ms systick timer

	HAL_SCU_SetNMI(SCU_NMI_T30INT_ENABLE); // Set TIMER30 Period Match and High Impedance Interrupt Enable
	HAL_TIMER3n_ConfigureInterrupt(TIMER30, (TIMER3n_INTERRUPT_Type) (TIMER3n_INTERRUPT_HIZ_INT) , ENABLE);	// Enable HIZ Match Interrupt

	NVIC_SetPriority(TIMER30_IRQn, 3);
	NVIC_EnableIRQ(TIMER30_IRQn);
	__enable_irq();

	// Clear Counter value and Start Motor PWM
	HAL_TIMER3n_ClearCounter(TIMER30);
	HAL_TIMER3n_Start(TIMER30);

	cputs("\n\rWait Pulse!\n\r");

	// Make Pulse (PC2 --> BLNK (PF5) pins are connected together.)
	HAL_GPIO_ClearPin(PC, _BIT(2));	// Low Singnal of PC2
	MilliSec = 10;		// 10ms
	while(MilliSec);
	HAL_GPIO_SetPin(PC, _BIT(2));		// High signal of PC2
	MilliSec = 10;		// 10ms
	while(MilliSec);
	HAL_GPIO_ClearPin(PC, _BIT(2));	// Low signal of PC2
	MilliSec = 10;		// 10ms
	while(MilliSec);
	HAL_GPIO_SetPin(PC, _BIT(2));		// High signal of PC2
	MilliSec = 10;		// 10ms
	while(MilliSec);
	HAL_GPIO_ClearPin(PC, _BIT(2));	// Low signal of PC2

	while(1);
}

/* --------------------------------- End Of File ------------------------------ */
