/*****************************************************************************
* @file     TestCode_TIMER3n_Polling.c
* @brief    Contains the external function prototypes for TestCode
* @author   AE Team, ABOV Semiconductor Co., Ltd.
* @version  V0.0.1
* @date     16. JUL. 2019
*
* Copyright(C) 2019, ABOV Semiconductor
* All rights reserved.
*
*
********************************************************************************
* DISCLAIMER
*
* THE PRESENT FIRMWARE WHICH IS FOR GUIDANCE ONLY AIMS AT PROVIDING CUSTOMERS
* WITH CODING INFORMATION REGARDING THEIR PRODUCTS IN ORDER FOR THEM TO SAVE
* TIME. AS A RESULT, ABOV SEMICONDUCTOR DISCLAIMS ALL LIABILITIES FROM ANY
* DIRECT, INDIRECT OR CONSEQUENTIAL DAMAGES WITH RESPECT TO ANY CLAIMS ARISING
* FROM THE CONTENT OF SUCH FIRMWARE AND/OR THE USE MADE BY CUSTOMERS OF THE
* CODING INFORMATION CONTAINED HEREIN IN CONNECTION WITH THEIR PRODUCTS.
*
*******************************************************************************/

/*******************************************************************************
* Pre-processor Definition & Macro
*******************************************************************************/
#include "TestCode_TIMER3n_Polling.h"
#include "main_conf.h"


/*******************************************************************************
* Private Typedef
*******************************************************************************/

/*******************************************************************************
* Private Variable
*******************************************************************************/

/*******************************************************************************
* Private Function Prototype
*******************************************************************************/
static Bool Timer3nPeriodFlag;
static TIMER3n_CFG_Type TIMER3n_Config;
static TIMER3n_OUTPUT_CFG_Type TIMER3n_OutputA_Config;
static TIMER3n_OUTPUT_CFG_Type TIMER3n_OutputB_Config;
/*******************************************************************************
* Public Function
*******************************************************************************/

/**********************************************************************
 * @brief		TestCode TIMER3n Polling Test
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void TestCode_TIMER3n_Polling(void) {
	uint32_t Status;

	// Deinitialization of TIMER30
	HAL_TIMER3n_DeInit(TIMER30);

	// Set test output pin (PE2)
	HAL_GPIO_ConfigOutput(PE, 2, PCU_MODE_PUSH_PULL);
	HAL_GPIO_ConfigPullup(PE, 2, PCU_PUPD_DISABLE);
	HAL_GPIO_ClearPin(PE, _BIT(2));

	// Set PWM pin
	// PWM30AA(PE0)
	HAL_GPIO_ConfigOutput(PE, 0, PCU_MODE_ALT_FUNC);
	HAL_GPIO_ConfigFunction(PE, 0, PCU_ALT_FUNCTION_1);
	// PWM30AB(PE1)
	HAL_GPIO_ConfigOutput(PE, 1, PCU_MODE_ALT_FUNC);
	HAL_GPIO_ConfigFunction(PE, 1, PCU_ALT_FUNCTION_1);
	// PWM30BA(PE2)
	HAL_GPIO_ConfigOutput(PE, 2, PCU_MODE_ALT_FUNC);
	HAL_GPIO_ConfigFunction(PE, 2, PCU_ALT_FUNCTION_1);
	// PWM30BB(PE3)
	HAL_GPIO_ConfigOutput(PE, 3, PCU_MODE_ALT_FUNC);
	HAL_GPIO_ConfigFunction(PE, 3, PCU_ALT_FUNCTION_1);
	// PWM30CA(PE4)
	HAL_GPIO_ConfigOutput(PE, 4, PCU_MODE_ALT_FUNC);
	HAL_GPIO_ConfigFunction(PE, 4, PCU_ALT_FUNCTION_1);
	// PWM30CB(PE5)
	HAL_GPIO_ConfigOutput(PE, 5, PCU_MODE_ALT_FUNC);
	HAL_GPIO_ConfigFunction(PE, 5, PCU_ALT_FUNCTION_1);

	// Initialize Motor PWM
    #if 1
	HAL_SCU_Timer30_ClockConfig(T30CLK_PCLK); // PCLK 32MHz
    #else
//  HAL_SCU_HSI_ClockConfig(HSI_EN);
    HAL_SCU_MiscClockConfig(2, TIMER3n_TYPE, CLKSRC_HSI, 1);   // HSI 32MHz
	HAL_SCU_Timer30_ClockConfig(T30CLK_MCCR2);
    #endif

	TIMER3n_Config.ExtClock = DISABLE;
//	TIMER3n_Config.Mode = TIMER3n_MODE_INTERVAL;
	TIMER3n_Config.Mode = TIMER3n_MODE_BACKTOBACK;
	TIMER3n_Config.OutputMode = TIMER3n_OUTPUT_MODE_6_CHANNEL;
	TIMER3n_Config.DelayInsertion = DISABLE;
	TIMER3n_Config.DelayPosition = TIMER3n_DELAY_POSITION_FRONT;
	TIMER3n_Config.ReloadTime = TIMER3n_DATA_RELOAD_TIME_WRITING;
	TIMER3n_Config.MatchIntInterval = TIMER3n_MATCH_INT_INTERVAL_1;
	TIMER3n_Config.PData = 1000; // 1kHz, 1ms(Back to Back)
	TIMER3n_Config.AData = 300; // 30.0%
	TIMER3n_Config.BData = 500; // 50.0%
	TIMER3n_Config.CData = 700; // 70.0%
	TIMER3n_Config.PrescalerData = 31; // 32MHz / (31 + 1) = 1MHz
	TIMER3n_Config.DelayData = 0; // (0 + 1) / 1MHz = 1us
	HAL_TIMER3n_Init(TIMER30, &TIMER3n_Config);

	TIMER3n_OutputA_Config.StartLevel = TIMER3n_LEVEL_LOW;
//	TIMER3n_OutputA_Config.StartLevel = TIMER3n_LEVEL_HIGH;
	TIMER3n_OutputA_Config.Output_PWM30Ax = ENABLE;
	TIMER3n_OutputA_Config.Output_PWM30Bx = ENABLE;
	TIMER3n_OutputA_Config.Output_PWM30Cx = ENABLE;
	TIMER3n_OutputA_Config.DisableLevel_PWM30Ax = TIMER3n_LEVEL_LOW;
	TIMER3n_OutputA_Config.DisableLevel_PWM30Bx = TIMER3n_LEVEL_LOW;
	TIMER3n_OutputA_Config.DisableLevel_PWM30Cx = TIMER3n_LEVEL_LOW;
	TIMER3n_OutputB_Config.StartLevel = TIMER3n_LEVEL_LOW;
	TIMER3n_OutputB_Config.Output_PWM30Ax = DISABLE;
	TIMER3n_OutputB_Config.Output_PWM30Bx = DISABLE;
	TIMER3n_OutputB_Config.Output_PWM30Cx = DISABLE;
	TIMER3n_OutputB_Config.DisableLevel_PWM30Ax = TIMER3n_LEVEL_LOW;
	TIMER3n_OutputB_Config.DisableLevel_PWM30Bx = TIMER3n_LEVEL_LOW;
	TIMER3n_OutputB_Config.DisableLevel_PWM30Cx = TIMER3n_LEVEL_LOW;
	HAL_TIMER3n_ConfigureOutput(TIMER30, TIMER3n_OUTPUT_CTRL_ALL, &TIMER3n_OutputA_Config, &TIMER3n_OutputB_Config);

	// Clear Counter value and Start Motor PWM
	HAL_TIMER3n_ClearCounter(TIMER30);
	HAL_TIMER3n_Start(TIMER30);
	cputs("TIMER30 Start\r\n");

	while(1) {
		Status = HAL_TIMER3n_GetStatus(TIMER30); // Get Status Flag
		if (Status & TIMER3n_STATUS_PMATCH_INT) {
			if(Timer3nPeriodFlag == TRUE) {
				Timer3nPeriodFlag = FALSE;
				HAL_GPIO_ClearPin(PE, _BIT(2));
			} else {
				Timer3nPeriodFlag = TRUE;
				HAL_GPIO_SetPin(PE, _BIT(2));
			}
			HAL_TIMER3n_ClearStatus(TIMER30, (TIMER3n_STATUS_Type)Status);
		}
	} /* while */
}

/* --------------------------------- End Of File ------------------------------ */
