/*****************************************************************************
* @file     TestCode_TIMER3n_Sync.c
* @brief    Contains the external function prototypes for TestCode
* @author   AE Team, ABOV Semiconductor Co., Ltd.
* @version  V0.0.1
* @date     16. JUL. 2019
*
* Copyright(C) 2019, ABOV Semiconductor
* All rights reserved.
*
*
********************************************************************************
* DISCLAIMER
*
* THE PRESENT FIRMWARE WHICH IS FOR GUIDANCE ONLY AIMS AT PROVIDING CUSTOMERS
* WITH CODING INFORMATION REGARDING THEIR PRODUCTS IN ORDER FOR THEM TO SAVE
* TIME. AS A RESULT, ABOV SEMICONDUCTOR DISCLAIMS ALL LIABILITIES FROM ANY
* DIRECT, INDIRECT OR CONSEQUENTIAL DAMAGES WITH RESPECT TO ANY CLAIMS ARISING
* FROM THE CONTENT OF SUCH FIRMWARE AND/OR THE USE MADE BY CUSTOMERS OF THE
* CODING INFORMATION CONTAINED HEREIN IN CONNECTION WITH THEIR PRODUCTS.
*
*******************************************************************************/

/*******************************************************************************
* Pre-processor Definition & Macro
*******************************************************************************/
#include "TestCode_TIMER3n_Sync.h"
#include "main_conf.h"

#define TEST_TIMER1n	(TIMER1n_IP_INDEX_TIMER10)
//#define TEST_TIMER1n	(TIMER1n_IP_INDEX_TIMER11)
//#define TEST_TIMER1n	(TIMER1n_IP_INDEX_TIMER12)
//#define TEST_TIMER1n	(TIMER1n_IP_INDEX_TIMER13)
//#define TEST_TIMER1n	(TIMER1n_IP_INDEX_TIMER14)
//#define TEST_TIMER1n	(TIMER1n_IP_INDEX_TIMER15)
//#define TEST_TIMER1n	(TIMER1n_IP_INDEX_TIMER16)

/*******************************************************************************
* Private Typedef
*******************************************************************************/
// Timer1n PWM demo configuration structure definition
typedef struct {
	// base address
	TIMER1n_Type *pTIMER1x;
 	// Output port
	PORT_Type *pOutputPort_Group;
	uint32_t OutputPort_Number;
	PCU_ALT_FUNCTION_Type OutputPort_AltFunc;
	IRQn_Type IRQ_Type;
	uint8_t TimerName[8];
} TIMER1n_PORT_Type;

/*******************************************************************************
* Private Variable
*******************************************************************************/
static uint32_t MilliSec;

static Bool Timer3nPeriodFlag;
static Bool Timer1nMatchFlag;
static uint32_t Timer1nCount;
static uint32_t Timer30Count;

static TIMER3n_CFG_Type TIMER3n_Config;
static TIMER3n_OUTPUT_CFG_Type TIMER3n_OutputA_Config;
static TIMER3n_OUTPUT_CFG_Type TIMER3n_OutputB_Config;
static TIMER3n_HiZ_CFG_Type TIMER3n_HiZ_Config;
static TIMER3n_ADC_TRIGGER_CFG_Type TIMER3n_AdcTrigger_Config;
static TIMER1n_CFG_Type TIMER1n_Config;
static const TIMER1n_PORT_Type TIMER1n_PORT_CFG[TIMER1n_IP_INDEX_MAX] = {
	{TIMER10,              PE,  6, PCU_ALT_FUNCTION_1, TIMER10_IRQn, "TIMER10"},
	{TIMER11,              PE,  7, PCU_ALT_FUNCTION_1, TIMER11_IRQn, "TIMER11"},
	{TIMER12,              PA,  5, PCU_ALT_FUNCTION_1, TIMER12_IRQn, "TIMER12"},
	{TIMER13, (PORT_Type *)PF, 10, PCU_ALT_FUNCTION_1, TIMER13_IRQn, "TIMER13"},
	{TIMER14, (PORT_Type *)PF, 11, PCU_ALT_FUNCTION_1, TIMER14_IRQn, "TIMER14"},
	{TIMER15,              PB,  8, PCU_ALT_FUNCTION_1, TIMER15_IRQn, "TIMER15"},
	{TIMER16,              PB,  9, PCU_ALT_FUNCTION_1, TIMER16_IRQn, "TIMER16"}
};
#ifdef TEST_TIMER1n
static const TIMER1n_PORT_Type * pConfig = &(TIMER1n_PORT_CFG[TEST_TIMER1n]);
#endif
/*******************************************************************************
* Private Function Prototype
*******************************************************************************/

/*******************************************************************************
* Public Function
*******************************************************************************/
/*******************************************************************************
* @brief      SysTick interrupt handler @ 1000Hz(1ms)
* @param      None
* @return     None
*******************************************************************************/
void Systick_Handler_Sync(void) {
	if (MilliSec) {
		MilliSec--;
	}
}

/*******************************************************************************
* @brief      Timer30 interrupt handler
* @param      None
* @return     None
*******************************************************************************/
void TIMER30_IRQHandler_Sync(void)
{
	static uint32_t Status;

	Status = HAL_TIMER3n_GetStatus(TIMER30);

	// TIMER30 C channel Match Interrupt
	if (Status & TIMER3n_STATUS_CMATCH_INT) {
		HAL_TIMER3n_ClearStatus(TIMER30, TIMER3n_STATUS_CMATCH_INT);
	}

	// TIMER30 B channel Match Interrupt
	if (Status & TIMER3n_STATUS_BMATCH_INT) {
		HAL_TIMER3n_ClearStatus(TIMER30, TIMER3n_STATUS_BMATCH_INT);
	}

	// TIMER30 A channel Match Interrupt
	if (Status & TIMER3n_STATUS_AMATCH_INT) {
		HAL_TIMER3n_ClearStatus(TIMER30, TIMER3n_STATUS_AMATCH_INT);
	}

	// Period Match Interrupt
	if (Status & TIMER3n_STATUS_PMATCH_INT) {
		if(Timer3nPeriodFlag == TRUE) {
			Timer3nPeriodFlag = FALSE;
			HAL_GPIO_ClearPin(PC, _BIT(0));
		} else {
			Timer3nPeriodFlag = TRUE;
			HAL_GPIO_SetPin(PC, _BIT(0));
		}
		Timer30Count++;
		if (Timer30Count >= 10) {
			HAL_TIMER3n_Stop(TIMER30);
			HAL_GPIO_ClearPin(PC, _BIT(2));
		}
		HAL_TIMER3n_ClearStatus(TIMER30, TIMER3n_STATUS_PMATCH_INT);
	}

	// TIMER30 Bottom Match Interrupt
	if (Status & TIMER3n_STATUS_BOTTOM_INT) {
		HAL_TIMER3n_ClearStatus(TIMER30, TIMER3n_STATUS_BOTTOM_INT);
	}

	// TIMER30 High-Impedence Interrupt
	if (Status & TIMER3n_STATUS_HIZ_INT) {
		HAL_TIMER3n_ClearStatus(TIMER30, TIMER3n_STATUS_HIZ_INT);
	}
}

/*******************************************************************************
* @brief      Timer1n IRQ Handler
* @param      None
* @return     None
*******************************************************************************/
static void Main_Timer1nIRQHandler(TIMER1n_Type *pTIMER1x){
	if (HAL_TIMER1n_GetStatus(pTIMER1x) & TIMER1n_STATUS_MATCH_INT) {
		if(Timer1nMatchFlag == TRUE) {
			Timer1nMatchFlag = FALSE;
			HAL_GPIO_ClearPin(PC, _BIT(1));
		} else {
			Timer1nMatchFlag = TRUE;
			HAL_GPIO_SetPin(PC, _BIT(1));
		}

		Timer1nCount++;
		if (Timer1nCount >= 100) {	// When this count value is set, TIMER30 stops before the TIMER1n.
//		if (Timer1nCount >= 30) {	// When this count value is set, TIMER1n stops before the TIMER30.
			HAL_TIMER1n_Stop(pTIMER1x);
		}
		HAL_TIMER1n_ClearStatus(pTIMER1x, TIMER1n_STATUS_MATCH_INT);
	}
}
/*******************************************************************************
* @brief      Timer10 interrupt handler
* @param      None
* @return     None
*******************************************************************************/
void TIMER10_IRQHandler_Sync(void)
{
	Main_Timer1nIRQHandler(TIMER10);
}

/*******************************************************************************
* @brief      Timer11 interrupt handler
* @param      None
* @return     None
*******************************************************************************/
void TIMER11_IRQHandler_Sync(void)
{
	Main_Timer1nIRQHandler(TIMER11);
}

/*******************************************************************************
* @brief      Timer12 interrupt handler
* @param      None
* @return     None
*******************************************************************************/
void TIMER12_IRQHandler_Sync(void)
{
	Main_Timer1nIRQHandler(TIMER12);
}

/*******************************************************************************
* @brief      Timer13 interrupt handler
* @param      None
* @return     None
*******************************************************************************/
void TIMER13_IRQHandler_Sync(void)
{
	Main_Timer1nIRQHandler(TIMER13);
}

/*******************************************************************************
* @brief      Timer14 interrupt handler
* @param      None
* @return     None
*******************************************************************************/
void TIMER14_IRQHandler_Sync(void)
{
	Main_Timer1nIRQHandler(TIMER14);
}

/*******************************************************************************
* @brief      Timer15 interrupt handler
* @param      None
* @return     None
*******************************************************************************/
void TIMER15_IRQHandler_Sync(void)
{
	Main_Timer1nIRQHandler(TIMER15);
}

/*******************************************************************************
* @brief      Timer16 interrupt handler
* @param      None
* @return     None
*******************************************************************************/
void TIMER16_IRQHandler_Sync(void)
{
	Main_Timer1nIRQHandler(TIMER16);
}

/**********************************************************************
 * @brief		TestCode TIMER3n Sync with TIMER1n Test
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void TestCode_TIMER3n_Sync(void) {

	//////////////// TIMER1n ///////////////////////////////////////////////////////
	HAL_TIMER1n_DeInit(pConfig->pTIMER1x); // Deinitialization of TIMER1n

	// TIMER1n Output port
	HAL_GPIO_ConfigOutput(pConfig->pOutputPort_Group, pConfig->OutputPort_Number, PCU_MODE_ALT_FUNC);
	HAL_GPIO_ConfigFunction(pConfig->pOutputPort_Group, pConfig->OutputPort_Number, pConfig->OutputPort_AltFunc);

	// Initialize and configure timer1n for ADC trigger
	HAL_SCU_HSE_ClockConfig(HSE_EN);
	HAL_SCU_MiscClockConfig(1, TIMER1n_TYPE, CLKSRC_HSE, 8);
	HAL_SCU_Timer1n_ClockConfig(T1NCLK_MCCR1);

	TIMER1n_Config.StartSync = ENABLE; // Start Sync
	TIMER1n_Config.ClearSync = DISABLE; // Clear Sync
	TIMER1n_Config.ExtClock = DISABLE;
	TIMER1n_Config.MatchInterrupt = ENABLE; // Enable Match Interrupt
	TIMER1n_Config.CaptureInterrupt = DISABLE;
	TIMER1n_Config.PrescalerData = 100 - 1; // 1MHz / 100 = 10kHz -> 100us
	TIMER1n_Config.AData = 2000 - 1; // 200us * 2000 = 200ms
	TIMER1n_Config.BData = 1000 - 1; // 100us * 1000 = 100ms // When TIMER1n runs in REPEAT Mode, This value is applied to Timer1n Output Signal.
	HAL_TIMER1n_Init(pConfig->pTIMER1x, TIMER1n_PERIODIC_MODE, &TIMER1n_Config);

	//////////////// TIMER30 ///////////////////////////////////////////////////////
	HAL_TIMER3n_DeInit(TIMER30); // Deinitialization of TIMER30

	// Set test output pin (PC0) - for TIMER30 Period Match
	HAL_GPIO_ConfigOutput(PC, 0, PCU_MODE_PUSH_PULL);
	HAL_GPIO_ConfigPullup(PC, 0, PCU_PUPD_DISABLE);
	HAL_GPIO_ClearPin(PC, _BIT(0));

	// Set test output pin (PC1) - for TIMER1n Match
	HAL_GPIO_ConfigOutput(PC, 1, PCU_MODE_PUSH_PULL);
	HAL_GPIO_ConfigPullup(PC, 1, PCU_PUPD_DISABLE);
	HAL_GPIO_ClearPin(PC, _BIT(1));

	// Set test output pin (PC2) - for TIMER30 Start/End
	HAL_GPIO_ConfigOutput(PC, 2, PCU_MODE_PUSH_PULL);
	HAL_GPIO_ConfigPullup(PC, 2, PCU_PUPD_DISABLE);
	HAL_GPIO_ClearPin(PC, _BIT(2));

	// Set PWM pin
	// PWM30AA(PE0)
	HAL_GPIO_ConfigOutput(PE, 0, PCU_MODE_ALT_FUNC);
	HAL_GPIO_ConfigFunction(PE, 0, PCU_ALT_FUNCTION_1);
	// PWM30AB(PE1)
	HAL_GPIO_ConfigOutput(PE, 1, PCU_MODE_ALT_FUNC);
	HAL_GPIO_ConfigFunction(PE, 1, PCU_ALT_FUNCTION_1);
	// PWM30BA(PE2)
	HAL_GPIO_ConfigOutput(PE, 2, PCU_MODE_ALT_FUNC);
	HAL_GPIO_ConfigFunction(PE, 2, PCU_ALT_FUNCTION_1);
	// PWM30BB(PE3)
	HAL_GPIO_ConfigOutput(PE, 3, PCU_MODE_ALT_FUNC);
	HAL_GPIO_ConfigFunction(PE, 3, PCU_ALT_FUNCTION_1);
	// PWM30CA(PE4)
	HAL_GPIO_ConfigOutput(PE, 4, PCU_MODE_ALT_FUNC);
	HAL_GPIO_ConfigFunction(PE, 4, PCU_ALT_FUNCTION_1);
	// PWM30CB(PE5)
	HAL_GPIO_ConfigOutput(PE, 5, PCU_MODE_ALT_FUNC);
	HAL_GPIO_ConfigFunction(PE, 5, PCU_ALT_FUNCTION_1);

	// Initialize Motor PWM
	HAL_SCU_Timer30_ClockConfig(T30CLK_PCLK);

	TIMER3n_Config.ExtClock = DISABLE;
	TIMER3n_Config.Mode = TIMER3n_MODE_INTERVAL; // Set TIMER30 Operationg Mode
	TIMER3n_Config.OutputMode = TIMER3n_OUTPUT_MODE_6_CHANNEL;
	TIMER3n_Config.DelayInsertion = DISABLE;
	TIMER3n_Config.DelayPosition = TIMER3n_DELAY_POSITION_FRONT;
	TIMER3n_Config.ReloadTime = TIMER3n_DATA_RELOAD_TIME_MATCH;
	TIMER3n_Config.MatchIntInterval = TIMER3n_MATCH_INT_INTERVAL_1;
	TIMER3n_Config.PrescalerData = 3199; // 32MHz / (31999 + 1) = 10kHz = 0.1ms
	TIMER3n_Config.PData = 10000; // 0.1ms * 10000 = 1s
	TIMER3n_Config.AData = 0; // 30.0%
	TIMER3n_Config.BData = 0; // 50.0%
	TIMER3n_Config.CData = 0; // 70.0%
	TIMER3n_Config.DelayData = 0; // (0 + 1) / 1MHz = 1us
	HAL_TIMER3n_Init(TIMER30, &TIMER3n_Config);

	// Setting High Impedence control
	TIMER3n_OutputA_Config.StartLevel = TIMER3n_LEVEL_LOW;
	TIMER3n_OutputA_Config.Output_PWM30Ax = DISABLE;
	TIMER3n_OutputA_Config.Output_PWM30Bx = DISABLE;
	TIMER3n_OutputA_Config.Output_PWM30Cx = ENABLE;
	TIMER3n_OutputA_Config.DisableLevel_PWM30Ax = TIMER3n_LEVEL_LOW;
	TIMER3n_OutputA_Config.DisableLevel_PWM30Bx = TIMER3n_LEVEL_LOW;
	TIMER3n_OutputA_Config.DisableLevel_PWM30Cx = TIMER3n_LEVEL_LOW;
	TIMER3n_OutputB_Config.StartLevel = TIMER3n_LEVEL_LOW;
	TIMER3n_OutputB_Config.Output_PWM30Ax = DISABLE;
	TIMER3n_OutputB_Config.Output_PWM30Bx = DISABLE;
	TIMER3n_OutputB_Config.Output_PWM30Cx = DISABLE;
	TIMER3n_OutputB_Config.DisableLevel_PWM30Ax = TIMER3n_LEVEL_HIGH;
	TIMER3n_OutputB_Config.DisableLevel_PWM30Bx = TIMER3n_LEVEL_HIGH;
	TIMER3n_OutputB_Config.DisableLevel_PWM30Cx = TIMER3n_LEVEL_HIGH;
	HAL_TIMER3n_ConfigureOutput(TIMER30, TIMER3n_OUTPUT_CTRL_ALL, &TIMER3n_OutputA_Config, &TIMER3n_OutputB_Config);

	TIMER3n_HiZ_Config.HiZ = DISABLE;
	TIMER3n_HiZ_Config.BLNK_Edge = TIMER3n_BLNK_EDGE_FALLING;
	HAL_TIMER3n_ConfigureHiZ(TIMER30, &TIMER3n_HiZ_Config);
	HAL_TIMER3n_SetHiZOutput(TIMER30, RESET);

	// ADC Trigger with Period Match Interrupt
	TIMER3n_AdcTrigger_Config.Trigger_Bottom = DISABLE;
	TIMER3n_AdcTrigger_Config.Trigger_PMatch = DISABLE;
	TIMER3n_AdcTrigger_Config.Trigger_AMatch = DISABLE;
	TIMER3n_AdcTrigger_Config.Trigger_BMatch = DISABLE;
	TIMER3n_AdcTrigger_Config.Trigger_CMatch = DISABLE;
	HAL_TIMER3n_ConfigureADCTrigger(TIMER30, &TIMER3n_AdcTrigger_Config);

	//////////////// Interrupts ///////////////////////////////////////////////////////
	// Systick interrupt
	SysTick_Config(SystemCoreClock / 1000);	//  1ms systick timer

	// TIMER1n Interrupt
	NVIC_SetPriority(pConfig->IRQ_Type, 3);
	NVIC_EnableIRQ(pConfig->IRQ_Type);

	// TIMER30 Interrupt
	HAL_TIMER3n_ConfigureInterrupt(TIMER30, TIMER3n_INTERRUPT_PMATCH_INT, ENABLE);
	NVIC_SetPriority(TIMER30_IRQn, 3);
	NVIC_EnableIRQ(TIMER30_IRQn);

	// Enable Interrupt Vector
	__enable_irq();

	// Clear Counter value, Flag Status of TIMER1n
	HAL_TIMER1n_ClearStatus(pConfig->pTIMER1x, TIMER1n_STATUS_MATCH_INT);
	HAL_TIMER1n_ClearCounter(pConfig->pTIMER1x);

	// Clear Counter value, Flag Status of TIMER30
	HAL_TIMER3n_ClearStatus(TIMER30, (TIMER3n_STATUS_Type)TIMER3n_INTFLAG_STATUS_CLEAR_MASK);
	HAL_TIMER3n_ClearCounter(TIMER30);

	HAL_TIMER1n_Start(pConfig->pTIMER1x);	// Run TIMER1n

	MilliSec = 200;
	while(MilliSec);

	HAL_GPIO_SetPin(PC, _BIT(2)); // Run TIMER30 after 200ms delay
	HAL_TIMER3n_Start(TIMER30);	// Run TIMER30

	cprintf("%s Runs...\r\n", pConfig->TimerName);
	while (1);
}

/* --------------------------------- End Of File ------------------------------ */
