/*****************************************************************************
* @file     TestCode_TS_Polling.c
* @brief    Contains the external function prototypes for TestCode
* @author   AE Team, ABOV Semiconductor Co., Ltd.
* @version  V0.0.1
* @date     25. MAR. 2020
*
* Copyright(C) 2019, ABOV Semiconductor
* All rights reserved.
*
*
********************************************************************************
* DISCLAIMER
*
* THE PRESENT FIRMWARE WHICH IS FOR GUIDANCE ONLY AIMS AT PROVIDING CUSTOMERS
* WITH CODING INFORMATION REGARDING THEIR PRODUCTS IN ORDER FOR THEM TO SAVE
* TIME. AS A RESULT, ABOV SEMICONDUCTOR DISCLAIMS ALL LIABILITIES FROM ANY
* DIRECT, INDIRECT OR CONSEQUENTIAL DAMAGES WITH RESPECT TO ANY CLAIMS ARISING
* FROM THE CONTENT OF SUCH FIRMWARE AND/OR THE USE MADE BY CUSTOMERS OF THE
* CODING INFORMATION CONTAINED HEREIN IN CONNECTION WITH THEIR PRODUCTS.
*
*******************************************************************************/

/*******************************************************************************
* Pre-processor Definition & Macro
*******************************************************************************/
#include "TestCode_TS_Polling.h"
#include "main_conf.h"

/*******************************************************************************
* Private Typedef
*******************************************************************************/
#define TEST_TRIAL      (100)
#define USE_SYSTICK
/*******************************************************************************
* Private Variable
*******************************************************************************/
static uint32_t MilliSec;
uint32_t TS_BUFFER[TEST_TRIAL] = {0, };

/*******************************************************************************
* Private Function Prototype
*******************************************************************************/
uint32_t getAverage(uint32_t * buf, int num);

/*******************************************************************************
* Public Function
*******************************************************************************/
/*******************************************************************************
* @brief      SysTick interrupt handler @ 1000Hz(1ms)
* @param      None
* @return     None
*******************************************************************************/
void SysTick_Handler_TSPolling(void) {
	if (MilliSec) {
		MilliSec--;
	}
}

/*******************************************************************************
* @brief      Main_TempsensorPolling
* @param      None
* @return     None
*******************************************************************************/
void Main_TempsensorPolling(void) {
	uint32_t Count = 0;
    int i = 0;
	TS_CFG_Type TS_Config;


#if 1
    // TEST MCLK CLock
    cputs("MCLK REF CLK\r\n");
	HAL_SCU_TS_ClockConfig(ENABLE, SCU_TS_REFERENCE_MCLK, SCU_TS_SESNING_LSI_TS);
    TS_Config.ReferencePeriod = 48000; // 48MHz / 48000 = 1kHz (1ms)
#endif
#if 0
    // TEST HSE Clock
    cputs("HSE REF CLK\r\n");
	HAL_SCU_HSE_ClockConfig(HSE_EN);    // 8MHz = 0.125us
	HAL_SCU_TS_ClockConfig(ENABLE, SCU_TS_REFERENCE_HSE, SCU_TS_SESNING_LSI_TS);
    TS_Config.ReferencePeriod = 8000; // 0.125us * 8000 = 1kHz (1ms)
#endif
#if 0
    // TEST HSI Clock
    cputs("HSI REF CLK\r\n");
	HAL_SCU_HSI_ClockConfig(HSI_EN);    // 32MHz = 31.25ns
	HAL_SCU_TS_ClockConfig(ENABLE, SCU_TS_REFERENCE_HSI, SCU_TS_SESNING_LSI_TS);
    TS_Config.ReferencePeriod = 32000; // 31.25ns * 32000 = 1kHz (1ms)
#endif
#if 0
    // TEST LSE Clock
    cputs("LSE REF CLK\r\n");
	HAL_SCU_LSE_ClockConfig(LSE_EN);
	HAL_SCU_TS_ClockConfig(ENABLE, SCU_TS_REFERENCE_LSE, SCU_TS_SESNING_LSI_TS);
	TS_Config.ReferencePeriod = 16384; // (1/32.768kHz) * 16384 = 0.5s
#endif
#if 0
    cputs("MCLK REF CLK\r\n");
    HAL_SCU_SetPLLFreq(XTAL8MHz, PLL48MHz);   // Use PLL with HSE clock
	HAL_SCU_TS_ClockConfig(ENABLE, SCU_TS_REFERENCE_MCLK, SCU_TS_SESNING_HSI);
    TS_Config.ReferencePeriod = 48000; // 48MHz / 48000 = 1kHz (1ms)
#endif


	// TS peripheral setting
	TS_Config.TsInterrupt = DISABLE;
	HAL_TS_Init(&TS_Config);

#ifdef USE_SYSTICK
	SysTick_Config(SystemPeriClock/1000); // 1ms systick
    __enable_irq();
#endif
    cputs("Temp Sensor Start...\r\n");

    HAL_TS_Start();     // Start Temp Sensor
	while(1) {
		if (HAL_TS_GetStatus()) {

			Count = HAL_TS_GetSensingCount();
            TS_BUFFER[i] = Count;
#if 1
            cprintf("%d, Count, 0x%08X, %d\r\n", i, Count, Count);
#endif
			HAL_TS_ClearStatus();

            if (i == TEST_TRIAL) {
                HAL_TS_Stop();
                i = 0;
                cputs("TS Sensing Test is Finished!\r\n");
                cprintf("Average TS Sensing Value: %d\r\n", getAverage(TS_BUFFER, TEST_TRIAL));

                __disable_irq();
                break;
            } else {
                i++;
            }

#ifdef USE_SYSTICK
			MilliSec = 100;
			while(MilliSec);
#endif
		}
	}
}

uint32_t getAverage(uint32_t * buf, int num) {
    int i;
    uint32_t result;

    for(i = 0; i < num; i++) {
        result += buf[i];
    }
    result /= num;

    return result;
}
/* --------------------------------- End Of File ------------------------------ */
