/**********************************************************************
* @file		main.c
* @brief	Contains all macro definitions and function prototypes
* 			support for PCU firmware library
* @version	1.0
* @date		
* @author	ABOV M team
*
* Copyright(C) 2019, ABOV Semiconductor
* All rights reserved.
*
**********************************************************************/
#include "main_conf.h"

/* Private typedef ---------------------------------------------------*/
/* Private define ----------------------------------------------------*/
/* Private macro -----------------------------------------------------*/
/* Private variables -------------------------------------------------*/
/* Private define ----------------------------------------------------*/


/* Private function prototypes ---------------------------------------*/
void DEBUG_MenuPrint(void);
void DEBUG_Init(void);
void GPIO_Configure(void);
void UART_Configure(void);
void UART_Transmit(void);
void UART_ReceivRun(void);
void mainloop(void);
int main (void);
void Error_Handler(void);

/* Private variables ---------------------------------------------------*/
UART_CFG_Type UARTConfigStruct;
uint8_t ch;

extern uint8_t uart_mode;

/**********************************************************************
 * @brief		GPIO_Configure
 * @param[in]	None
 * @return 	None
 **********************************************************************/
void GPIO_Configure(void)
{
	/*
	 * Initialize UART1 pin connect
	 */
	HAL_GPIO_ConfigOutput(PB, 7, PCU_MODE_ALT_FUNC);
	HAL_GPIO_ConfigFunction(PB, 7, PCU_ALT_FUNCTION_1);
	HAL_GPIO_ConfigPullup(PB, 7, PCU_PUPD_PULL_UP);
	
	HAL_GPIO_ConfigOutput(PB, 6, PCU_MODE_ALT_FUNC);
	HAL_GPIO_ConfigFunction(PB, 6, PCU_ALT_FUNCTION_1);
}

/**********************************************************************
 * @brief		UART1_Configure
 * @param[in]	None
 * @return 	None
 **********************************************************************/

void UART_Configure(void)
{   
    uart_mode = UART_POLLING;  // Polling Mode
 // default: 38400-8-N-1	
	HAL_UART_ConfigStructInit(&UARTConfigStruct);	    
	if(HAL_UART_Init(UART1, &UARTConfigStruct) != HAL_OK)
	{
		/* Initialization Error */
    Error_Handler();
	}
}
	

/**********************************************************************
 * @brief		UART1_SendRun
 * @param[in]	None
 * @return 	None
 **********************************************************************/
void UART_Transmit(void)
{	
    uputs(UART1, "Polling Test\r\n");
}

/**********************************************************************
 * @brief		UART1_ReceivRun
 * @param[in]	None
 * @return 	None
 **********************************************************************/
void UART_ReceivRun(void)
{
    ch = ugetc(UART1);

    if (ch == 0x0d) {
        uputc(UART1, '\n');
    }
        uputc(UART1, ch);
}

/**********************************************************************
 * @brief		Main loop
 * @param[in]	None
 * @return	None
 **********************************************************************/
void mainloop(void)
{

	/*Configure port peripheral*/
	GPIO_Configure();
	
	/*Configure UART1 peripheral*/
	UART_Configure();
	
	/*UART send*/
	UART_Transmit();
  
	/* Enable IRQ Interrupts */
	__enable_irq();

	while(1)
	{
   /* Infinite loop */
		UART_ReceivRun();
	}
}

/**********************************************************************
 * @brief		Main program
 * @param[in]	None
 * @return	None
 **********************************************************************/
int main (void)
{
	 /* Initialize all port */
	Port_Init(); 

	/* Configure the system clock to HSI 32MHz */
	SystemClock_Config();
	
	/* Infinite loop */
	mainloop();  

	return (0);
}

/**********************************************************************
  * @brief  Reports the name of the source file and the source line number
  *   where the check_param error has occurred.
  * @param file: pointer to the source file name
  * @param line: check_param error line source number
  * @retval : None
 **********************************************************************/
void Error_Handler(void)
{
    while (1)
    {
    }
}

#ifdef  USE_FULL_ASSERT
/**********************************************************************
  * @brief  Reports the name of the source file and the source line number
  *   where the assert_param error has occurred.
  * @param file: pointer to the source file name
  * @param line: assert_param error line source number
  * @retval : None
 **********************************************************************/
void check_failed(uint8_t* file, uint32_t line)
{ 
  /* User can add his own implementation to report the file name and line number,
     ex: printf("Wrong parameters value: file %s on line %d\r\n", file, line) */

   /* Infinite loop */
   while (1)
   {
   }
}
#endif
