/*****************************************************************************
* @file     TestCode_USART_PollingDma.c
* @brief    Contains the external function prototypes for TestCode
* @author   AE Team, ABOV Semiconductor Co., Ltd.
* @version  V0.0.1
* @date     24. JUN. 2019
*
* Copyright(C) 2019, ABOV Semiconductor
* All rights reserved.
*
*
********************************************************************************
* DISCLAIMER
*
* THE PRESENT FIRMWARE WHICH IS FOR GUIDANCE ONLY AIMS AT PROVIDING CUSTOMERS
* WITH CODING INFORMATION REGARDING THEIR PRODUCTS IN ORDER FOR THEM TO SAVE
* TIME. AS A RESULT, ABOV SEMICONDUCTOR DISCLAIMS ALL LIABILITIES FROM ANY
* DIRECT, INDIRECT OR CONSEQUENTIAL DAMAGES WITH RESPECT TO ANY CLAIMS ARISING
* FROM THE CONTENT OF SUCH FIRMWARE AND/OR THE USE MADE BY CUSTOMERS OF THE
* CODING INFORMATION CONTAINED HEREIN IN CONNECTION WITH THEIR PRODUCTS.
*
*******************************************************************************/

/*******************************************************************************
* Pre-processor Definition & Macro
*******************************************************************************/
#include "TestCode_UART_PollingDma.h"
#include "main_conf.h"

/* Definition Master and Slave Channel */
//#define TEST_UART     (UART0)
#define TEST_UART       (UART1)

#define USED_DMAC_TX					(DMAC4) // DMAC0 ~ DMAC7
#define USED_DMAC_RX					(DMAC5) // DMAC0 ~ DMAC7
/*******************************************************************************
* Private Typedef
*******************************************************************************/

/*******************************************************************************
* Private Variable
*******************************************************************************/
extern uint8_t uart_mode;


static uint8_t TxData[14] = {
	'0', '1', '2', '3', '4', '5', '6', '7', '8', '9', 'A', 'B', '\r', '\n'
};

static uint8_t RxData[14] = {0, };

/*******************************************************************************
* Private Function Prototype
*******************************************************************************/

/*******************************************************************************
* Public Function
*******************************************************************************/


/**********************************************************************
 * @brief		Main_UartPollingDmaTest
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void Main_UartPollingDmaTest(void) { 
    
    int i;
        
	UART_CFG_Type UART_Config;
  	DMAC_CFG_Type DMAC_Config;
        
    uart_mode = UART_POLLING;   // Set UART Mode    
    
	/*
	 * Initialize UART1 pin connect
	 */
	HAL_GPIO_ConfigOutput(PB, 7, PCU_MODE_ALT_FUNC);
	HAL_GPIO_ConfigFunction(PB, 7, PCU_ALT_FUNCTION_1);
	HAL_GPIO_ConfigPullup(PB, 7, PCU_PUPD_PULL_UP);
	
	HAL_GPIO_ConfigOutput(PB, 6, PCU_MODE_ALT_FUNC);
	HAL_GPIO_ConfigFunction(PB, 6, PCU_ALT_FUNCTION_1);    
    
    #if 1        
    // Customize UART Setting
    UART_Config.Baud_rate = 38400;
    UART_Config.Parity = UART_PARITY_NONE;
    UART_Config.Stopbits = UART_STOPBIT_1;
    UART_Config.Databits = UART_DATABIT_8; 
    #endif 
   
    // Setting Data buffer
    if (uart_mode == UART_POLLING) {
        UART_Config.RxTxBuffer =  NULL;
        UART_Config.RxBufferSize = 0;
        UART_Config.TxBufferSize = 0;
    }

    //  Update UART Channel Configuration  
    HAL_UART_Init(UART1, &UART_Config);

    // UART Put string test
    uputs(UART1, (char *)"Polling Dma test\r\n");

	// DMAC Tx channel setting 
	DMAC_Config.Peripheral = DMAC_PERIPHERAL_UART1_TX;
	DMAC_Config.Width = DMAC_WIDTH_BYTE;
	DMAC_Config.Direction = DMAC_DIRECTION_TX;
	DMAC_Config.PeripheralAddress = (uint32_t)(&(UART1->THR));
	HAL_DMAC_Init(USED_DMAC_TX, &DMAC_Config);
    
	// DMAC Rx channel setting 
	DMAC_Config.Peripheral = DMAC_PERIPHERAL_UART1_RX;
	DMAC_Config.Width = DMAC_WIDTH_BYTE;
	DMAC_Config.Direction = DMAC_DIRECTION_RX;
	DMAC_Config.PeripheralAddress = (uint32_t)(&(UART1->RBR));
	HAL_DMAC_Init(USED_DMAC_RX, &DMAC_Config);    

    // DMA Tx
    cputs("DMA Tx Test \r\n");
	while (1) {
		HAL_DMAC_SetTransferCount(USED_DMAC_TX, sizeof(TxData));
		HAL_DMAC_SetMemoryAddress(USED_DMAC_TX, ((uint32_t)(&(TxData[0])) & 0x0000FFFF));
		HAL_DMAC_Start(USED_DMAC_TX);
		while (HAL_DMAC_GetStatus(USED_DMAC_TX) == RESET);
		HAL_DMAC_Stop(USED_DMAC_TX);
        break;
	}
    // Print the result of DMA Tx 
    for(i = 0; i < sizeof(TxData); i++) {
        cprintf("%d Transmitted data througth DMA\r\n", sizeof(TxData));
        cprintf("0x%08X : 0x%X '%c'\r\n", &TxData[i], TxData[i], TxData[i]);
    }
    
    // DMA Rx
    cputs("DMA Rx Test \r\n");
    while (1) {
		HAL_DMAC_SetTransferCount(USED_DMAC_RX, sizeof(RxData));
		HAL_DMAC_SetMemoryAddress(USED_DMAC_RX, ((uint32_t)(&(RxData[0])) & 0x0000FFFF));
		HAL_DMAC_Start(USED_DMAC_RX);
		while (HAL_DMAC_GetStatus(USED_DMAC_RX) == RESET);
		HAL_DMAC_Stop(USED_DMAC_RX);
        break;
    }
    
    // Print the result of DMA Tx 
    for(i = 0; i < sizeof(RxData); i++) {
        cprintf("%d Received data through DMA\r\n", sizeof(RxData));
        cprintf("0x%08X : 0x%X '%c'\r\n", &RxData[i], RxData[i], RxData[i]);        
    }    

}


/* --------------------------------- End Of File ------------------------------ */
