/*****************************************************************************
* @file     TestCode_USART_uartInterrupt.c
* @brief    Contains the external function prototypes for TestCode
* @author   AE Team, ABOV Semiconductor Co., Ltd.
* @version  V0.0.1
* @date     24. JUN. 2019
*
* Copyright(C) 2019, ABOV Semiconductor
* All rights reserved.
*
*
********************************************************************************
* DISCLAIMER
*
* THE PRESENT FIRMWARE WHICH IS FOR GUIDANCE ONLY AIMS AT PROVIDING CUSTOMERS
* WITH CODING INFORMATION REGARDING THEIR PRODUCTS IN ORDER FOR THEM TO SAVE
* TIME. AS A RESULT, ABOV SEMICONDUCTOR DISCLAIMS ALL LIABILITIES FROM ANY
* DIRECT, INDIRECT OR CONSEQUENTIAL DAMAGES WITH RESPECT TO ANY CLAIMS ARISING
* FROM THE CONTENT OF SUCH FIRMWARE AND/OR THE USE MADE BY CUSTOMERS OF THE
* CODING INFORMATION CONTAINED HEREIN IN CONNECTION WITH THEIR PRODUCTS.
*
*******************************************************************************/

/*******************************************************************************
* Pre-processor Definition & Macro
*******************************************************************************/
#include "TestCode_USART_UartInterrupt.h"
#include "main_conf.h"

/* Definition of USART Master and Slave Channel */
#define UST10   (0)
#define UST11   (1)
#define UST12   (2)
#define UST13   (3)

// Select Test Channel
#define TEST_USART_MASTER   (UST10)
#define TEST_USART_SLAVE    (UST11)

#define TX_BUF_SIZE         (64)
#define RX_BUF_SIZE         (64)
static uint8_t test_buffer10[TX_BUF_SIZE + RX_BUF_SIZE];
static uint8_t test_buffer11[TX_BUF_SIZE + RX_BUF_SIZE];
static uint8_t test_buffer12[TX_BUF_SIZE + RX_BUF_SIZE];
static uint8_t test_buffer13[TX_BUF_SIZE + RX_BUF_SIZE];

extern uint8_t usart_mode;
/*******************************************************************************
* Private Typedef
*******************************************************************************/
/**
 * @brief USART Port Structure
 */
typedef struct {
	USART_Type *USARTx;     	// Base Address
	PORT_Type *RxdPort;          // SS (Master)
	uint32_t RxdPort_Number;
	PORT_Type *TxdPort;         // SCK
	uint32_t TxdPort_Number;
	IRQn_Type IRQ_Type; 	    // Interrupt Handler
	uint8_t UsartName[8];         // Module Name
} USART_PORT_Type;


/*******************************************************************************
* Private Variable
*******************************************************************************/
static const USART_PORT_Type USART_PORT_CFG[USART_IP_INDEX_MAX] = {
	{USART10, PB,  0, PB,  1,  USART10_IRQn,             "USART10"},
	{USART11, PD,  2, PD,  3,  USART11_IRQn,             "USART11"},
	{USART12, PE, 12, PE, 13,  USART12_13_SPI21_IRQn,    "USART12"},
	{USART13, PE,  9, PE,  8,  USART12_13_SPI21_IRQn,    "USART13"},
};
 
static const USART_PORT_Type * Master = &(USART_PORT_CFG[TEST_USART_MASTER]);
static const USART_PORT_Type * Slave = &(USART_PORT_CFG[TEST_USART_SLAVE]);
void USART_Configuration(const USART_PORT_Type * Uart, USART_CFG_Type * UsartConfig);

static uint8_t tx_ch = 0;
static uint8_t rx_ch = 0;

extern uint8_t usart_mode;

/*******************************************************************************
* Private Function Prototype
*******************************************************************************/

/*******************************************************************************
* Public Function
*******************************************************************************/
/**********************************************************************
 * @brief		Main_UsartUartInterruptTest
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void Main_UsartUartInterruptTest(void) { 
	USART_CFG_Type USARTConfigStruct;
    
    usart_mode = USART_INTERRUPT;
        
    // USART Channel Configurations
    USART_Configuration(Master, &USARTConfigStruct);
    USART_Configuration(Slave, &USARTConfigStruct);
    
 	__enable_irq();
        
    // Test Start
	cputs(">> USART USRT TEST..............\r\n");
	cprintf("Master Channel: %s\r\n", Master->UsartName);
	cprintf("Slave Channel: %s\r\n\r\n", Slave->UsartName);
       
	while(1){
        tx_ch = cgetchar(); // Get a character from debug channel
        if ((tx_ch) > 0 && (tx_ch < 0x7F)) {
                       
            usputc(Master->USARTx, tx_ch);
            cprintf("Tx DATA = %c : %X\r\n", tx_ch, tx_ch);
            
            rx_ch = usgetc(Slave->USARTx);
            cprintf("Rx DATA = %c : %X\r\n\r\n", rx_ch, rx_ch);
        }
	}
}


/***************************************************************************//**
* @brief      Determines best dividers to get a target baud rate
* @param      Uart : UART Port Type struct selected, should be
* @param      UartConfig : Uart Channel Options
* @return     None
*******************************************************************************/
void USART_Configuration(const USART_PORT_Type * Uart, USART_CFG_Type * UsartConfig) {
	// Initialize Rxd port connect
	HAL_GPIO_ConfigOutput(Uart->RxdPort, Uart->RxdPort_Number, PCU_MODE_ALT_FUNC);
	HAL_GPIO_ConfigFunction(Uart->RxdPort, Uart->RxdPort_Number, PCU_ALT_FUNCTION_1);
	HAL_GPIO_ConfigPullup(Uart->RxdPort, Uart->RxdPort_Number, PCU_PUPD_PULL_UP);

	// Initialize Txd port connect
	HAL_GPIO_ConfigOutput(Uart->TxdPort, Uart->TxdPort_Number, PCU_MODE_ALT_FUNC);
	HAL_GPIO_ConfigFunction(Uart->TxdPort, Uart->TxdPort_Number, PCU_ALT_FUNCTION_1);
	
    HAL_USART_UART_Mode_Config(UsartConfig);   // Updated Uart default option
        
    usart_mode = USART_INTERRUPT;              // Set Interrupt Mode
    
    if (usart_mode == USART_INTERRUPT) {
        
        if (Uart->USARTx == USART10) {
            UsartConfig->RxTxBuffer = test_buffer10;
        } else if (Uart->USARTx == USART11) {
            UsartConfig->RxTxBuffer = test_buffer11;
        } else if (Uart->USARTx == USART12) {
            UsartConfig->RxTxBuffer = test_buffer12;
        } else if (Uart->USARTx == USART13) {
            UsartConfig->RxTxBuffer = test_buffer13;
        } else {
            UsartConfig->RxTxBuffer = NULL;        	
	        UsartConfig->RxBufferSize = 0;
    	    UsartConfig->TxBufferSize = 0;            
        }

        UsartConfig->RxBufferSize = RX_BUF_SIZE;
        UsartConfig->TxBufferSize = TX_BUF_SIZE;
    }

	HAL_USART_Init(Uart->USARTx, UsartConfig);       // Initialize USART channel and Interrupt
}

/* --------------------------------- End Of File ------------------------------ */
