/*****************************************************************************
* @file     TestCode_USART_uartInterruptDma.c
* @brief    Contains the external function prototypes for TestCode
* @author   AE Team, ABOV Semiconductor Co., Ltd.
* @version  V0.0.1
* @date     24. JUN. 2019
*
* Copyright(C) 2019, ABOV Semiconductor
* All rights reserved.
*
*
********************************************************************************
* DISCLAIMER
*
* THE PRESENT FIRMWARE WHICH IS FOR GUIDANCE ONLY AIMS AT PROVIDING CUSTOMERS
* WITH CODING INFORMATION REGARDING THEIR PRODUCTS IN ORDER FOR THEM TO SAVE
* TIME. AS A RESULT, ABOV SEMICONDUCTOR DISCLAIMS ALL LIABILITIES FROM ANY
* DIRECT, INDIRECT OR CONSEQUENTIAL DAMAGES WITH RESPECT TO ANY CLAIMS ARISING
* FROM THE CONTENT OF SUCH FIRMWARE AND/OR THE USE MADE BY CUSTOMERS OF THE
* CODING INFORMATION CONTAINED HEREIN IN CONNECTION WITH THEIR PRODUCTS.
*
*******************************************************************************/

/*******************************************************************************
* Pre-processor Definition & Macro
*******************************************************************************/
#include "TestCode_USART_UartInterruptDma.h"
#include "main_conf.h"

/* Definition of USART Master and Slave Channel */
/* Definition of USART Master and Slave Channel */
#define UST10   (0)
#define UST11   (1)
#define UST12   (2)
#define UST13   (3)

#define TEST_USART           (UST10)

/* Definition of DMAC Channel*/
#define DMA_TX_CH			(DMAC0)
#define DMA_RX_CH			(DMAC1)
//#define DMA_TX_CH			(DMAC2)
//#define DMA_RX_CH			(DMAC3)
//#define DMA_TX_CH			(DMAC4)
//#define DMA_RX_CH			(DMAC5)
//#define DMA_TX_CH			(DMAC6)
//#define DMA_RX_CH			(DMAC7)

/*******************************************************************************
* Private Typedef
*******************************************************************************/
/**
 * @brief USART Port Structure
 */
typedef struct {
	USART_Type *USARTx;     	// Base Address
	PORT_Type *RxdPort;          // SS (Master)
	uint32_t RxdPort_Number;
	PORT_Type *TxdPort;         // SCK
	uint32_t TxdPort_Number;
	IRQn_Type IRQ_Type; 	    // Interrupt Handler
	uint8_t UsartName[8];         // Module Name
} USART_PORT_Type;


/*******************************************************************************
* Private Variable
*******************************************************************************/
// Usart Channels
static const USART_PORT_Type USART_PORT_CFG[USART_IP_INDEX_MAX] = {
	{USART10, PB,  0, PB,  1,  USART10_IRQn,             "USART10"},
	{USART11, PD,  2, PD,  3,  USART11_IRQn,             "USART11"},
	{USART12, PE, 12, PE, 13,  USART12_13_SPI21_IRQn,    "USART12"},
	{USART13, PE,  9, PE,  8,  USART12_13_SPI21_IRQn,    "USART13"},
};
 
static const USART_PORT_Type * Master = &(USART_PORT_CFG[TEST_USART]);

// Test Buffer
static uint8_t TestData[] = "Hello, World!";
static uint8_t Tx_TestData[sizeof(TestData)] = { 0, };
static uint8_t Rx_TestData[sizeof(TestData)] = { 0, };

// DMA Interrupt Flag
Bool DMA_Tx_Done;
Bool DMA_Rx_Done;

/*******************************************************************************
* Private Function Prototype
*******************************************************************************/
void USART_Configuration(const USART_PORT_Type * Uart, USART_CFG_Type * UsartConfig);

/*******************************************************************************
* Public Function
*******************************************************************************/
/*********************************************************************//**
 * @brief		USART1n interrupt handler sub-routine (Common)
 * @param[in]	None
 * @return 		None
 **********************************************************************/
static void USART1n_IRQHandler_UartInterruptTestDma(USART_Type * USTx) {    
	uint32_t intsrc, tmp;

	/* Determine the interrupt source */
	intsrc = HAL_USART_GetStatus(USTx);
	tmp = intsrc & USART_ST_BITMASK;

	// DMA TX Interrupt Flag is occurred
	if ((tmp & USART_ST_DMATXF) == USART_ST_DMATXF){
		DMA_Tx_Done = TRUE;									// Set User DMA TX Interrupt Flag
        HAL_USART_ClearStatus(USTx, USART_STATUS_DMATXF);
	}

	// DMA RX Interrupt Flag is occurred
	if ((tmp & USART_ST_DMARXF) == USART_ST_DMARXF){
		DMA_Rx_Done = TRUE;									// Set User DMA RX Interrupt Flag
        HAL_USART_ClearStatus(USTx, USART_STATUS_DMARXF);
	}

	// Receive Data Available or Character time-out
	if ((tmp & USART_ST_RXC) == USART_ST_RXC){
        HAL_USART_ClearStatus(USTx, USART_STATUS_RXC);
	}

	// Transmit Holding Empty
	if ((tmp & USART_ST_TXC) == USART_ST_TXC){
        HAL_USART_ClearStatus(USTx, USART_STATUS_TXC);
	}

	// DMA TX Interrupt Flag is occurred
	if ((tmp & USART_ST_DRE) == USART_ST_DRE){
        HAL_USART_ClearStatus(USTx, USART_STATUS_DRE);
	}
}

/*********************************************************************//**
 * @brief		USART10_IRQHandler_UartInterruptDma
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void USART10_IRQHandler_UartInterruptDma(void) {
    USART1n_IRQHandler_UartInterruptTestDma(USART10);
}


/*********************************************************************//**
 * @brief		USART11_IRQHandler_UartInterruptDma
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void USART11_IRQHandler_UartInterruptDma(void) {
    USART1n_IRQHandler_UartInterruptTestDma(USART11);
}


/*********************************************************************//**
 * @brief		USART12_13_SPI21_IRQHandler_UartInterruptDma
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void USART12_13_SPI21_IRQHandler_UartInterruptDma(void) {
  if (TEST_USART == UST12) {
    USART1n_IRQHandler_UartInterruptTestDma(USART12);
  } else if (TEST_USART == UST13) {
    USART1n_IRQHandler_UartInterruptTestDma(USART13);    
  }
}

/**********************************************************************
 * @brief		Main_UsartUartInterruptTest
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void Main_UsartUartInterruptDmaTest(void) { 
	USART_CFG_Type USARTConfigStruct;
	DMAC_CFG_Type DMA_Config_rx;
	DMAC_CFG_Type DMA_Config_tx;
	int i;    
        
    // USART Channel Configurations
    USART_Configuration(Master, &USARTConfigStruct);

    // Interrupt Setting
    // Master DMA Interrupts
	HAL_USART_DmaConfig(Master->USARTx, USART_CR2_DMARXIE, ENABLE);		// USART10 DMA(Rx) Interrupt Enable
	HAL_USART_DmaConfig(Master->USARTx, USART_CR2_DMATXIE, ENABLE);		// USART10 DMA(Tx) Interrupt Enable   
    
    NVIC_SetPriority(Master->IRQ_Type, 3);
    NVIC_EnableIRQ(Master->IRQ_Type); 

 	__enable_irq();
     
	HAL_USART_ClearStatus(Master->USARTx, USART_STATUS_MASK);

	HAL_DMAC_DeInit(DMA_TX_CH); 	// Disabled DMAC Tx Channel
	HAL_DMAC_DeInit(DMA_RX_CH);	    // Disabled DMAC Rx Channel

	// DMAC Tx Channel Configuration
	HAL_DMAC_SetTransferCount(DMA_TX_CH, sizeof(TestData));
	DMA_Config_tx.Peripheral = DMAC_PERIPHERAL_USART10_TX;	    	// Selected DMA USART10 TX
	DMA_Config_tx.Width = DMAC_WIDTH_BYTE;					    	// Half-Word byte transfer (16-bit)
	DMA_Config_tx.Direction = DMAC_DIRECTION_TX;			    	// Peripheral to designated address
	DMA_Config_tx.PeripheralAddress = (uint32_t)&(Master->USARTx->DR);   	// USART->DR register
	HAL_DMAC_Init(DMA_TX_CH, &DMA_Config_tx);							// DMA Channel Initization
	HAL_DMAC_SetMemoryAddress(DMA_TX_CH, (uint32_t)&Tx_TestData);		// Set the address of tx buffer
	HAL_DMAC_SetTransferCount(DMA_TX_CH, sizeof(TestData));				// Set the tx buffer size
	cputs("TX DMAC Initialized.\r\n");

	// DMAC Rx Channel Configuration : USART10 Rx
	HAL_DMAC_SetTransferCount(DMA_RX_CH, sizeof(TestData));
	DMA_Config_rx.Peripheral = DMAC_PERIPHERAL_USART10_RX;		    // Selected DMA USART10 RX
	DMA_Config_rx.Width = DMAC_WIDTH_BYTE;						    // Half-Word byte transfer (16-bit)
	DMA_Config_rx.Direction = DMAC_DIRECTION_RX;				    // Peripheral to designated address
	DMA_Config_rx.PeripheralAddress = (uint32_t)&(Master->USARTx->DR);	    // USART->DR register
	HAL_DMAC_Init(DMA_RX_CH, &DMA_Config_rx);							// DMA Channel Initization
	HAL_DMAC_SetMemoryAddress(DMA_RX_CH, (uint32_t)&Rx_TestData);		// Set the address of rx buffer
	HAL_DMAC_SetTransferCount(DMA_RX_CH, sizeof(TestData));				// Set the rx buffer size
	cputs("RX DMAC Channel Initialized.\r\n");

	DMA_Tx_Done = FALSE;	// Init tx flag
	DMA_Rx_Done = FALSE;	// Init rx flag

	cputs(">> USART(UART) DMA TEST.............. \r\n");

	while(1) {

		HAL_DMAC_SetTransferCount(DMA_TX_CH, sizeof(TestData));
		HAL_DMAC_SetMemoryAddress(DMA_TX_CH, (uint32_t)&Tx_TestData);
              
		for(i = 0 ; i < sizeof(TestData); i++) {
			Tx_TestData[i] = TestData[i];                   // Fill data to DMA buffer on sram
		}
        cprintf("%d\r\n", sizeof(TestData));
		HAL_DMAC_Start(DMA_TX_CH);	// Start DMA Tx Channel
        
		while(!DMA_Tx_Done);
		DMA_Tx_Done = FALSE;

		cputs(" - TX DMA Done!!\r\n");
		cputs("\tTx Data => ");
		for(i = 0 ; i < sizeof(TestData); i++) {
			cputc(Tx_TestData[i]);
		}
        cputs(" ");

		HAL_DMAC_SetTransferCount(DMA_RX_CH, sizeof(TestData));
		HAL_DMAC_SetMemoryAddress(DMA_RX_CH, (uint32_t)&Rx_TestData);

		HAL_DMAC_Start(DMA_RX_CH);
		cputs("\r\n Please insert the key as much as the buffer size on the usart10 channel.\r\n");

		while(1) {

                if (DMA_Rx_Done == TRUE) {
				cputs(" - RX DMA Done!!\r\n");
				cputs("\tRx Data => ");

				for(i = 0; i < sizeof(TestData); i++) {
					cputc(Rx_TestData[i]);
				}

				DMA_Rx_Done = FALSE;
           		HAL_DMAC_Stop(DMA_RX_CH);
				cputs("\r\n----------------------------------------------------------------------------------\r\n");
				break;
			}

		} /* while */

	} /* while */
}


/***************************************************************************//**
* @brief      Determines best dividers to get a target baud rate
* @param      Uart : UART Port Type struct selected, should be
* @param      UartConfig : Uart Channel Options
* @return     None
*******************************************************************************/
extern uint8_t usart_mode;
#define TX_BUFFER_SIZE      (64)
#define RX_BUFFER_SIZE      (64)
uint8_t test_buffer[TX_BUFFER_SIZE + RX_BUFFER_SIZE];

void USART_Configuration(const USART_PORT_Type * Uart, USART_CFG_Type * UsartConfig) {
	// Initialize Rxd port connect
	HAL_GPIO_ConfigOutput(Uart->RxdPort, Uart->RxdPort_Number, PCU_MODE_ALT_FUNC);
	HAL_GPIO_ConfigFunction(Uart->RxdPort, Uart->RxdPort_Number, PCU_ALT_FUNCTION_1);
	HAL_GPIO_ConfigPullup(Uart->RxdPort, Uart->RxdPort_Number, PCU_PUPD_PULL_UP);

	// Initialize Txd port connect
	HAL_GPIO_ConfigOutput(Uart->TxdPort, Uart->TxdPort_Number, PCU_MODE_ALT_FUNC);
	HAL_GPIO_ConfigFunction(Uart->TxdPort, Uart->TxdPort_Number, PCU_ALT_FUNCTION_1);

    HAL_USART_UART_Mode_Config(UsartConfig);   // Updated Uart default option\

//    usart_mode = USART_INTERRUPT;

//    if (usart_mode == USART_POLLING) {
//        UsartConfig->RxTxBuffer = test_buffer;
//        UsartConfig->RxBufferSize = TX_BUFFER_SIZE;
//        UsartConfig->TxBufferSize = RX_BUFFER_SIZE;
//    }

	HAL_USART_Init(Uart->USARTx, UsartConfig);       // Initialization and Enable
}


/* --------------------------------- End Of File ------------------------------ */
