/*****************************************************************************
* @file     TestCode_USART_UartPolling.c
* @brief    Contains the external function prototypes for TestCode
* @author   AE Team, ABOV Semiconductor Co., Ltd.
* @version  V0.0.1
* @date     24. JUN. 2019
*
* Copyright(C) 2019, ABOV Semiconductor
* All rights reserved.
*
*
********************************************************************************
* DISCLAIMER
*
* THE PRESENT FIRMWARE WHICH IS FOR GUIDANCE ONLY AIMS AT PROVIDING CUSTOMERS
* WITH CODING INFORMATION REGARDING THEIR PRODUCTS IN ORDER FOR THEM TO SAVE
* TIME. AS A RESULT, ABOV SEMICONDUCTOR DISCLAIMS ALL LIABILITIES FROM ANY
* DIRECT, INDIRECT OR CONSEQUENTIAL DAMAGES WITH RESPECT TO ANY CLAIMS ARISING
* FROM THE CONTENT OF SUCH FIRMWARE AND/OR THE USE MADE BY CUSTOMERS OF THE
* CODING INFORMATION CONTAINED HEREIN IN CONNECTION WITH THEIR PRODUCTS.
*
*******************************************************************************/

/*******************************************************************************
* Pre-processor Definition & Macro
*******************************************************************************/
#include "TestCode_USART_UartPolling.h"
#include "main_conf.h"

/* Definition of USART Master and Slave Channel */
#define UST10   (0)
#define UST11   (1)
#define UST12   (2)
#define UST13   (3)

// Select Test Channel
#define TEST_USART_MASTER   (UST10)
#define TEST_USART_SLAVE    (UST11)

/*******************************************************************************
* Private Typedef
*******************************************************************************/
/**
 * @brief USART Port Structure
 */
typedef struct {
	USART_Type *USARTx;     	// Base Address
	PORT_Type *RxdPort;          // SS (Master)
	uint32_t RxdPort_Number;
	PORT_Type *TxdPort;         // SCK
	uint32_t TxdPort_Number;
	IRQn_Type IRQ_Type; 	    // Interrupt Handler
	uint8_t UsartName[8];         // Module Name
} USART_PORT_Type;

/*******************************************************************************
* Private Variable
*******************************************************************************/
#define USART_IP_INDEX_MAX          (4)

static const USART_PORT_Type USART_PORT_CFG[USART_IP_INDEX_MAX] = {
	{USART10, PB,  0, PB,  1,  USART10_IRQn,             "USART10"},
	{USART11, PD,  2, PD,  3,  USART11_IRQn,             "USART11"},
	{USART12, PE, 12, PE, 13,  USART12_13_SPI21_IRQn,    "USART12"},
	{USART13, PE,  9, PE,  8,  USART12_13_SPI21_IRQn,    "USART13"},
};

static const USART_PORT_Type * Master = &(USART_PORT_CFG[TEST_USART_MASTER]);
static const USART_PORT_Type * Slave = &(USART_PORT_CFG[TEST_USART_SLAVE]);

static uint8_t tx_ch = 0;
static uint8_t rx_ch = 0;

extern uint8_t usart_mode;
/*******************************************************************************
* Private Function Prototype
*******************************************************************************/
void USART_Configuration(const USART_PORT_Type * USTx, USART_CFG_Type *UsartConfig);


/*******************************************************************************
* Public Function
*******************************************************************************/


/**********************************************************************
 * @brief		Main_UsartPollingTest
 * @param[in]	None
 * @return 		None
 **********************************************************************/

void Main_UsartUartPollingTest(void) {
	USART_CFG_Type USARTConfigStruct;

    USART_Configuration(Master, &USARTConfigStruct);
    USART_Configuration(Slave, &USARTConfigStruct);

	cputs(">> USART USRT TEST..............\r\n");
	cprintf("Master Channel: %s\r\n", Master->UsartName);
	cprintf("Slave Channel: %s\r\n\r\n", Slave->UsartName);

	while(1) {

        tx_ch = ugetchar(UART1);                                                     // Get a character from debug channel
        if ((tx_ch) > 0 && (tx_ch < 0x7F)) {
            if (tx_ch == 0x0d) {
                cputs("\r\n");
            } else {
                cprintf("Tx DATA = %c : %X\r\n", tx_ch, tx_ch);

                usputc(Master->USARTx, tx_ch);

                while(!(HAL_USART_GetStatus(Master->USARTx) & USART_ST_TXC)) {}     // wait until tx data
                while(!(HAL_USART_GetStatus(Slave->USARTx) & USART_ST_RXC)) {}      // wait until rx data
                rx_ch = usgetc(Slave->USARTx);

                cprintf("Rx DATA = %c : %X\r\n", rx_ch, rx_ch);
                cputs("\r\n");
            }
        }
	}
}

/***************************************************************************//**
* @brief      Determines best dividers to get a target baud rate
* @param      Uart : UART Port Type struct selected, should be
* @param      UartConfig : Uart Channel Options
* @return     None
*******************************************************************************/
void USART_Configuration(const USART_PORT_Type * Uart, USART_CFG_Type * UsartConfig) {
	// Initialize Rxd port connect
	HAL_GPIO_ConfigOutput(Uart->RxdPort, Uart->RxdPort_Number, PCU_MODE_ALT_FUNC);
	HAL_GPIO_ConfigFunction(Uart->RxdPort, Uart->RxdPort_Number, PCU_ALT_FUNCTION_1);
	HAL_GPIO_ConfigPullup(Uart->RxdPort, Uart->RxdPort_Number, PCU_PUPD_PULL_UP);

	// Initialize Txd port connect
	HAL_GPIO_ConfigOutput(Uart->TxdPort, Uart->TxdPort_Number, PCU_MODE_ALT_FUNC);
	HAL_GPIO_ConfigFunction(Uart->TxdPort, Uart->TxdPort_Number, PCU_ALT_FUNCTION_1);

    HAL_USART_UART_Mode_Config(UsartConfig);   // Updated Uart default option\

    usart_mode = USART_POLLING;

    if (usart_mode == USART_POLLING) {
        UsartConfig->RxTxBuffer = NULL;
        UsartConfig->RxBufferSize = 0;
        UsartConfig->TxBufferSize = 0;
    }

	HAL_USART_Init(Uart->USARTx, UsartConfig);       // Initialization and Enable
}

/* --------------------------------- End Of File ------------------------------ */
