/*****************************************************************************
* @file     TestCode_WDT_ClkSrc.c
* @brief    Contains the external function prototypes for TestCode
* @author   AE Team, ABOV Semiconductor Co., Ltd.
* @version  V0.0.1
* @date     12. JUN. 2019
*
* Copyright(C) 2019, ABOV Semiconductor
* All rights reserved.
*
*
********************************************************************************
* DISCLAIMER
*
* THE PRESENT FIRMWARE WHICH IS FOR GUIDANCE ONLY AIMS AT PROVIDING CUSTOMERS
* WITH CODING INFORMATION REGARDING THEIR PRODUCTS IN ORDER FOR THEM TO SAVE
* TIME. AS A RESULT, ABOV SEMICONDUCTOR DISCLAIMS ALL LIABILITIES FROM ANY
* DIRECT, INDIRECT OR CONSEQUENTIAL DAMAGES WITH RESPECT TO ANY CLAIMS ARISING
* FROM THE CONTENT OF SUCH FIRMWARE AND/OR THE USE MADE BY CUSTOMERS OF THE
* CODING INFORMATION CONTAINED HEREIN IN CONNECTION WITH THEIR PRODUCTS.
*
*******************************************************************************/

/*******************************************************************************
* Pre-processor Definition & Macro
*******************************************************************************/
#include "TestCode_WDT_ClkSrc.h"
#include "main_conf.h"

/*******************************************************************************
* Private Typedef
*******************************************************************************/

/*******************************************************************************
* Private Variable
*******************************************************************************/
static uint32_t MilliSec;       // Systick Counter Value

/*******************************************************************************
* Private Function Prototype
*******************************************************************************/

/*******************************************************************************
* Public Function
*******************************************************************************/

/*******************************************************************************
* @brief      SysTick interrupt handler @ 1000Hz(1ms)
* @param      None
* @return     None
*******************************************************************************/
void SysTick_Handler_ClkSrc(void) {
	if (MilliSec) {
		MilliSec--;
	}
}

/*******************************************************************************
* Public Function
*******************************************************************************/

/**********************************************************************
 * @brief		Watch-dog clock source selection and Wdt Reset
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void Main_WdtClkSrc(void) {
	uint8_t sel;

	WDT_CFG_Type WdtCfg;
	uint32_t WdtClk;

	// Test Pin setting PE0
	HAL_GPIO_ConfigOutput(PE, 0, PCU_MODE_PUSH_PULL);
	HAL_GPIO_ConfigPullup(PE, 0, PCU_PUPD_DISABLE);

	// Select WDT Clock
    cputs("Select a source clock to run WDT. (a to f)\r\n");
	cputs("a: LSI/16(31.250kHz, WDTRC)\r\nb: LSE(32.768kHz)\r\nc: MCLK\r\nd: HSI(32MHz)\r\ne: HSE(XTAL)\r\nf: PLL Clock(48MHz)\r\n");
	while(1) {
		sel = cgetchar();
        if ((sel > 0) && (sel <= 0x7F)) {
            cputs(">>");
            if(sel == 'a') {
                cputs("LSI is WDT Clock source\r\n");
                HAL_SCU_LSI_ClockConfig(LSI_EN);							// ENABLE LSI (500kHz)
                HAL_SCU_WDT_ClockConfig(WDTCLK_WDTRC);				// 31250Hz (500kHz LSI/16 = 31250Hz)
                WdtCfg.ClockDivider = WDT_DIVIDER_4;  		// WDTCLK = WDTCLKIN/PRS = 31250/4 = 7812Hz
                WdtClk = 78120;								// 10s
                break;
            } else if (sel == 'b') {
                cputs("LSE is WDT Clock source\r\n");
                HAL_SCU_LSE_ClockConfig(LSE_EN);							// ENABLE LSE (32768kHz)
                HAL_SCU_MiscClockConfig(3, WDT_TYPE, CLKSRC_LSE, 128);	// 32768Hz/128 = 256Hz
                HAL_SCU_WDT_ClockConfig(WDTCLK_MCCR3);				// Select Wdtclock source selected by MCCR3
                WdtCfg.ClockDivider = WDT_DIVIDER_256;  	// WDTCLK = WDTCLKIN/PRS = 256Hz/256 = 1Hz
                WdtClk = 10;								// 10s
                break;
            } else if (sel == 'c') {
                cputs("MCLK is WDT Clock source\r\n");		// MCLK (48MHz)
                HAL_SCU_MiscClockConfig(3, WDT_TYPE, CLKSRC_MCLK, 12);	// 48MHz/12 = 4MHz
                HAL_SCU_WDT_ClockConfig(WDTCLK_MCCR3);				// Select Wdtclock source selected by MCCR3
                WdtCfg.ClockDivider = WDT_DIVIDER_4;  		// WDTCLK = WDTCLKIN/PRS = 4MHz/4 = 1MHz
                WdtClk = 10000000;							// 10s
                break;
            } else if (sel == 'd') {
                HAL_SCU_HSI_ClockConfig(HSI_EN);							// Enable HSI (32MHz)
                cputs("HSI is WDT Clock source\r\n");		// HSI (32MHz)
                HAL_SCU_MiscClockConfig(3, WDT_TYPE, CLKSRC_HSI, 8);	// 32MHz/8 = 4MHz
                HAL_SCU_WDT_ClockConfig(WDTCLK_MCCR3);
                WdtCfg.ClockDivider = WDT_DIVIDER_4;  		// WDTCLK = WDTCLKIN/PRS = 4MHz/4 = 1MHz
                WdtClk = 10000000;							// 10s
                break;
            } else if (sel == 'e') {
                cputs("HSE is WDT Clock source\r\n");		// HSE (8MHz)
                HAL_SCU_MiscClockConfig(3, WDT_TYPE, CLKSRC_HSE, 2);	// 8MHz/2 = 4MHz
                HAL_SCU_WDT_ClockConfig(WDTCLK_MCCR3);
                WdtCfg.ClockDivider = WDT_DIVIDER_4;  		// WDTCLK = WDTCLKIN/PRS = 4MHz/4 = 1MHz
                WdtClk = 10000000;							// 10s
                break;
            } else if (sel == 'f') {
                cputs("PLL is WDT Clock source\r\n");		// PLL (48MHz)
                HAL_SCU_MiscClockConfig(3, WDT_TYPE, CLKSRC_PLL, 12);	// 48MHz/12 = 4MHz
                HAL_SCU_WDT_ClockConfig(WDTCLK_MCCR3);				// Select Wdtclock source selected by MCCR3
                WdtCfg.ClockDivider = WDT_DIVIDER_4;  		// WDTCLK = WDTCLKIN/PRS = 4MHz/4 = 1MHz
                WdtClk = 10000000;							// 10s
                break;
            }
        }
	}
	WdtCfg.EnableReset = TRUE;						// WDT Reset Enable
	WdtCfg.Data = (WdtClk/2) - 1; 					// 5.0s
	HAL_WDT_Init(&WdtCfg);								// Setting WDT configuration

	SysTick_Config(SystemCoreClock/1000);  			// 1ms interrupt
	__enable_irq();

	HAL_SCU_SetResetSrc(RST_WDTRST, ENABLE);			// Enable WDT Reset
	HAL_SCU_ClearResetStatus(0xFF); 					// Clear all reset status

	cputs("Watch-dog Reset will be counted after 5 sec ...\r\n\r\n");
	HAL_WDT_Start();											// Watch-dog Start

	while(1) {
		MilliSec = 500;
		while(MilliSec);
		HAL_GPIO_SetPin(PE, _BIT(0));
		MilliSec = 500;
		while(MilliSec);
		HAL_GPIO_ClearPin(PE, _BIT(0));
	}
}

/* --------------------------------- End Of File ------------------------------ */
