/*****************************************************************************
* @file     TestCode_WDT_Reload.c
* @brief    Contains the external function prototypes for TestCode
* @author   AE Team, ABOV Semiconductor Co., Ltd.
* @version  V0.0.1
* @date     16. MAY. 2019
*
* Copyright(C) 2019, ABOV Semiconductor
* All rights reserved.
*
*
********************************************************************************
* DISCLAIMER
*
* THE PRESENT FIRMWARE WHICH IS FOR GUIDANCE ONLY AIMS AT PROVIDING CUSTOMERS
* WITH CODING INFORMATION REGARDING THEIR PRODUCTS IN ORDER FOR THEM TO SAVE
* TIME. AS A RESULT, ABOV SEMICONDUCTOR DISCLAIMS ALL LIABILITIES FROM ANY
* DIRECT, INDIRECT OR CONSEQUENTIAL DAMAGES WITH RESPECT TO ANY CLAIMS ARISING
* FROM THE CONTENT OF SUCH FIRMWARE AND/OR THE USE MADE BY CUSTOMERS OF THE
* CODING INFORMATION CONTAINED HEREIN IN CONNECTION WITH THEIR PRODUCTS.
*
*******************************************************************************/

/*******************************************************************************
* Pre-processor Definition & Macro
*******************************************************************************/
#include "TestCode_WDT_Reload.h"
#include "main_conf.h"
/*******************************************************************************
* Private Typedef
*******************************************************************************/

/*******************************************************************************
* Private Variable
*******************************************************************************/
static uint32_t MilliSec;

/*******************************************************************************
* Private Function Prototype
*******************************************************************************/

/*******************************************************************************
* Public Function
*******************************************************************************/

/*******************************************************************************
* @brief      SysTick interrupt handler @ 1000Hz(1ms)
* @param      None
* @return     None
*******************************************************************************/
void SysTick_Handler_WdtReload(void) {
	if (MilliSec) {
		MilliSec--;
	}
}

/**********************************************************************
 * @brief		Watch-dog Reset Test
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void Main_WdtReload(void) {
	int i;
	WDT_CFG_Type WdtCfg;

	// Test Pin setting PE0
	HAL_GPIO_ConfigOutput(PE, 0, PCU_MODE_PUSH_PULL);
	HAL_GPIO_ConfigPullup(PE, 0, PCU_PUPD_DISABLE);
	HAL_GPIO_ClearPin(PE, _BIT(0));

	HAL_SCU_LSI_ClockConfig(LSI_EN);								// Enable LSI Clock
	HAL_SCU_WDT_ClockConfig(WDTCLK_WDTRC);					// 31250Hz clock as watch-dog clock

	WdtCfg.EnableReset = TRUE;
	WdtCfg.ClockDivider = WDT_DIVIDER_4;  			// WDTCLK = WDTCLKIN / PRS = 7812Hz
	WdtCfg.Data = 78120 - 1; 						// 10.0s @ 7812Hz
	WdtCfg.WindowData = 78120 - 1;					// 10.0s @ 7812Hz
	HAL_WDT_Init(&WdtCfg);								// Setting WDT configuration

	SysTick_Config(SystemCoreClock/1000);  			// 1ms interrupt
	__enable_irq();

	HAL_SCU_SetResetSrc(RST_WDTRST, ENABLE);		    // Enable WDT Reset
	HAL_SCU_ClearResetStatus(0xFF); 					// Clear all reset status

	_DBG("Watch-dog Reset Counter reloads WDT->DR value every 5 sec. (No Reset)\r\n");
	HAL_WDT_Start();									// Watch-dog Start

	for(i = 0; i <5; i++) {
		MilliSec = 5000;
		while(MilliSec);
		HAL_WDT_ReloadTimeCounter(ENABLE);					// Every 5 seconds, the same WDT counter value as the DR register value is reloaded.
	}

}

/* --------------------------------- End Of File ------------------------------ */
