/*****************************************************************************
* @file     TestCode_WDT_Reset1.c
* @brief    Contains the external function prototypes for TestCode
* @author   AE Team, ABOV Semiconductor Co., Ltd.
* @version  V0.0.1
* @date     31. MAY. 2019
*
* Copyright(C) 2019, ABOV Semiconductor
* All rights reserved.
*
*
********************************************************************************
* DISCLAIMER
*
* THE PRESENT FIRMWARE WHICH IS FOR GUIDANCE ONLY AIMS AT PROVIDING CUSTOMERS
* WITH CODING INFORMATION REGARDING THEIR PRODUCTS IN ORDER FOR THEM TO SAVE
* TIME. AS A RESULT, ABOV SEMICONDUCTOR DISCLAIMS ALL LIABILITIES FROM ANY
* DIRECT, INDIRECT OR CONSEQUENTIAL DAMAGES WITH RESPECT TO ANY CLAIMS ARISING
* FROM THE CONTENT OF SUCH FIRMWARE AND/OR THE USE MADE BY CUSTOMERS OF THE
* CODING INFORMATION CONTAINED HEREIN IN CONNECTION WITH THEIR PRODUCTS.
*
*******************************************************************************/

/*******************************************************************************
* Pre-processor Definition & Macro
*******************************************************************************/
#include "TestCode_WDT_Reset1.h"
#include "main_conf.h"

/*******************************************************************************
* Private Typedef
*******************************************************************************/

/*******************************************************************************
* Private Variable
*******************************************************************************/
static uint32_t MilliSec;

/*******************************************************************************
* Private Function Prototype
*******************************************************************************/

/*******************************************************************************
* Public Function
*******************************************************************************/

/*******************************************************************************
* @brief      SysTick interrupt handler @ 1000Hz(1ms)
* @param      None
* @return     None
*******************************************************************************/
void SysTick_Handler_WdtReset1(void) {
	if (MilliSec) {
		MilliSec--;
	}
}

/**********************************************************************
 * @brief		Watch-dog Reset Test
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void Main_WdtReset1(void) {
	int i;
	WDT_CFG_Type WdtCfg;

	// Test Pin setting PE0
	HAL_GPIO_ConfigOutput(PE, 0, PCU_MODE_PUSH_PULL);
	HAL_GPIO_ConfigPullup(PE, 0, PCU_PUPD_DISABLE);
	HAL_GPIO_ClearPin(PE, _BIT(0));

	HAL_SCU_LSI_ClockConfig(LSI_EN);								// Enable LSI Clock
	HAL_SCU_WDT_ClockConfig(WDTCLK_WDTRC);					// 31250Hz clock as watch-dog clock

	WdtCfg.EnableReset = TRUE;
	WdtCfg.ClockDivider = WDT_DIVIDER_4;  			// WDTCLK = WDTCLKIN / PRS = 7812Hz
#if 1
    WdtCfg.Data = (78120/2) - 1; 					// 5.0s @ 7812Hz
	WdtCfg.WindowData = 78120 - 1;					// 10.0s @ 7812Hz
#else
	WdtCfg.Data = (78120) - 1; 					    // 5.0s @ 7812Hz
	WdtCfg.WindowData = (78120/2) - 1;				// 10.0s @ 7812Hz
#endif
	HAL_WDT_Init(&WdtCfg);								// Setting WDT configuration

	SysTick_Config(SystemCoreClock/1000);  			// 1ms interrupt
	__enable_irq();


    cputs("--- DISABLE Peripherals Reset Disable (WDT / SCU)\r\n");
    SCU->PRER1 &= ~(SCU_PRER1_WDT_Msk | SCU_PRER1_SCU_Msk);

#if 0
    // for WDT Reset Masking Function
	SCU_SetResetSrc(RST_WDTRST, ENABLE);		    // Enable WDT Reset
	SCU_ClearResetStatus(0xFF); 					// Clear all reset status
#endif

	cputs("Watch-dog Reset will be counted after 5 sec ...\r\n");
	HAL_WDT_Start();									// Watch-dog Start

	for(i = 0; i <5; i++) {
		MilliSec = 500;
		while(MilliSec);
		HAL_GPIO_SetPin(PE, _BIT(0));
		MilliSec = 500;
		while(MilliSec);
		HAL_GPIO_ClearPin(PE, _BIT(0));
	}
}

/* --------------------------------- End Of File ------------------------------ */
