/*****************************************************************************
* @file     TestCode_WDT_UnderflowAndWinMatch.c
* @brief    Contains the external function prototypes for TestCode
* @author   AE Team, ABOV Semiconductor Co., Ltd.
* @version  V0.0.1
* @date     31. MAY. 2019
*
* Copyright(C) 2019, ABOV Semiconductor
* All rights reserved.
*
*
********************************************************************************
* DISCLAIMER
*
* THE PRESENT FIRMWARE WHICH IS FOR GUIDANCE ONLY AIMS AT PROVIDING CUSTOMERS
* WITH CODING INFORMATION REGARDING THEIR PRODUCTS IN ORDER FOR THEM TO SAVE
* TIME. AS A RESULT, ABOV SEMICONDUCTOR DISCLAIMS ALL LIABILITIES FROM ANY
* DIRECT, INDIRECT OR CONSEQUENTIAL DAMAGES WITH RESPECT TO ANY CLAIMS ARISING
* FROM THE CONTENT OF SUCH FIRMWARE AND/OR THE USE MADE BY CUSTOMERS OF THE
* CODING INFORMATION CONTAINED HEREIN IN CONNECTION WITH THEIR PRODUCTS.
*
*******************************************************************************/

/*******************************************************************************
* Pre-processor Definition & Macro
*******************************************************************************/
#include "TestCode_WDT_UnderflowAndWinMatch.h"
#include "main_conf.h"

/*******************************************************************************
* Private Typedef
*******************************************************************************/

/*******************************************************************************
* Private Variable
*******************************************************************************/
static uint32_t MilliSec;       // Systick Counter Value

static uint8_t wdtf_uf;  				// underflow flag
static uint8_t wdtf_wmf; 			    // window match flag
static int gCnt = 0;

/*******************************************************************************
* Private Function Prototype
*******************************************************************************/

/*******************************************************************************
* Public Function
*******************************************************************************/

/*******************************************************************************
* @brief      SysTick interrupt handler @ 1000Hz(1ms)
* @param      None
* @return     None
*******************************************************************************/
void SysTick_Handler_WdtUnderflowAndWinMatch(void) {
	if (MilliSec) {
		MilliSec--;
	}
}

/*******************************************************************************
* @brief      Watchdog interrupt handler
* @param      None
* @return     None
*******************************************************************************/
void WDT_Handler_WdtUnderflowAndWinMatch(void) {
	uint32_t status;

	status = HAL_WDT_GetStatus();						// Get WDT Status

	if (status & WDT_SR_UNFIFLAG){  				// UNFIFLAG
		cputs("Under-Flow Occurred!\r\n");
		if (wdtf_uf == 0){
			wdtf_uf = 1;
			HAL_GPIO_ClearPin(PE, _BIT(0));
		} else {
			wdtf_uf = 0;
			HAL_GPIO_SetPin(PE, _BIT(0));
		}
		HAL_WDT_ClearStatus(WDT_STATUS_UNDERFLOW_INT);
	}

	if (status & WDT_SR_WINMIFLAG) { 				// WINMIFLAG
		cputs("Window Match Occurred!\r\n");

		if (wdtf_wmf == 0){
			wdtf_wmf = 1;
			HAL_GPIO_ClearPin(PE, _BIT(1));
		} else {
			wdtf_wmf = 0;
			HAL_GPIO_SetPin(PE, _BIT(1));
		}
		HAL_WDT_ClearStatus(WDT_STATUS_WINDOW_MATCH_INT);
	}

	gCnt++;
	if(gCnt == 10) {
		cputs("WDT STOP \r\n");
		HAL_WDT_DeInit();
	}
}
/**********************************************************************
 * @brief		Watch-dog Underflow and Window Match Interrupt Test
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void Main_WdtUnderflowAndWinMatch(int TestNum) {
	WDT_CFG_Type WdtCfg;

	// Test Pin setting PE0
	HAL_GPIO_ConfigOutput(PE, 0, PCU_MODE_PUSH_PULL);
	HAL_GPIO_ConfigPullup(PE, 0, PCU_PUPD_DISABLE);
	HAL_GPIO_ClearPin(PE, _BIT(0));

	// Test Pin setting PE1
	HAL_GPIO_ConfigOutput(PE, 1, PCU_MODE_PUSH_PULL);
	HAL_GPIO_ConfigPullup(PE, 1, PCU_PUPD_DISABLE);
	HAL_GPIO_ClearPin(PE, _BIT(1));

	// Configured Watchdog Timing and operation mode
	HAL_SCU_LSI_ClockConfig(LSI_EN);									// Enable LSI Clock
	HAL_SCU_WDT_ClockConfig(WDTCLK_WDTRC);						// 31250Hz clock as watch-dog clock

	WdtCfg.EnableReset = FALSE;							// WDT Reset Disable
	WdtCfg.ClockDivider = WDT_DIVIDER_4;  				// WDTCLK = WDTCLKIN / PRS = 7812Hz

    switch(TestNum) {
        case 1:
            WdtCfg.Data = 78120 - 1; 					// 10s @ 7812Hz
            WdtCfg.WindowData = (78120/2) - 1;			// 5s @ 7812Hz
            break;
        case 2:
            WdtCfg.Data = (78120/2) - 1; 				// 5s @ 7812Hz
            WdtCfg.WindowData = 78120 - 1;				// 10s @ 7812Hz
            break;
        case 3:
            WdtCfg.Data = (78120/2) - 1; 				// 5s @ 7812Hz
            WdtCfg.WindowData = (78120/2) - 1;			// 5s @ 7812Hz
            break;
        default:
            break;
    }

	HAL_WDT_Init(&WdtCfg);  // Initialized WDT


	HAL_WDT_ConfigureInterrupt(WDT_INTCFG_WINMI_ENABLE, WDT_INTCFG_UNFI_ENABLE);    // WDT Interrupt Setting
    NVIC_SetPriority(WDT_IRQn, 3);
    NVIC_EnableIRQ(WDT_IRQn);

	__enable_irq();

	HAL_WDT_Start();    // Start WDT

	while(1);
}

/* --------------------------------- End Of File ------------------------------ */
