/*****************************************************************************
* @file     TestCode_WT_ClrSrc.c
* @brief    Contains the external function prototypes for TestCode
* @author   AE Team, ABOV Semiconductor Co., Ltd.
* @version  V0.0.1
* @date     16. MAY. 2019
*
* Copyright(C) 2019, ABOV Semiconductor
* All rights reserved.
*
*
********************************************************************************
* DISCLAIMER
*
* THE PRESENT FIRMWARE WHICH IS FOR GUIDANCE ONLY AIMS AT PROVIDING CUSTOMERS
* WITH CODING INFORMATION REGARDING THEIR PRODUCTS IN ORDER FOR THEM TO SAVE
* TIME. AS A RESULT, ABOV SEMICONDUCTOR DISCLAIMS ALL LIABILITIES FROM ANY
* DIRECT, INDIRECT OR CONSEQUENTIAL DAMAGES WITH RESPECT TO ANY CLAIMS ARISING
* FROM THE CONTENT OF SUCH FIRMWARE AND/OR THE USE MADE BY CUSTOMERS OF THE
* CODING INFORMATION CONTAINED HEREIN IN CONNECTION WITH THEIR PRODUCTS.
*
*******************************************************************************/

/*******************************************************************************
* Pre-processor Definition & Macro
*******************************************************************************/
#include "TestCode_WT_ClkSrc.h"
#include "main_conf.h"

/*******************************************************************************
* Private Typedef
*******************************************************************************/

/*******************************************************************************
* Private Variable
*******************************************************************************/
static uint8_t WtFlag;  			// underflow flag

/*******************************************************************************
* Private Function Prototype
*******************************************************************************/

/*******************************************************************************
* Public Function
*******************************************************************************/
void WT_IRQHandler_WtClkSrc(void) {
	FlagStatus status;

	status = HAL_WT_GetStatus();				// Get WT Interrupt Status

	if (status){  									// WT Interrupt Flag
		cprintf("Watch-Timer Interrupt Occurred!\r\n");
		if (WtFlag == 0){
			WtFlag = 1;
			HAL_GPIO_ClearPin(PE, _BIT(0));
		} else {
			WtFlag = 0;
			HAL_GPIO_SetPin(PE, _BIT(0));
		}
		HAL_WT_ClearStatus();
	}
}

/**********************************************************************
 * @brief		Watch-Timer clock source selection and Interrupt Operation
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void Main_WtClkSrc(void) {
	uint8_t sel;

	WT_CFG_Type WtCfg;
	uint32_t WtClk;


	// Select WT Clock
	cputs(" a: LSI/16(31.250kHz, WDTRC)\r\n b: LSE(32.768kHz)\r\n c: MCLK\r\n d: HSI(32MHz)\r\n e: HSE(XTAL)\r\n f: PLL Clock(48MHz)\r\n");
	while(1)
	{
        sel = cgetchar();
        if (sel > 0 && sel <= 0x7F) {
            cputs(">>");
            if(sel == 'a') {
                cputs("LSI is WT Clock source\r\n");
                HAL_SCU_LSI_ClockConfig(LSI_EN);									// ENABLE LSI (500kHz)
                HAL_SCU_WT_ClockConfig(WTCLK_MCCR3);							// Setting Wtclock source selected by MCCR3
                HAL_SCU_MiscClockConfig(3, WT_TYPE, CLKSRC_LSI, 16);			// 500kHz/16 = 31250Hz
                WtCfg.ClockDivider = WT_DIVIDER_2_14_MUL_WTDR;		// 31250Hz/2^14(WT_DR+1) = 0.95Hz (1.04s)
                WtClk = 1;											//	WT Period = 1.04s
                break;
            } else if (sel == 'b') {
                cputs("LSE is WT Clock source\r\n");
                HAL_SCU_LSE_ClockConfig(LSE_EN);									// ENABLE LSE (32768kHz)
                HAL_SCU_WT_ClockConfig(WTCLK_MCCR3);							// Setting Wtclock source selected by MCCR3
                HAL_SCU_MiscClockConfig(3, WT_TYPE, CLKSRC_LSE, 2);			// 32768Hz/2 = 16384Hz
                WtCfg.ClockDivider = WT_DIVIDER_2_14;  				// WTCLK = WTCLKIN/PRS = 16384/(2^14) = 1Hz
                WtClk = 1;											// WT Period = 1.00s
                break;
            } else if (sel == 'c') {
                cputs("MCLK is WT Clock source\r\n");				// MCLK (48MHz)
                HAL_SCU_MiscClockConfig(3, WT_TYPE, CLKSRC_MCLK, 127);			// 48MHz/127 = 37.7952kHz
                HAL_SCU_WT_ClockConfig(WTCLK_MCCR3);							// Setting Wtclock source selected by MCCR3
                WtCfg.ClockDivider = WT_DIVIDER_2_14_MUL_WTDR;  	// WTCLK = WTCLKIN/PRS = 37.7952kHz/(2^14*(WT_DR+1))
                WtClk = 23;											// WT Period = 37.7952kHz/(2^14*(23+1)) = 1.04s
                break;
            } else if (sel == 'd') {
                cputs("HSI is WT Clock source\r\n");					// HSI (32MHz)
                HAL_SCU_HSI_ClockConfig(HSI_EN);									// Enable HSI (32MHz)
                HAL_SCU_MiscClockConfig(3, WT_TYPE, CLKSRC_HSI, 64);			// 32MHz/64 = 500000Hz
                HAL_SCU_WT_ClockConfig(WTCLK_MCCR3);							// Setting Wtclock source selected by MCCR3
                WtCfg.ClockDivider = WT_DIVIDER_2_14_MUL_WTDR;  	// WTCLK = WTCLKIN/PRS = 500000Hz/(2^14*(WT_DR+1))
                WtClk = 29;											// WT Period = 500000Hz/(2^14*(29+1)) = 1.01s
                break;
            } else if (sel == 'e') {
                cputs("HSE is WT Clock source\r\n");					// HSE (8MHz)
                HAL_SCU_HSE_ClockConfig(HSE_EN);									// Enable HSI (32MHz)
                HAL_SCU_MiscClockConfig(3, WT_TYPE, CLKSRC_HSE, 8);			// 8MHz/8 = 1MHz
                HAL_SCU_WT_ClockConfig(WTCLK_MCCR3);							// Setting Wtclock source selected by MCCR3
                WtCfg.ClockDivider = WT_DIVIDER_2_14_MUL_WTDR;  	// WTCLK = WTCLKIN/PRS = 1MHz/(2^14*(WT_DR+1))
                WtClk = 60;											// WT Period = 500000Hz/(2^14*(60+1)) = 1.00s
                break;
            } else if (sel == 'f') {
                cputs("PLL is WT Clock source\r\n");					// PLL (48MHz)
                HAL_SCU_MiscClockConfig(3, WT_TYPE, CLKSRC_PLL, 127);			// 48MHz/127 = 37.7952kHz
                HAL_SCU_WT_ClockConfig(WTCLK_MCCR3);							// Setting Wtclock source selected by MCCR3
                WtCfg.ClockDivider = WT_DIVIDER_2_14_MUL_WTDR;  	// WTCLK = WTCLKIN/PRS = 37.7952kHz/(2^14*(WT_DR+1))
                WtClk = 23;											// WT Period = 37.7952kHz/(2^14*(23+1)) = 1.04s
                break;
            }
        }
	}

	WtCfg.MatchData = WtClk; 							// Setting Period
	HAL_WT_Init(&WtCfg);								// Setting WT configuration

	// WT Interrupt Setting
	HAL_WT_ConfigInterrupt(ENABLE);
    NVIC_SetPriority(WT_IRQn, 3);
    NVIC_EnableIRQ(WT_IRQn);
	__enable_irq();

	cputs("Watch-timer event will be counted after 1 sec ...\r\n\r\n");
	HAL_WT_Start();											// WT Start

	while(1) {}
}

/* --------------------------------- End Of File ------------------------------ */
