/*****************************************************************************
* @file     TestCode_WT_Interrupt.c
* @brief    Contains the external function prototypes for TestCode
* @author   AE Team, ABOV Semiconductor Co., Ltd.
* @version  V0.0.1
* @date     16. MAY. 2019
*
* Copyright(C) 2019, ABOV Semiconductor
* All rights reserved.
*
*
********************************************************************************
* DISCLAIMER
*
* THE PRESENT FIRMWARE WHICH IS FOR GUIDANCE ONLY AIMS AT PROVIDING CUSTOMERS
* WITH CODING INFORMATION REGARDING THEIR PRODUCTS IN ORDER FOR THEM TO SAVE
* TIME. AS A RESULT, ABOV SEMICONDUCTOR DISCLAIMS ALL LIABILITIES FROM ANY
* DIRECT, INDIRECT OR CONSEQUENTIAL DAMAGES WITH RESPECT TO ANY CLAIMS ARISING
* FROM THE CONTENT OF SUCH FIRMWARE AND/OR THE USE MADE BY CUSTOMERS OF THE
* CODING INFORMATION CONTAINED HEREIN IN CONNECTION WITH THEIR PRODUCTS.
*
*******************************************************************************/

/*******************************************************************************
* Pre-processor Definition & Macro
*******************************************************************************/
#include "TestCode_WT_Interrupt.h"
#include "main_conf.h"

/*******************************************************************************
* Private Typedef
*******************************************************************************/

/*******************************************************************************
* Private Variable
*******************************************************************************/
static uint8_t WtFlag;  			// underflow flag
int test_cnt;
int i;
/*******************************************************************************
* Private Function Prototype
*******************************************************************************/

/*******************************************************************************
* Public Function
*******************************************************************************/
void WT_IRQHandler_WtInterrupt(void) {
	FlagStatus status;

	status = HAL_WT_GetStatus();				// Get WT Interrupt Status

	if (status){  									// WT Interrupt Flag
		_DBG("Watch-Timer Interrupt Occurred!\r\n");
		if (WtFlag == 0){
			WtFlag = 1;
			HAL_GPIO_ClearPin(PE, _BIT(0));
		} else {
			WtFlag = 0;
			HAL_GPIO_SetPin(PE, _BIT(0));
		}
		HAL_WT_ClearStatus();
	}

    test_cnt++;
}

/**********************************************************************
 * @brief		Watch-Timer Interrupt Test
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void Main_WtInterruptTest(void) {
	WT_CFG_Type WtCfg;
    uint32_t *wt_reg;

	// Test Pin setting PE0
	HAL_GPIO_ConfigOutput(PE, 0, PCU_MODE_PUSH_PULL);
	HAL_GPIO_ConfigPullup(PE, 0, PCU_PUPD_DISABLE);
	HAL_GPIO_ClearPin(PE, _BIT(0));

	// Configured Watchdog Timing and operation mode
	HAL_SCU_LSI_ClockConfig(LSI_EN);										// Enable LSI Clock
	HAL_SCU_WT_ClockConfig(WTCLK_WDTRC);								// 31250Hz clock as Watch-Timer clock

	WtCfg.ClockDivider = WT_DIVIDER_2_14_MUL_WTDR;  		// WTCLK = WTCLKIN / 2^14(WT_DR+1) = 0.95Hz = 1.04s
	WtCfg.MatchData = 1; 									// WT_DR = 1 (1.04s @ 0.95Hz)
	HAL_WT_Init(&WtCfg);										// WT Configuration Setting

	// WT Interrupt Setting
	HAL_WT_ConfigInterrupt(ENABLE);
    NVIC_SetPriority(WT_IRQn, 3);
    NVIC_EnableIRQ(WT_IRQn);
	__enable_irq();

	HAL_WT_Start();												// WT Start

    // WT Value Check
    wt_reg = (uint32_t *)WT_BASE;
    // WT
    cprintf("\r\nWT (Before Reset)\r\n");
    for(i = 0; i < 0xC; i += 4) {
        cprintf("0x%08X: 0x%08X\r\n", (WT_BASE + i), *(volatile unsigned int *)(wt_reg + i));
    }

	while(1) {
        if (test_cnt == 10) {

            // forcely reset
            // Software Reset Test
                if ((HAL_SCU_GetResetSrc()&SCU_RSSR_SWRST_Msk) == SCU_RSSR_SWRST_Msk)
                    HAL_SCU_SetResetSrc(RST_SWRST, DISABLE);

                cprintf("\r\n!!WARM RESET!!\r\n\r\n");
                for(i =0; i < 0x100000; i++);

                HAL_SCU_SetResetSrc(RST_SWRST, ENABLE);																				// Enable Software Reset
                SCU->SCR = (SCU_SCR_WTIDKY << SCU_SCR_WTIDKY_Pos) | (1 << SCU_SCR_SWRST_Pos);		// SW Reset Operation
        }
    }
}

/* --------------------------------- End Of File ------------------------------ */
