/*****************************************************************************
* @file     TestCode_WT_Polling.c
* @brief    Contains the external function prototypes for TestCode
* @author   AE Team, ABOV Semiconductor Co., Ltd.
* @version  V0.0.1
* @date     16. MAY. 2019
*
* Copyright(C) 2019, ABOV Semiconductor
* All rights reserved.
*
*
********************************************************************************
* DISCLAIMER
*
* THE PRESENT FIRMWARE WHICH IS FOR GUIDANCE ONLY AIMS AT PROVIDING CUSTOMERS
* WITH CODING INFORMATION REGARDING THEIR PRODUCTS IN ORDER FOR THEM TO SAVE
* TIME. AS A RESULT, ABOV SEMICONDUCTOR DISCLAIMS ALL LIABILITIES FROM ANY
* DIRECT, INDIRECT OR CONSEQUENTIAL DAMAGES WITH RESPECT TO ANY CLAIMS ARISING
* FROM THE CONTENT OF SUCH FIRMWARE AND/OR THE USE MADE BY CUSTOMERS OF THE
* CODING INFORMATION CONTAINED HEREIN IN CONNECTION WITH THEIR PRODUCTS.
*
*******************************************************************************/

/*******************************************************************************
* Pre-processor Definition & Macro
*******************************************************************************/
#include "TestCode_WT_Polling.h"
#include "main_conf.h"

/*******************************************************************************
* Private Typedef
*******************************************************************************/

/*******************************************************************************
* Private Variable
*******************************************************************************/
static uint8_t WtFlag;  			// underflow flag

/*******************************************************************************
* Private Function Prototype
*******************************************************************************/

/*******************************************************************************
* Public Function
*******************************************************************************/

/**********************************************************************
 * @brief		Watch-Timer Polling Test
 * @param[in]	None
 * @return 		None
 **********************************************************************/
void Main_WtPollingTest(void) {
	WT_CFG_Type WtCfg;
	FlagStatus	status;

	// Test Pin setting PE0
	HAL_GPIO_ConfigOutput(PE, 0, PCU_MODE_PUSH_PULL);
	HAL_GPIO_ConfigPullup(PE, 0, PCU_PUPD_DISABLE);
	HAL_GPIO_ClearPin(PE, _BIT(0));

	// Configured Watchdog Timing and operation mode
	HAL_SCU_LSI_ClockConfig(LSI_EN);									// Enable LSI Clock
	HAL_SCU_WT_ClockConfig(WTCLK_WDTRC);							// 31250Hz clock as Watch-Timer clock

	WtCfg.ClockDivider = WT_DIVIDER_2_14_MUL_WTDR;  		// WTCLK = WTCLKIN / 2^14(WT_DR+1) = 0.95Hz = 1.04s
	WtCfg.MatchData = 1; 									// 1.04s @ 095Hz
	HAL_WT_Init(&WtCfg);									// WT Configuration Setting

	HAL_WT_Start();											// WT Start

    WtFlag = 0;

	while(1) {
		status = HAL_WT_GetStatus();		// Get WT Flag

		if (status){
			cprintf("Watch-Timer Flag!\r\n");
			if (WtFlag == 0){
				WtFlag=1;
				HAL_GPIO_ClearPin(PE, _BIT(0));
			} else {
				WtFlag=0;
				HAL_GPIO_SetPin(PE, _BIT(0));
			}
			HAL_WT_ClearStatus();						// Clear WT Flag
		}
	}
}

/* --------------------------------- End Of File ------------------------------ */
